@@ -116,7 +116,7 @@ static void gretl_abort (char *line)
 
 static void noalloc (void)
 {
-    fputs(_("Out of memory\n"), stderr);
+    fputs(_("Out of memory!\n"), stderr);
     exit(EXIT_FAILURE);
 }
 
@@ -462,7 +462,7 @@ int main (int argc, char *argv[])
 	strcpy(given_file, filearg);
 	strcpy(paths.datfile, filearg);
 
-	ftype = detect_filetype(paths.datfile, &paths, prn);
+	ftype = detect_filetype(paths.datfile, &paths);
 
 	switch (ftype) {
 	case GRETL_UNRECOGNIZED:
@@ -674,7 +674,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     } else if (cmd->opt & OPT_O) {
 	ftype = GRETL_ODBC;
     } else {
-	ftype = detect_filetype(datfile, &paths, prn);
+	ftype = detect_filetype(datfile, &paths);
     }
 
     dbdata = (ftype == GRETL_NATIVE_DB || ftype == GRETL_NATIVE_DB_WWW ||
@@ -726,7 +726,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 
     data_status = 1;
 
-    if (pdinfo->v > 0 && !dbdata) {
+    if (pdinfo->v > 0 && !dbdata && !(cmd->opt & OPT_Q)) {
 	varlist(pdinfo, prn);
     }
 
@@ -860,44 +860,23 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     switch (cmd->ci) {
 
     case DELEET:
+	k = 0;
 	if (cmd->opt & OPT_D) {
 	    err = db_delete_series_by_name(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	if (get_matrix_by_name(cmd->param)) {
-	    err = user_matrix_destroy_by_name(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	if (get_string_by_name(cmd->param)) {
-	    err = delete_saved_string(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	if (get_list_by_name(cmd->extra)) {
+	} else if (*cmd->param != '\0') {
+	    err = gretl_delete_var_by_name(cmd->param, prn);
+	} else if (get_list_by_name(cmd->extra)) {
 	    err = delete_list_by_name(cmd->extra);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
+	} else {
+	    err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
 					    &k, prn);
+	}
 	if (err) {
 	    errmsg(err, prn);
-	} else {
-	    if (k) {
-		pputs(prn, _("Take note: variables have been renumbered"));
-		pputc(prn, '\n');
-		maybe_list_vars(pdinfo, prn);
-	    }
+	} else if (k) {
+	    pputs(prn, _("Take note: variables have been renumbered"));
+	    pputc(prn, '\n');
+	    maybe_list_vars(pdinfo, prn);
 	}
 	break;
 
@@ -925,7 +904,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	}
 
 	if (err) {
-	    pputs(prn, _("gnuplot command failed\n"));
+	    errmsg(err, prn);
 	} else if (batch) {
 	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
 	}
@@ -941,8 +920,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
 
     case NULLDATA:
-	k = gretl_int_from_string(cmd->param, (const double **) *pZ, 
-				  pdinfo, &err);
+	k = gretl_int_from_string(cmd->param, &err);
 	if (!err && k < 2) {
 	    err = 1;
 	}
@@ -953,7 +931,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	cli_clear_data(cmd, pZ, pdinfo, models);
 	err = open_nulldata(pZ, pdinfo, data_status, k, prn);
 	if (err) { 
-	    pputs(prn, _("Failed to create empty data set\n"));
+	    errmsg(err, prn);
 	} else {
 	    data_status = 1;
 	}
