@@ -28,7 +28,7 @@
 # include <windows.h>
 #endif
 
-#include "gtkplot-lite.h"
+#include "psplot.h"
 
 typedef struct {
     int n;
@@ -76,10 +76,9 @@ static int dump_boxplot (PLOTGROUP *grp);
 #ifdef G_OS_WIN32
 static int cb_copy_image (gpointer data);
 #else
-int plot_to_xpm (const char *fname, gpointer data);
+static int plot_to_xpm (const char *fname, gpointer data);
 #endif
 
-
 const char *get_boxdump_name (void)
 {
     return boxplottmp;
@@ -133,22 +132,16 @@ box_key_handler (GtkWidget *w, GdkEventKey *key, gpointer data)
 {
     if (key->keyval == GDK_q) {
 	gtk_widget_destroy(w);
-    } else if (key->keyval == GDK_s) {
-        file_selector(_("Save boxplot file"), SAVE_BOXPLOT_EPS, 
-		      FSEL_DATA_MISC, data);
     } else if (key->keyval == GDK_p) {  
 	six_numbers(data);
     }
+
 #ifdef G_OS_WIN32
-    else if (key->keyval == GDK_c) {  
+    if (key->keyval == GDK_c) {  
 	cb_copy_image(data);
     }
-#else
-    else if (key->keyval == GDK_c) { 
-        file_selector(_("Save boxplot file"), SAVE_BOXPLOT_XPM, 
-		      FSEL_DATA_MISC, data);	
-    }
 #endif
+
     return TRUE;
 }
 
@@ -167,11 +160,11 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
 	    grp->saved = 1;
 	}
     } else if (!strcmp(item, _("Save as EPS..."))) {
-        file_selector(_("Save boxplot file"), SAVE_BOXPLOT_EPS, 
-		      FSEL_DATA_MISC, ptr);
+        file_selector_with_parent(_("Save boxplot file"), SAVE_BOXPLOT_EPS, 
+				  FSEL_DATA_MISC, ptr, grp->window);
     } else if (!strcmp(item, _("Save as PS..."))) {
-        file_selector(_("Save boxplot file"), SAVE_BOXPLOT_PS, 
-		      FSEL_DATA_MISC, ptr);
+        file_selector_with_parent(_("Save boxplot file"), SAVE_BOXPLOT_PS, 
+				  FSEL_DATA_MISC, ptr, grp->window);
     } else if (!strcmp(item, _("Help"))) {
 	context_help(NULL, GINT_TO_POINTER(BXPLOT));
     } else if (!strcmp(item, _("Close"))) { 
@@ -187,8 +180,8 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
     }
 #else
     else if (!strcmp(item, _("Save as XPM..."))) {
-        file_selector(_("Save boxplot file"), SAVE_BOXPLOT_XPM, 
-		      FSEL_DATA_MISC, ptr);
+        file_selector_with_parent(_("Save boxplot file"), SAVE_BOXPLOT_XPM, 
+				  FSEL_DATA_MISC, ptr, grp->window);
     }
 #endif
 
@@ -198,7 +191,7 @@ static gint box_popup_activated (GtkWidget *w, gpointer data)
     return TRUE;
 }
 
-static GtkWidget *build_menu (PLOTGROUP *grp)
+static GtkWidget *build_boxplot_menu (PLOTGROUP *grp)
 {
     GtkWidget *menu, *item;    
     static char *items[] = {
@@ -241,8 +234,11 @@ static gint box_popup (GtkWidget *widget, GdkEventButton *event,
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
 
-    if (grp->popup) g_free(grp->popup);
-    grp->popup = build_menu(grp);
+    if (grp->popup) {
+	g_free(grp->popup);
+    }
+
+    grp->popup = build_boxplot_menu(grp);
     gtk_menu_popup(GTK_MENU(grp->popup), NULL, NULL, NULL, NULL,
 		   event->button, event->time);
     return TRUE;
@@ -250,24 +246,24 @@ static gint box_popup (GtkWidget *widget, GdkEventButton *event,
 
 static void
 setup_text (GtkWidget *area, GdkPixmap *pixmap,
-	    GdkGC *gc, GtkPlotPC *pc, const char *text, 
+	    GdkGC *gc, PSPlot *ps, const char *text, 
 	    double x, double y, unsigned just)
 {
-    if (pc != NULL) {
+    if (ps != NULL) {
 	GdkColor black, white;
 	
 	black.red = black.green = black.blue = 0;
 	white.red = white.green = white.blue = 65535.0;
 
-	gtk_plot_pc_draw_string (pc,
-				 x, y + 4,        /* alignment bodge */
-				 0,               /* angle */
-				 &black, &white,  /* fore, back */
-				 FALSE,           /* transparent? */
-				 GTK_PLOT_BORDER_NONE, 0, 1, 0,
-				 boxfont, boxfontsize,
-				 just,
-				 text);
+	ps_plot_draw_string (ps,
+			     x, y + 4,        /* alignment bodge */
+			     0,               /* angle */
+			     &black, &white,  /* fore, back */
+			     FALSE,           /* transparent? */
+			     PS_PLOT_BORDER_NONE, 0, 1, 0,
+			     boxfont, boxfontsize,
+			     just,
+			     text);
     } else {
 	PangoContext *context;
 	PangoLayout *pl;
@@ -293,10 +289,10 @@ setup_text (GtkWidget *area, GdkPixmap *pixmap,
 
 static void 
 draw_line (double *points, GtkWidget *area, GdkPixmap *pixmap,
-	   GdkGC *gc, GtkPlotPC *pc)
+	   GdkGC *gc, PSPlot *ps)
 {
-    if (pc != NULL) {
-	gtk_plot_pc_draw_line(pc, points[0], points[1], points[2], points[3]); 
+    if (ps != NULL) {
+	ps_plot_draw_line(ps, points[0], points[1], points[2], points[3]); 
     } else {
 	gdk_draw_line(pixmap, gc, points[0], points[1], points[2], points[3]);
     }
@@ -305,12 +301,12 @@ draw_line (double *points, GtkWidget *area, GdkPixmap *pixmap,
 static void
 draw_outlier (double x, double y,
 	      GtkWidget *area, GdkPixmap *pixmap, 
-	      GdkGC *gc, GtkPlotPC *pc)
+	      GdkGC *gc, PSPlot *ps)
 {
     int fullcircle = 360 * 64;
 
-    if (pc != NULL) {
-	gtk_plot_pc_draw_circle(pc, FALSE, x, y, 8);
+    if (ps != NULL) {
+	ps_plot_draw_circle(ps, FALSE, x, y, 8);
     } else {
 	gdk_draw_arc(pixmap, gc, FALSE, x - 4, y - 4, 8, 8, 0, fullcircle);
     }
@@ -319,11 +315,11 @@ draw_outlier (double x, double y,
 static void 
 draw_mean (double x, double y, 
 	   GtkWidget *area, GdkPixmap *pixmap, 
-	   GdkGC *gc, GtkPlotPC *pc)
+	   GdkGC *gc, PSPlot *ps)
 {
-    if (pc != NULL) {
-	gtk_plot_pc_draw_line(pc, x - 4, y, x + 4, y);
-	gtk_plot_pc_draw_line(pc, x, y + 4, x, y - 4);
+    if (ps != NULL) {
+	ps_plot_draw_line(ps, x - 4, y, x + 4, y);
+	ps_plot_draw_line(ps, x, y + 4, x, y - 4);
     } else {
 	gdk_draw_line(pixmap, gc, x - 4, y, x + 4, y);
 	gdk_draw_line(pixmap, gc, x, y + 4, x, y - 4);
@@ -355,7 +351,7 @@ place_plots (PLOTGROUP *plotgrp)
 }
 
 static void 
-gtk_boxplot_yscale (PLOTGROUP *grp, GtkPlotPC *pc)
+gtk_boxplot_yscale (PLOTGROUP *grp, PSPlot *ps)
 {
     double points[4];
     double top = (headroom / 2.0) * grp->height;
@@ -363,7 +359,7 @@ gtk_boxplot_yscale (PLOTGROUP *grp, GtkPlotPC *pc)
     gchar *tmp = NULL;
     GdkGC *gc = NULL;
 
-    if (pc == NULL) {
+    if (ps == NULL) {
 	gc = grp->window->style->fg_gc[GTK_STATE_NORMAL];
     }
 
@@ -371,42 +367,42 @@ gtk_boxplot_yscale (PLOTGROUP *grp, GtkPlotPC *pc)
     points[0] = points[2] = scalepos;
     points[1] = top;
     points[3] = bottom;
-    draw_line(points, grp->area, grp->pixmap, gc, pc);
+    draw_line(points, grp->area, grp->pixmap, gc, ps);
 
     /* draw backticks top and bottom */
     points[2] = points[0] - 5;
     points[1] = points[3] = top;
-    draw_line (points, grp->area, grp->pixmap, gc, pc);
+    draw_line (points, grp->area, grp->pixmap, gc, ps);
     points[1] = points[3] = bottom;
-    draw_line(points, grp->area, grp->pixmap, gc, pc);
+    draw_line(points, grp->area, grp->pixmap, gc, ps);
 
     /* draw backtick at middle */
     points[1] = points[3] = top + (bottom - top) / 2.0;
-    draw_line(points, grp->area, grp->pixmap, gc, pc);
+    draw_line(points, grp->area, grp->pixmap, gc, ps);
     
     /* mark three values on scale */
     tmp = g_strdup_printf("%.4g", grp->gmax);
     gretl_fix_exponent(tmp);
-    setup_text(grp->area, grp->pixmap, gc, pc, tmp, scalepos - 8, top, 
+    setup_text(grp->area, grp->pixmap, gc, ps, tmp, scalepos - 8, top, 
 	       GTK_JUSTIFY_RIGHT);
     g_free(tmp);
 
     tmp = g_strdup_printf("%.4g", grp->gmin);
     gretl_fix_exponent(tmp);
-    setup_text(grp->area, grp->pixmap, gc, pc, tmp, scalepos - 8, bottom, 
+    setup_text(grp->area, grp->pixmap, gc, ps, tmp, scalepos - 8, bottom, 
 	       GTK_JUSTIFY_RIGHT);
     g_free(tmp);
 
     tmp = g_strdup_printf("%.4g", (grp->gmax + grp->gmin) / 2.0);
     gretl_fix_exponent(tmp);
-    setup_text(grp->area, grp->pixmap, gc, pc, tmp, scalepos - 8, 
+    setup_text(grp->area, grp->pixmap, gc, ps, tmp, scalepos - 8, 
 	       top + (bottom - top) / 2.0, GTK_JUSTIFY_RIGHT);
     g_free(tmp);
 }
 
 static void 
 gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
-		  GtkStyle *style, GtkPlotPC *pc,
+		  GtkStyle *style, PSPlot *ps,
 		  int height, double boxwidth, 
 		  double gmax, double gmin, char *numbers)
 {
@@ -419,7 +415,7 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     double nameoff = headroom / 4.0;
     GdkGC *gc = NULL;
 
-    if (pc == NULL) {
+    if (ps == NULL) {
 	gc = style->fg_gc[GTK_STATE_NORMAL];
     }
 
@@ -456,12 +452,12 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 
     if (confhi == 0.) {
 	/* no notches: draw simple inter-quartile box */
-	if (pc != NULL) { 
-	    gtk_plot_pc_draw_rectangle (pc,
-					FALSE,
-					plot->xbase, uq,
-					boxwidth,
-					lq - uq);
+	if (ps != NULL) { 
+	    ps_plot_draw_rectangle (ps,
+				    FALSE,
+				    plot->xbase, uq,
+				    boxwidth,
+				    lq - uq);
 	} else {
 	    gdk_draw_rectangle (pixmap, 
 				gc, 
@@ -473,8 +469,8 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	}
     } else { 
 	/* draw notched boxes */
-	if (pc != NULL) {
-	    GtkPlotPoint points[10];
+	if (ps != NULL) {
+	    PlotPoint points[10];
 
 	    points[0].x = points[6].x = points[7].x = points[9].x = 
 		plot->xbase;
@@ -488,10 +484,10 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	    points[8].x = plot->xbase + .10 * boxwidth;
 	    points[3].x = plot->xbase + .90 * boxwidth;
 
-	    gtk_plot_pc_draw_polygon (pc,
-				      FALSE, /* filled ? */
-				      points,
-				      10);
+	    ps_plot_draw_polygon (ps,
+				  FALSE, /* filled ? */
+				  points,
+				  10);
 	} else {
 	    GdkPoint points[10];
 
@@ -519,11 +515,11 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     points[0] = plot->xbase + ((confhi > 0.)? (0.1 * boxwidth) : 0.0);
     points[1] = points[3] = median;
     points[2] = plot->xbase + ((confhi > 0.)? (0.9 * boxwidth) : boxwidth);
-    draw_line(points, area, pixmap, gc, pc); /* was whitegc */
+    draw_line(points, area, pixmap, gc, ps); /* was whitegc */
 
     /* draw '+' at mean */
     if (!na(mean)) {
-	draw_mean(xcenter, mean, area, pixmap, gc, pc);
+	draw_mean(xcenter, mean, area, pixmap, gc, ps);
     }
 
     /* insert numerical values for median and quartiles? */
@@ -532,13 +528,13 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	double x = xcenter + .55 * boxwidth;
 
 	sprintf(numstr, numbers, plot->uq);
-	setup_text (area, pixmap, gc, pc, numstr, 
+	setup_text (area, pixmap, gc, ps, numstr, 
 		    x, uq, GTK_JUSTIFY_LEFT);
 	sprintf(numstr, numbers, plot->median);
-	setup_text (area, pixmap, gc, pc, numstr, 
+	setup_text (area, pixmap, gc, ps, numstr, 
 		    x, median, GTK_JUSTIFY_LEFT);
 	sprintf(numstr, numbers, plot->lq);
-	setup_text (area, pixmap, gc, pc, numstr, 
+	setup_text (area, pixmap, gc, ps, numstr, 
 		    x, lq, GTK_JUSTIFY_LEFT);	
     }
 
@@ -546,25 +542,25 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     points[0] = points[2] = xcenter;
     points[1] = maxval;
     points[3] = uq;
-    draw_line(points, area, pixmap, gc, pc);
+    draw_line(points, area, pixmap, gc, ps);
 
     /* plus a little crossbar */
     points[0] = xcenter - 5.0;
     points[2] = xcenter + 5.0;
     points[1] = points[3] = maxval;
-    draw_line(points, area, pixmap, gc, pc);
+    draw_line(points, area, pixmap, gc, ps);
 
     /* draw line to minimum value */
     points[0] = points[2] = xcenter;
     points[1] = lq;
     points[3] = minval;
-    draw_line(points, area, pixmap, gc, pc);
+    draw_line(points, area, pixmap, gc, ps);
 
     /* plus a little crossbar */
     points[0] = xcenter - 5.0;
     points[2] = xcenter + 5.0;
     points[1] = points[3] = minval;
-    draw_line(points, area, pixmap, gc, pc);
+    draw_line(points, area, pixmap, gc, ps);
 
     /* draw outliers, if any */
     if (plot->outliers != NULL) {
@@ -575,7 +571,7 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
 	    /* fprintf(stderr, "outlier: %g\n", plot->outliers->vals[i]); */
 	    y = ybase + (gmax - plot->outliers->vals[i]) * scale;
 	    if (y == ybak) y += 1.0;
-	    draw_outlier(xcenter, y, area, pixmap, gc, pc);
+	    draw_outlier(xcenter, y, area, pixmap, gc, ps);
 	    ybak = y;
 	}
     }
@@ -584,10 +580,10 @@ gtk_area_boxplot (BOXPLOT *plot, GtkWidget *area, GdkPixmap *pixmap,
     if (plot->bool) {
 	nameoff = headroom / 3.5;
     }
-    setup_text(area, pixmap, gc, pc, plot->varname, xcenter, 
+    setup_text(area, pixmap, gc, ps, plot->varname, xcenter, 
 	       height * (1.0 - nameoff), GTK_JUSTIFY_CENTER);
     if (plot->bool) {
-	setup_text(area, pixmap, gc, pc, plot->bool, xcenter, 
+	setup_text(area, pixmap, gc, ps, plot->bool, xcenter, 
 		   height * (1.0 - headroom/6.0), GTK_JUSTIFY_CENTER);
     }
 }
@@ -788,54 +784,78 @@ median_interval (double *x, int n, double *low, double *high)
     return err;
 }
 
-int ps_print_plots (const char *fname, int flag, gpointer data) 
+static int ps_print_plots (const char *fname, int flag, gpointer data) 
 {
     PLOTGROUP *grp = (PLOTGROUP *) data;
-    GtkPlotPS *ps;
-    int i, eps = 1, orient = GTK_PLOT_PORTRAIT;
-    gdouble pscale = 0.8;
+    FILE *fp;
+    PSPlot *ps;
+    gdouble epswidth, epsheight, pscale;
+    int i, eps, orient;
 
-    if (flag == SAVE_BOXPLOT_PS) {
-	pscale = 1.2;
+    fp = gretl_fopen(fname, "w");
+    if (fp == NULL) {
+	file_write_errbox(fname);
+	return E_FOPEN;
+    }
+
+    if (flag == SAVE_BOXPLOT_EPS) {
+	eps = 1;
+	orient = PS_PLOT_PORTRAIT;
+	pscale = 0.8;
+	epswidth = pscale * grp->width;
+	epsheight = pscale * grp->height;
+    } else {
+	/* full page PS */
 	eps = 0;
-	orient = GTK_PLOT_LANDSCAPE;
+	orient = PS_PLOT_LANDSCAPE;
+	pscale = 1.2;
+	epswidth = epsheight = 0;
+    } 
+
+    ps = ps_plot_new(fname,
+		     fp, 
+		     orient, 
+		     eps, 
+		     PS_PLOT_LETTER, 
+		     epswidth, epsheight,
+		     pscale, pscale);
+
+    if (ps == NULL) {
+	fclose(fp);
+	remove(fname);
+	return E_ALLOC;
     }
 
-    ps = GTK_PLOT_PS(gtk_plot_ps_new (fname, 
-				      orient, 
-				      eps, 
-				      GTK_PLOT_LETTER, 
-				      pscale, pscale));
+    gretl_push_c_numeric_locale();
+    ps_plot_init(ps);
 
-    if (ps == NULL) return 1;
-
-    if (eps) {
-	ps->page_width = pscale * grp->width;
-	ps->page_height = pscale * grp->height;
+    for (i=0; i<grp->nplots; i++) {
+	gtk_area_boxplot(&grp->plots[i], 
+			 NULL, NULL, NULL, ps, 
+			 grp->height, grp->boxwidth, 
+			 grp->gmax, grp->gmin, grp->numbers);
     }
-
-    if (!psinit(GTK_PLOT_PC(ps))) {
-	return 1;
-    }
-
-    gtk_psfont_init();
-
-    for (i=0; i<grp->nplots; i++)
-	gtk_area_boxplot (&grp->plots[i], 
-			  NULL, NULL, NULL, &ps->pc, 
-			  grp->height, grp->boxwidth, 
-			  grp->gmax, grp->gmin, grp->numbers);
     
-    gtk_boxplot_yscale(grp, &ps->pc);
+    gtk_boxplot_yscale(grp, ps);
 
-    psleave(GTK_PLOT_PC(ps));
-
-    gtk_object_destroy(GTK_OBJECT(ps));
-    gtk_psfont_unref();
+    ps_plot_finalize(ps);
+    gretl_pop_c_numeric_locale();
+    fclose(fp);
 
     return 0;
 }
 
+int print_boxplot_group (const char *fname, int flag, gpointer data)
+{
+#ifndef G_OS_WIN32
+    if (flag == SAVE_BOXPLOT_XPM) {
+	return plot_to_xpm(fname, data);
+    }
+#endif
+
+    return ps_print_plots(fname, flag, data);
+}
+
 static void real_six_numbers (BOXPLOT *plt, int offset, int do_mean,
 			      PRN *prn)
 {
@@ -1230,7 +1250,7 @@ static int cb_copy_image (gpointer data)
 
 #else /* end G_OS_WIN32 */
 
-int plot_to_xpm (const char *fname, gpointer data)
+static int plot_to_xpm (const char *fname, gpointer data)
 {    
     PLOTGROUP *grp = (PLOTGROUP *) data;
     GdkImage *image;
@@ -1253,8 +1273,8 @@ int plot_to_xpm (const char *fname, gpointer data)
 	    "\". c black\",\n"
 	    "/* pixels */\n", grp->width, grp->height);
 
-    image = gdk_image_get (grp->pixmap, 0, 0, 
-			   grp->width, grp->height);
+    image = gdk_image_get(grp->pixmap, 0, 0, 
+			  grp->width, grp->height);
 
     white_pixel = pow(2, image->depth) - 1;
 
@@ -1262,10 +1282,9 @@ int plot_to_xpm (const char *fname, gpointer data)
 	fprintf(fp, "\"");
 	for (j=0; j<grp->width; j++) {
 	    pixel = gdk_image_get_pixel(image, j, i);
-	    if (pixel == white_pixel) fprintf(fp, " ");
-	    else fprintf(fp, ".");
+	    fputc((pixel == white_pixel)? ' ' : '.', fp);
 	}
-	fprintf(fp, "\"%s\n", (i<grp->height-1)? "," : "};");
+	fprintf(fp, "\"%s\n", (i < grp->height - 1)? "," : "};");
     }
 
     g_object_unref(G_OBJECT(image));
@@ -1319,29 +1338,22 @@ static void read_boxrc (PLOTGROUP *grp)
 		val[31] = '\0';
 		if (!strcmp(key, "max")) { 
 		    grp->gmax = atof(val);
-		}
-		else if (!strcmp(key, "min")) { 
+		} else if (!strcmp(key, "min")) { 
 		    grp->gmin = atof(val);
-		}
-		else if (!strcmp(key, "font")) { 
+		} else if (!strcmp(key, "font")) { 
 		    strncpy(boxfont, val, 63);
 		    boxfont[63] = '\0';
-		}
-		else if (!strcmp(key, "fontsize")) {
+		} else if (!strcmp(key, "fontsize")) {
 		    boxfontsize = atoi(val);
-		}
-		else if (!strcmp(key, "width") && atoi(val) > 0) {
+		} else if (!strcmp(key, "width") && atoi(val) > 0) {
 		    grp->width = atoi(val);
-		}
-		else if (!strcmp(key, "height") && atoi(val) > 0) { 
+		} else if (!strcmp(key, "height") && atoi(val) > 0) { 
 		    grp->height = atoi(val);
-		}
-		else if (!strcmp(key, "numbers") && 
+		} else if (!strcmp(key, "numbers") && 
 			 (grp->numbers = malloc(8))) {
 		    grp->numbers[0] = 0;
 		    strncat(grp->numbers, val, 7);
-		}
-		else if (!strcmp(key, "outliers") && !strcmp(val, "true")) {
+		} else if (!strcmp(key, "outliers") && !strcmp(val, "true")) {
 		    grp->show_outliers = 1;
 		}
 	    }
@@ -1682,7 +1694,9 @@ static char *boxplots_fix_parentheses (const char *line)
     }
 
     s = malloc(flen + 1);
-    if (s == NULL) return NULL;
+    if (s == NULL) {
+	return NULL;
+    }
 
     p = s;
     for (i=0; i<len; i++) {
@@ -1726,11 +1740,16 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
     char *tok, *s = NULL, **bools = NULL;
     int *list = NULL;
 
-    if (!strncmp(str, "boxplots ", 9)) str += 9;
-    else if (!strncmp(str, "boxplot ", 8)) str += 8;
+    if (!strncmp(str, "boxplots ", 9)) {
+	str += 9;
+    } else if (!strncmp(str, "boxplot ", 8)) {
+	str += 8;
+    }
 
     s = boxplots_fix_parentheses(str);
-    if (s == NULL) return 1;
+    if (s == NULL) {
+	return 1;
+    }
 
     nvars = special_varcount(s);
     if (nvars == 0) {
@@ -1740,7 +1759,10 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 
     list = malloc((nvars + 1) * sizeof *list);
     bools = malloc(nvars * sizeof *bools);
+
     if (list == NULL || bools == NULL) {
+	free(list);
+	free(bools);
 	free(s);
 	return 1;
     }
@@ -1769,15 +1791,17 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	} else {
 	    if (isdigit(tok[0])) { 
 		v = atoi(tok);
-		if (v < origv) list[++i] = v;
-		else {
+		if (v < origv) {
+		    list[++i] = v;
+		} else {
 		    errbox(_("got invalid variable number %d"), v);
 		    err = 1;
 		}
 	    } else if (isalpha(tok[0])) {
-		v = varindex(pdinfo, tok);
-		if (v < origv) list[++i] = v;
-		else {
+		v = series_index(pdinfo, tok);
+		if (v < origv) {
+		    list[++i] = v;
+		} else {
 		    errbox(_("got invalid varname '%s'"), tok);
 		    err = 1;
 		}
@@ -1812,10 +1836,11 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 		err = 1;
 	    } else {
 		for (t=0; t<n; t++) {
-		    if ((*pZ)[k][t] == 1.0) 
+		    if ((*pZ)[k][t] == 1.0) {
 			(*pZ)[k][t] = (*pZ)[list[i]][t];
-		    else 
+		    } else { 
 			(*pZ)[k][t] = NADBL;
+		    }
 		}
 		strcpy(pdinfo->varname[k], pdinfo->varname[list[i]]);
 		list[i] = k++;
