@@ -26,6 +26,7 @@
 #include "gretl_www.h"
 #include "menustate.h"
 #include "treeutils.h"
+#include "winstack.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -50,6 +51,7 @@ static int add_rats_db_series_list (windata_t *vwin);
 static int add_pcgive_db_series_list (windata_t *vwin);
 static dbwrapper *get_series_info (windata_t *vwin, int action);
 static int *db_get_selection_list (windata_t *vwin);
+static void gui_get_db_series (windata_t *vwin, int cmd);
 
 enum db_data_actions {
     DB_DISPLAY,
@@ -298,7 +300,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	/* is there already a var of this name? */
-	dbv = varindex(datainfo, sinfo->varname);
+	dbv = series_index(datainfo, sinfo->varname);
 	if (dbv < datainfo->v) {
 	    if (dw->nv == 1) {
 		resp = yes_no_dialog ("gretl",                      
@@ -324,7 +326,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd);
 	} else if (sinfo->pd > datainfo->pd) {
 	    if (!chosen) {
-		data_compact_dialog(vwin->w, sinfo->pd, &datainfo->pd, NULL, 
+		data_compact_dialog(vwin->main, sinfo->pd, &datainfo->pd, NULL, 
 				    &method, NULL);
 		if (method == COMPACT_NONE) {
 		    if (!overwrite) {
@@ -433,17 +435,17 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
 
 static void gui_display_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_DISPLAY, NULL);
+    gui_get_db_series(vwin, DB_DISPLAY);
 }
 
 static void gui_graph_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_GRAPH, NULL);
+    gui_get_db_series(vwin, DB_GRAPH);
 }
 
 static void gui_import_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT);
 }
 
 void sync_db_windows (void)
@@ -490,7 +492,7 @@ static void gui_delete_series (GtkWidget *w, windata_t *vwin)
 
 void import_db_series (windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT);
 }
 
 static int diffdate (double d1, double d0, int pd)
@@ -596,9 +598,8 @@ static DATAINFO *new_dataset_from_dbwrapper (dbwrapper *dw,
     return dinfo;
 }
 
-void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
+static void gui_get_db_series (windata_t *vwin, int cmd)
 {
-    windata_t *vwin = (windata_t *) p;
     int dbcode = vwin->role;
     DATAINFO *dbinfo = NULL;
     double **dbZ = NULL;
@@ -631,7 +632,7 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
 	    err = get_pcgive_db_data(vwin->fname, sinfo, dbZ);
 	}
 
-	if (action == DB_IMPORT && err == DB_MISSING_DATA) {
+	if (cmd == DB_IMPORT && err == DB_MISSING_DATA) {
 	    warnbox(_("Warning: series has missing observations"));
 	}
 
@@ -644,11 +645,11 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
 	strcpy(VARLABEL(dbinfo, i+1), sinfo->descrip);
     }
 
-    if (action == DB_DISPLAY) {
+    if (cmd == DB_DISPLAY) {
 	display_dbdata((const double **) dbZ, dbinfo);
-    } else if (action == DB_GRAPH) {
+    } else if (cmd == DB_GRAPH) {
 	graph_dbdata(&dbZ, dbinfo);
-    } else if (action == DB_IMPORT) { 
+    } else if (cmd == DB_IMPORT) { 
 	add_dbdata(vwin, dbZ, dbinfo, dw, &freeit);
     }
 
@@ -660,6 +661,29 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
     }
 
     dbwrapper_destroy(dw);
+}
+
+static int get_db_command (GtkAction *action)
+{
+    if (action == NULL) {
+	return DB_DISPLAY;
+    } else {
+	const gchar *s = gtk_action_get_name(action);
+
+	if (!strcmp(s, "SeriesDisplay"))
+	    return DB_DISPLAY;
+	else if (!strcmp(s, "SeriesGraph"))
+	    return DB_GRAPH;
+	if (!strcmp(s, "SeriesImport"))
+	    return DB_IMPORT;
+	else
+	    return DB_DISPLAY;
+    }
+}
+
+void db_series_callback (GtkAction *action, gpointer data)
+{
+    gui_get_db_series((windata_t *) data, get_db_command(action));
 } 
 
 static void db_view_codebook (GtkWidget *w, windata_t *vwin)
@@ -673,14 +697,9 @@ static void db_view_codebook (GtkWidget *w, windata_t *vwin)
 }
 
 static void 
-book_callback_wrapper (gpointer p, guint u, GtkWidget *w)
+book_callback_wrapper (GtkAction *action, gpointer p)
 {
-    db_view_codebook(w, p);
-}
-
-static void db_menu_find (GtkWidget *w, windata_t *vwin)
-{
-    menu_find(vwin, 1, NULL);
+    db_view_codebook(NULL, p);
 }
 
 static void build_db_popup (windata_t *vwin, int cb, int del)
@@ -705,9 +724,11 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
 		       G_CALLBACK(gui_delete_series), 
 		       vwin);
     }	
+
     add_popup_item(_("Find..."), vwin->popup, 
-		   G_CALLBACK(db_menu_find), 
+		   G_CALLBACK(listbox_find), 
 		   vwin);
+
     if (cb) {
 	add_popup_item(_("Codebook"), vwin->popup, 
 		       G_CALLBACK(db_view_codebook), 
@@ -715,48 +736,60 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     }
 }
 
-static void 
-delete_series_callback (gpointer p, guint u, GtkWidget *w)
+static void delete_series_callback (GtkAction *action, windata_t *vwin)
 {
-    gui_delete_series(NULL, p);
+    gui_delete_series(NULL, vwin);
 }
 
-static void 
-close_db_callback (windata_t *vwin, guint u, GtkWidget *w)
+static void close_db_callback (GtkAction *action, windata_t *vwin)
 {
-    gtk_widget_destroy(vwin->w);
+    gtk_widget_destroy(vwin->main);
 }
 
 static void set_up_db_menu (windata_t *vwin, int cb, int del)
 {
-    GtkItemFactoryEntry db_items[] = {
-	{ N_("/_File"), NULL, NULL, 0, "<Branch>", GNULL },
-	{ N_("/File/_Close"), NULL, close_db_callback, 0, "<StockItem>", GTK_STOCK_CLOSE },
-	{ N_("/_Series/_Display"), NULL, gui_get_db_series, DB_DISPLAY, NULL, GNULL },
-	{ N_("/_Series/_Graph"), NULL, gui_get_db_series, DB_GRAPH, NULL, GNULL },
-	{ N_("/_Series/_Import"), NULL, gui_get_db_series, DB_IMPORT, NULL, GNULL },
-	{ N_("/_Series/_Delete"), NULL, delete_series_callback, 0, NULL, GNULL },
-	{ N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
-	{ N_("/Find/_Find in window"), NULL, menu_find, 1, "<StockItem>", GTK_STOCK_FIND },
-	{ N_("/_Codebook"), NULL, NULL, 0, "<Branch>", GNULL },    
-	{ N_("/Codebook/_Open"), NULL, book_callback_wrapper, 0, "<StockItem>", 
-	  GTK_STOCK_OPEN }
+    GtkActionEntry db_items[] = {
+	{ "File", NULL, N_("_File"), NULL, NULL, NULL },
+	{ "FileClose", GTK_STOCK_CLOSE, N_("_Close"), NULL, NULL, G_CALLBACK(close_db_callback) },
+	{ "Series", NULL, N_("_Series"), NULL, NULL, NULL },
+	{ "SeriesDisplay", NULL, N_("_Display"), NULL, NULL, G_CALLBACK(db_series_callback) },
+	{ "SeriesGraph", NULL, N_("_Graph"), NULL, NULL, G_CALLBACK(db_series_callback) },
+	{ "SeriesImport", NULL, N_("_Import"), NULL, NULL, G_CALLBACK(db_series_callback) },
+	{ "SeriesDelete", NULL, N_("_Delete"), NULL, NULL, G_CALLBACK(delete_series_callback) },
+	{ "Find", NULL, N_("_Find"), NULL, NULL, NULL },
+	{ "WindowFind", GTK_STOCK_FIND, N_("_Find in window"), NULL, NULL, G_CALLBACK(listbox_find) },
     };
-    gint n_items = sizeof db_items / sizeof db_items[0];
+    GtkActionEntry cb_items[] = {
+	{ "Codebook", NULL, N_("_Codebook"), NULL, NULL, NULL },
+	{ "CodebookOpen", GTK_STOCK_OPEN, N_("_Open"), NULL, NULL, G_CALLBACK(book_callback_wrapper) },
+    };
+    const gchar *db_ui = 
+	"<ui>"
+	"  <menubar name='MenuBar'>"
+	"    <menu action='File'>"
+	"      <menuitem action='FileClose'/>"
+	"    </menu>"
+	"    <menu action='Series'>"
+	"      <menuitem action='SeriesDisplay'/>"
+	"      <menuitem action='SeriesGraph'/>"
+	"      <menuitem action='SeriesImport'/>"
+	"      <menuitem action='SeriesDelete'/>"
+	"    </menu>"
+	"    <menu action='Find'>"
+	"      <menuitem action='WindowFind'/>"
+	"    </menu>"
+	"  </menubar>"
+	"</ui>";
 
-    if (!cb) {
-	n_items -= 2;
+    vwin_add_ui(vwin, db_items, G_N_ELEMENTS(db_items), db_ui);
+
+    if (cb) {
+	vwin_menu_add_menu(vwin, "/MenuBar", &cb_items[0]);
+	vwin_menu_add_item(vwin, "/MenuBar/Codebook", &cb_items[1]);
     }
 
-    vwin->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);
-#ifdef ENABLE_NLS
-    gtk_item_factory_set_translate_func(vwin->ifac, menu_translate, NULL, NULL);
-#endif
-    gtk_item_factory_create_items(vwin->ifac, n_items, db_items, vwin);
-    vwin->mbar = gtk_item_factory_get_widget(vwin->ifac, "<main>");
-
     if (!del) {
-	flip(vwin->ifac, "/Series/Delete", FALSE);
+	flip(vwin->ui, "/MenuBar/Series/SeriesDelete", FALSE);
     }
 }
 
@@ -862,11 +895,11 @@ make_db_series_window (int action, char *fname, char *buf)
 {
     GtkWidget *listbox;
     GtkWidget *w, *main_vbox;
-    char *titlestr;
+    gchar *title;
     windata_t *vwin;
     int db_width = 700, db_height = 420;
     int cb = 0, del = 0;
-    int err = 0;
+    int record, err = 0;
 
     w = match_window_by_filename(fname);
     if (w != NULL) {
@@ -878,37 +911,25 @@ make_db_series_window (int action, char *fname, char *buf)
 	return 1;
     }
 
-    vwin = mymalloc(sizeof *vwin);
+    if (buf == NULL && strrchr(fname, SLASH) != NULL) {
+	title = strrchr(fname, SLASH) + 1;
+    } else {
+	title = fname;
+    } 
+
+    record = (action == NATIVE_SERIES);
+
+    vwin = gretl_browser_new(action, title, record);
     if (vwin == NULL) {
 	return 1;
     }
 
-    windata_init(vwin);
-    vwin->role = action;
-
-    vwin->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-
     db_width *= gui_scale;
     db_height *= gui_scale;
-    gtk_window_set_default_size(GTK_WINDOW(vwin->w), db_width, db_height);
+    gtk_window_set_default_size(GTK_WINDOW(vwin->main), db_width, db_height);
     
-    if (buf == NULL && strrchr(fname, SLASH) != NULL) {
-	titlestr = strrchr(fname, SLASH) + 1;
-    } else {
-	titlestr = fname;
-    }
-
-    gtk_window_set_title(GTK_WINDOW(vwin->w), titlestr);
-
-    if (action == NATIVE_SERIES) {
-	g_object_set_data(G_OBJECT(vwin->w), "object", vwin);
-	g_object_set_data(G_OBJECT(vwin->w), "role", 
-			  GINT_TO_POINTER(vwin->role));
-	winstack_add(vwin->w);
-	g_signal_connect(G_OBJECT(vwin->w), "destroy",
-			 G_CALLBACK(free_windata), vwin);	
-    } else {
-	g_signal_connect(G_OBJECT(vwin->w), "destroy", 
+    if (action != NATIVE_SERIES) {
+	g_signal_connect(G_OBJECT(vwin->main), "destroy", 
 			 G_CALLBACK(destroy_db_win), vwin);
     }
 
@@ -921,7 +942,7 @@ make_db_series_window (int action, char *fname, char *buf)
     /* set up grids */
     main_vbox = gtk_vbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
-    gtk_container_add(GTK_CONTAINER(vwin->w), main_vbox);
+    gtk_container_add(GTK_CONTAINER(vwin->main), main_vbox);
 
     cb = db_has_codebook(fname);
     del = db_is_writable(action, fname);
@@ -956,9 +977,9 @@ make_db_series_window (int action, char *fname, char *buf)
     }
 
     if (err) {
-	gtk_widget_destroy(vwin->w);
+	gtk_widget_destroy(vwin->main);
     } else {
-	gtk_widget_show_all(vwin->w); 
+	gtk_widget_show_all(vwin->main); 
 	maybe_adjust_descrip_column(vwin);
 	db_select_first_series(vwin);
     }
@@ -1587,8 +1608,8 @@ void open_db_index (GtkWidget *w, gpointer data)
     make_db_series_window(action, dbfile, NULL); 
 
 #ifndef KEEP_BROWSER_OPEN
-    if (vwin != NULL && vwin->w != NULL && GTK_IS_WIDGET(vwin->w)) {
-	gtk_widget_destroy(GTK_WIDGET(vwin->w));
+    if (vwin != NULL && vwin->main != NULL && GTK_IS_WIDGET(vwin->main)) {
+	gtk_widget_destroy(GTK_WIDGET(vwin->main));
     }
 #endif
 }
@@ -2582,7 +2603,7 @@ void do_compact_data_set (void)
 	pmonstart = &monstart;
     }
 
-    data_compact_dialog(mdata->w, datainfo->pd, &newpd, pmonstart, 
+    data_compact_dialog(mdata->main, datainfo->pd, &newpd, pmonstart, 
 			&method, &repday);
 
     if (method == COMPACT_NONE) {
@@ -2608,7 +2629,7 @@ void do_expand_data_set (void)
 	return;
     }
 
-    data_expand_dialog(mdata->w, datainfo->pd, &newpd);
+    data_expand_dialog(mdata->main, datainfo->pd, &newpd);
     if (newpd < 0) {
 	return;
     }
