@@ -19,7 +19,7 @@ gint populate_remote_db_list (windata_t *vwin);
 
 gint populate_remote_func_list (windata_t *win);
 
-void gui_get_db_series (gpointer p, guint action, GtkWidget *w);
+void db_series_callback (GtkAction *action, gpointer data);
 
 void import_db_series (windata_t *vwin);
 
