@@ -26,19 +26,6 @@
 
 #include "system.h"
 
-GtkWidget *windata_get_toplevel (windata_t *vwin)
-{
-    GtkWidget *top = NULL;
-
-    if (vwin->w != NULL) {
-	top = gtk_widget_get_toplevel(vwin->w);
-    } else if (vwin->dialog != NULL) {
-	top = gtk_widget_get_toplevel(vwin->dialog);
-    }
-
-    return top;
-}
-
 void set_window_busy (windata_t *vwin)
 {
     vwin->flags |= VWIN_BUSY;
@@ -306,6 +293,8 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
     
     if (flags & GRETL_DLG_MODAL) {
 	gretl_set_window_modal(d);
+    } else if (flags & GRETL_DLG_QUASI_MODAL) {
+	gretl_set_window_quasi_modal(d);
     }
 
     if (!(flags & GRETL_DLG_RESIZE)) {
@@ -372,6 +361,17 @@ static void cancel_on_delete (GtkDialog *d, int resp, int *c)
     }
 }
 
+gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key, 
+			   gpointer unused)
+{
+    if (key->keyval == GDK_Escape) { 
+        gtk_widget_destroy(w);
+	return TRUE; /* handled! */
+    } else {
+	return FALSE;
+    }
+}
+
 static dialog_t *
 dialog_data_new (gpointer p, gint code, const char *title,
 		 int *canceled)
@@ -405,6 +405,8 @@ dialog_data_new (gpointer p, gint code, const char *title,
 
     g_signal_connect(G_OBJECT(d->dialog), "destroy", 
 		     G_CALLBACK(destroy_dialog_data), d);
+    g_signal_connect(G_OBJECT(d->dialog), "key_press_event", 
+		     G_CALLBACK(esc_kills_window), NULL);
 
     return d;
 }
@@ -529,7 +531,7 @@ static void dialog_table_setup (dialog_t *dlg, int hsize)
 {
     GtkWidget *sw;
 
-    sw = gtk_scrolled_window_new (NULL, NULL);
+    sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_set_size_request(sw, hsize, 200);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dialog)->vbox), 
 		       sw, TRUE, TRUE, FALSE);
@@ -696,9 +698,9 @@ edit_dialog_popup_handler (GtkWidget *w, GdkEventButton *event, dialog_t *d)
 	if (d->popup != NULL) {
 	    gtk_menu_popup(GTK_MENU(d->popup), NULL, NULL, NULL, NULL,
 			   event->button, event->time);
-	    gtk_signal_connect(GTK_OBJECT(d->popup), "destroy",
-			       GTK_SIGNAL_FUNC(gtk_widget_destroyed), 
-			       &d->popup);
+	    g_signal_connect(G_OBJECT(d->popup), "destroy",
+			     G_CALLBACK(gtk_widget_destroyed), 
+			     &d->popup);
 	}
 	return TRUE;
     }
@@ -981,13 +983,13 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
 	}
     }
 
-    gtk_combo_box_set_active(GTK_COMBO_BOX(w), method);
-
     g_object_set_data(G_OBJECT(w), "bt", bt);
     g_object_set_data(G_OBJECT(w), "bv", bv);
     g_signal_connect(G_OBJECT(w), "changed",
 		     G_CALLBACK(set_sys_method), d);
 
+    gtk_combo_box_set_active(GTK_COMBO_BOX(w), method);
+
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);  
 }
@@ -1007,6 +1009,7 @@ static void dlg_display_sys (dialog_t *d)
     d->edit = dlg_text_edit_new(&hsize, FALSE);
     dialog_table_setup(d, hsize);
     dlg_text_set_from_sys(sys, d); 
+    gretl_dialog_set_resizeable(d->dialog, TRUE);
 }
 
 static void raise_and_focus_dialog (GtkEditable *editable, gpointer p)
@@ -1128,6 +1131,7 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 
 	d->edit = dlg_text_edit_new(&hsize, TRUE);
 	dialog_table_setup(d, hsize);
+	gretl_dialog_set_resizeable(d->dialog, TRUE);
 
 	/* insert previous text, if any and if the command
 	   is the same as previously -- or insert skeleton
@@ -1145,7 +1149,8 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     } else {
 	if (info != NULL) {
 	    w = gtk_label_new(info);
-	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
+	    /* GTK_JUSTIFY_CENTER? */
+	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
 	    gtk_widget_show(w);
 	}
@@ -1211,10 +1216,8 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	gtk_widget_show(w);  
     }    
 
-    /* "Cancel" button? */
-    if (canceled != NULL) {
-	cancel_delete_button(button_box, d->dialog, canceled);
-    }
+    /* "Cancel" button */
+    cancel_delete_button(button_box, d->dialog, canceled);
 
     /* "OK" button */
     w = ok_button(button_box);
@@ -1238,13 +1241,11 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 
     /* schedule destruction */
     if (cmdcode == MODEL_GENR || cmdcode == RESTRICT) {
-	GtkWidget *w = windata_get_toplevel(okptr);
+	windata_t *vwin = (windata_t *) okptr;
 
-	if (w != NULL) {
-	    g_signal_connect(G_OBJECT(w), "destroy",
-			     G_CALLBACK(cancel_vwin_edit), 
-			     NULL);
-	}
+	g_signal_connect(G_OBJECT(vwin->main), "destroy",
+			 G_CALLBACK(cancel_vwin_edit), 
+			 NULL);
     }
 
     gtk_window_set_destroy_with_parent(GTK_WINDOW(d->dialog), TRUE);
@@ -1287,3 +1288,46 @@ char *entry_box_get_trimmed_text (GtkWidget *w)
 
     return ret;
 }
+
+void set_combo_box_strings_from_list (GtkComboBox *box, GList *list)
+{
+    GList *mylist = list;
+
+    while (mylist != NULL) {
+	gtk_combo_box_append_text(box, mylist->data);
+	mylist = mylist->next;
+    }
+}
+
+void set_combo_box_default_text (GtkComboBox *box, const char *s)
+{
+    GtkWidget *entry = gtk_bin_get_child(GTK_BIN(box));
+
+    gtk_entry_set_text(GTK_ENTRY(entry), s);
+}
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
+
+gchar *gtk_combo_box_get_active_text (GtkComboBox *box)
+{
+    GtkWidget *w = gtk_bin_get_child(GTK_BIN(box));
+    gchar *ret = NULL;
+
+    if (GTK_IS_ENTRY(w)) {
+	const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
+
+	if (s != NULL) {
+	    ret = g_strdup(s);
+	}
+    } else {
+	GtkTreeModel *model = gtk_combo_box_get_model(box);
+	GtkTreeIter iter;
+
+	gtk_combo_box_get_active_iter(box, &iter);
+	gtk_tree_model_get(model, &iter, 0, &ret, -1);
+    }
+
+    return ret;
+}
+
+#endif
