@@ -21,9 +21,10 @@
 #define DLGUTILS_H
 
 enum {
-    GRETL_DLG_MODAL  = 1 << 0,
-    GRETL_DLG_BLOCK  = 1 << 1,
-    GRETL_DLG_RESIZE = 1 << 2
+    GRETL_DLG_MODAL       = 1 << 0,
+    GRETL_DLG_BLOCK       = 1 << 1,
+    GRETL_DLG_RESIZE      = 1 << 2,
+    GRETL_DLG_QUASI_MODAL = 1 << 3,
 };
 
 enum {
@@ -39,8 +40,6 @@ struct combo_opts_ {
 
 typedef struct combo_opts_ combo_opts;
 
-GtkWidget *windata_get_toplevel (windata_t *vwin);
-
 void set_window_busy (windata_t *vwin);
 
 void unset_window_busy (windata_t *vwin);
@@ -70,6 +69,9 @@ GtkWidget *next_button (GtkWidget *hbox);
 
 GtkWidget *back_button (GtkWidget *hbox);
 
+gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key, 
+			   gpointer unused);
+
 void edit_dialog (const char *title, const char *info, const char *deflt, 
 		  void (*okfunc)(), void *okptr,
 		  guint hlpcode, guint varclick,
@@ -102,4 +104,15 @@ dialog_opts *dialog_opts_new (int n, int type,
 
 void dialog_opts_free (dialog_opts *opts);
 
+void set_combo_box_strings_from_list (GtkComboBox *box, GList *list);
+
+void set_combo_box_default_text (GtkComboBox *box, const char *s);
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
+
+gchar *gtk_combo_box_get_active_text (GtkComboBox *box);
+
+#endif
+
+
 #endif /* DLGUTILS_H */
