@@ -32,6 +32,13 @@
 #include "database.h"
 #include "bootstrap.h"
 
+#include <sys/stat.h>
+#include <unistd.h>
+
+#ifdef G_OS_WIN32
+# include <gdk/gdkwin32.h>
+#endif
+
 #define IS_DAT_ACTION(i) (i == SAVE_DATA || \
                           i == SAVE_DATA_AS || \
                           i == OPEN_DATA)
@@ -210,6 +217,13 @@ static const char *get_ext (int action, gpointer data)
     return s;
 }
 
+static int isdir (const char *path)
+{
+    struct stat buf;
+
+    return (stat(path, &buf) == 0 && S_ISDIR(buf.st_mode)); 
+}
+
 static int check_maybe_add_ext (char *fname, int action, gpointer data)
 {
     const char *ext = NULL;
@@ -251,7 +265,7 @@ static void script_window_update (windata_t *vwin, const char *fname)
 
     /* update the window title */
     title = g_strdup_printf("gretl: %s", trfname);
-    gtk_window_set_title(GTK_WINDOW(vwin->dialog), title);
+    gtk_window_set_title(GTK_WINDOW(vwin->main), title);
     g_free(trfname);
     g_free(title);
 
@@ -264,10 +278,10 @@ static void script_window_update (windata_t *vwin, const char *fname)
 	vwin->role = EDIT_SCRIPT;
     }
 
-    mark_content_saved(vwin);
+    mark_vwin_content_saved(vwin);
 
     /* make the window editable */
-    if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->w))) {
+    if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->text))) {
 	view_window_set_editable(vwin);
     }
 }
@@ -281,7 +295,7 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
     gchar *trbuf;
 #endif
 
-    buf = textview_get_text(vwin->w);
+    buf = textview_get_text(vwin->text);
     if (buf == NULL) {
 	errbox("Couldn't retrieve buffer");
 	return;
@@ -477,9 +491,13 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	verify_open_data(NULL, action);
     } else if (APPEND_DATA_ACTION(action)) {
 	strcpy(tryfile, fname);
-	do_open_data(NULL, NULL, action);
+	do_open_data(NULL, action);
     } else if (action == OPEN_SCRIPT) {
-	filesel_open_script(fname);
+	if (src == FSEL_DATA_FNPKG) {
+	    fnsave_set_script(fname, data);
+	} else {
+	    filesel_open_script(fname);
+	}
     } else if (action == OPEN_SESSION) {
 	filesel_open_session(fname);
     } else if (action == OPEN_MARKERS) {
@@ -516,13 +534,10 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	do_store(fname, save_action_to_opt(action, data));
     } else if (action == SAVE_GNUPLOT) {
 	save_graph_to_file(data, fname);
-    } else if (action == SAVE_BOXPLOT_EPS || action == SAVE_BOXPLOT_PS) {
-	int err;
-
-	err = ps_print_plots(fname, action, data);
-	if (err) {
-	    errbox(_("boxplot save failed"));
-	}
+    } else if (action == SAVE_BOXPLOT_EPS || 
+	       action == SAVE_BOXPLOT_PS ||
+	       action == SAVE_BOXPLOT_XPM) {
+	print_boxplot_group(fname, action, data);
     } else if (action == SAVE_SESSION) {
 	save_session(fname);
     } else if (action == SAVE_FUNCTIONS) {
@@ -574,7 +589,7 @@ static struct winfilter get_gp_filter (int ttype)
 	{ N_("PNG files (*.png)"), "*.png" },
 	{ N_("Windows metafiles (*.emf)"), "*.emf" },
 	{ N_("gnuplot files (*.plt)"), "*.plt" },
-	{ N_("all files (*.*)"), "*.*" }
+	{ N_("all files (*.*)"), "*" }
     };
 
     if (ttype == GP_TERM_EPS) 
@@ -643,7 +658,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	{ SET_PROG,         { N_("program files (*.exe)"), "*.exe" }}
     };
     static struct winfilter default_filter = {
-	N_("all files (*.*)"), "*.*" 
+	N_("all files (*.*)"), "*" 
     };
     struct winfilter filt;
     int i;
@@ -666,7 +681,7 @@ static struct winfilter get_filter (int action, gpointer data)
     return filt;
 }
 
-static char *make_winfilter (int action, gpointer data)
+static char *make_winfilter (int action, FselDataSrc src, gpointer data)
 {
     struct winfilter filter;
     char *ret, *p;
@@ -688,7 +703,7 @@ static char *make_winfilter (int action, gpointer data)
     p += strlen(p) + 1;
     strcpy(p, filter.pat);
 
-    if (action == OPEN_SCRIPT) {
+    if (action == OPEN_SCRIPT && src != FSEL_DATA_FNPKG) {
 	p += strlen(p) + 1;
 	strcpy(p, I_("gnuplot files (*.plt)"));
 	p += strlen(p) + 1;
@@ -703,7 +718,7 @@ static char *make_winfilter (int action, gpointer data)
 	p += strlen(p) + 1;
 	strcpy(p, I_("all files (*.*)"));
 	p += strlen(p) + 1;
-	strcpy(p, "*.*");
+	strcpy(p, "*");
     }
 
     return ret;
@@ -739,7 +754,7 @@ static int select_dirname (char *fname, char *trmsg)
 }
 
 static void win32_file_selector (const char *msg, int action, FselDataSrc src, 
-				 gpointer data) 
+				 gpointer data, GtkWidget *parent) 
 {
     OPENFILENAME of;
     int retval;
@@ -795,8 +810,12 @@ static void win32_file_selector (const char *msg, int action, FselDataSrc src,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = NULL;
-	filter = make_winfilter(action, data);
+	if (parent != NULL) {
+	    of.hwndOwner = GDK_WINDOW_HWND(parent->window);
+	} else {
+	    of.hwndOwner = NULL;
+	}
+	filter = make_winfilter(action, src, data);
 	of.lpstrFilter = filter;
 	of.lpstrCustomFilter = NULL;
 	of.nFilterIndex = 1;
@@ -866,6 +885,7 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
     GtkFileFilter *filter;
     GtkFileChooserAction fa;
     const gchar *okstr;
+    gint response;
 
     set_startdir(startdir, action);
 
@@ -881,15 +901,10 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
     } else {
 	fa = GTK_FILE_CHOOSER_ACTION_SAVE;
 	okstr = GTK_STOCK_SAVE;
-	parent = NULL;
     }
 
-    /* FIXME: parent window below should not always be mdata->w,
-       in particular when action == SAVE_GNUPLOT
-    */
-
     if (parent == NULL) {
-	parent = mdata->w;
+	parent = mdata->main;
     }
 
     filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
@@ -914,9 +929,9 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
     if (action == OPEN_ASCII || action == APPEND_ASCII) {
 	filter = gtk_file_filter_new();
 	gtk_file_filter_set_name(filter, _("all files (*.*)"));
-	gtk_file_filter_add_pattern(filter, "*.*");
+	gtk_file_filter_add_pattern(filter, "*");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    } else if (action == OPEN_SCRIPT) {
+    } else if (action == OPEN_SCRIPT && src != FSEL_DATA_FNPKG) {
 	filter = gtk_file_filter_new();
 	gtk_file_filter_set_name(filter, _("GNU R files (*.R)"));
 	gtk_file_filter_add_pattern(filter, "*.R");
@@ -925,7 +940,7 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
 	gtk_file_filter_set_name(filter, _("gnuplot files (*.plt)"));
 	gtk_file_filter_add_pattern(filter, "*.plt");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }    
+    } 
 
     /* FIXME session dir */
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
@@ -970,26 +985,35 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
 					  fname);
     } 
 
-    if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
-	char *fname;
+    response = gtk_dialog_run(GTK_DIALOG(filesel));
 
+    if (response == GTK_RESPONSE_ACCEPT) {
+	gchar *fname;
+	
 	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
-	gtk_widget_destroy(filesel);
 	file_selector_process_result(fname, action, src, data);
 	g_free(fname);
-    } else {
-	gtk_widget_destroy(filesel);
-    }
+    } 
+
+    gtk_widget_destroy(filesel);
 }
 
 #endif /* end of non-MS Windows code */
 
 void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 {
+    GtkWidget *w = NULL;
+
+    if (src == FSEL_DATA_VWIN) {
+	windata_t *vwin = (windata_t *) data;
+
+	w = vwin->main;
+    }
+
 #ifdef G_OS_WIN32
-    win32_file_selector(msg, action, src, data);
+    win32_file_selector(msg, action, src, data, w);
 #else
-    gtk_file_selector(msg, action, src, data, NULL);
+    gtk_file_selector(msg, action, src, data, w);
 #endif
 }
 
@@ -997,7 +1021,7 @@ void file_selector_with_parent (const char *msg, int action, FselDataSrc src,
 				gpointer data, GtkWidget *w)
 {
 #ifdef G_OS_WIN32
-    win32_file_selector(msg, action, src, data);
+    win32_file_selector(msg, action, src, data, w);
 #else
     gtk_file_selector(msg, action, src, data, w);
 #endif
