@@ -24,6 +24,13 @@
 #include "filters.h"
 #include "libset.h"
 
+enum {
+    FILTER_SMA = 1,
+    FILTER_EMA,
+    FILTER_HP,
+    FILTER_BK
+};
+
 enum {
     FILTER_SAVE_NONE = 0,
     FILTER_SAVE_TREND = 1 << 0,
@@ -261,7 +268,7 @@ static int varname_error (filter_info *finfo, int i)
     if (*s == 0) {
 	errbox(_("Variable name is missing"));
 	return 1;
-    } else if (validate_varname(s)) {
+    } else if (gui_validate_varname(s, GRETL_TYPE_SERIES)) {
 	return 1;
     } else if (i == 2 && (finfo->save_opt & FILTER_SAVE_TREND)) {
 	if (!strcmp(s, finfo->save1)) {
@@ -444,7 +451,7 @@ static int filter_dialog (filter_info *finfo)
     GtkWidget *w;
     int ret = 0;
 
-    dlg = gretl_dialog_new(_("gretl: time-series filter"), mdata->w,
+    dlg = gretl_dialog_new(_("gretl: time-series filter"), mdata->main,
 			   GRETL_DLG_BLOCK);
     finfo->dlg = dlg;
 
@@ -700,7 +707,7 @@ static int save_filtered_var (filter_info *finfo, double *x, int i,
 {
     const char *vname = (i == FILTER_SAVE_TREND)? finfo->save1 :
 	finfo->save2;
-    int v = varindex(datainfo, vname);
+    int v = series_index(datainfo, vname);
     int err = 0;
 
     if (v == datainfo->v) {
@@ -708,7 +715,6 @@ static int save_filtered_var (filter_info *finfo, double *x, int i,
     } else {
 	free(Z[v]);
 	Z[v] = x;
-	set_var_scalar(datainfo, v, 0);
 	set_var_discrete(datainfo, v, 0);
     }
 
@@ -873,20 +879,37 @@ static int calculate_filter (filter_info *finfo)
     return err;
 }
 
-void filter_callback (gpointer p, guint code, GtkWidget *w)
+static int filter_code (GtkAction *action)
+{
+    const gchar *s = gtk_action_get_name(action);
+
+    if (!strcmp(s, "FilterSMA")) 
+	return FILTER_SMA;
+    else if (!strcmp(s, "FilterEMA")) 
+	return FILTER_EMA;
+    else if (!strcmp(s, "FilterHP")) 
+	return FILTER_HP;
+    else if (!strcmp(s, "FilterBK")) 
+	return FILTER_BK;
+    else
+	return FILTER_SMA;
+}
+
+void filter_callback (GtkAction *action)
 {
     filter_info finfo;
     int v = mdata_active_var();
     int t1 = datainfo->t1;
     int t2 = datainfo->t2;
-    int cancel = 0;
+    int code, cancel = 0;
     int err = 0;
 
-    if (var_is_scalar(datainfo, v)) {
-	errbox(_("This variable is a scalar"));
+    if (reject_scalar(v)) {
 	return;
     }
 
+    code = filter_code(action);
+
     err = array_adjust_t1t2(Z[v], &t1, &t2);
     if (err) {
 	gui_errmsg(E_MISSDATA);
