@@ -23,6 +23,7 @@
 #include "textbuf.h"
 #include "fileselect.h"
 #include "gretl_www.h"
+#include "winstack.h"
 #include "fnsave.h"
 
 #ifdef G_OS_WIN32
@@ -43,20 +44,23 @@ struct function_info_ {
     GtkWidget *ifsel;
     GtkWidget *codesel;
     GtkWidget *check;
+    windata_t *samplewin;
     fnpkg *pkg;
     char *fname;
     char *author;
     char *version;
     char *date;
     char *pkgdesc;
+    char *sample;
     char *help;
     int pub;
     int *privlist;
     int iface;
     FuncDataReq dreq;
-    float minver;
+    int minver;
     int upload;
     int saveas;
+    const char *openscript;
 };
 
 struct login_info_ {
@@ -83,15 +87,20 @@ function_info *finfo_new (void)
     finfo->version = NULL;
     finfo->date = NULL;
     finfo->pkgdesc = NULL;
+    finfo->sample = NULL;
     finfo->upload = 0;
     finfo->saveas = 0;
     finfo->iface = -1;
 
+    finfo->samplewin = NULL;
+
     finfo->help = NULL;
     finfo->pub = -1;
     finfo->privlist = NULL;
     finfo->dreq = 0;
-    finfo->minver = 1.6;
+    finfo->minver = 10600;
+
+    finfo->openscript = NULL;
 
     return finfo;
 }
@@ -103,9 +112,14 @@ static void finfo_free (function_info *finfo)
     free(finfo->version);
     free(finfo->date);
     free(finfo->pkgdesc);
+    free(finfo->sample);
     free(finfo->privlist);
     free(finfo->help);
 
+    if (finfo->samplewin != NULL) {
+	gtk_widget_destroy(finfo->samplewin->main);
+    }
+
     free(finfo);
 }
 
@@ -269,8 +283,8 @@ static void real_finfo_save (function_info *finfo)
     }
 
     if (finfo->saveas) {
-	file_selector(_("Save function package"), SAVE_FUNCTIONS, 
-		      FSEL_DATA_MISC, finfo);
+	file_selector_with_parent(_("Save function package"), SAVE_FUNCTIONS, 
+				  FSEL_DATA_MISC, finfo, finfo->dlg);
     } else {
 	maybe_revise_package_name(finfo);
 	err = save_user_functions(finfo->fname, finfo);
@@ -312,10 +326,10 @@ const char *print_today (void)
     return timestr;
 }
 
-static gboolean update_iface (GtkOptionMenu *menu, 
+static gboolean update_iface (GtkComboBox *menu, 
 			      function_info *finfo)
 {
-    int i = gtk_option_menu_get_history(menu);
+    int i = gtk_combo_box_get_active(menu);
 
     if (i == 0) {
 	finfo->iface = finfo->pub;
@@ -359,11 +373,134 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
     if (vwin != NULL) {
 	build_path(vwin->fname, paths.dotdir, "pkgedit", NULL);
 	gretl_tempname(vwin->fname);
-	g_object_set_data(G_OBJECT(vwin->w), "iface", 
+	g_object_set_data(G_OBJECT(vwin->text), "iface", 
 			  GINT_TO_POINTER(finfo->iface));
     }
 }
 
+void update_sample_script (windata_t *vwin)
+{
+    function_info *finfo;
+
+    finfo = g_object_get_data(G_OBJECT(vwin->main), "finfo");
+
+    if (finfo != NULL) {
+	gchar *text = textview_get_text(vwin->text);
+
+	free(finfo->sample);
+	finfo->sample = gretl_strdup(text);
+	g_free(text);
+    }
+}
+
+static void
+nullify_sample_window (GtkWidget *w, function_info *finfo)
+{
+    finfo->samplewin = NULL;
+}
+
+void fnsave_set_script (const char *fname, gpointer p)
+{
+    function_info *finfo = (function_info *) p;
+
+    finfo->openscript = fname;
+}
+
+static int maybe_open_script (function_info *finfo, char **fname)
+{
+    int resp = yes_no_dialog("gretl",
+			     "This package does not yet contain a sample script.\n"
+			     "Add one now?", 
+			     0);
+
+    if (resp != GRETL_YES) {
+	return GRETL_CANCEL;
+    }
+	
+    resp = yes_no_dialog("gretl",
+			 "Start from an existing script?",
+			 1);
+    if (resp == GRETL_CANCEL) {
+	return GRETL_CANCEL;
+    } else if (resp == GRETL_YES) {
+	file_selector_with_parent(_("Open script file"), OPEN_SCRIPT, 
+				  FSEL_DATA_FNPKG, finfo, finfo->dlg);
+	if (finfo->openscript != NULL) {
+	    *fname = g_strdup(finfo->openscript);
+	    finfo->openscript = NULL;
+	}
+    }
+
+    return 0;
+}
+
+static void edit_sample_callback (GtkWidget *w, function_info *finfo)
+{
+    const char *pkgname = NULL;
+    gchar *fname = NULL;
+    gchar *title;
+    PRN *prn = NULL;
+
+    if (finfo->samplewin != NULL) {
+	gtk_window_present(GTK_WINDOW(finfo->samplewin->main));
+	return;
+    }
+
+    if (finfo->sample == NULL) {
+	if (maybe_open_script(finfo, &fname) == GRETL_CANCEL) {
+	    return;
+	}
+    }
+
+    if (finfo->sample != NULL && bufopen(&prn)) {
+	return;
+    }
+
+    if (finfo->pkg != NULL) {
+	gretl_function_get_info(finfo->pub, "pkgname", &pkgname);
+    } 
+
+    if (pkgname != NULL) {
+	title = g_strdup_printf("%s-sample", pkgname);
+    } else {
+	title = g_strdup("sample script");
+    }
+
+    if (finfo->sample != NULL) {
+	pputs(prn, finfo->sample);
+	pputc(prn, '\n');
+    } else if (fname != NULL) {
+	gchar *tmp = NULL;
+	char *cont = NULL;
+
+	gretl_file_get_contents(fname, &tmp);
+	if (tmp != NULL) {
+	    cont = gretl_strdup(tmp);
+	    g_free(tmp);
+	}
+	if (cont != NULL) {
+	    prn = gretl_print_new_with_buffer(cont);
+	} else {
+	    bufopen(&prn);
+	}
+    }
+
+    finfo->samplewin = view_buffer(prn, 78, 350, title,
+				   EDIT_FUNC_CODE, finfo);
+
+    if (fname != NULL) {
+	mark_vwin_content_changed(finfo->samplewin);
+	g_free(fname);
+    }
+
+    g_object_set_data(G_OBJECT(finfo->samplewin->main), "finfo",
+		      finfo);
+    g_signal_connect(G_OBJECT(finfo->samplewin->main), "destroy",
+		     G_CALLBACK(nullify_sample_window), finfo);
+
+    g_free(title);
+}
+
 static void gfn_to_script_callback (GtkWidget *w, function_info *finfo)
 {
     int n = 0;
@@ -381,8 +518,8 @@ static void gfn_to_script_callback (GtkWidget *w, function_info *finfo)
 	return;
     }
 
-    file_selector(_("Save function package"), SAVE_FUNCTIONS_AS, 
-		  FSEL_DATA_MISC, finfo);
+    file_selector_with_parent(_("Save function package"), SAVE_FUNCTIONS_AS, 
+			      FSEL_DATA_MISC, finfo, finfo->dlg);
 }
 
 static GtkWidget *label_hbox (GtkWidget *w, const char *txt)
@@ -429,36 +566,32 @@ enum {
 
 static GtkWidget *interface_selector (function_info *finfo, int iface)
 {
-    GtkWidget *ifmenu, *menu, *tmp;
+    GtkWidget *ifmenu;
     const char *fnname;
     int i;
 
     finfo->iface = finfo->pub;
-
-    ifmenu = gtk_option_menu_new();
-    menu = gtk_menu_new();
+    ifmenu = gtk_combo_box_new_text();
 
     fnname = user_function_name_by_index(finfo->pub);
-    tmp = gtk_menu_item_new_with_label(fnname);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), tmp);
+    gtk_combo_box_append_text(GTK_COMBO_BOX(ifmenu), fnname);
 
     if (iface == IFACE_ALL && finfo->privlist != NULL) {
 	for (i=1; i<=finfo->privlist[0]; i++) {
 	    fnname = user_function_name_by_index(finfo->privlist[i]);
-	    tmp = gtk_menu_item_new_with_label(fnname);
-	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), tmp);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(ifmenu), fnname);
 	}
-    }	
+    }
 
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(ifmenu), menu);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(ifmenu), 0);
     gtk_widget_show_all(ifmenu);
 
     return ifmenu;
 }
 
-static void dreq_select (GtkOptionMenu *menu, function_info *finfo)
+static void dreq_select (GtkComboBox *menu, function_info *finfo)
 {
-    finfo->dreq = gtk_option_menu_get_history(menu);
+    finfo->dreq = gtk_combo_box_get_active(menu);
 }
 
 static void add_data_requirement_menu (GtkWidget *tbl, int i, 
@@ -470,21 +603,18 @@ static void add_data_requirement_menu (GtkWidget *tbl, int i,
 	N_("Quarterly or monthly data"),
 	N_("Panel data")
     };
-    GtkWidget *menu, *datamenu, *tmp;
+    GtkWidget *datamenu, *tmp;
     int j;
 
     tmp = gtk_label_new(_("Data requirement"));
     gtk_table_attach_defaults(GTK_TABLE(tbl), tmp, 0, 1, i, i+1);
     gtk_widget_show(tmp);
 
-    datamenu = gtk_option_menu_new();
-    menu = gtk_menu_new();
+    datamenu = gtk_combo_box_new_text();
     for (j=0; j<=FN_NEEDS_PANEL; j++) {
-	tmp = gtk_menu_item_new_with_label(_(datareq[j]));
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), tmp);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(datamenu), _(datareq[j]));
     }
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(datamenu), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(datamenu), finfo->dreq);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(datamenu), finfo->dreq);
 
     tmp = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(tmp), datamenu, FALSE, FALSE, 0);
@@ -495,17 +625,11 @@ static void add_data_requirement_menu (GtkWidget *tbl, int i,
 		     G_CALLBACK(dreq_select), finfo);
 }
 
-static void get_maj_min_pl (float minver, int *maj, int *min, int *pl)
+static void get_maj_min_pl (int v, int *maj, int *min, int *pl)
 {
-    char vstr[5], minstr[2], plstr[2];
-
-    gretl_push_c_numeric_locale();
-    sprintf(vstr, "%.2f", (double) minver);
-    gretl_pop_c_numeric_locale();
-
-    sscanf(vstr, "%d.%1s%1s", maj, minstr, plstr);
-    *min = atoi(minstr);
-    *pl = atoi(plstr);
+    *maj = v / 10000;
+    *min = (v - *maj * 10000) / 100;
+    *pl = v % 10;
 }
 
 static void adjust_minver (GtkWidget *w, function_info *finfo)
@@ -517,11 +641,11 @@ static void adjust_minver (GtkWidget *w, function_info *finfo)
     get_maj_min_pl(finfo->minver, &maj, &min, &pl);
 
     if (lev == 1) {
-	finfo->minver = (float) val + min / 10.0 + pl / 100.0;
+	finfo->minver = 10000 * val + 100 * min + pl;
     } else if (lev == 2) {
-	finfo->minver = (float) maj + val / 10.0 + pl / 100.0;
+	finfo->minver = 10000 * maj + 100 * val + pl;
     } else if (lev == 3) {
-	finfo->minver = (float) maj + min / 10.0 + val / 100.0;
+	finfo->minver = 10000 * maj + 100 * min + val;
     }
 }
 
@@ -650,7 +774,6 @@ static void finfo_dialog (function_info *finfo)
 
 	entry = gtk_entry_new();
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 40);
-	gtk_entry_set_editable(GTK_ENTRY(entry), TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), entry, 1, 2, i, i+1);
 	gtk_widget_show(entry); 
 
@@ -706,7 +829,7 @@ static void finfo_dialog (function_info *finfo)
     if (finfo->privlist != NULL) {
 	finfo->codesel = interface_selector(finfo, IFACE_ALL);
 	gtk_box_pack_start(GTK_BOX(hbox), finfo->codesel, FALSE, FALSE, 5);
-	g_signal_connect(G_OBJECT(GTK_OPTION_MENU(finfo->codesel)), "changed",
+	g_signal_connect(G_OBJECT(finfo->codesel), "changed",
 			 G_CALLBACK(update_iface), finfo);
     } else {
 	finfo->iface = finfo->pub;
@@ -719,6 +842,15 @@ static void finfo_dialog (function_info *finfo)
 
     gtk_widget_show_all(hbox);
 
+    /* edit sample script button */
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    button = gtk_button_new_with_label(_("Edit sample script"));
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(edit_sample_callback), finfo);
+    gtk_widget_show_all(hbox);
+
     /* check box for upload option */
     finfo->check = button_in_hbox(vbox, CHECK_BUTTON, 
 				  _("Upload package to server on save"));
@@ -726,7 +858,7 @@ static void finfo_dialog (function_info *finfo)
     /* control button area */
     hbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 10);
+    gtk_box_set_spacing(GTK_BOX(hbox), 10);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
     /* Save button */
@@ -751,7 +883,7 @@ static void finfo_dialog (function_info *finfo)
 
 static void web_get_login (GtkWidget *w, gpointer p)
 {
-    browser_open("http://ricardo.ecn.wfu.edu/gretl/apply/");
+    browser_open("http://gretl.ecn.wfu.edu/apply/");
 }
 
 static void login_dialog (login_info *linfo)
@@ -784,7 +916,6 @@ static void login_dialog (login_info *linfo)
 
 	entry = gtk_entry_new();
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 34);
-	gtk_entry_set_editable(GTK_ENTRY(entry), TRUE);
 	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), entry, 1, 2, i, i+1);
 	if (src != NULL) {
@@ -804,7 +935,7 @@ static void login_dialog (login_info *linfo)
 
     hbox = label_hbox(GTK_DIALOG(linfo->dlg)->vbox, 
 		      _("If you don't have a login to the gretl server\n"
-			"please see http://ricardo.ecn.wfu.edu/gretl/apply/.\n"
+			"please see http://gretl.ecn.wfu.edu/apply/.\n"
 			"The 'Website' button below should open this page\n"
 			"in your web browser."));
     gtk_widget_show(hbox);
@@ -1038,7 +1169,7 @@ int save_user_functions (const char *fname, gpointer p)
     fprintf(stderr, "finfo->pub = %d\n", finfo->pub);
     printlist(finfo->privlist, "finfo->privlist");
     fprintf(stderr, "dreq=%d\n", finfo->dreq);
-    fprintf(stderr, "minver=%.2f\n", (double) finfo->minver);
+    fprintf(stderr, "minver=%d\n", finfo->minver);
 #endif
 		
     err = write_function_package(finfo->pkg,
@@ -1049,6 +1180,7 @@ int save_user_functions (const char *fname, gpointer p)
 				 finfo->version,
 				 finfo->date,
 				 finfo->pkgdesc,
+				 finfo->sample,
 				 finfo->dreq,
 				 finfo->minver);
 
@@ -1057,7 +1189,11 @@ int save_user_functions (const char *fname, gpointer p)
     } else {
 	maybe_update_func_files_window(1);
 	if (finfo->upload) {
-	    do_upload(fname);
+	    if (finfo->sample == NULL) {
+		warnbox("Please add a sample script for this package");
+	    } else {
+		do_upload(fname);
+	    }
 	}
     }
 
@@ -1092,6 +1228,16 @@ int save_user_functions_as_script (const char *fname, gpointer p)
 	gretl_function_print_code(finfo->pub, prn);
     }
 
+    if (finfo->sample != NULL) {
+	int n = strlen(finfo->sample);
+
+	pputs(prn, "\n# sample function call\n");
+	pputs(prn, finfo->sample);
+	if (finfo->sample[n-1] != '\n') {
+	    pputc(prn, '\n');
+	}
+    }
+
     gretl_print_destroy(prn);
 
     return 0;
@@ -1175,6 +1321,7 @@ void edit_function_package (const char *fname, int *loaderr)
 				    &finfo->version,
 				    &finfo->date,
 				    &finfo->pkgdesc,
+				    &finfo->sample,
 				    &finfo->dreq,
 				    &finfo->minver);
 
@@ -1210,7 +1357,7 @@ int no_user_functions_check (void)
 			       "Do you want to write a function now?"),
 			     0);
 	if (resp == GRETL_YES) {
-	    do_new_script(NULL, FUNC, NULL);
+	    do_new_script(FUNC);
 	}
     } 
 
