@@ -224,7 +224,7 @@ void plot_label_position_click (GtkWidget *w, png_plot *plot)
 
 	cursor = gdk_cursor_new(GDK_CROSSHAIR);
 	gdk_window_set_cursor(plot->canvas->window, cursor);
-	gdk_cursor_destroy(cursor);
+	gdk_cursor_unref(cursor);
 	entry = g_object_get_data(G_OBJECT(w), "labelpos_entry");
 	plot->labelpos_entry = entry;
 	plot->status |= PLOT_POSITIONING;
@@ -280,7 +280,7 @@ add_or_remove_png_term (const char *fname, int action, GPT_SPEC *spec)
 {
     FILE *fsrc, *ftmp;
     char temp[MAXLEN], fline[MAXLEN];
-    char restore_line[MAXLEN] = {0};
+    char restore_line[MAXLEN];
     GptFlags flags = 0;
 
     sprintf(temp, "%sgpttmp", paths.dotdir);
@@ -294,17 +294,17 @@ add_or_remove_png_term (const char *fname, int action, GPT_SPEC *spec)
 	fclose(ftmp);
 	return 1;
     }
+    
+    *restore_line = '\0';
 
     if (action == ADD_PNG) {
 	/* see if there's already a png term setting, possibly commented
 	   out, that can be reused */
 	while (fgets(fline, sizeof fline, fsrc)) {
-	    if (is_png_term_line(fline)) {
+	    if (is_png_term_line(fline) && *restore_line == '\0') {
 		strcat(restore_line, fline);
-		break;
 	    } else if (commented_term_line(fline) && *restore_line == '\0') {
 		strcat(restore_line, fline + 2);
-		break;
 	    } else if (strstr(fline, "letterbox")) {
 		flags = GPT_LETTERBOX;
 	    } else if (!strncmp(fline, "plot", 4)) {
@@ -326,24 +326,26 @@ add_or_remove_png_term (const char *fname, int action, GPT_SPEC *spec)
 	    pline = get_gretl_png_term_line(ptype, flags);
 	    fprintf(ftmp, "%s\n", pline);
 	}	    
-	fprintf(ftmp, "set output '%sgretltmp.png'\n", 
-		paths.dotdir);
+	fprintf(ftmp, "set output '%sgretltmp.png'\n", paths.dotdir);
     }
 
     /* now for the body of the plot file */
 
     if (action == ADD_PNG) {
-	int got_set_print = 0;
-
 	while (fgets(fline, sizeof fline, fsrc)) {
 	    if (set_print_line(fline)) {
-		got_set_print = 1;
-		fputs(fline, ftmp);
-	    } else if (!commented_term_line(fline) && !set_output_line(fline)) {
+		; /* skip it (portability) */
+	    } else if (is_png_term_line(fline)) {
+		; /* handled above */
+	    } else if (commented_term_line(fline)) {
+		; /* handled above */
+	    } else if (set_output_line(fline)) {
+		; /* handled above */
+	    } else {
 		fputs(fline, ftmp);
 	    }
 	}
-	if (gnuplot_has_bbox() && !got_set_print) {
+	if (gnuplot_has_bbox()) {
 	    print_plot_bounding_box_request(ftmp);
 	}
     } else {
@@ -1711,7 +1713,9 @@ static int parse_gp_line_line (const char *s, GPT_SPEC *spec)
 
 static int plot_ols_var_ok (const char *vname, int v)
 {
-    int vi = varindex(datainfo, vname);
+    int vi = series_index(datainfo, vname);
+
+    /* FIXME WTF?? */
 
     if (vi <= datainfo->v && !strcmp(datainfo->varname[vi], vname)) {
 	return 1;
@@ -2196,12 +2200,12 @@ static void draw_selection_rectangle (png_plot *plot,
 		       FALSE,
 		       rx, ry, rw, rh);
     /* show the modified pixmap */
-    gdk_window_copy_area(plot->canvas->window,
-			 plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
-			 0, 0,
-			 plot->pixmap,
-			 0, 0,
-			 plot->pixel_width, plot->pixel_height);
+    gdk_draw_drawable(plot->canvas->window,
+		      plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
+		      plot->pixmap,
+		      0, 0,
+		      0, 0,
+		      plot->pixel_width, plot->pixel_height);
     /* draw (invert) again to erase the rectangle */
     gdk_draw_rectangle(plot->pixmap,
 		       plot->invert_gc,
@@ -2262,12 +2266,12 @@ write_label_to_plot (png_plot *plot, int i, gint x, gint y)
     gdk_draw_layout(plot->pixmap, plot->invert_gc, x, y, pl);
 
     /* show the modified pixmap */
-    gdk_window_copy_area(plot->canvas->window,
-			 plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
-			 0, 0,
-			 plot->pixmap,
-			 0, 0,
-			 plot->pixel_width, plot->pixel_height);
+    gdk_draw_drawable(plot->canvas->window,
+		      plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
+		      plot->pixmap,
+		      0, 0,
+		      0, 0,
+		      plot->pixel_width, plot->pixel_height);
 
     /* trash the pango layout */
     g_object_unref(G_OBJECT(pl));
@@ -2523,25 +2527,25 @@ static gint color_popup_activated (GtkWidget *w, gpointer data)
     gpointer ptr = g_object_get_data(G_OBJECT(w), "plot");
     png_plot *plot = (png_plot *) ptr;
     GtkWidget *parent = (GTK_MENU(w->parent))->parent_menu_item;
-    gchar *parent_item = g_object_get_data(G_OBJECT(parent), "string");
+    gchar *up_item = g_object_get_data(G_OBJECT(parent), "string");
 
     if (!strcmp(item, _("monochrome"))) {
 	plot->spec->flags |= GPT_MONO;
     }
 
-    if (!strcmp(parent_item, _("Save as postscript (EPS)..."))) {
+    if (!strcmp(up_item, _("Save as postscript (EPS)..."))) {
 	plot->spec->termtype = GP_TERM_EPS;
-	file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
-		      FSEL_DATA_MISC, plot->spec);
-    } else if (!strcmp(parent_item, _("Save as Windows metafile (EMF)..."))) {
+	file_selector_with_parent(_("Save gnuplot graph"), SAVE_GNUPLOT, 
+				  FSEL_DATA_MISC, plot->spec, plot->shell);
+    } else if (!strcmp(up_item, _("Save as Windows metafile (EMF)..."))) {
 	plot->spec->termtype = GP_TERM_EMF;
-	file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
-		      FSEL_DATA_MISC, plot->spec);
+	file_selector_with_parent(_("Save gnuplot graph"), SAVE_GNUPLOT, 
+				  FSEL_DATA_MISC, plot->spec, plot->shell);
     } 
 #ifdef G_OS_WIN32
-    else if (!strcmp(parent_item, _("Copy to clipboard"))) {
+    else if (!strcmp(up_item, _("Copy to clipboard"))) {
 	win32_process_graph(plot->spec, WIN32_TO_CLIPBOARD);
-    } else if (!strcmp(parent_item, _("Print"))) {
+    } else if (!strcmp(up_item, _("Print"))) {
 	win32_process_graph(plot->spec, WIN32_TO_PRINTER);
     }    
 #endif   
@@ -2630,15 +2634,15 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
     plot->popup = NULL;
 
     if (!strcmp(item, _("Add another curve..."))) {
-	stats_calculator(plot, CALC_GRAPH_ADD, NULL);
+	dist_graph_add(plot);
     } else if (!strcmp(item, _("Save as PNG..."))) {
 	plot->spec->termtype = GP_TERM_PNG;
-        file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
-		      FSEL_DATA_MISC, plot->spec);
+        file_selector_with_parent(_("Save gnuplot graph"), SAVE_GNUPLOT, 
+				  FSEL_DATA_MISC, plot->spec, plot->shell);
     } else if (!strcmp(item, _("Save as PDF..."))) {
 	plot->spec->termtype = GP_TERM_PDF;
-        file_selector(_("Save gnuplot graph"), SAVE_GNUPLOT, 
-		      FSEL_DATA_MISC, plot->spec);
+        file_selector_with_parent(_("Save gnuplot graph"), SAVE_GNUPLOT, 
+				  FSEL_DATA_MISC, plot->spec, plot->shell);
     } else if (!strcmp(item, _("Save to session as icon"))) { 
 	add_to_session_callback(plot->spec);
     } else if (plot_is_range_mean(plot) && !strcmp(item, _("Help"))) { 
@@ -2655,7 +2659,7 @@ static gint plot_popup_activated (GtkWidget *w, gpointer data)
 
 	cursor = gdk_cursor_new(GDK_CROSSHAIR);
 	gdk_window_set_cursor(plot->canvas->window, cursor);
-	gdk_cursor_destroy(cursor);
+	gdk_cursor_unref(cursor);
 	plot->status |= PLOT_ZOOMING;
 	gtk_statusbar_push(GTK_STATUSBAR(plot->statusbar), plot->cid, 
 			   _(" Drag to define zoom rectangle"));
@@ -3078,12 +3082,12 @@ void plot_expose (GtkWidget *widget, GdkEventExpose *event,
 
     /* Refresh double buffer, then copy the "dirtied" area to
        the on-screen GdkWindow */
-    gdk_window_copy_area(widget->window,
-			 widget->style->fg_gc[GTK_STATE_NORMAL],
-			 event->area.x, event->area.y,
-			 dbuf_pixmap,
-			 event->area.x, event->area.y,
-			 event->area.width, event->area.height);
+    gdk_draw_drawable(widget->window,
+		      widget->style->fg_gc[GTK_STATE_NORMAL],
+		      dbuf_pixmap,
+		      event->area.x, event->area.y,
+		      event->area.x, event->area.y,
+		      event->area.width, event->area.height);
 }
 
 static GdkPixbuf *gretl_pixbuf_new_from_file (const gchar *fname)
@@ -3165,22 +3169,23 @@ static int render_pngfile (png_plot *plot, int view)
 	plot->format &= ~PLOT_MARKERS_UP;
     }
 
-    gdk_pixbuf_render_to_drawable(pbuf, plot->pixmap, 
-				  plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
-				  0, 0, 0, 0, width, height,
-				  GDK_RGB_DITHER_NONE, 0, 0);
+    gdk_draw_pixbuf(plot->pixmap, 
+		    plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
+		    pbuf, 0, 0, 0, 0, width, height,
+		    GDK_RGB_DITHER_NONE, 0, 0);
 
     g_object_unref(pbuf);
     remove(pngname);
     
     if (view != PNG_START) { 
 	/* we're changing the view, so refresh the whole canvas */
-	gdk_window_copy_area(plot->canvas->window,
-			     plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
-			     0, 0,
-			     plot->pixmap,
-			     0, 0,
-			     plot->pixel_width, plot->pixel_height);
+	gdk_draw_drawable(plot->canvas->window,
+			  plot->canvas->style->fg_gc[GTK_STATE_NORMAL],
+			  plot->pixmap,
+			  0, 0,
+			  0, 0,
+			  plot->pixel_width, plot->pixel_height);
+
 	if (view == PNG_ZOOM) {
 	    plot->status |= PLOT_ZOOMED;
 	} else if (view == PNG_UNZOOM) {
@@ -3214,10 +3219,10 @@ static void destroy_png_plot (GtkWidget *w, png_plot *plot)
     }
 
     if (plot->invert_gc != NULL) {
-	gdk_gc_destroy(plot->invert_gc);
+	g_object_unref(plot->invert_gc);
     }
 
-    gtk_widget_unref(plot->shell);
+    g_object_unref(plot->shell);
 
     free(plot);
 }
@@ -3645,7 +3650,7 @@ static int gnuplot_show_png (const char *plotfile, GPT_SPEC *spec, int saved)
     plot->shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
     /* note need for corresponding unref */
-    gtk_widget_ref(plot->shell);
+    g_object_ref(plot->shell);
 
     gtk_window_set_title(GTK_WINDOW(plot->shell), _("gretl: gnuplot graph")); 
     gtk_window_set_resizable(GTK_WINDOW(plot->shell), FALSE);
