@@ -20,16 +20,8 @@
 #ifndef GUI_UTILS_H
 #define GUI_UTILS_H
 
-void gretl_stock_icons_init (void);
-
-#ifdef ENABLE_NLS
-gchar *menu_translate (const gchar *path, gpointer p);
-#endif
-
 int copyfile (const char *src, const char *dest);
 
-int isdir (const char *path);
-
 FILE *gretl_tempfile_open (char *fname);
 
 int gretl_tempname (char *fname);
@@ -48,35 +40,27 @@ void register_data (int flag);
 
 void register_startup_data (const char *fname);
 
-void do_open_data (GtkWidget *w, gpointer data, int code);
+void do_open_data (windata_t *vwin, int code);
 
-void verify_open_data (gpointer userdata, int code);
+void verify_open_data (windata_t *vwin, int code);
 
 void verify_open_session (void);
 
-void close_window (gpointer data, guint win_code, GtkWidget *widget);
-
-void windata_init (windata_t *mydata);
-
 void free_windata (GtkWidget *w, gpointer data);
 
-void winstack_init (void);
+void mark_vwin_content_saved (windata_t *vwin);
 
-void winstack_destroy (void);
+void mark_vwin_content_changed (windata_t *vwin);
 
-void winstack_add (GtkWidget *w);
+int latex_is_ok (void);
 
-void winstack_remove (GtkWidget *w);
+gint query_save_text (GtkWidget *w, GdkEvent *event, windata_t *vwin);
 
-int winstack_match_data (const gpointer p);
+void auto_save_plot (windata_t *vwin);
 
-GtkWidget *match_window_by_data (const gpointer p);
+void view_window_save (GtkWidget *widget, windata_t *vwin);
 
-GtkWidget *match_window_by_filename (const char *fname);
-
-GtkWidget *build_text_popup (windata_t *vwin);
-
-void mark_content_saved (windata_t *vwin);
+void buf_edit_save (GtkWidget *widget, windata_t *vwin);
 
 windata_t *view_buffer (PRN *prn, int hsize, int vsize, 
 			const char *title, int role,
@@ -86,7 +70,8 @@ windata_t *view_file (const char *filename, int editable, int del_file,
 		      int hsize, int vsize, int role);
 
 windata_t *
-view_help_file (const char *filename, int role, GtkItemFactoryEntry *menu_items);
+view_help_file (const char *filename, int role, GtkActionEntry *menu_items,
+		const gchar *ui_info);
 
 windata_t *edit_buffer (char **pbuf, int hsize, int vsize, 
 			char *title, int role);
@@ -96,11 +81,9 @@ windata_t *vwin_first_child (windata_t *vwin);
 int view_model (PRN *prn, MODEL *pmod, int hsize, int vsize, 
 		char *title);
 
-int highest_numbered_variable_in_winstack (void);
-
 void view_window_set_editable (windata_t *vwin);
 
-int validate_varname (const char *varname);
+int gui_validate_varname (const char *varname, GretlType t);
 
 gretlopt get_tex_eqn_opt (void);
 
@@ -110,8 +93,6 @@ gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
 void add_popup_item (const gchar *label, GtkWidget *menu,
 		     GCallback callback, gpointer data);
 
-void get_stats_table (void);
-
 void *gui_get_plugin_function (const char *funcname, 
 			       void **phandle);
 
