@@ -826,7 +826,7 @@ void winprint (char *fullbuf, char *selbuf)
     g_signal_connect(op, "draw_page", G_CALLBACK(draw_text_page), &pinfo);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-				  GTK_WINDOW(mdata->w), &err);
+				  GTK_WINDOW(mdata->main), &err);
 
     if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
 	errbox("Error printing:\n%s", err->message);
@@ -972,7 +972,7 @@ void gtk_print_graph (const char *fname)
     g_signal_connect(op, "draw_page", G_CALLBACK(draw_image), NULL);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-				  GTK_WINDOW(mdata->w), &err);
+				  GTK_WINDOW(mdata->main), &err);
 
     if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
 	errbox("Error printing:\n%s", err->message);
@@ -2008,12 +2008,8 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     }
 
     for (i=1; i<=l0; i++) {
-	if (var_is_series(datainfo, list[i])) {
-	    pmax[i-1] = get_precision(&Z[list[i]][datainfo->t1], 
-				      tsamp, 8);
-	} else {
-	    pmax[i-1] = SCALAR_DIGITS;
-	}
+	pmax[i-1] = get_precision(&Z[list[i]][datainfo->t1], 
+				  tsamp, 8);
     }	
 
     if (datainfo->decpoint != ',') {
@@ -2035,8 +2031,7 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     for (t=datainfo->t1; t<=datainfo->t2; t++) {
 	csv_obs_to_prn(t, datainfo, prn);
 	for (i=1; i<=l0; i++) { 
-	    xx = (var_is_series(datainfo, list[i]))? 
-		Z[list[i]][t] : Z[list[i]][0];
+	    xx = Z[list[i]][t];
 	    if (na(xx)) {
 		pputs(prn, "NA");
 	    } else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
@@ -2174,3 +2169,47 @@ int csv_copy_listed_vars (windata_t *vwin, int fmt, int action)
 
     return err;
 }
+
+int font_has_minus (PangoFontDescription *desc)
+{
+    GtkWidget *widget;
+    PangoContext *context = NULL;
+    PangoLayout *layout = NULL;
+    PangoLanguage *lang = NULL;
+    PangoCoverage *coverage = NULL;
+    PangoFont *pfont = NULL;
+    int ret = 0;
+
+    if (desc == NULL) {
+	return -1;
+    }
+
+    widget = gtk_label_new(NULL);  
+    context = gtk_widget_get_pango_context(widget); 
+
+    if (context == NULL) {
+	gtk_widget_destroy(widget);
+	return 0;
+    }    
+
+    layout = pango_layout_new(context); 
+    lang = pango_language_from_string("eng");
+
+    if (layout != NULL && lang != NULL) {
+	pfont = pango_context_load_font(context, desc);
+	if (pfont != NULL) {
+	    coverage = pango_font_get_coverage(pfont, lang);
+	    if (coverage != NULL) {
+		/* U+2212 = minus sign */
+		ret = (pango_coverage_get(coverage, 0x2212) == PANGO_COVERAGE_EXACT);
+	    }
+	}
+    } 
+
+    pango_coverage_unref(coverage);
+    g_object_unref(G_OBJECT(layout));
+    g_object_unref(G_OBJECT(context));
+    gtk_widget_destroy(widget);    
+
+    return ret;
+}
