@@ -24,6 +24,7 @@
 #include "gretl_www.h"
 #include "treeutils.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 #ifdef G_OS_WIN32
 # include "gretlwin32.h"
@@ -63,7 +64,7 @@ static help_head **en_cli_heads, **en_gui_heads;
 
 static windata_t *make_helpwin (int flags);
 static void real_do_help (int hcode, int pos, int flags);
-static void en_text_cmdref (gpointer p, guint u, GtkWidget *w);
+static void en_text_cmdref (GtkAction *a);
 
 /* searching stuff */
 static void find_in_text (GtkWidget *widget, gpointer data);
@@ -74,11 +75,31 @@ static GtkWidget *find_window = NULL;
 static GtkWidget *find_entry;
 static char *needle;
 
-GtkItemFactoryEntry help_menu_items[] = {
-    { N_("/_Topics"), NULL, NULL, 0, "<Branch>", GNULL },    
-    { N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
-    { N_("/Find/_Find in window"), NULL, menu_find, 0, "<StockItem>", GTK_STOCK_FIND },
-    { NULL, NULL, NULL, 0, NULL, GNULL }
+const gchar *help_ui = 
+    "<ui>"
+    " <menubar name='MenuBar'>"
+    "   <menu action='Topics'/>"
+    "   <menu action='Find'>"
+    "     <menuitem action='WindowFind'/>"
+    "   </menu>"
+    "   <menu action='Zoom'>"
+    "     <menuitem action='ZoomIn'/>"
+    "     <menuitem action='ZoomOut'/>"
+    "   </menu>"
+    " </menubar>"
+    "</ui>";
+
+GtkActionEntry help_menu_items[] = {
+    { "Topics", NULL, N_("_Topics"), NULL, NULL, NULL },
+    { "Find", NULL, N_("_Find"), NULL, NULL, NULL },
+    { "WindowFind", GTK_STOCK_FIND, N_("_Find in window"), NULL, NULL,
+      G_CALLBACK(text_find) },
+    { "Zoom", NULL, N_("_Text"), NULL, NULL, NULL },
+    { "ZoomIn", GTK_STOCK_ZOOM_IN, N_("_Larger"), NULL, NULL,
+      G_CALLBACK(text_zoom) },
+    { "ZoomOut", GTK_STOCK_ZOOM_OUT, N_("_Smaller"), NULL, NULL,
+      G_CALLBACK(text_zoom) },
+    { NULL, NULL, NULL, NULL, NULL, NULL }
 };
 
 struct gui_help_item {
@@ -381,14 +402,6 @@ static int get_byte_positions (help_head **heads, int nh, int gui, FILE *fp)
 		m++;
 	    }
 
-#if HDEBUG
-	    if (i >= 0 && j >= 0) {
-		fprintf(stderr, "%s: setting heads[%d]->pos[%d] = %d\n", word, i, j, pos);
-	    } else {
-		fprintf(stderr, "%s: pos = %d\n", word, pos);
-	    }
-#endif
-
 	    if (err || m == n) {
 		break;
 	    }
@@ -514,10 +527,6 @@ static void add_topic_to_head (help_head *head, int *i, const char *word,
 	    head->topicnames[*i] = label;
 	}
 	*i += 1;
-#if HDEBUG
-	fprintf(stderr, "add_topic_to_head: topic %d: word = %s: hnum = %d\n", 
-		*i, word, hnum);
-#endif
     } else {
 	fprintf(stderr, "helpfile error: word='%s' not recognized\n", word);
 	head->ntopics -= 1;
@@ -843,29 +852,33 @@ static int help_pos_from_function_index (int fnum)
 
 /* end apparatus for genr functions help */
 
-static void do_gui_help (gpointer p, guint pos, GtkWidget *w) 
+static void do_gui_help (GtkAction *action, gpointer p) 
 {
+    int pos = atoi(gtk_action_get_name(action));
     int hcode = GPOINTER_TO_INT(p);
 
     real_do_help(hcode, pos, 0);
 }
 
-static void do_en_gui_help (gpointer p, guint pos, GtkWidget *w) 
+static void do_en_gui_help (GtkAction *action, gpointer p) 
 {
+    int pos = atoi(gtk_action_get_name(action));
     int hcode = GPOINTER_TO_INT(p);
 
     real_do_help(hcode, pos, HELP_EN);
 }
 
-static void do_cli_help (gpointer p, guint pos, GtkWidget *w) 
+static void do_cli_help (GtkAction *action, gpointer p) 
 {
+    int pos = atoi(gtk_action_get_name(action));
     int hcode = GPOINTER_TO_INT(p);
 
     real_do_help(hcode, pos, HELP_CLI);
 }
 
-static void do_en_cli_help (gpointer p, guint pos, GtkWidget *w) 
+static void do_en_cli_help (GtkAction *action, gpointer p) 
 {
+    int pos = atoi(gtk_action_get_name(action));
     int hcode = GPOINTER_TO_INT(p);
 
     real_do_help(hcode, pos, HELP_CLI | HELP_EN);
@@ -901,7 +914,6 @@ static char *get_compat_help_string (int pos)
     return NULL;
 }
 
-
 static int gui_pos_from_cmd (int cmd, int en)
 {
     help_head **heads;
@@ -984,32 +996,69 @@ static void en_help_callback (gpointer p, int cli, GtkWidget *w)
 
 static void add_en_help_item (windata_t *hwin, int cli)
 {
-    GtkItemFactoryEntry item;
+    GtkActionEntry item;
+    const gchar *path;
 
-    item.accelerator = NULL;
+    action_entry_init(&item);
 
-    item.item_type = "<Branch>";
-    item.callback = NULL;
-    item.callback_action = 0; 
-    item.path = "/_English";
-    gtk_item_factory_create_item(hwin->ifac, &item, NULL, 1);
+    path = "/MenuBar/English";
+    item.name = item.label = "English";
+    vwin_menu_add_menu(hwin, path, &item);
 
-    item.item_type = NULL;
-    item.callback = en_help_callback; 
-    item.callback_action = cli; 
-    item.path = g_strdup_printf("/English/%s", _("Show English help"));
-    gtk_item_factory_create_item(hwin->ifac, &item, hwin, 1);
-    g_free(item.path);
+    item.name = "EnCliHelp";
+    item.label = _("Show English help");
+    item.callback = G_CALLBACK(en_help_callback); 
+    vwin_menu_add_item(hwin, "/MenuBar/English", &item);
+}
+
+static const gchar *help_item_get_word (help_head **hds,
+					int i, int j, int tnum,
+					int cli)
+{
+    const gchar *hword = NULL;
+
+    if (hds[i]->topicnames != NULL) {
+	hword = hds[i]->topicnames[j];
+    } else if (tnum < NC) {
+	/* a regular gretl command */
+	hword = gretl_command_word(tnum);
+    } else if (!cli) {
+	/* a gui special item? */
+	hword = get_gui_help_string(hds[i]->pos[j]);
+    } else {
+	hword = get_compat_help_string(hds[i]->pos[j]);
+    }
+
+    return hword;
+}
+
+static int help_menu_add_item (windata_t *vwin, const gchar *path, 
+			       GtkActionEntry *entry)
+{
+    guint id = gtk_ui_manager_new_merge_id(vwin->ui);
+    GtkActionGroup *actions;
+
+    actions = gtk_action_group_new("AdHoc");
+
+    gtk_action_group_add_actions(actions, entry, 1, vwin);
+    gtk_ui_manager_add_ui(vwin->ui, id, path, entry->name, entry->name,
+			  GTK_UI_MANAGER_MENUITEM, FALSE);
+    gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
+    g_object_unref(actions);
+
+    return id;
 }
 
 static void add_help_topics (windata_t *hwin, int flags)
 {
-    GtkItemFactoryEntry hitem;
-    const gchar *mpath = N_("/_Topics");
+    GtkActionEntry hitem;
+    const gchar *mpath = "/MenuBar/Topics";
     help_head **hds = NULL;
     int en = (flags & HELP_EN);
     int cli = (flags & HELP_CLI);
     int funcs = (flags & HELP_FUNCS);
+    const gchar *hword;
+    gchar *path;
     int i, j;
 
     if (en) {
@@ -1018,22 +1067,19 @@ static void add_help_topics (windata_t *hwin, int flags)
 	hds = (cli)? cli_heads : (funcs)? NULL : gui_heads;
     }
 
-    hitem.accelerator = NULL;
+    action_entry_init(&hitem);
 
     if (cli || funcs) {
 	/* Add general index as (first) "topic" */
-	hitem.callback_action = 0; 
-	hitem.item_type = NULL;
-	hitem.path = g_strdup_printf("%s/%s", mpath, (en)? "Index" : _("Index"));
+	hitem.name = "0";
+	hitem.label = N_("Index");
 	if (funcs) {
-	    hitem.callback = genr_funcs_ref;
+	    hitem.callback = G_CALLBACK(genr_funcs_ref);
 	} else {
-	    hitem.callback = (en)? en_text_cmdref : plain_text_cmdref;
+	    hitem.callback = (en)? G_CALLBACK(en_text_cmdref) : 
+		G_CALLBACK(plain_text_cmdref);
 	}
-	gtk_item_factory_create_item(hwin->ifac, &hitem, 
-				     GINT_TO_POINTER(0), 
-				     1);
-	g_free(hitem.path);
+	vwin_menu_add_item(hwin, mpath, &hitem);
     }
 
     /* Are there any actual topics to add? */
@@ -1044,90 +1090,36 @@ static void add_help_topics (windata_t *hwin, int flags)
     /* put the topics under the menu heading */
     for (i=0; hds[i] != NULL; i++) {
 	const char *headname = (en)? hds[i]->name : _(hds[i]->name);
+	char aname[16];
 
 	if (hds[i]->ntopics == 0) {
 	    continue;
 	}
 
-	hitem.callback_action = 0; 
-	hitem.item_type = "<Branch>";
-	hitem.path = g_strdup_printf("%s/%s", mpath, headname);
-	hitem.callback = NULL; 
-
-	gtk_item_factory_create_item(hwin->ifac, &hitem, NULL, 1);
-	g_free(hitem.path);
+	hitem.name = headname;
+	hitem.label = _(headname);
+	hitem.callback = NULL;
+	vwin_menu_add_menu(hwin, mpath, &hitem);
 
 	for (j=0; j<hds[i]->ntopics; j++) {
-	    int tnum = -1;
-	    int topic_ok = 1;
+	    int tnum = hds[i]->topics[j];
 
-	    hitem.callback_action = hds[i]->pos[j]; 
-	    hitem.item_type = NULL;
-	    hitem.path = NULL;
-
-	    tnum = hds[i]->topics[j];
-
-	    if (hds[i]->topicnames != NULL) {
-		hitem.path = 
-		    g_strdup_printf("%s/%s/%s", 
-				    mpath, headname, 
-				    hds[i]->topicnames[j]);
-#if HDEBUG
-		fprintf(stderr, "(1) Built help topic path from\n"
-			" '%s', '%s' and '%s'\n", mpath, headname,
-			hds[i]->topicnames[j]);
-#endif
-	    } else {
-		if (tnum < NC) {
-		    /* a regular gretl command */
-		    hitem.path = 
-			g_strdup_printf("%s/%s/%s", 
-					mpath, headname, 
-					gretl_command_word(tnum));
-#if HDEBUG
-		    fprintf(stderr, "(2) Built help topic path from\n"
-			    " '%s', '%s' and '%s'\n", mpath, headname,
-			    gretl_command_word(tnum));
-#endif
-		} else if (!cli) {
-		    /* a gui special item? */
-		    char *gstr = get_gui_help_string(hds[i]->pos[j]);
-
-		    if (gstr != NULL) {
-			hitem.path = 
-			    g_strdup_printf("%s/%s/%s", 
-					    mpath, headname, gstr);
-#if HDEBUG
-			fprintf(stderr, "(3) Built help topic path from\n"
-				" '%s', '%s' and '%s'\n", mpath, headname,
-				gstr);
-#endif
-		    } else {
-			topic_ok = 0;
-		    }
-		} else {
-		    char *cstr = get_compat_help_string(hds[i]->pos[j]);
-
-		    if (cstr != NULL) {
-			hitem.path = 
-			    g_strdup_printf("%s/%s/%s", 
-					    mpath, headname, cstr);
-		    } else {		    
-			topic_ok = 0;
-		    }
-		}
-	    }
-
-	    if (topic_ok) {
+	    hword = help_item_get_word(hds, i, j, tnum, cli);
+					    
+	    if (hword != NULL) {
+		path = g_strdup_printf("%s/%s", mpath, headname);
+		sprintf(aname, "%d", hds[i]->pos[j]);
+		hitem.name = aname;
+		hitem.label = hword;
 		if (en) {
-		    hitem.callback = (cli)? do_en_cli_help : do_en_gui_help; 
+		    hitem.callback = (cli)? G_CALLBACK(do_en_cli_help) : 
+			G_CALLBACK(do_en_gui_help); 
 		} else {
-		    hitem.callback = (cli)? do_cli_help : do_gui_help; 
+		    hitem.callback = (cli)? G_CALLBACK(do_cli_help) : 
+			G_CALLBACK(do_gui_help); 
 		}
-		gtk_item_factory_create_item(hwin->ifac, &hitem, 
-					     GINT_TO_POINTER(tnum), 
-					     1);
-		g_free(hitem.path);
+		help_menu_add_item(hwin, path, &hitem);
+		g_free(path);
 	    }
 	}
     }
@@ -1165,7 +1157,7 @@ static windata_t *make_helpwin (int flags)
     }
 
     if (helpfile != NULL) {
-	vwin = view_help_file(helpfile, role, help_menu_items);
+	vwin = view_help_file(helpfile, role, help_menu_items, help_ui);
     }
 
     if (vwin != NULL) {
@@ -1263,11 +1255,11 @@ static void real_do_help (int hcode, int pos, int flags)
 		    phwin = &gui_hwin;
 		}
 	    }		
-	    g_signal_connect(G_OBJECT(hwin->w), "destroy",
+	    g_signal_connect(G_OBJECT(hwin->main), "destroy",
 			     G_CALLBACK(nullify_hwin), phwin);
 	}
     } else {
-	gtk_window_present(GTK_WINDOW(gtk_widget_get_toplevel(hwin->w)));
+	gtk_window_present(GTK_WINDOW(hwin->main));
     }
 
 #if HDEBUG
@@ -1282,24 +1274,36 @@ static void real_do_help (int hcode, int pos, int flags)
 }
 
 /* called from main menu in gretl.c; also used as callback from
-   command ref index in textbuf.c
+   help window topics menu
 */
 
-void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
+void plain_text_cmdref (GtkAction *action)
 {
     /* pos = 0 gives index of commands */
-    int pos = 0;
+    const gchar *aname = NULL;
     int flags = HELP_CLI;
-    int en = 0;
-
-    if (w == NULL && p != NULL) {
-	en = GPOINTER_TO_INT(p);
+    int cmdnum = 0, pos = 0;
+ 
+    if (action != NULL) {
+	aname = gtk_action_get_name(action);
+	cmdnum = atoi(aname);
     }
 
-#if HDEBUG
-    fprintf(stderr, "plain_text_cmdref: p=%p, cmdnum=%d, w=%p\n", 
-	    (void *) p, cmdnum, (void *) w);
-#endif
+    if (cmdnum > 0) {
+	pos = cli_pos_from_cmd(cmdnum, 0);
+	if (pos < 0 && translated_helpfile == 1) {
+	    /* no translated entry: fall back on English */
+	    flags |= HELP_EN;
+	    pos = cli_pos_from_cmd(cmdnum, 1);
+	}
+    }
+
+    real_do_help(cmdnum, pos, flags);
+} 
+
+void command_help_callback (int cmdnum, int en)
+{
+    int pos = 0, flags = HELP_CLI;
 
     if (cmdnum > 0) {
 	pos = cli_pos_from_cmd(cmdnum, en);
@@ -1315,20 +1319,28 @@ void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w)
     }
 
     real_do_help(cmdnum, pos, flags);
-} 
+}
+
+void function_help_callback (int fnum)
+{
+    int pos = help_pos_from_function_index(fnum);
+
+    real_do_help(fnum, pos, HELP_FUNCS); 
+}
 
 /* called from main menu in gretl.c; also used as callback from
-   command ref index in textbuf.c
+   help window topics menu
 */
 
-void genr_funcs_ref (gpointer p, guint fnum, GtkWidget *w)
+void genr_funcs_ref (GtkAction *action)
 {
+    int fnum = atoi(gtk_action_get_name(action));
     int pos = help_pos_from_function_index(fnum);
 
     real_do_help(fnum, pos, HELP_FUNCS);    
 }
 
-static void en_text_cmdref (gpointer p, guint u, GtkWidget *w)
+static void en_text_cmdref (GtkAction *action)
 {
     real_do_help(0, 0, HELP_CLI | HELP_EN);
 } 
@@ -1378,7 +1390,7 @@ gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
 	GtkTextBuffer *buf;
 	GtkTextIter iter;
 
-	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	gtk_text_buffer_get_iter_at_mark(buf, &iter,
 					 gtk_text_buffer_get_insert(buf));
 
@@ -1438,7 +1450,7 @@ gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
 
 	g_free(text);
 	unset_window_help_active(vwin);
-	text_set_cursor(vwin->w, 0);
+	text_set_cursor(vwin->text, 0);
 
 	if (pos <= 0) {
 	    warnbox(_("Sorry, help not found"));
@@ -1453,25 +1465,16 @@ gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
     return FALSE;
 }
 
-void menu_find (gpointer data, guint db, GtkWidget *widget)
+void text_find (gpointer unused, gpointer data)
 {
-    if (db) {
-	find_string_dialog(find_in_listbox, data);
-    } else {
-	find_string_dialog(find_in_text, data);
-    }
+    find_string_dialog(find_in_text, data);
 }
 
-void datafile_find (GtkWidget *widget, gpointer data)
+void listbox_find (gpointer unused, gpointer data)
 {
     find_string_dialog(find_in_listbox, data);
 }
 
-void find_var (gpointer p, guint u, GtkWidget *w)
-{
-    find_string_dialog(find_in_listbox, mdata);
-}
-
 static gint close_find_dialog (GtkWidget *widget, gpointer data)
 {
     find_window = NULL;
@@ -1572,7 +1575,7 @@ static void find_in_text (GtkWidget *widget, gpointer data)
 	return;
     }
 
-    found = real_find_in_text(GTK_TEXT_VIEW(vwin->w), needle, TRUE);
+    found = real_find_in_text(GTK_TEXT_VIEW(vwin->text), needle, TRUE);
 
     if (!found) {
 	infobox(_("String was not found."));
@@ -1688,17 +1691,10 @@ static void cancel_find (GtkWidget *widget, gpointer data)
 
 static void parent_find (GtkWidget *finder, windata_t *caller)
 {
-    GtkWidget *w = NULL;
-
-    if (caller->dialog != NULL) {
-	w = caller->dialog;
-    } else if (caller->w != NULL) {
-	w = caller->w;
-    }
+    GtkWidget *w = caller->main;
 
     if (w != NULL) {
-	gtk_window_set_transient_for(GTK_WINDOW(finder),
-				     GTK_WINDOW(w));
+	gtk_window_set_transient_for(GTK_WINDOW(finder), GTK_WINDOW(w));
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(finder), TRUE);
     }
 }
@@ -1767,24 +1763,6 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
     gtk_widget_show(find_window);
 }
 
-void text_find_callback (GtkWidget *w, windata_t *vwin)
-{
-    find_string_dialog(find_in_text, vwin);
-}
-
-static GtkTooltips *gretl_tips;
-
-void gretl_tooltips_init (void)
-{
-    gretl_tips = gtk_tooltips_new();
-    gtk_tooltips_enable(gretl_tips); /* redundant? */
-}
-
-void gretl_tooltips_add (GtkWidget *w, const gchar *str)
-{
-    gtk_tooltips_set_tip(gretl_tips, w, str, NULL);
-}
-
 enum {
     EN_LETTER,
     EN_A4,
@@ -1896,14 +1874,17 @@ static int find_or_download_pdf (int uguide, int i, char *fullpath)
     return err;
 }
 
-void display_pdf_help (gpointer p, guint uguide, GtkWidget *w)
+void display_pdf_help (GtkAction *action)
 {
     char fname[FILENAME_MAX];
-    int pref, err = 0;
+    int err, uguide = 1;
 
-    pref = get_manpref();
+    if (action != NULL && strcmp(gtk_action_get_name(action), "UserGuide")) {
+	/* PDF command ref wanted */
+	uguide = 0;
+    }
 
-    err = find_or_download_pdf(uguide, pref, fname);
+    err = find_or_download_pdf(uguide, get_manpref(), fname);
     if (err) {
 	return;
     }
