@@ -45,6 +45,8 @@ static void mtable_errmsg (char *msg, int gui)
     } else {
 	gretl_errmsg_set(msg);
     }
+
+    fprintf(stderr, "%s\n", msg);
 }
 
 static int real_table_n_models (void)
@@ -242,6 +244,7 @@ static int real_add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
 {
     if (model_table_precheck(pmod, add_mode)) {
+	fprintf(stderr, "add_to_model_table: precheck failed\n");
 	return 1;
     }
 
@@ -549,44 +552,38 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
     }
 }
 
-static int any_log_lik (void)
+enum {
+    MT_LNL,
+    MT_RSQ,
+    MT_FSTAT
+};
+
+static int any_stat (int s)
 {
     int i;
 
     for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) {
-	    continue;
-	}
-	if (!na(table_models[i]->lnL)) {
-	    return 1;
+	if (table_models[i] != NULL) {
+	    if (s == MT_LNL && !na(table_models[i]->lnL)) {
+		return 1;
+	    } else if (s == MT_RSQ && !na(table_models[i]->rsq)) {
+		return 1;
+	    } else if (s == MT_FSTAT && !na(table_models[i]->fstt)) {
+		return 1;
+	    }
 	}
     }
 
     return 0;
 }
 
-static int any_r_squared (void)
-{
-    int i;
-
-    for (i=0; i<n_models; i++) {
-	if (table_models[i] == NULL) {
-	    continue;
-	}
-	if (!na(table_models[i]->rsq)) {
-	    return 1;
-	}
-    }
-
-    return 0;
-}
-
-static void print_n_r_squared (int width0, PRN *prn, int *binary)
+static void print_equation_stats (int width0, PRN *prn, int *binary)
 {
     const MODEL *pmod;
     int same_df, any_R2, any_ll;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
+    double rsq;
     int j;
 
     if (rtf) {
@@ -623,8 +620,9 @@ static void print_n_r_squared (int width0, PRN *prn, int *binary)
     }
 
     same_df = common_df();
-    any_R2 = any_r_squared();
-    any_ll = any_log_lik();
+
+    any_R2 = any_stat(MT_RSQ);
+    any_ll = any_stat(MT_LNL);
 
     if (any_R2) {
 	/* print R^2 values */
@@ -641,7 +639,12 @@ static void print_n_r_squared (int width0, PRN *prn, int *binary)
 	for (j=0; j<n_models; j++) {
 	    pmod = table_models[j];
 	    if (pmod == NULL) continue;
-	    if (na(pmod->rsq)) {
+	    if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
+		rsq = pmod->rsq;
+	    } else {
+		rsq = (same_df)? pmod->rsq : pmod->adjrsq;
+	    }
+	    if (na(rsq)) {
 		if (tex) {
 		    pputs(prn, "& ");
 		} else if (rtf) {
@@ -660,8 +663,6 @@ static void print_n_r_squared (int width0, PRN *prn, int *binary)
 		    pprintf(prn, "%#*.4g", colwidth, pmod->rsq);
 		}
 	    } else {
-		double rsq = (same_df)? pmod->rsq : pmod->adjrsq;
-
 		if (tex) {
 		    pprintf(prn, "& %.4f ", rsq);
 		} else if (rtf) {
@@ -686,7 +687,6 @@ static void print_n_r_squared (int width0, PRN *prn, int *binary)
 
     if (any_ll) {
 	/* print log-likelihoods */
-
 	if (tex) {
 	    pputs(prn, "$\\ell$");
 	} else if (rtf) {
@@ -817,7 +817,7 @@ int display_model_table (int gui)
     pputc(prn, '\n'); 
 
     print_model_table_coeffs(namelen, prn);
-    print_n_r_squared(namelen + 1, prn, &binary);
+    print_equation_stats(namelen + 1, prn, &binary);
 
     if (use_tstats) {
 	pprintf(prn, "%s\n", _("t-statistics in parentheses"));
@@ -916,7 +916,7 @@ static int tex_print_model_table (PRN *prn)
     pputs(prn, " [6pt] \n");   
 
     print_model_table_coeffs(0, prn);
-    print_n_r_squared(0, prn, &binary);
+    print_equation_stats(0, prn, &binary);
 
     pputs(prn, "\\end{longtable}\n\n");
     pputs(prn, "\\vspace{1em}\n");
@@ -1021,11 +1021,16 @@ static int rtf_print_model_table (PRN *prn)
     }
 
     print_model_table_coeffs(0, prn);
-    print_n_r_squared(0, prn, &binary);
+    print_equation_stats(0, prn, &binary);
 
     pputs(prn, "}\n\n");
 
-    pprintf(prn, "\\par \\qc %s\n", I_("Standard errors in parentheses"));
+    if (use_tstats) {
+	pprintf(prn, "\\par \\qc %s\n", I_("t-statistics in parentheses"));
+    } else {
+	pprintf(prn, "\\par \\qc %s\n", I_("Standard errors in parentheses"));
+    }
+
     pprintf(prn, "\\par \\qc %s\n", 
 	    I_("* indicates significance at the 10 percent level"));
     pprintf(prn, "\\par \\qc %s\n", 
