@@ -24,9 +24,13 @@
 
 #include "gretl.h"
 
-#include "obsbutton.h"
+#include <gdk/gdk.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkadjustment.h>
 #include <gtk/gtkmain.h>
 
+#include "obsbutton.h"
+
 #define MIN_OBS_BUTTON_WIDTH              30
 #define OBS_BUTTON_INITIAL_TIMER_DELAY    200
 #define OBS_BUTTON_TIMER_DELAY            20
@@ -39,6 +43,35 @@ enum {
   PROP_VALUE
 };
 
+struct _ObsButton
+{
+    GtkEntry entry;
+    GtkAdjustment *adjustment;
+    const DATAINFO *pdinfo;
+    GdkWindow *panel;
+    guint32 timer;
+    gdouble timer_step;
+    guint in_child : 2;
+    guint click_child : 2; /* valid: GTK_ARROW_UP=0, GTK_ARROW_DOWN=1 or 2=NONE/BOTH */
+    guint button : 2;
+    guint need_timer : 1;
+    guint timer_calls : 3;
+};
+
+struct _ObsButtonClass
+{
+    GtkEntryClass parent_class;
+
+    gint (*input)  (ObsButton *obs_button,
+		    gdouble       *new_value);
+    gint (*output) (ObsButton *obs_button);
+    void (*value_changed) (ObsButton *obs_button);
+
+    /* Action signals for keybindings, do not connect to these */
+    void (*change_value) (ObsButton *obs_button,
+			  GtkScrollType scroll);
+};
+
 static void obs_button_class_init     (ObsButtonClass *klass);
 static void obs_button_editable_init  (GtkEditableClass   *iface);
 static void obs_button_init           (ObsButton      *obs_button);
@@ -113,56 +146,52 @@ static GtkEntryClass *parent_class = NULL;
 
 #define NO_ARROW 2
 
-GType
-obs_button_get_type (void)
+GType obs_button_get_type (void)
 {
-  static GType obs_button_type = 0;
+    static GType obs_button_type = 0;
 
-  if (!obs_button_type)
-    {
-      static const GTypeInfo obs_button_info =
-      {
-	sizeof (ObsButtonClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) obs_button_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (ObsButton),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) obs_button_init,
-	NULL
-      };
+    if (!obs_button_type) {
+	static const GTypeInfo obs_button_info = {
+	    sizeof (ObsButtonClass),
+	    NULL,		/* base_init */
+	    NULL,		/* base_finalize */
+	    (GClassInitFunc) obs_button_class_init,
+	    NULL,		/* class_finalize */
+	    NULL,		/* class_data */
+	    sizeof (ObsButton),
+	    0,		/* n_preallocs */
+	    (GInstanceInitFunc) obs_button_init,
+	    NULL
+	};
 
-      static const GInterfaceInfo editable_info =
-      {
-	(GInterfaceInitFunc) obs_button_editable_init, /* interface_init */
-	NULL, /* interface_finalize */
-	NULL  /* interface_data */
-      };
+	static const GInterfaceInfo editable_info = {
+	    (GInterfaceInitFunc) obs_button_editable_init, /* interface_init */
+	    NULL, /* interface_finalize */
+	    NULL  /* interface_data */
+	};
 
-      obs_button_type =
-	g_type_register_static (GTK_TYPE_ENTRY, "ObsButton",
-				&obs_button_info, 0);
+	obs_button_type =
+	    g_type_register_static(GTK_TYPE_ENTRY, "ObsButton",
+				   &obs_button_info, 0);
 
-      g_type_add_interface_static (obs_button_type,
-				   GTK_TYPE_EDITABLE,
-				   &editable_info);
+	g_type_add_interface_static(obs_button_type,
+				    GTK_TYPE_EDITABLE,
+				    &editable_info);
     }
-  return obs_button_type;
+    return obs_button_type;
 }
 
 static void
 obs_button_class_init (ObsButtonClass *class)
 {
-  GObjectClass     *gobject_class = G_OBJECT_CLASS (class);
+  GObjectClass     *gobject_class = G_OBJECT_CLASS(class);
   GtkObjectClass   *object_class;
   GtkWidgetClass   *widget_class;
   GtkEntryClass    *entry_class;
 
-  object_class   = (GtkObjectClass*)   class;
-  widget_class   = (GtkWidgetClass*)   class;
-  entry_class    = (GtkEntryClass*)    class;
+  object_class = (GtkObjectClass*)   class;
+  widget_class = (GtkWidgetClass*)   class;
+  entry_class  = (GtkEntryClass*)    class;
 
   parent_class = g_type_class_peek_parent (class);
 
@@ -227,10 +256,10 @@ obs_button_set_property (GObject      *object,
 			 const GValue *value,
 			 GParamSpec   *pspec)
 {
-    ObsButton *obs_button = OBS_BUTTON (object);
+    ObsButton *obs_button = OBS_BUTTON(object);
 
     if (prop_id == PROP_VALUE) {
-	obs_button_set_value (obs_button, g_value_get_double (value));
+	obs_button_set_value(obs_button, g_value_get_double(value));
     }
 }
 
@@ -267,34 +296,34 @@ obs_button_init (ObsButton *obs_button)
 static void
 obs_button_finalize (GObject *object)
 {
-    obs_button_set_adjustment (OBS_BUTTON (object), NULL);
+    obs_button_set_adjustment(OBS_BUTTON(object), NULL);
   
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+    G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
 static void
 obs_button_destroy (GtkObject *object)
 {
-    obs_button_stop_spinning (OBS_BUTTON (object));
+    obs_button_stop_spinning(OBS_BUTTON(object));
   
-    GTK_OBJECT_CLASS (parent_class)->destroy (object);
+    GTK_OBJECT_CLASS(parent_class)->destroy(object);
 }
 
 static void
 obs_button_map (GtkWidget *widget)
 {
-    if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget)) {
-	GTK_WIDGET_CLASS (parent_class)->map (widget);
-	gdk_window_show (OBS_BUTTON (widget)->panel);
+    if (GTK_WIDGET_REALIZED(widget) && !GTK_WIDGET_MAPPED(widget)) {
+	GTK_WIDGET_CLASS(parent_class)->map(widget);
+	gdk_window_show(OBS_BUTTON(widget)->panel);
     }
 }
 
 static void
 obs_button_unmap (GtkWidget *widget)
 {
-    if (GTK_WIDGET_MAPPED (widget)) {
-	gdk_window_hide (OBS_BUTTON (widget)->panel);
-	GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+    if (GTK_WIDGET_MAPPED(widget)) {
+	gdk_window_hide(OBS_BUTTON(widget)->panel);
+	GTK_WIDGET_CLASS(parent_class)->unmap(widget);
     }
 }
 
@@ -307,22 +336,22 @@ obs_button_realize (GtkWidget *widget)
     guint real_width;
     gint arrow_size;
 
-    obs_button = OBS_BUTTON (widget);
-    arrow_size = obs_button_get_arrow_size (obs_button);
+    obs_button = OBS_BUTTON(widget);
+    arrow_size = obs_button_get_arrow_size(obs_button);
 
     real_width = widget->allocation.width;
     widget->allocation.width -= arrow_size + 2 * widget->style->xthickness;
-    gtk_widget_set_events (widget, gtk_widget_get_events (widget) |
-			   GDK_KEY_RELEASE_MASK);
-    GTK_WIDGET_CLASS (parent_class)->realize (widget);
+    gtk_widget_set_events(widget, gtk_widget_get_events(widget) |
+			  GDK_KEY_RELEASE_MASK);
+    GTK_WIDGET_CLASS(parent_class)->realize(widget);
 
     widget->allocation.width = real_width;
   
     attributes.window_type = GDK_WINDOW_CHILD;
     attributes.wclass = GDK_INPUT_OUTPUT;
-    attributes.visual = gtk_widget_get_visual (widget);
-    attributes.colormap = gtk_widget_get_colormap (widget);
-    attributes.event_mask = gtk_widget_get_events (widget);
+    attributes.visual = gtk_widget_get_visual(widget);
+    attributes.colormap = gtk_widget_get_colormap(widget);
+    attributes.event_mask = gtk_widget_get_events(widget);
     attributes.event_mask |= GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK 
 	| GDK_BUTTON_RELEASE_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK 
 	| GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK;
@@ -337,27 +366,27 @@ obs_button_realize (GtkWidget *widget)
     attributes.width = arrow_size + 2 * widget->style->xthickness;
     attributes.height = widget->requisition.height;
   
-    obs_button->panel = gdk_window_new (gtk_widget_get_parent_window (widget), 
-					&attributes, attributes_mask);
-    gdk_window_set_user_data (obs_button->panel, widget);
+    obs_button->panel = gdk_window_new(gtk_widget_get_parent_window(widget), 
+				       &attributes, attributes_mask);
+    gdk_window_set_user_data(obs_button->panel, widget);
 
-    gtk_style_set_background (widget->style, obs_button->panel, GTK_STATE_NORMAL);
+    gtk_style_set_background(widget->style, obs_button->panel, GTK_STATE_NORMAL);
 
-    obs_button_default_output (obs_button);
+    obs_button_default_output(obs_button);
 
-    gtk_widget_queue_resize (GTK_WIDGET (obs_button));
+    gtk_widget_queue_resize(GTK_WIDGET(obs_button));
 }
 
 static void
 obs_button_unrealize (GtkWidget *widget)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
-    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+    GTK_WIDGET_CLASS(parent_class)->unrealize(widget);
 
     if (spin->panel) {
-	gdk_window_set_user_data (spin->panel, NULL);
-	gdk_window_destroy (spin->panel);
+	gdk_window_set_user_data(spin->panel, NULL);
+	gdk_window_destroy(spin->panel);
 	spin->panel = NULL;
     }
 }
@@ -387,29 +416,29 @@ obs_button_size_request (GtkWidget      *widget,
 	gboolean interior_focus;
 	gint focus_width;
 
-	gtk_widget_style_get (widget,
-			      "interior-focus", &interior_focus,
-			      "focus-line-width", &focus_width,
-			      NULL);
+	gtk_widget_style_get(widget,
+			     "interior-focus", &interior_focus,
+			     "focus-line-width", &focus_width,
+			     NULL);
 
-	context = gtk_widget_get_pango_context (widget);
-	metrics = pango_context_get_metrics (context,
-					     widget->style->font_desc,
-					     pango_context_get_language (context));
+	context = gtk_widget_get_pango_context(widget);
+	metrics = pango_context_get_metrics(context,
+					    widget->style->font_desc,
+					    pango_context_get_language(context));
 
-	digit_width = pango_font_metrics_get_approximate_digit_width (metrics);
+	digit_width = pango_font_metrics_get_approximate_digit_width(metrics);
 	digit_width = PANGO_SCALE *
 	    ((digit_width + PANGO_SCALE - 1) / PANGO_SCALE);
 
-	pango_font_metrics_unref (metrics);
+	pango_font_metrics_unref(metrics);
       
 	/* Get max of MIN_OBS_BUTTON_WIDTH, size of upper, size of lower */
       
 	width = MIN_OBS_BUTTON_WIDTH;
 	max_string_len = OBSLEN;
 	string_len = strlen(obs_button->pdinfo->endobs) + 1;
-	w = PANGO_PIXELS (MIN (string_len, max_string_len) * digit_width);
-	width = MAX (width, w);
+	w = PANGO_PIXELS(MIN(string_len, max_string_len) * digit_width);
+	width = MAX(width, w);
       
 	requisition->width = width + /* INNER_BORDER */ 2 * 2;
 	if (!interior_focus)
@@ -429,11 +458,11 @@ obs_button_size_allocate (GtkWidget     *widget,
     gint arrow_size;
     gint panel_width;
 
-    g_return_if_fail (GTK_IS_OBS_BUTTON (widget));
-    g_return_if_fail (allocation != NULL);
+    g_return_if_fail(GTK_IS_OBS_BUTTON(widget));
+    g_return_if_fail(allocation != NULL);
 
-    spin = OBS_BUTTON (widget);
-    arrow_size = obs_button_get_arrow_size (spin);
+    spin = OBS_BUTTON(widget);
+    arrow_size = obs_button_get_arrow_size(spin);
     panel_width = arrow_size + 2 * widget->style->xthickness;
   
     widget->allocation = *allocation;
@@ -441,7 +470,7 @@ obs_button_size_allocate (GtkWidget     *widget,
     entry_allocation = *allocation;
     entry_allocation.width -= panel_width;
 
-    if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) {
+    if (gtk_widget_get_direction(widget) == GTK_TEXT_DIR_RTL) {
 	entry_allocation.x += panel_width;
 	panel_allocation.x = allocation->x;
     } else {
@@ -449,22 +478,22 @@ obs_button_size_allocate (GtkWidget     *widget,
     }
 
     panel_allocation.width = panel_width;
-    panel_allocation.height = MIN (widget->requisition.height, allocation->height);
+    panel_allocation.height = MIN(widget->requisition.height, allocation->height);
 
     panel_allocation.y = allocation->y + (allocation->height -
 					  panel_allocation.height) / 2;
 
-    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, &entry_allocation);
+    GTK_WIDGET_CLASS(parent_class)->size_allocate(widget, &entry_allocation);
 
-    if (GTK_WIDGET_REALIZED (widget)) {
-	gdk_window_move_resize (OBS_BUTTON (widget)->panel, 
-				panel_allocation.x,
-				panel_allocation.y,
-				panel_allocation.width,
-				panel_allocation.height); 
+    if (GTK_WIDGET_REALIZED(widget)) {
+	gdk_window_move_resize(OBS_BUTTON(widget)->panel, 
+			       panel_allocation.x,
+			       panel_allocation.y,
+			       panel_allocation.width,
+			       panel_allocation.height); 
     }
 
-    obs_button_redraw (spin);
+    obs_button_redraw(spin);
 }
 
 static gint
@@ -473,17 +502,17 @@ obs_button_expose (GtkWidget      *widget,
 {
     ObsButton *spin;
 
-    g_return_val_if_fail (GTK_IS_OBS_BUTTON (widget), FALSE);
-    g_return_val_if_fail (event != NULL, FALSE);
+    g_return_val_if_fail(GTK_IS_OBS_BUTTON(widget), FALSE);
+    g_return_val_if_fail(event != NULL, FALSE);
 
-    spin = OBS_BUTTON (widget);
+    spin = OBS_BUTTON(widget);
 
-    if (GTK_WIDGET_DRAWABLE (widget)) {
+    if (GTK_WIDGET_DRAWABLE(widget)) {
 	GtkShadowType shadow_type;
 	GdkRectangle rect;
 
 	if (event->window != spin->panel)
-	    GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+	    GTK_WIDGET_CLASS(parent_class)->expose_event(widget, event);
 
 	/* we redraw the panel even if it wasn't exposed. This is
 	 * because spin->panel is not a child window of widget->window,
@@ -492,23 +521,23 @@ obs_button_expose (GtkWidget      *widget,
 	rect.x = 0;
 	rect.y = 0;
 
-	gdk_drawable_get_size (spin->panel, &rect.width, &rect.height);
+	gdk_drawable_get_size(spin->panel, &rect.width, &rect.height);
 
-	shadow_type = obs_button_get_shadow_type (spin);
+	shadow_type = obs_button_get_shadow_type(spin);
       
-	gdk_window_begin_paint_rect (spin->panel, &rect);      
+	gdk_window_begin_paint_rect(spin->panel, &rect);      
 
 	if (shadow_type != GTK_SHADOW_NONE) {
-	    gtk_paint_box (widget->style, spin->panel,
-			   GTK_STATE_NORMAL, shadow_type,
-			   NULL, widget, "spinbutton",
-			   rect.x, rect.y, rect.width, rect.height);
+	    gtk_paint_box(widget->style, spin->panel,
+			  GTK_STATE_NORMAL, shadow_type,
+			  NULL, widget, "spinbutton",
+			  rect.x, rect.y, rect.width, rect.height);
 	}
 
-	obs_button_draw_arrow (spin, GTK_ARROW_UP);
-	obs_button_draw_arrow (spin, GTK_ARROW_DOWN);
+	obs_button_draw_arrow(spin, GTK_ARROW_UP);
+	obs_button_draw_arrow(spin, GTK_ARROW_DOWN);
 
-	gdk_window_end_paint (spin->panel);
+	gdk_window_end_paint(spin->panel);
     }
   
     return FALSE;
@@ -593,8 +622,7 @@ obs_button_draw_arrow (ObsButton *obs_button,
 	    if (obs_button->click_child == arrow_type) {
 		state_type = GTK_STATE_ACTIVE;
 		shadow_type = GTK_SHADOW_IN;
-	    }
-	    else {
+	    } else {
 		if (obs_button->in_child == arrow_type &&
 		    obs_button->click_child == NO_ARROW) {
 		    state_type = GTK_STATE_PRELIGHT;
@@ -651,13 +679,13 @@ static gint
 obs_button_enter_notify (GtkWidget        *widget,
 			 GdkEventCrossing *event)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
     if (event->window == spin->panel) {
 	gint x;
 	gint y;
 
-	gdk_window_get_pointer (spin->panel, &x, &y, NULL);
+	gdk_window_get_pointer(spin->panel, &x, &y, NULL);
 
 	if (y <= widget->requisition.height / 2)
 	    spin->in_child = GTK_ARROW_UP;
@@ -674,7 +702,7 @@ static gint
 obs_button_leave_notify (GtkWidget        *widget,
 			 GdkEventCrossing *event)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
     spin->in_child = NO_ARROW;
     obs_button_redraw (spin);
@@ -686,21 +714,21 @@ static gint
 obs_button_focus_out (GtkWidget     *widget,
 		      GdkEventFocus *event)
 {
-    if (GTK_ENTRY (widget)->editable)
-	obs_button_update (OBS_BUTTON (widget));
+    if (GTK_ENTRY(widget)->editable)
+	obs_button_update(OBS_BUTTON(widget));
 
-    return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, event);
+    return GTK_WIDGET_CLASS(parent_class)->focus_out_event(widget, event);
 }
 
 static void
 obs_button_grab_notify (GtkWidget *widget,
 			gboolean   was_grabbed)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
     if (!was_grabbed) {
-	obs_button_stop_spinning (spin);
-	obs_button_redraw (spin);
+	obs_button_stop_spinning(spin);
+	obs_button_redraw(spin);
     }
 }
 
@@ -708,11 +736,11 @@ static void
 obs_button_state_changed (GtkWidget    *widget,
 			  GtkStateType  previous_state)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
-    if (!GTK_WIDGET_IS_SENSITIVE (widget)) {
-	obs_button_stop_spinning (spin);    
-	obs_button_redraw (spin);
+    if (!GTK_WIDGET_IS_SENSITIVE(widget)) {
+	obs_button_stop_spinning(spin);    
+	obs_button_redraw(spin);
     }
 }
 
@@ -742,7 +770,7 @@ static void
 obs_button_stop_spinning (ObsButton *spin)
 {
     if (spin->timer) {
-	gtk_timeout_remove (spin->timer);
+	g_source_remove (spin->timer);
 	spin->timer = 0;
 	spin->timer_calls = 0;
 	spin->need_timer = FALSE;
@@ -770,8 +798,8 @@ start_spinning (ObsButton *spin,
     if (!spin->timer) {
 	spin->timer_step = step;
 	spin->need_timer = TRUE;
-	spin->timer = gtk_timeout_add (OBS_BUTTON_INITIAL_TIMER_DELAY, 
-				       (GtkFunction) obs_button_timer, (gpointer) spin);
+	spin->timer = g_timeout_add (OBS_BUTTON_INITIAL_TIMER_DELAY, 
+				     (GSourceFunc) obs_button_timer, (gpointer) spin);
     }
 
     obs_button_redraw (spin);
@@ -891,7 +919,7 @@ obs_button_motion_notify (GtkWidget      *widget,
     return GTK_WIDGET_CLASS (parent_class)->motion_notify_event (widget, event);
 }
 
-static gint
+static gboolean
 obs_button_timer (ObsButton *obs_button)
 {
     gboolean retval = FALSE;
@@ -906,8 +934,8 @@ obs_button_timer (ObsButton *obs_button)
 
 	if (obs_button->need_timer) {
 	    obs_button->need_timer = FALSE;
-	    obs_button->timer = gtk_timeout_add 
-		(OBS_BUTTON_TIMER_DELAY, (GtkFunction) obs_button_timer, 
+	    obs_button->timer = g_timeout_add 
+		(OBS_BUTTON_TIMER_DELAY, (GSourceFunc) obs_button_timer, 
 		 (gpointer) obs_button);
 	} else {
 	    retval = TRUE;
@@ -923,13 +951,13 @@ static void
 obs_button_value_changed (GtkAdjustment *adjustment,
 			  ObsButton *obs_button)
 {
-    g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+    g_return_if_fail(GTK_IS_ADJUSTMENT(adjustment));
 
-    obs_button_default_output (obs_button);
+    obs_button_default_output(obs_button);
 
-    obs_button_redraw (obs_button);
+    obs_button_redraw(obs_button);
   
-    g_object_notify (G_OBJECT (obs_button), "value");
+    g_object_notify(G_OBJECT(obs_button), "value");
 }
 
 static void
@@ -994,7 +1022,7 @@ static gint
 obs_button_key_release (GtkWidget   *widget,
 			GdkEventKey *event)
 {
-    ObsButton *spin = OBS_BUTTON (widget);
+    ObsButton *spin = OBS_BUTTON(widget);
 
     /* We only get a release at the end of a key repeat run, so reset the timer_step */
     spin->timer_step = spin->adjustment->step_increment;
@@ -1007,10 +1035,10 @@ static void
 obs_button_activate (GtkEntry *entry)
 {
     if (entry->editable)
-	obs_button_update (OBS_BUTTON (entry));
+	obs_button_update(OBS_BUTTON(entry));
 
     /* Chain up so that entry->activates_default is honored */
-    parent_class->activate (entry);
+    parent_class->activate(entry);
 }
 
 static void
@@ -1019,10 +1047,10 @@ obs_button_insert_text (GtkEditable *editable,
 			gint         new_text_length,
 			gint        *position)
 {
-    GtkEditableClass *parent_editable_iface = g_type_interface_peek (parent_class, GTK_TYPE_EDITABLE);
+    GtkEditableClass *parent_editable_iface = g_type_interface_peek(parent_class, GTK_TYPE_EDITABLE);
  
-    parent_editable_iface->insert_text (editable, new_text,
-					new_text_length, position);
+    parent_editable_iface->insert_text(editable, new_text,
+				       new_text_length, position);
 }
 
 static void
@@ -1037,10 +1065,9 @@ obs_button_real_spin (ObsButton *obs_button,
     new_value = adj->value + increment;
 
     if (increment > 0) {
-	new_value = MIN (new_value, adj->upper);
-    }
-    else if (increment < 0) {
-	new_value = MAX (new_value, adj->lower);
+	new_value = MIN(new_value, adj->upper);
+    } else if (increment < 0) {
+	new_value = MAX(new_value, adj->lower);
     }
 
     if (fabs (new_value - adj->value) > EPSILON)
@@ -1051,7 +1078,7 @@ obs_button_real_spin (ObsButton *obs_button,
 
 static gint
 obs_button_default_input (ObsButton *obs_button,
-			  gdouble       *new_val)
+			  gdouble *new_val)
 {
     *new_val = dateton(gtk_entry_get_text(GTK_ENTRY (obs_button)), 
 		       obs_button->pdinfo);
@@ -1073,8 +1100,8 @@ obs_button_default_output (ObsButton *obs_button)
 
     ntodate_full(buf, ival, obs_button->pdinfo);
 
-    if (strcmp (buf, gtk_entry_get_text (GTK_ENTRY (obs_button))))
-	gtk_entry_set_text (GTK_ENTRY (obs_button), buf);
+    if (strcmp(buf, gtk_entry_get_text(GTK_ENTRY(obs_button))))
+	gtk_entry_set_text(GTK_ENTRY(obs_button), buf);
 
     data = g_object_get_data(G_OBJECT(obs_button), "rset");
     if (data != NULL) {
@@ -1096,16 +1123,16 @@ obs_button_new (GtkAdjustment *adjustment, const DATAINFO *pdinfo)
     ObsButton *spin;
 
     if (adjustment)
-	g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);
+	g_return_val_if_fail(GTK_IS_ADJUSTMENT(adjustment), NULL);
 
-    spin = g_object_new (GTK_TYPE_OBS_BUTTON, NULL);
+    spin = g_object_new(GTK_TYPE_OBS_BUTTON, NULL);
 
     spin->pdinfo = pdinfo;
 
-    obs_button_set_adjustment (spin, adjustment);
-    gtk_adjustment_value_changed (adjustment);
+    obs_button_set_adjustment(spin, adjustment);
+    gtk_adjustment_value_changed(adjustment);
 
-    return GTK_WIDGET (spin);
+    return GTK_WIDGET(spin);
 }
 
 /* Callback used when the spin button's adjustment changes.  We need to redraw
@@ -1114,10 +1141,10 @@ obs_button_new (GtkAdjustment *adjustment, const DATAINFO *pdinfo)
 static void
 adjustment_changed_cb (GtkAdjustment *adjustment, gpointer data)
 {
-    ObsButton *obs_button = OBS_BUTTON (data);
+    ObsButton *obs_button = OBS_BUTTON(data);
 
     obs_button->timer_step = obs_button->adjustment->step_increment;
-    gtk_widget_queue_resize (GTK_WIDGET (obs_button));
+    gtk_widget_queue_resize(GTK_WIDGET(obs_button));
 }
 
 /**
@@ -1146,7 +1173,11 @@ obs_button_set_adjustment (ObsButton *obs_button,
 	obs_button->adjustment = adjustment;
 	if (adjustment) {
 	    g_object_ref (adjustment);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 10)
 	    gtk_object_sink (GTK_OBJECT (adjustment));
+#else
+	    g_object_ref_sink (GTK_OBJECT (adjustment));
+#endif
 	    g_signal_connect (adjustment, "value_changed",
 			      G_CALLBACK (obs_button_value_changed),
 			      obs_button);
@@ -1189,7 +1220,7 @@ obs_button_set_value (ObsButton *obs_button,
 {
     g_return_if_fail (GTK_IS_OBS_BUTTON (obs_button));
 
-    if (fabs (value - obs_button->adjustment->value) > EPSILON) {
+    if (fabs(value - obs_button->adjustment->value) > EPSILON) {
 	gtk_adjustment_set_value (obs_button->adjustment, value);
     } else {
 	obs_button_default_output (obs_button);
@@ -1212,7 +1243,7 @@ obs_button_get_shadow_type (ObsButton *spin_button)
 {
     GtkShadowType rc_shadow_type;
 
-    gtk_widget_style_get (GTK_WIDGET (spin_button), "shadow_type", &rc_shadow_type, NULL);
+    gtk_widget_style_get(GTK_WIDGET(spin_button), "shadow_type", &rc_shadow_type, NULL);
 
     return rc_shadow_type;
 }
@@ -1230,12 +1261,12 @@ obs_button_update (ObsButton *obs_button)
     gint error = 0;
     gint return_val;
 
-    g_return_if_fail (GTK_IS_OBS_BUTTON (obs_button));
+    g_return_if_fail(GTK_IS_OBS_BUTTON(obs_button));
 
-    return_val = obs_button_default_input (obs_button, &val);
+    return_val = obs_button_default_input(obs_button, &val);
     error = (return_val == GTK_INPUT_ERROR);
     
-    obs_button_redraw (obs_button);
+    obs_button_redraw(obs_button);
 
     if (val < obs_button->adjustment->lower)
 	val = obs_button->adjustment->lower;
@@ -1243,9 +1274,9 @@ obs_button_update (ObsButton *obs_button)
 	val = obs_button->adjustment->upper;
 
     if (fabs(val - obs_button->adjustment->value) > EPSILON) {
-	gtk_adjustment_set_value (obs_button->adjustment, val);
+	gtk_adjustment_set_value(obs_button->adjustment, val);
     } else {
-	obs_button_default_output (obs_button);
+	obs_button_default_output(obs_button);
     }
 }
 
@@ -1254,14 +1285,14 @@ obs_button_redraw (ObsButton *obs_button)
 {
     GtkWidget *widget;
 
-    widget = GTK_WIDGET (obs_button);
+    widget = GTK_WIDGET(obs_button);
 
-    if (GTK_WIDGET_DRAWABLE (widget)) {
-	gtk_widget_queue_draw (widget);
+    if (GTK_WIDGET_DRAWABLE(widget)) {
+	gtk_widget_queue_draw(widget);
 
 	/* We must invalidate the panel window ourselves, because it
 	 * is not a child of widget->window
 	 */
-	gdk_window_invalidate_rect (obs_button->panel, NULL, TRUE);
+	gdk_window_invalidate_rect(obs_button->panel, NULL, TRUE);
     }
 }        
