@@ -94,8 +94,6 @@ static int series_view_allocate (series_view *sview)
     if (sview->npoints != 0) {
 	/* already allocated */
 	return 0;
-    } else if (var_is_scalar(datainfo, sview->varnum)) {
-	return 0;
     } else {
 	int t, tp, T = datainfo->t2 - datainfo->t1 + 1;
 	int v = sview->varnum;
@@ -196,7 +194,7 @@ static void series_view_print (windata_t *vwin)
 	
     /* print formatted data to buffer */
 
-    if (var_is_series(datainfo, sview->varnum)) {
+    if (1) { /* FIXME scalars */
 	pprintf(prn, "\n%*s ", obslen, _("Obs"));
 	pprintf(prn, "%13s\n\n", datainfo->varname[sview->varnum]);
 	for (t=0; t<sview->npoints; t++) {
@@ -225,7 +223,7 @@ static void series_view_print (windata_t *vwin)
     }
 
     pbuf = gretl_print_get_buffer(prn);
-    textview_set_text(vwin->w, pbuf);
+    textview_set_text(vwin->text, pbuf);
 
     gretl_print_destroy(prn);
 }
@@ -270,7 +268,7 @@ static void multi_series_view_print_sorted (windata_t *vwin)
 	gui_errmsg(err);
     } else {
 	pbuf = gretl_print_get_buffer(prn);
-	textview_set_text(vwin->w, pbuf);
+	textview_set_text(vwin->text, pbuf);
     }
 
     free(obsvec);
@@ -471,22 +469,10 @@ int has_sortable_data (windata_t *vwin)
 multi_series_view *multi_series_view_new (const int *list)
 {
     multi_series_view *mview = NULL;
-    int i, err = 0;
 
     if (list == NULL) {
-	err = 1;
-    } else {
-	for (i=1; i<=list[0]; i++) {
-	    if (var_is_scalar(datainfo, list[i])) {
-		err = 1;
-		break;
-	    }
-	} 
-    }
-
-    if (err) {
 	return NULL;
-    }
+    } 
 
     mview = malloc(sizeof *mview);
 
@@ -548,7 +534,7 @@ void series_view_format_dialog (GtkWidget *src, windata_t *vwin)
     /* spinner for number of digits */
     hbox = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("Show"));
-    adj = gtk_adjustment_new(sview->digits, 1, 10, 1, 1, 1);
+    adj = gtk_adjustment_new(sview->digits, 1, 10, 1, 1, 0);
     sview->digit_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
     g_signal_connect (adj, "value_changed",
 		      G_CALLBACK(series_view_get_figures), sview);
