@@ -35,6 +35,8 @@
 #include "dlgutils.h"
 #include "fileselect.h"
 #include "menustate.h"
+#include "toolbar.h"
+#include "winstack.h"
 #include "lib_private.h"
 
 #include "var.h"
@@ -46,6 +48,7 @@
 #include "modelspec.h"
 #include "matrix_extra.h"
 #include "cmd_private.h"
+#include "gretl_scalar.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -104,7 +107,7 @@ struct SESSION_TEXT_ {
 };
 
 struct SESSION_MODEL_ {
-    char name[MAXSAVENAME];
+    char *name;
     void *ptr;
     GretlObjType type;
 };
@@ -139,12 +142,6 @@ enum {
     ICON_ADD_SINGLE
 } icon_add_modes;
 
-enum {
-    SAVEFILE_SESSION,
-    SAVEFILE_SCRIPT,
-    SAVEFILE_ERROR
-} savefile_returns;
-
 static char *global_items[] = {
     N_("Save session"),
     N_("Arrange icons"),
@@ -248,6 +245,7 @@ static gboolean session_view_click (GtkWidget *widget,
 				    gpointer data);
 static int real_delete_model_from_session (SESSION_MODEL *model);
 static void make_short_label_string (char *targ, const char *src);
+static gui_obj *get_gui_obj_by_data (void *finddata);
 
 static int session_graph_count;
 static int session_bplot_count;
@@ -262,7 +260,7 @@ void mark_session_changed (void)
     session.status = SESSION_CHANGED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, TRUE);
-	flip(mdata->ifac, "/File/Session files/Save session", TRUE);
+	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", TRUE);
     }
 }
 
@@ -271,7 +269,7 @@ static void mark_session_saved (void)
     session.status = SESSION_SAVED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, FALSE);
-	flip(mdata->ifac, "/File/Session files/Save session", FALSE);
+	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", FALSE);
     }
 }
 
@@ -285,12 +283,10 @@ static void free_session_text (SESSION_TEXT *text)
 
 static void free_session_model (SESSION_MODEL *mod)
 {
-    /* note: remove a reference to this model */
 #if SESSION_DEBUG
-    fprintf(stderr, "free_session_model: unref'ing ptr at %p\n", (void *) mod->ptr);
-    fprintf(stderr, " also freeing session_model at %p\n", mod);
+    fprintf(stderr, "free_session_model: ptr at %p\n", (void *) mod->ptr);
 #endif
-    gretl_object_unref(mod->ptr, mod->type);
+    gretl_object_remove_from_stack(mod->ptr, mod->type);
     free(mod);
 }
 
@@ -302,15 +298,11 @@ static SESSION_MODEL *session_model_new (void *ptr, const char *name,
     if (mod != NULL) {
 	mod->ptr = ptr;
 	mod->type = type;
-	if (name == NULL) {
-	    gretl_object_compose_name(ptr, type);
-	    name = gretl_object_get_name(ptr, type);
-	} 
-	*mod->name = 0;
-	strncat(mod->name, name, MAXSAVENAME - 1);
-
-	/* note: take care of adding a refence to model */
-	gretl_object_ref(ptr, type);
+	gretl_stack_object_as(ptr, type, name);
+	mod->name = gretl_object_get_name(ptr, type);
+	/* note: we don't add a reference here: that's handled by the
+	   mechanism in objstack.c 
+	*/
     }
 
     return mod;
@@ -456,8 +448,8 @@ static void edit_session_notes (void)
 	vwin = edit_buffer(&session.notes, 80, 400, 
 			   _("gretl: session notes"),
 			   EDIT_NOTES);
-	notes_window = vwin->dialog;
-	g_signal_connect(G_OBJECT(vwin->dialog), "destroy",
+	notes_window = vwin->main;
+	g_signal_connect(G_OBJECT(vwin->main), "destroy",
 			 G_CALLBACK(gtk_widget_destroyed),
 			 &notes_window);
     } else {
@@ -612,14 +604,14 @@ void save_output_as_text_icon (windata_t *vwin)
     gchar *buf;
     int err;
 
-    buf = textview_get_text(vwin->w);
+    buf = textview_get_text(vwin->text);
     if (buf == NULL) {
 	errbox("Couldn't retrieve buffer");
 	return;
     }
 
-    if (vwin->dialog != NULL) {
-	title = gtk_window_get_title(GTK_WINDOW(vwin->dialog));
+    if (vwin->main != NULL) {
+	title = gtk_window_get_title(GTK_WINDOW(vwin->main));
     }
 
     if (title != NULL && !strncmp(title, "gretl: ", 7)) {
@@ -707,7 +699,7 @@ static int session_dir_ok (void)
 
 	errno = 0;
 
-	sprintf(session.dirname, ".gretl-%d", (int) p); /* added dot! */
+	sprintf(session.dirname, ".gretl-%d", (int) p);
 	if (chdir(paths.dotdir)) {
 	    perror("moving to user directory");
 	    ret = 0;
@@ -893,7 +885,16 @@ static int model_type_from_vwin (windata_t *vwin)
     }
 }
 
-void model_add_as_icon (gpointer p, guint u, GtkWidget *w)
+static int close_on_add (GtkAction *action)
+{
+    if (action == NULL) {
+	return 1;
+    } else {
+	return (strstr(gtk_action_get_name(action), "Close") != NULL);
+    }
+}
+
+void model_add_as_icon (GtkAction *action, gpointer p)
 {
     windata_t *vwin = (windata_t *) p;
     void *ptr = vwin->data;
@@ -921,17 +922,10 @@ void model_add_as_icon (gpointer p, guint u, GtkWidget *w)
     }
 
     mark_session_changed();
-}
 
-void model_add_as_icon_and_close (gpointer p, guint u, GtkWidget *w)
-{
-    windata_t *vwin = (windata_t *) p;
-
-    model_add_as_icon(p, u, w);
-
-    if (!window_is_busy(vwin)) {
-	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(vwin->w)));
-    } 
+    if (close_on_add(action) && !window_is_busy(vwin)) {
+	gtk_widget_destroy(vwin->main);
+    } 	
 }
 
 static void
@@ -955,20 +949,29 @@ session_name_from_session_file (char *sname, const char *fname)
 	    fname, sname);
 }
 
-static int unzip_session_file (const char *fname)
+static int unzip_session_file (const char *fname, char **zdirname)
 {
-    int (*gretl_unzip_file)(const char *, GError **);
-    void *handle;
-    FILE *fp;
+    gchar *(*gretl_zipfile_get_topdir) (const char *);
+    int (*gretl_unzip_file) (const char *, GError **);
+    void *handle = NULL;
     GError *gerr = NULL;
-    int err, ret = SAVEFILE_SESSION;
+    int err = 0;
 
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	return SAVEFILE_ERROR;
+    gretl_zipfile_get_topdir = gui_get_plugin_function("gretl_zipfile_get_topdir",
+						       &handle);
+    if (gretl_zipfile_get_topdir == NULL) {
+        return 1;
     }
-    fclose(fp);
 
+    *zdirname = gretl_zipfile_get_topdir(fname);
+    close_plugin(handle);
+    handle = NULL;
+
+    if (zdirname == NULL) {
+	errbox("Couldn't read session directory name");
+	return 1;
+    }
+    
     chdir(paths.dotdir);
 
     gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
@@ -980,18 +983,58 @@ static int unzip_session_file (const char *fname)
     err = (*gretl_unzip_file)(fname, &gerr);
     if (gerr != NULL) {
 	fprintf(stderr, "gretl_unzip_file: '%s'\n", gerr->message);
-	ret = SAVEFILE_ERROR;
+	if (!err) {
+	    err = 1;
+	}
 	g_error_free(gerr);
     } else if (err) {
 	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
-	ret = SAVEFILE_ERROR;
     }
 
-    /* FIXME backward compat: try fallback to old-style session file */
-
     close_plugin(handle);
 
-    return ret;
+    fprintf(stderr, "unzip_session_file: returning %d\n", err);
+
+    return err;
+}
+
+/* remedial action in case of mis-coded filename */
+
+static int get_session_dataname (char *fname, struct sample_info *sinfo)
+{
+    struct dirent *dirent;
+    DIR *dir;
+    FILE *fp;
+    gchar *tmp;
+    int n, err = E_FOPEN;
+
+    tmp = g_strdup(session.dirname);
+    n = strlen(tmp);
+
+    if (tmp[n-1] == '/' || tmp[n-1] == '\\') {
+	tmp[n-1] = '\0';
+    }
+
+    dir = opendir(tmp);
+
+    if (dir != NULL) {
+	while ((dirent = readdir(dir)) != NULL) {
+	    if (has_suffix(dirent->d_name, ".gdt")) {
+		session_file_make_path(fname, dirent->d_name);
+		fp = gretl_fopen(fname, "r");
+		if (fp != NULL) {
+		    fclose(fp);
+		    err = 0;
+		} 
+		break;
+	    }
+	}	
+	closedir(dir);
+    }
+
+    g_free(tmp);
+
+    return err;
 }
 
 static void sinfo_init (struct sample_info *sinfo)
@@ -1004,28 +1047,20 @@ static void sinfo_init (struct sample_info *sinfo)
     sinfo->resample_n = 0;
 }
 
-static int set_session_dirname (const char *sname)
+static int set_session_dirname (const char *zdirname)
 {
     char test[MAXLEN];
     FILE *fp;
     int err = 0;
 
-    sprintf(session.dirname, ".%s", sname);
+    strcpy(session.dirname, zdirname);
     sprintf(test, "%s%csession.xml", session.dirname, SLASH);
     fp = gretl_fopen(test, "r");
 
-    if (fp != NULL) {
-	fclose(fp);
+    if (fp == NULL) {
+	err = E_FOPEN;
     } else {
-	/* try without leading dot */
-	strcpy(session.dirname, sname);
-	sprintf(test, "%s%csession.xml", session.dirname, SLASH);
-	fp = gretl_fopen(test, "r");
-	if (fp != NULL) {
-	    fclose(fp);
-	} else {
-	    err = E_FOPEN;
-	}
+	fclose(fp);
     }
 
     return err;
@@ -1036,8 +1071,9 @@ void do_open_session (void)
     struct sample_info sinfo;
     char sname[MAXLEN];
     char fname[MAXLEN];
+    gchar *zdirname = NULL;
     FILE *fp;
-    int status, err = 0;
+    int err = 0;
 
     sinfo_init(&sinfo);
 
@@ -1047,7 +1083,6 @@ void do_open_session (void)
     } else {
 	file_read_errbox(tryfile);
 	delete_from_filelist(FILE_LIST_SESSION, tryfile);
-	delete_from_filelist(FILE_LIST_SCRIPT, tryfile);
 	return;
     }
 
@@ -1058,53 +1093,68 @@ void do_open_session (void)
     fprintf(stderr, I_("\nReading session file %s\n"), sessionfile);
 
     chdir(paths.dotdir);
-    status = unzip_session_file(sessionfile);
+    err = unzip_session_file(sessionfile, &zdirname);
 
-    if (status == SAVEFILE_ERROR) {
+    if (err) {
 	/* FIXME more explicit error message */
+	g_free(zdirname);
 	file_read_errbox(sessionfile);
 	return;
-    } else if (status == SAVEFILE_SCRIPT) {
-	strcpy(scriptfile, sessionfile);
-	*sessionfile = '\0';
-	do_open_script(EDIT_SCRIPT);
-	return;
-    }
+    } 
 
     session_name_from_session_file(sname, sessionfile);
     strcpy(session.name, sname);
-    
-    err = set_session_dirname(sname);
+
+    err = set_session_dirname(zdirname);
     if (err) {
+	g_free(zdirname);
 	fprintf(stderr, "Failed on set_session_dirname\n");
 	file_read_errbox("session.xml");
 	return;
     }
 
+    g_free(zdirname);
     session_file_make_path(fname, "session.xml");
     err = read_session_xml(fname, &sinfo);
 
     if (err) {
 	/* FIXME more explicit error message */
-	fprintf(stderr, "Failed on read_session_xml\n");
+	fprintf(stderr, "Failed on read_session_xml: err = %d\n", err);
 	file_read_errbox("session.xml");
 	return;
     }
 
     session_file_make_path(paths.datfile, sinfo.datafile);
-    err = gretl_read_gdt(paths.datfile, &paths, &Z, datainfo, 
-			 OPT_P, NULL);
+    fp = gretl_fopen(paths.datfile, "r");
+
+    if (fp != NULL) {
+	/* OK */
+	strcpy(fname, paths.datfile);
+	fclose(fp);
+    } else {
+	/* try remedial action */
+	fprintf(stderr, "'%s' : not found, trying to fix\n", paths.datfile);
+	err = get_session_dataname(fname, &sinfo);
+    }
+
+    if (!err) {
+	err = gretl_read_gdt(fname, NULL, &Z, datainfo, OPT_P, NULL);
+    }
+
     if (err) {
 	/* FIXME more explicit error message */
 	file_read_errbox(sinfo.datafile);
 	return;
     } else {
-	fprintf(stderr, "Opened session datafile '%s'\n", paths.datfile);
+	fprintf(stderr, "Opened session datafile '%s'\n", fname);
     }
 
     session_file_make_path(fname, "matrices.xml");
     err = maybe_read_matrix_file(fname);
 
+    session_file_make_path(fname, "scalars.xml");
+    err = maybe_read_scalars_file(fname);
+
     session_file_make_path(fname, "functions.xml");
     err = maybe_read_functions_file(fname);
 
@@ -1298,7 +1348,7 @@ int highest_numbered_variable_in_session (void)
 
 int session_file_is_open (void)
 {
-    return session_file_open;
+    return (session_file_open && *sessionfile != '\0');
 }
 
 void gui_clear_dataset (void)
@@ -1548,7 +1598,10 @@ int save_session (char *fname)
 	close_plugin(handle);
     }
 
-    if (!err) {
+    if (gerr != NULL) {
+	errbox(gerr->message);
+	g_error_free(gerr);
+    } else {
 	mkfilelist(FILE_LIST_SESSION, fname);
 	mark_session_saved();
     }
@@ -1556,9 +1609,19 @@ int save_session (char *fname)
     return err;
 }
 
-void save_session_callback (GtkWidget *w, guint code, gpointer data)
+void save_session_callback (GtkAction *action)
 {
-    if (code == SAVE_AS_IS && session_file_open && sessionfile[0]) {
+    int as_is = 0;
+
+    if (session_file_is_open() && action != NULL) {
+	const gchar *s = gtk_action_get_name(action);
+
+	if (!strcmp(s, "SaveSession")) {
+	    as_is = 1;
+	}
+    }
+
+    if (as_is) {
 	save_session(sessionfile);
     } else {
 	file_selector(_("Save session"), SAVE_SESSION, FSEL_DATA_NONE, NULL);
@@ -1733,12 +1796,13 @@ void session_model_callback (void *ptr, int action)
     if (action == OBJ_ACTION_SHOW) {
 	display_session_model(mod);
     } else if (action == OBJ_ACTION_FREE) {
-#if SESSION_DEBUG
-	fprintf(stderr, "session_model_callback: gretl_object_unref at %p\n", 
-		(void *) ptr);
-#endif
-	/* FIXME not sure here: should we trash the icon too */
-	gretl_object_unref(ptr, mod->type);
+	gui_obj *gobj = get_gui_obj_by_data(mod);
+
+	if (gobj != NULL) {
+	    /* model icon is shown: delete it */
+	    session_delete_icon(gobj);
+	}	
+	real_delete_model_from_session(mod);
     } 
 }	
 
@@ -1808,18 +1872,22 @@ static int real_delete_model_from_session (SESSION_MODEL *model)
 
 static int real_delete_text_from_session (SESSION_TEXT *junk)
 {
-    if (session.ntexts == 1) {
+    int nt = session.ntexts;
+
+    if (nt == 1) {
 	free_session_text(session.texts[0]);
+	free(session.texts);
+	session.texts = NULL;
     } else {
 	SESSION_TEXT **pptext;
 	int i, j;
 
-	pptext = mymalloc((session.ntexts - 1) * sizeof *pptext);
-	if (session.ntexts > 1 && pptext == NULL) {
+	pptext = mymalloc((nt - 1) * sizeof *pptext);
+	if (pptext == NULL) {
 	    return 1;
 	}
 	j = 0;
-	for (i=0; i<session.ntexts; i++) {
+	for (i=0; i<nt; i++) {
 	    if (session.texts[i] != junk) {
 		pptext[j++] = session.texts[i];
 	    } else {
@@ -1830,7 +1898,7 @@ static int real_delete_text_from_session (SESSION_TEXT *junk)
 	session.texts = pptext;
     }
 
-    session.ntexts -= 1;
+    session.ntexts = nt - 1;
     mark_session_changed();
 
     return 0;
@@ -1847,20 +1915,23 @@ static void remove_session_graph_file (const char *gfname)
 
 static int real_delete_graph_from_session (SESSION_GRAPH *junk)
 {
+    int ng = session.ngraphs;
     int i, j;
 
-    if (session.ngraphs == 1) {
+    if (ng == 1) {
 	remove_session_graph_file(session.graphs[0]->fname);
 	free(session.graphs[0]);
+	free(session.graphs);
+	session.graphs = NULL;	
     } else {
 	SESSION_GRAPH **ppgr;
 
-	ppgr = mymalloc((session.ngraphs - 1) * sizeof *ppgr);
-	if (session.ngraphs > 1 && ppgr == NULL) {
+	ppgr = mymalloc((ng - 1) * sizeof *ppgr);
+	if (ppgr == NULL) {
 	    return 1;
 	}
 	j = 0;
-	for (i=0; i<session.ngraphs; i++) {
+	for (i=0; i<ng; i++) {
 	    if (session.graphs[i]->ID != junk->ID) { 
 		ppgr[j++] = session.graphs[i];
 	    } else {
@@ -1872,7 +1943,7 @@ static int real_delete_graph_from_session (SESSION_GRAPH *junk)
 	session.graphs = ppgr;
     }
 
-    session.ngraphs -= 1;
+    session.ngraphs = ng - 1;
     mark_session_changed();
 
     return 0;
@@ -1898,7 +1969,7 @@ static int delete_session_object (gui_obj *obj)
 
 static void maybe_delete_session_object (gui_obj *obj)
 {
-    gpointer p;
+    gpointer p = NULL;
     gchar *msg;
 
     if (obj->sort == GRETL_OBJ_EQN || obj->sort == GRETL_OBJ_SYS || 
@@ -1910,10 +1981,10 @@ static void maybe_delete_session_object (gui_obj *obj)
 	p = obj->data;
     }
 
-    if (winstack_match_data(p)) {
-	errbox(_("Please close this object's window first"));
+    if (p != NULL && winstack_match_data(p)) {
+	warnbox(_("Please close this object's window first"));
 	return;
-    }    
+    }
 
     msg = g_strdup_printf(_("Really delete %s?"), obj->name);
 
@@ -1975,6 +2046,18 @@ static void rename_session_graph (SESSION_GRAPH *graph, const char *newname)
     }
 }
 
+static void maybe_sync_model_window_name (SESSION_MODEL *sm)
+{
+    GtkWidget *w = match_window_by_data(sm->ptr);
+
+    if (w != NULL) {
+	gchar *title = g_strdup_printf("gretl: %s", sm->name);
+
+	gtk_window_set_title(GTK_WINDOW(w), title);
+	g_free(title);
+    }
+}
+
 static int rename_session_object (gui_obj *obj, const char *newname)
 {
     int err = 0;
@@ -1989,8 +2072,7 @@ static int rename_session_object (gui_obj *obj, const char *newname)
 	} else {
 	    sm = obj->data;
 	    gretl_object_rename(sm->ptr, sm->type, newname);
-	    sm->name[0] = '\0';
-	    strncat(sm->name, newname, MAXSAVENAME - 1);
+	    maybe_sync_model_window_name(sm);
 	}
     } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
 	SESSION_GRAPH *sg;
@@ -2054,11 +2136,11 @@ static void rename_object_callback (GtkWidget *widget, dialog_t *dlg)
 
 static void rename_object_dialog (gui_obj *obj) 
 {
-    edit_dialog (_("gretl: rename object"), 
-		 _("Enter new name\n"
-		   "(max. 31 characters)"),
-		 obj->name, rename_object_callback, obj, 
-		 0, VARCLICK_NONE, NULL);
+    edit_dialog(_("gretl: rename object"), 
+		_("Enter new name\n"
+		  "(max. 31 characters)"),
+		obj->name, rename_object_callback, obj, 
+		0, VARCLICK_NONE, NULL);
 }
 
 void delete_text_from_session (void *p)
@@ -2429,7 +2511,7 @@ static gboolean session_view_click (GtkWidget *widget,
 	    open_matrix(obj); 
 	    break;
 	case GRETL_OBJ_INFO:
-	    open_info(NULL, 0, NULL); 
+	    open_info(); 
 	    break;
 	case GRETL_OBJ_NOTES:
 	    edit_session_notes(); 
@@ -2441,10 +2523,10 @@ static gboolean session_view_click (GtkWidget *widget,
 	    display_graph_page(); 
 	    break;
 	case GRETL_OBJ_CORR:
-	    do_menu_op(NULL, ALL_CORR, NULL); 
+	    do_menu_op(ALL_CORR, NULL, OPT_NONE); 
 	    break;
 	case GRETL_OBJ_STATS:
-	    do_menu_op(NULL, ALL_SUMMARY, NULL); 
+	    do_menu_op(ALL_SUMMARY, NULL, OPT_NONE); 
 	    break;
 	}
 	return TRUE;
@@ -2493,9 +2575,9 @@ static void info_popup_activated (GtkWidget *widget, gpointer data)
     gchar *item = (gchar *) data;
 
     if (!strcmp(item, _("View"))) {
-	open_info(NULL, 0, NULL);
+	open_info();
     } else if (!strcmp(item, _("Edit"))) { 
-	edit_header(NULL, 0, NULL);
+	edit_header(NULL);
     }
 }
 
@@ -2534,9 +2616,9 @@ static void data_popup_activated (GtkWidget *widget, gpointer data)
     if (!strcmp(item, _("Edit"))) {
 	show_spreadsheet(SHEET_EDIT_DATASET);
     } else if (!strcmp(item, _("Save..."))) {
-	file_save(mdata, SAVE_DATA, NULL);
+	file_save(mdata, SAVE_DATA);
     } else if (!strcmp(item, _("Export as CSV..."))) {
-	file_save(mdata, EXPORT_CSV, NULL);
+	file_save(mdata, EXPORT_CSV);
     } else if (!strcmp(item, _("Copy as CSV..."))) {
 	csv_to_clipboard();
     }
