@@ -25,22 +25,66 @@ static int check_graph_file (const char *fname)
     return err;
 }
 
+/* Arrange things so that when a session file is opened, any graph
+   files are numbered consecutively, starting at 1.  This avoids a
+   situation where adding a graph to an existing session results in
+   over-writing an existing graph file.  (The graph files in a saved
+   session may not be numbered consecutively if some graphs were
+   initially saved, then deleted; and it's easier to fix this on
+   opening a session file than on saving one, since on opening we're
+   constructing the graph list de novo.)
+*/
+
+static void normalize_graph_filename (char *fname, int *gnum, int *pnum)
+{
+    char s[16];
+    int i;
+
+    if (sscanf(fname, "%15[^.].%d", s, &i) == 2) {
+	char oldname[MAXLEN];
+	char newname[MAXLEN];
+	gchar *tmp = NULL;
+
+	if (!strcmp(s, "plot")) {
+	    if (i > *pnum) {
+		session_file_make_path(oldname, fname);
+		tmp = g_strdup_printf("%s.%d", s, *pnum);
+		session_file_make_path(newname, tmp);
+	    }
+	    *pnum += 1;
+	} else {
+	    if (i > *gnum) {
+		session_file_make_path(oldname, fname);
+		tmp = g_strdup_printf("%s.%d", s, *gnum);
+		session_file_make_path(newname, tmp);
+	    }
+	    *gnum += 1;
+	}
+
+	if (tmp != NULL) {
+	    rename(oldname, newname);
+	    strcpy(fname, tmp);
+	    g_free(tmp);
+	}
+    }
+}
+
 static int restore_session_graphs (xmlNodePtr node)
 {
     xmlNodePtr cur;
-    int i = 0;
+    int gnum = 1, pnum = 1;
     int inpage = 0;
-    int err = 0;
+    int errs = 0;
 
     /* reset prior to parsing */
     session.ngraphs = 0;
 
     cur = node->xmlChildrenNode;
 
-    while (cur != NULL && !err) {
+    while (cur != NULL) {
 	xmlChar *name = NULL;
 	xmlChar *fname = NULL;
-	int type, ID;
+	int type, err = 0;
 
 	name = xmlGetProp(cur, (XUC) "name");
 	if (name == NULL) {
@@ -53,11 +97,13 @@ static int restore_session_graphs (xmlNodePtr node)
 		err = 1;
 	    } else {
 		err = check_graph_file((const char *) fname);
+		if (!err) {
+		    normalize_graph_filename((char *) fname, &gnum, &pnum);
+		}
 	    } 
 	}
 
-	if (!err && (!gretl_xml_get_prop_as_int(cur, "ID", &ID) ||
-	    !gretl_xml_get_prop_as_int(cur, "type", &type))) {
+	if (!err && !gretl_xml_get_prop_as_int(cur, "type", &type)) {
 	    err = 1;
 	}
 
@@ -76,29 +122,32 @@ static int restore_session_graphs (xmlNodePtr node)
 	    }
 	}
 
+	if (err) {
+	    errs++;
+	}
+
 	free(name);
 	free(fname);
 
 	cur = cur->next;
-	i++;
     }
 
-    return err;
+    return errs;
 }
 
 static int restore_session_texts (xmlNodePtr node, xmlDocPtr doc)
 {
     xmlNodePtr cur;
-    int i = 0;
-    int err = 0;
+    int errs = 0;
 
     session.ntexts = 0;
 
     cur = node->xmlChildrenNode;
 
-    while (cur != NULL && !err) {
+    while (cur != NULL) {
 	xmlChar *name = NULL;
 	xmlChar *buf = NULL;
+	int err = 0;
 
 	name = xmlGetProp(cur, (XUC) "name");
 	if (name == NULL) {
@@ -113,13 +162,16 @@ static int restore_session_texts (xmlNodePtr node, xmlDocPtr doc)
 	    }
 	}
 
+	if (err) {
+	    errs++;
+	}
+
 	free(name);
 
 	cur = cur->next;
-	i++;
     }
 
-    return err;
+    return errs;
 }
 
 static int data_submask_from_xml (xmlNodePtr node, xmlDocPtr doc,
@@ -136,76 +188,79 @@ static int data_submask_from_xml (xmlNodePtr node, xmlDocPtr doc,
     return err;
 }
 
-static gpointer rebuild_session_model (const char *fname, 
-				       GretlObjType type,
-				       SavedObjectFlags *flags,
-				       int *err)
+static int rebuild_session_model (const char *fname, 
+				  const char *name, 
+				  GretlObjType type)
 {
     gpointer ptr = NULL;
+    SavedObjectFlags flags = 0;
     xmlDocPtr doc;
     xmlNodePtr node;
+    int iflag, err;
 
-    *err = gretl_xml_open_doc_root(fname, 
-				   (type == GRETL_OBJ_EQN)? "gretl-model" :
-				   (type == GRETL_OBJ_VAR)? "gretl-VAR" :
-				   "gretl-equation-system", &doc, &node);
-    if (*err) {
+    err = gretl_xml_open_doc_root(fname, 
+				  (type == GRETL_OBJ_EQN)? "gretl-model" :
+				  (type == GRETL_OBJ_VAR)? "gretl-VAR" :
+				  "gretl-equation-system", &doc, &node);
+    if (err) {
 	fprintf(stderr, "Failed on gretl_xml_open_doc_root\n");
-	return NULL;
+	return err;
     }
 
-    if (!gretl_xml_get_prop_as_int(node, "saveflags", (int *) flags)) {
-	*flags = IN_GUI_SESSION;
-    }    
+    if (gretl_xml_get_prop_as_int(node, "saveflags", &iflag)) {
+	flags = iflag;
+    } else {
+	flags = IN_GUI_SESSION | IN_NAMED_STACK;
+    }
 
     if (type == GRETL_OBJ_EQN) {
-	ptr = gretl_model_from_XML(node, doc, err);
+	ptr = gretl_model_from_XML(node, doc, &err);
     } else if (type == GRETL_OBJ_VAR) {
-	ptr = gretl_VAR_from_XML(node, doc, err);
+	ptr = gretl_VAR_from_XML(node, doc, &err);
     } else {
-	ptr = equation_system_from_XML(node, doc, err);
+	ptr = equation_system_from_XML(node, doc, &err);
     }
 
     xmlFreeDoc(doc);
 
-    if (ptr != NULL) {
-	gretl_object_ref(ptr, type); /* FIXME do this later? */
-    }
+    if (!err && ptr != NULL) {
+#if 1
+	fprintf(stderr, "rebuild_session_model: IN_GUI_SESSION %s, IN_MODEL_TABLE %s\n"
+		" IN_NAMED_STACK %s, IS_LAST_MODEL %s\n",
+		(flags & IN_GUI_SESSION)? "yes" : "no",
+		(flags & IN_MODEL_TABLE)? "yes" : "no",
+		(flags & IN_NAMED_STACK)? "yes" : "no",
+		(flags & IS_LAST_MODEL)? "yes" : "no");
+#endif
+
+	if (flags & IN_GUI_SESSION) {
+	    SESSION_MODEL *smod;
+
+	    smod = session_model_new(ptr, name, type);
+	    if (smod == NULL) {
+		fprintf(stderr, "error from session_model_new\n");
+		err = E_ALLOC;
+	    } else {
+		err = session_append_model(smod);
+		fprintf(stderr, "error %d from session_append_model\n", err);
+	    }
+	}
+
+	if (!err && (flags & IN_MODEL_TABLE)) {
+	    add_to_model_table(ptr, MODEL_ADD_BY_CMD, NULL);
+	}
+
+	if (!err && (flags & IN_NAMED_STACK)) {
+	    err = gretl_stack_object(ptr, type);
+	} 
+
+	if (!err && (flags & IS_LAST_MODEL)) {
+	    set_as_last_model(ptr, type);
+	} 
+    }   
 
     /* need to clean up on error here (also: clean up XML parser?) */
 
-    return ptr;
-}
-
-static int 
-reattach_model (void *ptr, GretlObjType type, const char *name, 
-		SavedObjectFlags flags)
-{
-    int err = 0;
-
-    if (flags & IN_GUI_SESSION) {
-	SESSION_MODEL *smod;
-
-	smod = session_model_new(ptr, name, type);
-	if (smod == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    err = session_append_model(smod);
-	}
-    }
-
-    if (!err && (flags & IN_MODEL_TABLE)) {
-	err = add_to_model_table(ptr, MODEL_ADD_BY_CMD, NULL);
-    }
-
-    if (!err && (flags & IN_NAMED_STACK)) {
-	err = gretl_stack_object(ptr, type);
-    } 
-
-    if (!err && (flags & IS_LAST_MODEL)) {
-	set_as_last_model(ptr, type);
-    }     
-
     return err;
 }
 
@@ -213,20 +268,18 @@ static int restore_session_models (xmlNodePtr node, xmlDocPtr doc)
 {
     char fullname[MAXLEN];
     xmlNodePtr cur;
-    int i, err = 0;
+    int errs = 0;
 
     /* reset prior to parsing */
     session.nmodels = 0;
 
     cur = node->xmlChildrenNode;
-    i = 0;
 
-    while (cur != NULL && !err) {
-	SavedObjectFlags flags;
+    while (cur != NULL) {
 	xmlChar *fname = NULL;
 	xmlChar *name = NULL;
-	gpointer ptr = NULL;
 	int type = GRETL_OBJ_EQN;
+	int err = 0;
 
 	fname = xmlGetProp(cur, (XUC) "fname");
 	if (fname == NULL) {
@@ -242,35 +295,28 @@ static int restore_session_models (xmlNodePtr node, xmlDocPtr doc)
 	    session_file_make_path(fullname, (const char *) fname);
 	    gretl_xml_get_prop_as_int(cur, "type", &type);
 	    fprintf(stderr, "model file: fname='%s', type=%d\n", fullname, type);
-	    ptr = rebuild_session_model(fullname, type, &flags, &err);
+	    err = rebuild_session_model(fullname, (const char *) name, type);
 	}
 
 	if (!err) {
-	    fprintf(stderr, "reattaching model to session\n");
-	    err = reattach_model(ptr, type, (const char *) name, flags);
-	    if (!err) {
-		model_count_plus();
-	    } else {
-		fprintf(stderr, "reattach_model: failed on %s (err = %d)\n",
-			name, err);
-	    }
+	    model_count_plus();
 	} else {
 	    fprintf(stderr, "rebuild_session_model: failed on %s (err = %d)\n",
 		    fullname, err);
+	    errs++;
 	}
 
 	free(fname);
 	free(name);
 
 	cur = cur->next;
-	i++;
     }
 
 #if SESSION_DEBUG
-    fprintf(stderr, "restore_session_models: returning %d\n", err);
+    fprintf(stderr, "restore_session_models: %d errors\n", errs);
 #endif
 
-    return err;
+    return errs;
 }
 
 static int 
@@ -279,6 +325,7 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
     xmlDocPtr doc = NULL;
     xmlNodePtr cur = NULL;
     xmlChar *tmp;
+    int object_errs = 0;
     int err = 0;
 
     LIBXML_TEST_VERSION
@@ -340,7 +387,7 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 		session.nmodels = atoi((const char *) tmp);
 		free(tmp);
 		if (session.nmodels > 0) {
-		    err = restore_session_models(cur, doc);
+		    object_errs += restore_session_models(cur, doc);
 		}		
 	    }
         } else if (!xmlStrcmp(cur->name, (XUC) "graphs")) {
@@ -349,7 +396,7 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 		session.ngraphs = atoi((const char *) tmp);
 		free(tmp);
 		if (session.ngraphs > 0) {
-		    err = restore_session_graphs(cur);
+		    object_errs += restore_session_graphs(cur);
 		}
 	    }	    
 	} else if (!xmlStrcmp(cur->name, (XUC) "texts")) {
@@ -358,14 +405,14 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 		session.ntexts = atoi((const char *) tmp);
 		free(tmp);
 		if (session.ntexts > 0) {
-		    err = restore_session_texts(cur, doc);
+		    object_errs += restore_session_texts(cur, doc);
 		}		
 	    }
 	} else if (!xmlStrcmp(cur->name, (XUC) "notes")) {
 	    session.notes = 
 		(char *) xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
 	    if (session.notes == NULL) {
-		err = 1;
+		object_errs++;
 	    }
 	}
 	if (!err) {
@@ -378,6 +425,10 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
 	xmlCleanupParser();
     }
 
+    if (!err && object_errs > 0) {
+	errbox("%d session object(s) could not be rebuilt", object_errs);
+    }
+
     return err;
 }
 
@@ -396,6 +447,21 @@ static int maybe_read_matrix_file (const char *fname)
     return load_user_matrix_file(fname);
 }
 
+static int maybe_read_scalars_file (const char *fname) 
+{
+    FILE *fp;
+
+    fp = gretl_fopen(fname, "r");
+    if (fp == NULL) {
+	/* nothing to be read */
+	return 0;
+    }
+
+    fclose(fp);
+
+    return load_user_scalars_file(fname);
+}
+
 static int maybe_read_functions_file (const char *fname) 
 {
     FILE *fp;
@@ -484,6 +550,27 @@ static int maybe_write_matrix_file (void)
     return 0;
 }
 
+static int maybe_write_scalars_file (void)
+{
+    char fullname[MAXLEN];
+    FILE *fp;
+
+    if (n_saved_scalars() == 0) {
+	return 0;
+    }
+
+    session_file_make_path(fullname, "scalars.xml");
+    fp = gretl_fopen(fullname, "w");
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
+
+    write_scalars_to_file(fp);
+    fclose(fp);
+
+    return 0;
+}
+
 static int maybe_write_function_file (void)
 {
     char fullname[MAXLEN];
@@ -508,7 +595,8 @@ static int write_session_xml (const char *datname)
     FILE *fp, *fq;
     int nmodels;
     int tabmodels;
-    int i, err = 0;
+    int i, modnum;
+    int err = 0;
 
     chdir(paths.dotdir);
 
@@ -547,19 +635,21 @@ static int write_session_xml (const char *datname)
 
     fprintf(fp, " <models count=\"%d\">\n", nmodels);
 
+    modnum = 1;
+
     for (i=0; i<session.nmodels && !err; i++) {
 	int type = session.models[i]->type;
 	void *ptr = session.models[i]->ptr;
 	SavedObjectFlags sflags;
 
-	sprintf(tmpname, "%s%cmodel.%d", session.dirname, SLASH, i+1);
-	fq = fopen(tmpname, "w");
+	sprintf(tmpname, "%s%cmodel.%d", session.dirname, SLASH, modnum);
+	fq = gretl_fopen(tmpname, "w");
 
 	if (fq == NULL) {
 	    file_write_errbox(tmpname);
 	    err = E_FOPEN;
 	} else {
-	    sprintf(tmpname, "model.%d", i + 1);
+	    sprintf(tmpname, "model.%d", modnum++);
 	    fprintf(fp, "  <session-model name=\"%s\" fname=\"%s\" type=\"%d\"/>\n", 
 		    session.models[i]->name, tmpname, type);
 	    gretl_xml_header(fq);
@@ -578,13 +668,13 @@ static int write_session_xml (const char *datname)
     for (i=0; i<tabmodels && !err; i++) {
 	pmod = model_table_model_by_index(i);
 	if (!model_in_session(pmod)) {
-	    sprintf(tmpname, "%s%cmodel.%d", session.dirname, SLASH, i+1);
-	    fq = fopen(tmpname, "w");
+	    sprintf(tmpname, "%s%cmodel.%d", session.dirname, SLASH, modnum);
+	    fq = gretl_fopen(tmpname, "w");
 	    if (fq == NULL) {
 		file_write_errbox(tmpname);
 		err = E_FOPEN;
 	    } else {
-		sprintf(tmpname, "model.%d", i+1);
+		sprintf(tmpname, "model.%d", modnum++);
 		fprintf(fp, "  <session-model name=\"%s\" fname=\"%s\" type=\"%d\"/>\n", 
 			(pmod->name != NULL)? pmod->name : "none", tmpname, 
 			GRETL_OBJ_EQN);
@@ -637,6 +727,7 @@ static int write_session_xml (const char *datname)
     fclose(fp);
 
     maybe_write_matrix_file();
+    maybe_write_scalars_file();
     maybe_write_function_file();
     maybe_write_lists_file();
 
