@@ -22,12 +22,12 @@
 #include "gretl.h"
 #include "filelists.h"
 #include "gretl_www.h"
-#include "toolbar.h"
 #include "dlgutils.h"
 #include "fileselect.h"
 #include "menustate.h"
 #include "session.h"
 #include "textbuf.h"
+#include "ssheet.h"
 
 #include "libset.h"
 #include "version.h"
@@ -51,8 +51,6 @@
 char rcfile[MAXLEN];
 #endif
 
-extern int want_toolbar;
-
 char dbproxy[21];
 int use_proxy;
 
@@ -86,7 +84,6 @@ static char appfontname[MAXLEN] = "Sans 10";
 PangoFontDescription *fixed_font;
 
 static int usecwd;
-static int useqr;
 static int shellok;
 static int manpref;
 char gpcolors[32];
@@ -94,6 +91,7 @@ static char datapage[24];
 static char scriptpage[24];
 
 static int hc_by_default;
+static int langpref;
 static char hc_xsect[5] = "HC1";
 static char hc_tseri[5] = "HAC";
 static char hc_panel[9] = "Arellano";
@@ -157,8 +155,6 @@ RCVAR rc_vars[] = {
       INVISET, MAXLEN, TAB_MAIN, NULL },
     { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
       BOOLSET, 0, TAB_MAIN, NULL },
-    { "toolbar", N_("Show gretl toolbar"), NULL, &want_toolbar, 
-      BOOLSET, 0, TAB_MAIN, NULL },
 #ifndef G_OS_WIN32
     { "winsize", N_("Remember main window size"), NULL, &winsize, 
       BOOLSET, 0, TAB_MAIN, NULL },
@@ -177,10 +173,16 @@ RCVAR rc_vars[] = {
 #endif
     { "shellok", N_("Allow shell commands"), NULL, &shellok, 
       BOOLSET, 0, TAB_MAIN, NULL },
+#ifdef ENABLE_NLS
+    { "langpref", N_("Language preference"), NULL, &langpref, 
+      RADIOSET | INTSET, 0, TAB_MAIN, NULL },
+#endif
     { "usecwd", N_("Set working directory from shell"), NULL, &usecwd, 
       INVISET | BOOLSET, 0, TAB_NONE, NULL },
+#ifndef G_OS_WIN32 
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       MACHSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
+#endif
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "latex", N_("Command to compile TeX files"), NULL, latex, 
@@ -217,8 +219,6 @@ RCVAR rc_vars[] = {
       USERSET, 21, TAB_DBS, NULL },
     { "useproxy", N_("Use HTTP proxy"), NULL, &use_proxy, 
       BOOLSET, 1, TAB_DBS, NULL },
-    { "useqr", N_("Use QR decomposition"), N_("Use Cholesky decomposition"), &useqr, 
-      BOOLSET, 0, TAB_MAIN, NULL },
     { "Fixed_font", N_("Fixed font"), NULL, fixedfontname, 
       USERSET, MAXLEN, TAB_NONE, NULL },
 #if !defined(USE_GNOME)
@@ -570,7 +570,7 @@ static void set_tramo_x12a_status (void)
     }
 
     if (mdata != NULL) {
-	flip(mdata->ifac, "/Variable/TRAMO analysis", ok);
+	flip(mdata->ui, "/Variable/TRAMO analysis", ok);
     }
 #endif /* TRAMO */
 
@@ -586,7 +586,7 @@ static void set_tramo_x12a_status (void)
     }
 
     if (mdata != NULL) {
-	flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", ok);
+	flip(mdata->ui, "/Variable/X-12-ARIMA analysis", ok);
     }    
 #endif /* X12A */
 }
@@ -782,7 +782,7 @@ int options_dialog (int page, const char *varname)
 	return 0;
     }
 
-    dialog = gretl_dialog_new(_("gretl: options"), mdata->w, GRETL_DLG_BLOCK);
+    dialog = gretl_dialog_new(_("gretl: options"), mdata->main, GRETL_DLG_BLOCK);
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 2);
 
@@ -868,20 +868,24 @@ void set_path_callback (char *setvar, char *setting)
 
 static void browse_button_callback (GtkWidget *w, RCVAR *rc)
 {
+    GtkWidget *top = g_object_get_data(G_OBJECT(w), "parent");
     int code = SET_PROG;
 
     if (strstr(rc->description, "directory") != NULL) {
 	code = SET_DIR;
     }
 
-    file_selector(_(rc->description), code, FSEL_DATA_MISC, rc->var);
+    file_selector_with_parent(_(rc->description), code, 
+			      FSEL_DATA_MISC, rc->var, top);
 }
 
-static GtkWidget *make_path_browse_button (RCVAR *rc)
+static GtkWidget *make_path_browse_button (RCVAR *rc, GtkWidget *w)
 {
+    GtkWidget *top = gtk_widget_get_toplevel(w);
     GtkWidget *b;
 
     b = gtk_button_new_with_label(_("Browse..."));
+    g_object_set_data(G_OBJECT(b), "parent", top);
     g_signal_connect(G_OBJECT(b), "clicked",
 		     G_CALLBACK(browse_button_callback), 
 		     rc);
@@ -904,6 +908,11 @@ static const char **get_radio_setting_strings (void *var, int *n)
         N_("Italian"),
 	N_("Spanish")
     };
+    static const char *lang_strs[] = {
+        N_("Use local language if possible"),
+        N_("Use English"),
+	N_("Use Basque")
+    };
     const char **strs = NULL;
 
     *n = 0;
@@ -914,6 +923,9 @@ static const char **get_radio_setting_strings (void *var, int *n)
 #if HIDE_SPANISH_MANUAL
 	*n -= 1;
 #endif
+    } else if (var == &langpref) {
+	strs = lang_strs;
+	*n = sizeof lang_strs / sizeof lang_strs[0];
     }
 
     return strs;
@@ -974,6 +986,10 @@ static void radio_change_value (GtkWidget *w, int *v)
 {
     if (GTK_TOGGLE_BUTTON(w)->active) {
 	gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
+
+	if (v == &langpref) {
+	    takes_effect_on_restart();
+	}
 	*v = i;
     }
 }
@@ -982,11 +998,12 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 {
     GtkWidget *b_table = NULL, *s_table = NULL;
     GtkWidget *box, *w = NULL;
-    int s_len = 1, b_len = 0, b_col = 0;
+    int s_len = 1, b_len = 0;
+    int s_cols, b_cols = 0;
+    int b_col = 0;
     int n_str = 0;
     int n_bool = 0;
     int n_browse = 0;
-    int s_cols;
     RCVAR *rc;
     int i;
    
@@ -1020,9 +1037,10 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	gtk_box_pack_start(GTK_BOX(box), s_table, FALSE, FALSE, 0);
 	gtk_widget_show(s_table);
     }
-    
+
     if (n_bool > 0) {
-	b_table = gtk_table_new(1, 2, FALSE);
+	b_cols = 2;
+	b_table = gtk_table_new(1, b_cols, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(b_table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(b_table), 5);
 	gtk_box_pack_start(GTK_BOX(box), b_table, FALSE, FALSE, 10);
@@ -1142,20 +1160,20 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	} else if (rc->flags & LISTSET) {
 	    char *strvar = (char *) rc->var;
 	    const char **strs;
-	    int j, nopt, active = 0;
+	    int j, ww, nopt, active = 0;
 
 	    s_len++;
 
 	    gtk_table_resize(GTK_TABLE(s_table), s_len, s_cols);
 	    w = gtk_label_new(_(rc->description));
-	    gtk_misc_set_alignment(GTK_MISC(w), 0.75, 0.5);
+	    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
 	    gtk_table_attach_defaults(GTK_TABLE(s_table), 
 				      w, 0, 1, s_len - 1, s_len);
 	    gtk_widget_show(w);
 
 	    rc->widget = gtk_combo_box_new_text();
 	    gtk_table_attach(GTK_TABLE(s_table), rc->widget, 
-			     1, 2, s_len-1, s_len,
+			     1, 2, s_len - 1, s_len,
 			     0, 0, 0, 0);
 	    strs = get_list_setting_strings(rc->var, &nopt);
 	    for (j=0; j<nopt; j++) {
@@ -1166,17 +1184,35 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		}
 	    }
 	    gtk_combo_box_set_active(GTK_COMBO_BOX(rc->widget), active);
+	    /* this will need generalizing if we add another LISTSET
+	       variable */
+	    ww = get_string_width("XXArellanoXXXXX");
+	    gtk_widget_set_size_request(rc->widget, ww, -1);
 	    gtk_widget_show(rc->widget);
 	} else if (rc->flags & RADIOSET) {
 	    int nopt, j, rcval = *(int *) (rc->var);
+	    int rcol;
 	    GtkWidget *b;
 	    GSList *group = NULL;
 	    const char **strs;
 
+	    if (b_len > 0) {
+		/* there are buttons above: add a separator and
+		   make this section full-width */
+		rcol = b_cols;
+		b_len++;
+		w = gtk_hseparator_new();
+		gtk_table_attach_defaults(GTK_TABLE(b_table), w, 
+					  0, rcol, b_len - 1, b_len);  
+		gtk_widget_show(w);
+	    } else {
+		rcol = b_col + 1;
+	    }
+
 	    b_len++;
 	    b = gtk_label_new(_(rc->description));
 	    gtk_table_attach_defaults(GTK_TABLE(b_table), b, 
-				      b_col, b_col + 1, 
+				      b_col, rcol, 
 				      b_len - 1, b_len);
 	    gtk_widget_show(b);
 
@@ -1187,16 +1223,16 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		gtk_table_resize(GTK_TABLE(b_table), b_len, 2);
 		b = gtk_radio_button_new_with_label(group, _(strs[j]));
 		gtk_table_attach_defaults(GTK_TABLE(b_table), b, 
-					  b_col, b_col + 1, 
+					  b_col, rcol, 
 					  b_len - 1, b_len);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), j == rcval);
 		g_object_set_data(G_OBJECT(b), "action", 
 				  GINT_TO_POINTER(j));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), j == rcval);
+		gtk_widget_show(b);
+		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
 		g_signal_connect(G_OBJECT(b), "clicked",
 				 G_CALLBACK(radio_change_value),
 				 rc->var);
-		gtk_widget_show(b);
-		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b));
 	    }
 	} else if (!(rc->flags & INVISET)) { 
 	    /* visible string variable */
@@ -1213,15 +1249,15 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 
 	    rc->widget = gtk_entry_new();
 	    gtk_table_attach_defaults(GTK_TABLE(s_table), 
-				      rc->widget, 1, 2, s_len-1, s_len);
+				      rc->widget, 1, 2, s_len - 1, s_len);
 	    gtk_entry_set_text(GTK_ENTRY(rc->widget), strvar);
 	    gtk_widget_show(rc->widget);
 
 	    if (rc->flags & BROWSER) {
 		/* add path browse button */
-		w = make_path_browse_button(rc);
+		w = make_path_browse_button(rc, notebook);
 		gtk_table_attach_defaults(GTK_TABLE(s_table), 
-					  w, 2, 3, s_len-1, s_len);
+					  w, 2, 3, s_len - 1, s_len);
 		gtk_widget_show(w);
 	    }
 
@@ -1272,6 +1308,7 @@ static void set_lcnumeric (void)
 	    { "pl", "Polish" },
 	    { "de", "German" },
 	    { "pt", "Portuguese" },
+	    { "ru", "Russian" },
 	    { NULL, NULL }
 	};
 	char *lang = getenv("LANG");
@@ -1419,10 +1456,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     write_rc(); /* note: calls gretl_set_paths */
 
-    show_or_hide_toolbar(want_toolbar);
-
     /* register these for session using libset apparatus */
-    libset_set_bool(USE_QR, useqr);
     libset_set_bool(SHELL_OK, shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
@@ -1567,7 +1601,6 @@ static int common_read_rc_setup (void)
 {
     int err = 0;
 
-    libset_set_bool(USE_QR, useqr);
     libset_set_bool(SHELL_OK, shellok);
     libset_set_bool(USE_CWD, usecwd);
     set_gp_colors();
@@ -1592,6 +1625,9 @@ static int common_read_rc_setup (void)
 
 # ifdef ENABLE_NLS
     set_lcnumeric();
+    if (langpref > 0) {
+	force_language(langpref);
+    }
 # endif
 
     return err;
@@ -1839,16 +1875,27 @@ static void read_rc (void)
 
 #endif /* end of non-Windows versions of read_rc, write_rc */
 
+static int fontsel_code (GtkAction *action)
+{
+    const gchar *s = gtk_action_get_name(action);
+    
+    if (!strcmp(s, "MenuFont")) {
+	return APP_FONT_SELECTION;
+    } else {
+	return FIXED_FONT_SELECTION;
+    }
+}
+
 /* font selection: non-Windows, gtk-2.0 version first */
 
 #ifndef G_OS_WIN32
 
-static void font_selection_ok (GtkWidget *w, GtkFontSelectionHackDialog *fs)
+static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
 {
     guint which = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(fs), "which"));
     gchar *fontname;
 
-    fontname = gtk_font_selection_hack_dialog_get_font_name(fs);
+    fontname = gtk_fontsel_hack_dialog_get_font_name(fs);
 
     if (*fontname == 0) {
 	g_free(fontname);
@@ -1862,14 +1909,6 @@ static void font_selection_ok (GtkWidget *w, GtkFontSelectionHackDialog *fs)
 	write_rc();
     } 
 
-# ifdef PLOT_FONT_SELECTOR
-    else if (which == GRAPH_FONT_SELECTION) {
-	GtkWidget *fentry = g_object_get_data(G_OBJECT(fs), "font_entry");
-
-	gtk_entry_set_text(GTK_ENTRY(fentry), fontname);
-    }
-# endif
-
 # ifndef USE_GNOME /* gnome handles the app font */
     else if (which == APP_FONT_SELECTION) {
 	set_app_font(fontname);
@@ -1886,9 +1925,10 @@ static void fontsel_quit (GtkWidget *w, gpointer p)
     gtk_main_quit();
 }
 
-void font_selector (gpointer data, guint which, GtkWidget *widget)
+void font_selector (GtkAction *action)
 {
     static GtkWidget *fontsel = NULL;
+    int which = fontsel_code(action);
     int filter = GTK_FONT_HACK_LATIN;
     char *title = NULL;
     const char *fontname = NULL;
@@ -1898,7 +1938,9 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 # endif
 
     if (fontsel != NULL) {
-	if (!GTK_WIDGET_VISIBLE(fontsel)) gtk_widget_show (fontsel);
+	if (!GTK_WIDGET_VISIBLE(fontsel)) {
+	    gtk_widget_show(fontsel);
+	}
         gdk_window_raise(fontsel->window);
         return;
     }
@@ -1907,50 +1949,40 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 	title = _("Font for gretl output windows");
 	filter = GTK_FONT_HACK_LATIN_MONO;
 	fontname = fixedfontname;
-    }
+    } 
+
 # ifndef USE_GNOME
     else if (which == APP_FONT_SELECTION) {
 	title = _("Font for menus and labels");
 	fontname = appfontname;
     }
 # endif
-# ifdef PLOT_FONT_SELECTOR
-    else if (which == GRAPH_FONT_SELECTION) {
-	fontname = paths.pngfont;
-    }
-# endif
 
-    fontsel = gtk_font_selection_hack_dialog_new(title);
-    gtk_font_selection_hack_dialog_set_filter
-	(GTK_FONT_SELECTION_HACK_DIALOG (fontsel), filter);
-    gtk_font_selection_hack_dialog_set_font_name 
-	(GTK_FONT_SELECTION_HACK_DIALOG (fontsel), fontname); 
+    fontsel = gtk_fontsel_hack_dialog_new(title);
+    gtk_fontsel_hack_dialog_set_filter
+	(GTK_FONTSEL_HACK_DIALOG(fontsel), filter);
+    gtk_fontsel_hack_dialog_set_font_name 
+	(GTK_FONTSEL_HACK_DIALOG(fontsel), fontname); 
     g_object_set_data(G_OBJECT(fontsel), "which", GINT_TO_POINTER(which));
 
-# ifdef PLOT_FONT_SELECTOR
-    if (which == GRAPH_FONT_SELECTION) {
-	 g_object_set_data(G_OBJECT(fontsel), "font_entry", widget);
-    }
-# endif
+    gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
 
-    gtk_window_set_position (GTK_WINDOW (fontsel), GTK_WIN_POS_MOUSE);
-
-    g_signal_connect (G_OBJECT(fontsel), "destroy",
-		      G_CALLBACK(gtk_widget_destroyed),
-		      &fontsel);
-    g_signal_connect (G_OBJECT(fontsel), "destroy",
-		      G_CALLBACK(fontsel_quit),
-		      NULL);
-    g_signal_connect (G_OBJECT(GTK_FONT_SELECTION_HACK_DIALOG(fontsel)->ok_button),
-		      "clicked", 
-		      G_CALLBACK(font_selection_ok),
-		      fontsel);
-    g_signal_connect(G_OBJECT(GTK_FONT_SELECTION_HACK_DIALOG(fontsel)->cancel_button),
+    g_signal_connect(G_OBJECT(fontsel), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed),
+		     &fontsel);
+    g_signal_connect(G_OBJECT(fontsel), "destroy",
+		     G_CALLBACK(fontsel_quit),
+		     NULL);
+    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_ok_button(fontsel)),
+		     "clicked", 
+		     G_CALLBACK(font_selection_ok),
+		     fontsel);
+    g_signal_connect(G_OBJECT(gtk_fontsel_hack_dialog_cancel_button(fontsel)),
 		     "clicked", 
 		     G_CALLBACK(delete_widget),
 		     fontsel);
 
-    gtk_widget_show (fontsel);
+    gtk_widget_show(fontsel);
 
     gtk_main();
 }
@@ -1992,8 +2024,9 @@ static void fontname_to_win32 (const char *src, int fixed,
     }
 }
 
-void font_selector (gpointer data, guint which, GtkWidget *widget)
+void font_selector (GtkAction *action)
 {
+    int which = fontsel_code(action);
     CHOOSEFONT cf;            /* common dialog box structure */
     LOGFONT lf;               /* logical font structure */
     char fontname[48];
@@ -2014,15 +2047,9 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
     if (which == FIXED_FONT_SELECTION) {
 	cf.Flags |= CF_FIXEDPITCHONLY;
 	fontname_to_win32(fixedfontname, 1, lf.lfFaceName, &(cf.iPointSize));
-    } 
-    else if (which == APP_FONT_SELECTION) {
+    } else if (which == APP_FONT_SELECTION) {
 	fontname_to_win32(appfontname, 0, lf.lfFaceName, &(cf.iPointSize));
     } 
-# ifdef PLOT_FONT_SELECTOR
-    else if (which == GRAPH_FONT_SELECTION) {
-	fontname_to_win32(paths.pngfont, 0, lf.lfFaceName, &(cf.iPointSize));
-    }
-# endif
 
     cf.lpLogFont = &lf;
 
@@ -2034,16 +2061,10 @@ void font_selector (gpointer data, guint which, GtkWidget *widget)
 	    strcpy(fixedfontname, fontname);
 	    set_fixed_font();
 	    write_rc();
-	} 
-	else if (which == APP_FONT_SELECTION) {
+	} else if (which == APP_FONT_SELECTION) {
 	    set_app_font(fontname);
 	    write_rc();
-	}
-# ifdef PLOT_FONT_SELECTOR
-	else if (which == GRAPH_FONT_SELECTION) {
-	    gtk_entry_set_text(GTK_ENTRY(widget), fontname);
-	}
-# endif
+	} 	    
     }
 }
 
@@ -2280,7 +2301,7 @@ int gui_set_working_dir (char *dirname)
 
 struct wdir_setter {
     GtkWidget *dialog;
-    GtkWidget *wdir_entry;
+    GtkWidget *wdir_combo;
     GtkWidget *r1, *r2;
 };
 
@@ -2288,17 +2309,15 @@ struct wdir_setter {
 
 void set_working_dir_callback (GtkWidget *w, char *path)
 {
-    GtkWidget *entry = GTK_COMBO(w)->entry;
-
-    gtk_entry_set_text(GTK_ENTRY(entry), path);
+    set_combo_box_default_text(GTK_COMBO_BOX(w), path);
 }
 
 static void wdir_browse_callback (GtkWidget *w, struct wdir_setter *wset)
 {
-    GtkWidget *entry = wset->wdir_entry;
+    GtkWidget *combo = wset->wdir_combo;
     const char *s = N_("gretl working directory");
 
-    file_selector_with_parent(_(s), SET_WDIR, FSEL_DATA_MISC, entry, 
+    file_selector_with_parent(_(s), SET_WDIR, FSEL_DATA_MISC, combo, 
 			      wset->dialog);
 }
 
@@ -2326,14 +2345,12 @@ add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset)
     trim_slash(tmp);
  
     /* combo + browse button for current working dir */
-    w = gtk_combo_new();
+    w = gtk_combo_box_entry_new_text();
     gtk_container_add(GTK_CONTAINER(hbox), w);
-    gtk_combo_set_popdown_strings(GTK_COMBO(w), list);
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(w)->entry), tmp);
-    gtk_entry_set_width_chars(GTK_ENTRY(GTK_COMBO(w)->entry), 32);
-    gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(w)->entry), 
-			      TRUE);
-    wset->wdir_entry = w;
+    set_combo_box_strings_from_list(GTK_COMBO_BOX(w), list);
+    set_combo_box_default_text(GTK_COMBO_BOX(w), tmp);
+    gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(w))), 32);
+    wset->wdir_combo = w;
     w = gtk_button_new_with_label(_("Browse..."));
     g_signal_connect(G_OBJECT(w), "clicked",
 		     G_CALLBACK(wdir_browse_callback), wset);
@@ -2372,15 +2389,16 @@ add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset)
 static void 
 apply_wdir_changes (GtkWidget *w, struct wdir_setter *wset)
 { 
-    GtkWidget *entry;
     char tmp[MAXLEN];
-    const char *str;
+    gchar *str;
     int err;
 
-    entry = GTK_COMBO(wset->wdir_entry)->entry;
-    str = gtk_entry_get_text(GTK_ENTRY(entry));
+    str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(wset->wdir_combo));
     *tmp = '\0';
-    strncat(tmp, str, MAXLEN - 2);
+    if (str != NULL) {
+	strncat(tmp, str, MAXLEN - 2);
+	g_free(str);
+    }
 
     err = set_gretl_work_dir(tmp, &paths);
     if (err) {
@@ -2406,7 +2424,7 @@ void working_dir_dialog (void)
     }
 
     dialog = gretl_dialog_new(_("gretl: working directory"), 
-			      mdata->w, GRETL_DLG_BLOCK);
+			      mdata->main, GRETL_DLG_BLOCK);
     gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 5);
     g_signal_connect(G_OBJECT(dialog), "destroy", 
 		     G_CALLBACK(gtk_widget_destroyed), 
