@@ -24,8 +24,10 @@
 #include "menustate.h"
 #include "selector.h"
 #include "session.h"
+#include "winstack.h"
 #include "usermat.h"
 #include "matrix_extra.h"
+#include "gretl_scalar.h"
 
 #include <errno.h>
 #include <ctype.h>
@@ -64,7 +66,7 @@ typedef struct {
     GtkWidget *entry;
     GtkWidget *popup;
     GtkWidget *save;
-    GtkItemFactory *ifac;
+    GtkUIManager *ui;
     GtkCellRenderer *dumbcell;
     GtkCellRenderer *datacell;
     gchar location[64];
@@ -87,8 +89,8 @@ typedef struct {
 
 #define MATRIX_DIGITS DBL_DIG
 
-static void sheet_add_var_callback (gpointer data, guint code, GtkWidget *w);
-static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w);
+static void sheet_add_var_callback (GtkAction *action, gpointer data);
+static void sheet_add_obs_callback (GtkAction *action, gpointer data);
 static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet);
 static void set_up_sheet_column (GtkTreeViewColumn *column, gint width, 
 				 gboolean expand);
@@ -97,61 +99,72 @@ static int add_data_column (Spreadsheet *sheet);
 static void create_sheet_cell_renderers (Spreadsheet *sheet);
 static void set_dataset_locked (gboolean s);
 
-static void matrix_fill_callback (gpointer data, guint u, GtkWidget *w);
-static void matrix_props_callback (gpointer data, guint u, GtkWidget *w);
-static void matrix_edit_callback (gpointer data, guint u, GtkWidget *w);
+static void matrix_fill_callback (GtkAction *action, gpointer data);
+static void matrix_props_callback (GtkAction *action, gpointer data);
+static void matrix_edit_callback (GtkAction *action, gpointer data);
 static int update_sheet_from_matrix (Spreadsheet *sheet);
 static void size_matrix_window (Spreadsheet *sheet);
 
-enum {
-    MATRIX_FILL_IDENTITY,
-    MATRIX_FILL_UNIFORM,
-    MATRIX_FILL_NORMAL
+static void 
+spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col);
+
+const gchar *sheet_ui = 
+    "<ui>"
+    "  <menubar name='MenuBar'>"
+    "    <menu action='ObsMenu'>"
+    "      <menuitem action='AppendObs'/>"
+    "      <menuitem action='InsertObs'/>"
+    "    </menu>"
+    "    <menu action='VarMenu'>"
+    "      <menuitem action='VarAdd'/>"
+    "    </menu>"
+    "  </menubar>"
+    "</ui>";
+
+static GtkActionEntry sheet_items[] = {
+    { "ObsMenu", NULL, N_("_Observation"), NULL, NULL, NULL },
+    { "AppendObs", NULL, N_("_Append obs"), NULL, NULL, G_CALLBACK(sheet_add_obs_callback) },
+    { "InsertObs", NULL, N_("_Insert obs"), NULL, NULL, G_CALLBACK(sheet_add_obs_callback) },
+    { "VarMenu", NULL, N_("_Variable"), NULL, NULL, NULL },
+    { "VarAdd", NULL, N_("_Add"), NULL, NULL, G_CALLBACK(sheet_add_var_callback) }
 };
 
-enum {
-    MATRIX_EDIT_XTX,
-    MATRIX_EDIT_TRANSPOSE,
-    MATRIX_EDIT_CHOLESKY,
-    MATRIX_EDIT_INVERT,
-    MATRIX_EDIT_MULTIPLY,
-    MATRIX_EDIT_DIVIDE
-};
+const gchar *matrix_ui = 
+    "<ui>"
+    "  <menubar name='MenuBar'>"
+    "    <menu action='Fill'>"
+    "      <menuitem action='FillIdentity'/>"
+    "      <menuitem action='FillUniform'/>"
+    "      <menuitem action='FillNormal'/>"
+    "    </menu>"
+    "    <menu action='Properties'>"
+    "      <menuitem action='PropsView'/>"
+    "    </menu>"
+    "    <menu action='Transform'>"
+    "      <menuitem action='XTX'/>"
+    "      <menuitem action='Transpose'/>"
+    "      <menuitem action='Cholesky'/>"
+    "      <menuitem action='Invert'/>"
+    "      <menuitem action='ScalarMult'/>"
+    "      <menuitem action='ScalarDiv'/>"
+    "    </menu>"
+    "  </menubar>"
+    "</ui>";
 
-static GtkItemFactoryEntry sheet_items[] = {
-    { N_("/_Observation"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Observation/_Append obs"), NULL, sheet_add_obs_callback, SHEET_AT_END, 
-      NULL, GNULL },
-    { N_("/Observation/_Insert obs"), NULL, sheet_add_obs_callback, SHEET_AT_POINT, 
-      NULL, GNULL },
-    { N_("/_Variable"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Variable/_Add"), NULL, sheet_add_var_callback, 0, NULL, GNULL }
-};
-
-static GtkItemFactoryEntry matrix_items[] = {
-    { N_("/_Fill"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Fill/_Identity matrix"), NULL, matrix_fill_callback, 
-      MATRIX_FILL_IDENTITY, NULL, GNULL },
-    { N_("/Fill/_Uniform random"), NULL, matrix_fill_callback, 
-      MATRIX_FILL_UNIFORM, NULL, GNULL },
-    { N_("/Fill/_Normal random"), NULL, matrix_fill_callback, 
-      MATRIX_FILL_NORMAL, NULL, GNULL },
-    { N_("/_Properties"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Properties/_View"), NULL, matrix_props_callback, 
-      0, NULL, GNULL },
-    { N_("/_Transform"), NULL, NULL, 0, "<Branch>", GNULL },
-    { N_("/Transform/_X'X"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_XTX, NULL, GNULL },
-    { N_("/Transform/_Transpose"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_TRANSPOSE, NULL, GNULL },
-    { N_("/Transform/_Cholesky"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_CHOLESKY, NULL, GNULL },
-    { N_("/Transform/_Invert"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_INVERT, NULL, GNULL },
-    { N_("/Transform/_Multiply by scalar"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_MULTIPLY, NULL, GNULL },
-    { N_("/Transform/_Divide by scalar"), NULL, matrix_edit_callback, 
-      MATRIX_EDIT_DIVIDE, NULL, GNULL },
+static GtkActionEntry matrix_items[] = {
+    { "Fill", NULL, N_("_Fill"), NULL, NULL, NULL },
+    { "FillIdentity", NULL, N_("_Identity matrix"), NULL, NULL, G_CALLBACK(matrix_fill_callback) },
+    { "FillUniform", NULL, N_("_Uniform random"), NULL, NULL, G_CALLBACK(matrix_fill_callback) },
+    { "FillNormal", NULL, N_("_Normal random"), NULL, NULL, G_CALLBACK(matrix_fill_callback) },
+    { "Properties", NULL, N_("_Properties"), NULL, NULL, NULL },
+    { "PropsView", NULL, N_("_View"), NULL, NULL, G_CALLBACK(matrix_props_callback) },
+    { "Transform", NULL, N_("_Transform"), NULL, NULL, NULL },
+    { "XTX", NULL, N_("_X'X"), NULL, NULL, G_CALLBACK(matrix_edit_callback) },
+    { "Transpose", NULL, N_("_Transpose"), NULL, NULL, G_CALLBACK(matrix_edit_callback) },
+    { "Cholesky", NULL, N_("_Cholesky"), NULL, NULL, G_CALLBACK(matrix_edit_callback) },
+    { "Invert", NULL, N_("_Invert"), NULL, NULL, G_CALLBACK(matrix_edit_callback) },
+    { "ScalarMult", NULL, N_("_Multiply by scalar"), NULL, NULL, G_CALLBACK(matrix_edit_callback) },
+    { "ScalarDiv", NULL, N_("_Divide by scalar"), NULL, NULL, G_CALLBACK(matrix_edit_callback) }
 };
 
 #define sheet_is_modified(s) (s->flags & SHEET_MODIFIED)
@@ -169,32 +182,28 @@ static void sheet_set_modified (Spreadsheet *sheet, gboolean s)
     }
 }
 
-static void disable_obs_menu (GtkItemFactory *ifac)
+static void disable_obs_menu (GtkUIManager *ui)
 {
-    GtkWidget *w = gtk_item_factory_get_item(ifac, "/Observation");
+    const gchar *path = "/MenuBar/ObsMenu";
+    GtkWidget *w = gtk_ui_manager_get_widget(ui, path);
 
-    gtk_widget_set_sensitive(w, FALSE);
-}
-
-static void disable_insert_obs_item (GtkItemFactory *ifac)
-{
-    GtkWidget *w = 
-	gtk_item_factory_get_item(ifac, "/Observation/Insert obs");
-
-    gtk_widget_set_sensitive(w, FALSE);
-}
-
-static int spreadsheet_hide (int i, const DATAINFO *pdinfo)
-{
-    int ret = 0;
-
-    if (var_is_scalar(pdinfo, i)) {
-	ret = 1;
-    } else if (var_is_hidden(pdinfo, i)) {
-	ret = 1;
+    if (w != NULL) {
+	gtk_widget_set_sensitive(w, FALSE);
+    } else {
+	fprintf(stderr, "Couldn't get widget for '%s'\n", path);
     }
+}
 
-    return ret;
+static void disable_insert_obs_item (GtkUIManager *ui)
+{
+    const gchar *path = "/MenuBar/ObsMenu/InsertObs";
+    GtkWidget *w = gtk_ui_manager_get_widget(ui, path);
+
+    if (w != NULL) {
+	gtk_widget_set_sensitive(w, FALSE);
+    } else {
+	fprintf(stderr, "Couldn't get widget for '%s'\n", path);
+    }
 }
 
 static char *single_underscores (char *targ, const char *src)
@@ -547,7 +556,7 @@ add_treeview_column_with_title (Spreadsheet *sheet, const char *name)
     return column;
 }
 
-static int real_add_new_var (Spreadsheet *sheet, const char *varname)
+static int real_add_new_series (Spreadsheet *sheet, const char *varname)
 {
     GtkTreeViewColumn *column;
     char tmp[32];
@@ -572,6 +581,33 @@ static int real_add_new_var (Spreadsheet *sheet, const char *varname)
     return 0;
 }
 
+static int real_add_new_scalar (Spreadsheet *sheet, const char *vname)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkListStore *store;
+    GtkTreeIter iter;
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter, 0, vname, 1, "", -1);
+
+    sheet->datarows += 1;
+    spreadsheet_scroll_to_foot(sheet, sheet->datarows - 1, 1);
+
+    sheet_set_modified(sheet, TRUE);
+
+    return 0;
+}
+
+static int real_add_new_var (Spreadsheet *sheet, const char *varname)
+{
+    if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	return real_add_new_scalar(sheet, varname);
+    } else {
+	return real_add_new_series(sheet, varname);
+    }
+}
+
 static void 
 spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col)
 {
@@ -689,11 +725,17 @@ real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
 static void name_new_var (GtkWidget *widget, dialog_t *dlg) 
 {
     Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
+    GretlType t;
     const gchar *buf;
     char varname[VNAMELEN];
 
+    t = (sheet->cmd == SHEET_EDIT_SCALARS)? GRETL_TYPE_DOUBLE :
+	GRETL_TYPE_SERIES;
+
     buf = edit_dialog_get_text(dlg);
-    if (buf == NULL || validate_varname(buf)) return;
+    if (buf == NULL || gui_validate_varname(buf, t)) {
+	return;
+    }
 
     *varname = 0;
     strncat(varname, buf, VNAMELEN - 1);
@@ -724,23 +766,23 @@ static void name_new_obs (GtkWidget *widget, dialog_t *dlg)
 static void name_var_dialog (Spreadsheet *sheet) 
 {
     int cancel = 0;
-
-    edit_dialog (_("gretl: name variable"), 
-		 _("Enter name for new variable\n"
-		   "(max. 15 characters)"),
-		 NULL, name_new_var, sheet, 
-		 0, VARCLICK_NONE, &cancel);
+    
+    edit_dialog(_("gretl: name variable"), 
+		_("Enter name for new variable\n"
+		  "(max. 15 characters)"),
+		NULL, name_new_var, sheet, 
+		0, VARCLICK_NONE, &cancel);
 }
 
 static void new_case_dialog (Spreadsheet *sheet) 
 {
     int cancel = 0;
 
-    edit_dialog (_("gretl: case marker"), 
-		 _("Enter case marker for new obs\n"
-		   "(max. 8 characters)"),
-		 NULL, name_new_obs, sheet, 
-		 0, VARCLICK_NONE, &cancel);
+    edit_dialog(_("gretl: case marker"), 
+		_("Enter case marker for new obs\n"
+		  "(max. 8 characters)"),
+		NULL, name_new_obs, sheet, 
+		0, VARCLICK_NONE, &cancel);
 }
 
 static void name_matrix_col (GtkWidget *widget, dialog_t *dlg) 
@@ -750,7 +792,7 @@ static void name_matrix_col (GtkWidget *widget, dialog_t *dlg)
     char tmp[13], colname[24];
 
     buf = edit_dialog_get_text(dlg);
-    if (buf == NULL || validate_varname(buf)) {
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_NONE)) {
 	return;
     }
 
@@ -775,12 +817,12 @@ static void name_column_dialog (GtkTreeViewColumn *col, gpointer p)
 {
     int cancel = 0;
 
-    edit_dialog (_("gretl: name column"), 
-		 _("Enter name for column\n"
-		   "(max. 12 characters)"),
-		 gtk_tree_view_column_get_title(col),
-		 name_matrix_col, col, 
-		 0, VARCLICK_NONE, &cancel);
+    edit_dialog(_("gretl: name column"), 
+		_("Enter name for column\n"
+		  "(max. 12 characters)"),
+		gtk_tree_view_column_get_title(col),
+		name_matrix_col, col, 
+		0, VARCLICK_NONE, &cancel);
 }
 
 static GtkListStore *make_sheet_liststore (Spreadsheet *sheet)
@@ -884,14 +926,14 @@ static void sheet_get_scalar (GtkWidget *w, dialog_t *dlg)
     }
 }
 
-static void matrix_edit_callback (gpointer data, guint u, GtkWidget *w)
+static void matrix_edit_callback (GtkAction *action, gpointer data)
 {
+    const gchar *s = gtk_action_get_name(action);
     Spreadsheet *sheet = (Spreadsheet *) data;
     double x = NADBL;
     int err = 0;
 
-    if (u == MATRIX_EDIT_MULTIPLY ||
-	u == MATRIX_EDIT_DIVIDE) {
+    if (!strcmp(s, "ScalarMult") || !strcmp(s, "ScalarDiv")) {
 	int cancel = 0;
 
 	edit_dialog(_("gretl: specify scalar"), 
@@ -903,25 +945,18 @@ static void matrix_edit_callback (gpointer data, guint u, GtkWidget *w)
 	}
     }
 
-    switch (u) {
-    case MATRIX_EDIT_XTX:
+    if (!strcmp(s, "XTX")) {
 	err = matrix_XTX_in_place(sheet->matrix);
-	break;
-    case MATRIX_EDIT_TRANSPOSE:
+    } else if (!strcmp(s, "Transpose")) {
 	err = matrix_transpose_in_place(sheet->matrix);
-	break;
-    case MATRIX_EDIT_CHOLESKY:
+    } else if (!strcmp(s, "Cholesky")) {
 	err = matrix_cholesky_in_place(sheet->matrix);
-	break;
-    case MATRIX_EDIT_INVERT:
+    } else if (!strcmp(s, "Invert")) {
 	err = matrix_invert_in_place(sheet->matrix);
-	break;
-    case MATRIX_EDIT_MULTIPLY:
+    } else if (!strcmp(s, "ScalarMult")) {
 	gretl_matrix_multiply_by_scalar(sheet->matrix, x);
-	break;
-    case MATRIX_EDIT_DIVIDE:
+    } else if (!strcmp(s, "ScalarDiv")) {
 	gretl_matrix_divide_by_scalar(sheet->matrix, x);
-	break;
     }
 
     if (err) {
@@ -931,51 +966,44 @@ static void matrix_edit_callback (gpointer data, guint u, GtkWidget *w)
     }
 }
 
-static void matrix_props_callback (gpointer data, guint u, GtkWidget *w)
+static void matrix_props_callback (GtkAction *action, gpointer data)
 {
     Spreadsheet *sheet = (Spreadsheet *) data;
 
     view_matrix_properties(sheet->matrix, sheet->mname);
 }
 
-static void matrix_fill_callback (gpointer data, guint u, GtkWidget *w)
+static void matrix_fill_callback (GtkAction *action, gpointer data)
 {
+    const gchar *s = gtk_action_get_name(action);
     Spreadsheet *sheet = (Spreadsheet *) data;
     gretl_matrix *A = sheet->matrix;
     int mindim;
 
-    switch (u) {
-    case MATRIX_FILL_IDENTITY:
+    if (!strcmp(s, "FillIdentity")) {
 	mindim = (A->rows < A->cols)? A->rows : A->cols;
 	gretl_matrix_inscribe_I(A, 0, 0, mindim);
-	break;
-    case MATRIX_FILL_UNIFORM:
+    } else if (!strcmp(s, "FillUniform")) {
 	gretl_matrix_random_fill(A, D_UNIFORM);
-	break;
-    case MATRIX_FILL_NORMAL:
+    } else if (!strcmp(s, "FillNormal")) {
 	gretl_matrix_random_fill(A, D_NORMAL);
-	break;
     }
 
     update_sheet_from_matrix(sheet);
 }
 
-static void sheet_add_var_callback (gpointer data, guint u, GtkWidget *w)
+static void sheet_add_var_callback (GtkAction *action, gpointer data)
 {
     Spreadsheet *sheet = (Spreadsheet *) data;
 
     name_var_dialog(sheet);
 }
 
-static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w)
+static void sheet_add_obs_direct (Spreadsheet *sheet)
 {
-    Spreadsheet *sheet = (Spreadsheet *) data;
-
-    sheet->point = where;
-
     if (datainfo->markers) {
 	new_case_dialog(sheet);
-    } else if (where == SHEET_AT_END) {
+    } else if (sheet->point == SHEET_AT_END) {
 	int n = add_obs_dialog(NULL, 1);
 
 	if (n > 0) {
@@ -986,19 +1014,35 @@ static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w)
     }
 }
 
+static void sheet_add_obs_callback (GtkAction *action, gpointer data)
+{
+    const gchar *s = gtk_action_get_name(action);
+    Spreadsheet *sheet = (Spreadsheet *) data;
+
+    if (!strcmp(s, "AppendObs")) {
+	sheet->point = SHEET_AT_END;
+    } else {
+	sheet->point = SHEET_AT_POINT;
+    }
+
+    sheet_add_obs_direct(sheet);
+}
+
 static void popup_sheet_add_obs (GtkWidget *w, Spreadsheet *sheet)
 {
-    sheet_add_obs_callback(sheet, SHEET_AT_END, NULL);
+    sheet->point = SHEET_AT_END;
+    sheet_add_obs_direct(sheet);
 }
 
 static void popup_sheet_insert_obs (GtkWidget *w, Spreadsheet *sheet)
 {
-    sheet_add_obs_callback(sheet, SHEET_AT_POINT, NULL);
+    sheet->point = SHEET_AT_POINT;
+    sheet_add_obs_direct(sheet);
 }
 
 static void popup_sheet_add_var (GtkWidget *w, Spreadsheet *sheet)
 {
-    sheet_add_var_callback(sheet, 0, NULL);
+    name_var_dialog(sheet);
 }
 
 static void build_sheet_popup (Spreadsheet *sheet)
@@ -1080,6 +1124,41 @@ static void update_matrix_from_sheet_full (Spreadsheet *sheet)
     sheet_set_modified(sheet, FALSE);
 }
 
+/* put modified values from the spreadsheet into the array
+   of saved scalars */
+
+static void update_scalars_from_sheet (Spreadsheet *sheet)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gchar *vname, *val;
+    double x;
+    int i, err = 0;
+
+    model = gtk_tree_view_get_model(view);
+    gtk_tree_model_get_iter_first(model, &iter);
+
+    for (i=0; i<sheet->datarows && !err; i++) {
+	gtk_tree_model_get(model, &iter, 0, &vname, 1, &val, -1);
+	x = (*val == '\0')? NADBL : atof(val);
+	if (gretl_is_scalar(vname)) {
+	    gretl_scalar_set_value(vname, x);
+	} else {
+	    err = gretl_scalar_add(vname, x);
+	}
+	g_free(vname);
+	g_free(val);
+	gtk_tree_model_iter_next(model, &iter);
+    }
+
+    if (err) {
+	gui_errmsg(err);
+    }
+
+    sheet_set_modified(sheet, FALSE);
+}
+
 /* pull modified values from the data-editing spreadsheet
    into the main dataset */
 
@@ -1200,7 +1279,9 @@ static void matrix_new_name (GtkWidget *w, dialog_t *dlg)
     char *newname = (char *) edit_dialog_get_data(dlg);
     const gchar *buf = edit_dialog_get_text(dlg);
 
-    if (buf == NULL || validate_varname(buf)) return;
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_MATRIX)) {
+	return;
+    }
 
     *newname = 0;
     strncat(newname, buf, VNAMELEN - 1);
@@ -1208,18 +1289,6 @@ static void matrix_new_name (GtkWidget *w, dialog_t *dlg)
     close_dialog(dlg);
 }
 
-static int matrix_overwrite_check (const char *name)
-{
-    gchar *msg = g_strdup_printf(_("A matrix named %s already exists\n" 
-				   "OK to overwrite it?"), name);
-    int resp;
-
-    resp = yes_no_dialog("gretl", msg, 0);
-    g_free(msg);
-
-    return resp;
-}
-
 static void matrix_save_as (GtkWidget *w, Spreadsheet *sheet)
 {
     char newname[VNAMELEN];
@@ -1235,11 +1304,6 @@ static void matrix_save_as (GtkWidget *w, Spreadsheet *sheet)
 	user_matrix *u;
 	gchar *tmp;
 
-	if (get_matrix_by_name(newname) != NULL &&
-	    matrix_overwrite_check(newname) == GRETL_NO) {
-	    return;
-	}
-
 	m = gretl_matrix_copy(sheet->matrix);
 	add_or_replace_user_matrix(m, newname);
 	strcpy(sheet->mname, newname);
@@ -1266,6 +1330,8 @@ static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet)
 	infobox(_("No changes were made"));
     } else if (sheet->cmd == SHEET_EDIT_MATRIX) {
 	update_matrix_from_sheet_full(sheet);
+    } else if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	update_scalars_from_sheet(sheet);
     } else {
 	update_dataset_from_sheet(sheet);
     }
@@ -1280,7 +1346,9 @@ static void select_first_editable_cell (Spreadsheet *sheet)
     path = gtk_tree_path_new_from_string("0");
     column = gtk_tree_view_get_column(view, 1);
     gtk_tree_view_set_cursor(view, path, column, FALSE);
-    set_locator_label(sheet, path, column);
+    if (sheet->locator != NULL) {
+	set_locator_label(sheet, path, column);
+    }
 
     gtk_tree_path_free(path);
 }
@@ -1290,15 +1358,15 @@ static void set_ok_transforms (Spreadsheet *sheet)
     int z = gretl_is_zero_matrix(sheet->matrix);
     int s = gretl_matrix_get_structure(sheet->matrix);
 
-    flip(sheet->ifac, "/Transform/Multiply by scalar", !z);
-    flip(sheet->ifac, "/Transform/Divide by scalar", !z);
-    flip(sheet->ifac, "/Transform/X'X", 
+    flip(sheet->ui, "/MenuBar/Transform/ScalarMult", !z);
+    flip(sheet->ui, "/MenuBar/Transform/ScalarDiv", !z);
+    flip(sheet->ui, "/MenuBar/Transform/XTX", 
 	 s == 0 || (!z && s != GRETL_MATRIX_IDENTITY));
-    flip(sheet->ifac, "/Transform/Cholesky", 
+    flip(sheet->ui, "/MenuBar/Transform/Cholesky", 
 	 s == GRETL_MATRIX_SYMMETRIC && !z);
-    flip(sheet->ifac, "/Transform/Invert", s > 0 && !z &&
+    flip(sheet->ui, "/MenuBar/Transform/Invert", s > 0 && !z &&
 	 s != GRETL_MATRIX_IDENTITY);
-    flip(sheet->ifac, "/Transform/Transpose", 
+    flip(sheet->ui, "/MenuBar/Transform/Transpose", 
 	 s < GRETL_MATRIX_SYMMETRIC);
 }
 
@@ -1449,6 +1517,43 @@ static int add_matrix_data_to_sheet (Spreadsheet *sheet)
     return 0;
 }
 
+static int add_scalars_to_sheet (Spreadsheet *sheet)
+{
+    gchar vname[VNAMELEN];
+    gchar val[32];
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkTreeIter iter;
+    GtkListStore *store;
+    double x;
+    int i;
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
+    if (n_saved_scalars() == 0) {
+	/* starting from scratch */
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, sheet->mname, 1, "", -1);
+	return 0;
+    }
+
+    /* insert variable names and values */
+
+    for (i=0; i<sheet->datarows; i++) {
+	strcpy(vname, gretl_scalar_get_name(i)); /* underscores? */
+	x = gretl_scalar_get_value_by_index(i);
+	if (na(x)) {
+	    *val = '\0';
+	} else {
+	    sprintf(val, "%.*g", DBL_DIG, x);
+	}
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, vname, 1, val, -1);
+    }
+
+    return 0;
+}
+
 static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
 {
     gchar rowlabel[OBSLEN];
@@ -1474,7 +1579,7 @@ static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
 
     sheet->datarows = datainfo->t2 - datainfo->t1 + 1;
 
-    /* now insert data values */
+    /* insert data values */
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
@@ -1510,7 +1615,7 @@ static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
     return 0;
 }
 
-static gint get_string_width (const gchar *str)
+gint get_string_width (const gchar *str)
 {
     gint width;
 
@@ -1551,6 +1656,16 @@ static gint get_obs_col_width (void)
     return width;
 }
 
+static gint get_name_col_width (void)
+{
+    static gint width;
+
+    if (width == 0) {
+	width = get_string_width("XXXXXXXXXXXXXXX");
+    }
+    return width;
+}
+
 static gint get_data_col_width (void)
 {
     static gint width;
@@ -1881,15 +1996,16 @@ static int build_sheet_view (Spreadsheet *sheet)
     GtkWidget *view;
     GtkTreeViewColumn *column;
     GtkTreeSelection *select;
+    gchar *col0str = NULL;
     gchar tmpstr[32];
     gint i, width, colnum;
 
     if (sheet->varlist != NULL) {
 	sheet->datacols = 0;
 
-	for (i=1; i<=sheet->varlist[0]; i++) {
-	    if (spreadsheet_hide(sheet->varlist[i], datainfo)) {
-		gretl_list_delete_at_pos(sheet->varlist, i--);
+	for (i=sheet->varlist[0]; i>0; i--) {
+	    if (var_is_hidden(datainfo, sheet->varlist[i])) {
+		gretl_list_delete_at_pos(sheet->varlist, i);
 	    } else {
 		sheet->datacols += 1;
 	    }
@@ -1913,13 +2029,16 @@ static int build_sheet_view (Spreadsheet *sheet)
     /* build and attach the (two) cell renderers */
     create_sheet_cell_renderers(sheet);
 
-    /* construct the observation marker (first) column */
-    if (sheet->matrix == NULL) {
-	width = get_obs_col_width();
-    } else {
+    /* construct the first (text) column */
+    if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	col0str = _("Name");
+	width = get_name_col_width();
+    } else if (sheet->matrix != NULL) {
 	width = get_row_label_width();
+    } else {
+	width = get_obs_col_width();
     }
-    column = gtk_tree_view_column_new_with_attributes(NULL,
+    column = gtk_tree_view_column_new_with_attributes(col0str,
 						      sheet->dumbcell,
 						      "text", 0, 
 						      NULL);
@@ -1950,12 +2069,22 @@ static int build_sheet_view (Spreadsheet *sheet)
 			     G_CALLBACK(name_column_dialog), column);
 	}
 	sheet->colnames = NULL;
+    } else if (sheet->cmd == SHEET_EDIT_SCALARS) {
+	column = gtk_tree_view_column_new_with_attributes(_("Value"),
+							  sheet->datacell,
+							  "text", 
+							  1, 
+							  NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
+	set_up_sheet_column(column, width, TRUE);
+	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(1));
+	g_object_set_data(G_OBJECT(column), "sheet", sheet);
     } else {
 	colnum = 0;
 	for (i=1; i<=sheet->varlist[0]; i++) {
 	    int vi = sheet->varlist[i];
 
-	    if (spreadsheet_hide(vi, datainfo)) {
+	    if (var_is_hidden(datainfo, vi)) {
 		continue;
 	    }
 	    colnum++;
@@ -1975,8 +2104,10 @@ static int build_sheet_view (Spreadsheet *sheet)
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
     gtk_tree_selection_set_mode(select, GTK_SELECTION_NONE);
 
-    g_signal_connect(G_OBJECT(view), "cursor-changed",
-		     G_CALLBACK(update_cell_position), sheet);
+    if (sheet->cmd != SHEET_EDIT_SCALARS) {
+	g_signal_connect(G_OBJECT(view), "cursor-changed",
+			 G_CALLBACK(update_cell_position), sheet);
+    }
     g_signal_connect(G_OBJECT(view), "key-press-event",
 		     G_CALLBACK(catch_spreadsheet_key), sheet);
 
@@ -2037,19 +2168,10 @@ static int sheet_list_empty (Spreadsheet *sheet)
     if (sheet->varlist == NULL) {
 	free(sheet);
 	ret = 1;
-    } else {
-	int i, ns = 0;
-
-	for (i=1; i<=sheet->varlist[0]; i++) {
-	    if (var_is_series(datainfo, sheet->varlist[i])) {
-		ns++;
-	    }
-	}
-	if (ns == 0) {
-	    free(sheet->varlist);
-	    free(sheet);
-	    ret = 1;
-	}
+    } else if (sheet->varlist[0] == 0) {
+	free(sheet->varlist);
+	free(sheet);
+	ret = 1;
     }
 
     if (ret) {
@@ -2071,7 +2193,7 @@ static Spreadsheet *spreadsheet_new (SheetCmd c)
     sheet->locator = NULL;
     sheet->entry = NULL;
     sheet->popup = NULL;
-    sheet->ifac = NULL;
+    sheet->ui = NULL;
     sheet->save = NULL;
     sheet->dumbcell = NULL;
     sheet->datacell = NULL;
@@ -2091,7 +2213,7 @@ static Spreadsheet *spreadsheet_new (SheetCmd c)
 
     sheet->mname[0] = '\0';
 
-    if (c == SHEET_EDIT_MATRIX) {
+    if (c == SHEET_EDIT_MATRIX || c == SHEET_EDIT_SCALARS) {
 	return sheet;
     }
 
@@ -2207,11 +2329,11 @@ sheet_adjust_menu_state (Spreadsheet *sheet)
     if (complex_subsampled() || datainfo->t2 < datainfo->n - 1) {
 	sheet->flags &= ~SHEET_ADD_OBS_OK;
 	sheet->flags &= ~SHEET_INSERT_OBS_OK;
-	disable_obs_menu(sheet->ifac);
+	disable_obs_menu(sheet->ui);
     } else if ((sheet->flags & SHEET_SHORT_VARLIST) ||
 	       dataset_is_panel(datainfo)) {
 	sheet->flags &= ~SHEET_INSERT_OBS_OK;
-	disable_insert_obs_item(sheet->ifac);
+	disable_insert_obs_item(sheet->ui);
     }
 }
 
@@ -2237,6 +2359,24 @@ static void size_matrix_window (Spreadsheet *sheet)
     gtk_window_resize(GTK_WINDOW(sheet->win), w, h);
 }
 
+static void size_scalars_window (Spreadsheet *sheet)
+{
+    int nc = 2;
+    int w, h;
+
+    w = get_row_label_width() + nc * get_data_col_width() + 30;
+    if (w > 640) {
+	w = 640;
+    }
+
+    h = sheet->datarows * 20 + 160;
+    if (h > 480) {
+	h = 480;
+    }
+
+    gtk_window_resize(GTK_WINDOW(sheet->win), w, h);    
+}
+
 static void size_data_window (Spreadsheet *sheet)
 {
     int ocw = get_obs_col_width();
@@ -2279,6 +2419,57 @@ button_entered (GtkWidget *w, GdkEventCrossing *e, Spreadsheet *sheet)
     return FALSE;
 }
 
+static void sheet_add_menubar (Spreadsheet *sheet, GtkWidget *vbox)
+{
+    GtkActionGroup *actions;
+    GtkWidget *mbar;
+
+    sheet->ui = gtk_ui_manager_new();
+    actions = gtk_action_group_new("SheetActions");
+#ifdef ENABLE_NLS
+    gtk_action_group_set_translation_domain(actions, "gretl");
+#endif
+
+    if (sheet->matrix != NULL) {
+	gtk_action_group_add_actions(actions, matrix_items, 
+				     sizeof matrix_items / sizeof matrix_items[0],
+				     sheet);
+	gtk_ui_manager_add_ui_from_string(sheet->ui, matrix_ui, -1, NULL);
+    } else {
+	gtk_action_group_add_actions(actions, sheet_items, 
+				     sizeof sheet_items / sizeof sheet_items[0],
+				     sheet);
+	gtk_ui_manager_add_ui_from_string(sheet->ui, sheet_ui, -1, NULL);
+    }
+
+    gtk_ui_manager_insert_action_group(sheet->ui, actions, 0);
+    g_object_unref(actions);
+
+    mbar = gtk_ui_manager_get_widget(sheet->ui, "/MenuBar");
+    gtk_box_pack_start(GTK_BOX(vbox), mbar, FALSE, FALSE, 0);
+    gtk_widget_show(mbar);
+}
+
+static void sheet_add_locator (Spreadsheet *sheet, GtkWidget *vbox)
+{
+    GtkWidget *status_box;
+    gint w;
+
+    status_box = gtk_hbox_new(FALSE, 1);
+    gtk_container_set_border_width(GTK_CONTAINER(status_box), 0);
+    gtk_box_pack_start(GTK_BOX(vbox), status_box, FALSE, FALSE, 0);
+    gtk_widget_show(status_box);
+
+    sheet->locator = gtk_statusbar_new(); 
+    w = (sheet->matrix == NULL)? get_obs_col_width() : get_row_label_width();
+    gtk_widget_set_size_request(sheet->locator, 2 * w, 20);
+    gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(sheet->locator), FALSE);
+    sheet->cid = gtk_statusbar_get_context_id(GTK_STATUSBAR(sheet->locator), 
+					      "current row and column");
+    gtk_box_pack_start(GTK_BOX(status_box), sheet->locator, FALSE, FALSE, 0);
+    gtk_widget_show(sheet->locator);
+}
+
 /* this has to block when the user is defining a matrix in the
    course of responding to the function call dialog; otherwise
    it should not block 
@@ -2290,8 +2481,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
     Spreadsheet *sheet = *psheet;
     GtkWidget *tmp, *button_box;
     GtkWidget *scroller, *main_vbox;
-    GtkWidget *status_box, *mbar;
-    int w, err = 0;
+    int err = 0;
 
     sheet->win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
@@ -2304,12 +2494,16 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 	} else {
 	    gtk_window_set_title(GTK_WINDOW(sheet->win), _("gretl: edit matrix"));
 	}
+    } else if (c == SHEET_EDIT_SCALARS) {
+	gtk_window_set_title(GTK_WINDOW(sheet->win), _("gretl: edit scalars"));
     } else {
 	gtk_window_set_title(GTK_WINDOW(sheet->win), _("gretl: edit data"));
     }
 
     if (sheet->matrix != NULL) {
 	size_matrix_window(sheet);
+    } else if (c == SHEET_EDIT_SCALARS) {
+	size_scalars_window(sheet);
     } else {
 	size_data_window(sheet);
     }
@@ -2326,41 +2520,11 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
     gtk_container_add(GTK_CONTAINER(sheet->win), main_vbox);
     gtk_widget_show(main_vbox);
 
-    /* add menu bar */
-    sheet->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
-				       NULL);
-#ifdef ENABLE_NLS
-    gtk_item_factory_set_translate_func(sheet->ifac, menu_translate, NULL, NULL);
-#endif
-
-    if (sheet->matrix == NULL) {
-	gtk_item_factory_create_items(sheet->ifac, 
-				      sizeof sheet_items / sizeof sheet_items[0],
-				      sheet_items, sheet);
-    } else {
-	gtk_item_factory_create_items(sheet->ifac, 
-				      sizeof matrix_items / sizeof matrix_items[0],
-				      matrix_items, sheet);
+    if (c != SHEET_EDIT_SCALARS) {
+	sheet_add_menubar(sheet, main_vbox);
+	sheet_add_locator(sheet, main_vbox);
     }
 
-    mbar = gtk_item_factory_get_widget(sheet->ifac, "<main>");
-    gtk_box_pack_start(GTK_BOX(main_vbox), mbar, FALSE, FALSE, 0);
-    gtk_widget_show(mbar);
-
-    status_box = gtk_hbox_new(FALSE, 1);
-    gtk_container_set_border_width(GTK_CONTAINER(status_box), 0);
-    gtk_box_pack_start(GTK_BOX(main_vbox), status_box, FALSE, FALSE, 0);
-    gtk_widget_show(status_box);
-
-    sheet->locator = gtk_statusbar_new(); 
-    w = (sheet->matrix == NULL)? get_obs_col_width() : get_row_label_width();
-    gtk_widget_set_size_request(sheet->locator, 2 * w, 20);
-    gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(sheet->locator), FALSE);
-    sheet->cid = gtk_statusbar_get_context_id(GTK_STATUSBAR(sheet->locator), 
-					      "current row and column");
-    gtk_box_pack_start(GTK_BOX(status_box), sheet->locator, FALSE, FALSE, 0);
-    gtk_widget_show(sheet->locator);
-
     scroller = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -2374,23 +2538,25 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
     gtk_widget_show(sheet->view);
     gtk_widget_show(scroller);
 
-    if (sheet->matrix == NULL) {
+    if (sheet->matrix != NULL) {
+	g_signal_connect(G_OBJECT(sheet->win), "destroy",
+			 G_CALLBACK(free_matrix_sheet), sheet);
+    } else if (c == SHEET_EDIT_SCALARS) {
+	g_signal_connect(G_OBJECT(sheet->win), "destroy",
+			 G_CALLBACK(free_spreadsheet), psheet); /* FIXME? */
+    } else {
 	sheet_adjust_menu_state(sheet);
 
 	g_signal_connect(G_OBJECT(sheet->win), "destroy",
 			 G_CALLBACK(free_spreadsheet), psheet);
-    } else {
-	g_signal_connect(G_OBJECT(sheet->win), "destroy",
-			 G_CALLBACK(free_matrix_sheet), sheet);
-    }
+    } 
 
     /* control buttons */
 
     button_box = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
 			      GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box),
-			       10);
+    gtk_box_set_spacing(GTK_BOX(button_box), 10);
     gtk_widget_show(button_box);
     gtk_box_pack_start(GTK_BOX(main_vbox), button_box, FALSE, FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(button_box), 0);
@@ -2455,6 +2621,8 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 
     if (c == SHEET_EDIT_MATRIX) {
 	err = add_matrix_data_to_sheet(sheet);
+    } else if (c == SHEET_EDIT_SCALARS) {
+	err = add_scalars_to_sheet(sheet);
     } else {
 	err = add_data_to_sheet(sheet, c);
     }
@@ -2468,7 +2636,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 
     gtk_widget_show(sheet->win);
 
-    if (c != SHEET_EDIT_MATRIX) {
+    if (c != SHEET_EDIT_MATRIX && c != SHEET_EDIT_SCALARS) {
 	/* we can't have the user making confounding changes elsewhere,
 	   while editing the dataset here */
 	set_dataset_locked(TRUE);
@@ -2483,13 +2651,6 @@ void show_spreadsheet (SheetCmd c)
 {
     static Spreadsheet *sheet;    
 
-#ifdef OLD_WINDOWS
-    if (datainfo->t2 - datainfo->t1 > 1600) {
-	errbox(_("Sorry, can't edit more than 1600 rows"));
-	return;
-    }
-#endif
-
     if (datainfo->v == 1) {
 	warnbox(_("Please add a variable to the dataset first"));
 	return;
@@ -2508,6 +2669,86 @@ void show_spreadsheet (SheetCmd c)
     real_show_spreadsheet(&sheet, c, 0);
 }
 
+static void real_edit_scalars (const char *vname);
+
+static void start_scalars (GtkWidget *w, dialog_t *dlg) 
+{
+    const gchar *buf;
+    char varname[VNAMELEN];
+ 
+    buf = edit_dialog_get_text(dlg);
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_DOUBLE)) {
+	return;
+    }
+
+    *varname = 0;
+    strncat(varname, buf, VNAMELEN - 1);
+
+    close_dialog(dlg);
+    real_edit_scalars(varname);
+}
+
+static void maybe_add_scalar (void)
+{
+    int resp;
+
+    resp = yes_no_dialog(_("gretl: scalars"),
+			 _("No scalar variable are currently defined.\n"
+			   "Add one now?"),
+			 0);
+
+    if (resp == GRETL_YES) {
+	int cancel = 0;
+
+	edit_dialog(_("gretl: name variable"), 
+		    _("Enter name for new variable\n"
+		      "(max. 15 characters)"),
+		    NULL, start_scalars, NULL, 
+		    0, VARCLICK_NONE, &cancel);
+    }
+}
+
+static void real_edit_scalars (const char *vname) 
+{
+    static Spreadsheet *sheet; 
+    int ns;
+
+    if (sheet != NULL) {
+	gdk_window_raise(sheet->win->window);
+	return;
+    }
+
+    if (vname != NULL) {
+	ns = 1;
+    } else {
+	ns = n_saved_scalars();
+    }
+
+    if (ns == 0) {
+	maybe_add_scalar();
+	return;
+    }
+
+    sheet = spreadsheet_new(SHEET_EDIT_SCALARS);
+    if (sheet == NULL) {
+	return;
+    }
+
+    sheet->datarows = ns;
+    sheet->datacols = 1;
+
+    if (vname != NULL) {
+	strcpy(sheet->mname, vname);
+    }
+
+    real_show_spreadsheet(&sheet, SHEET_EDIT_SCALARS, 0);
+}
+
+void edit_scalars (void) 
+{
+    real_edit_scalars(NULL);
+}
+
 struct gui_matrix_spec {
     char name[VNAMELEN];
     int rows;
@@ -2534,9 +2775,8 @@ static void spin_call (GtkWidget *s, int *n)
 
 static void matrix_dialog_ok (GtkWidget *w, struct mdialog *mdlg)
 {
-    gretl_matrix *m = NULL;
     const char *etxt;
-    int err = 0;
+    int err;
 
     etxt = gtk_entry_get_text(GTK_ENTRY(mdlg->nentry));
 
@@ -2545,16 +2785,8 @@ static void matrix_dialog_ok (GtkWidget *w, struct mdialog *mdlg)
 	return;
     }
 
-    m = get_matrix_by_name(etxt);
-
-    if (!gretl_is_null_matrix(m) &&  
-	matrix_overwrite_check(etxt) == GRETL_NO) {
-	return;
-    }
-
-    err = check_varname(etxt);
+    err = gui_validate_varname(etxt, GRETL_TYPE_MATRIX);
     if (err) {
-	gui_errmsg(err);
 	return;
     }
 
@@ -2640,7 +2872,7 @@ static int new_matrix_dialog (struct gui_matrix_spec *spec)
     int maxdim = 1000;
     int canceled = 0;
 
-    dlg = gretl_dialog_new("Matrix", mdata->w, 
+    dlg = gretl_dialog_new("Matrix", mdata->main, 
 			   GRETL_DLG_BLOCK | GRETL_DLG_MODAL);
     vbox = GTK_DIALOG(dlg)->vbox;
     mdlg.dlg = dlg;
@@ -2975,7 +3207,7 @@ static int locked;
 static void set_dataset_locked (gboolean s)
 {
     locked = s;
-    flip(mdata->ifac, "/Sample", !s);
+    flip(mdata->ui, "/MenuBar/Sample", !s);
 }
 
 int dataset_locked (void)
