@@ -20,6 +20,7 @@
 #include "gretl.h"
 #include "textbuf.h"
 #include "gretl_func.h"
+#include "toolbar.h"
 
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
@@ -63,40 +64,42 @@ void text_set_cursor (GtkWidget *w, GdkCursorType cspec)
 	GdkCursor *cursor = gdk_cursor_new(cspec);
 
 	gdk_window_set_cursor(win, cursor);
-	gdk_cursor_destroy(cursor);
+	gdk_cursor_unref(cursor);
     } 
 }
 
 void cursor_to_top (windata_t *vwin)
 {
-    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w)); 
+    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text)); 
     GtkTextIter start;
     GtkTextMark *mark;
 
     gtk_text_buffer_get_start_iter(buf, &start);
     gtk_text_buffer_place_cursor(buf, &start);
     mark = gtk_text_buffer_create_mark(buf, NULL, &start, FALSE);
-    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->w), 
+    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->text), 
 				 mark, 0.0, FALSE, 0, 0);
 }
 
 void cursor_to_mark (windata_t *vwin, GtkTextMark *mark)
 {
-    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w)); 
+    GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text)); 
     GtkTextIter iter;
 
     gtk_text_buffer_get_iter_at_mark(buf, &iter, mark);
     gtk_text_buffer_place_cursor(buf, &iter);
-    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->w), 
+    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->text), 
 				 mark, 0.0, TRUE, 0, 0.1);
 }
 
-gint get_char_width (GtkWidget *widget)
+static void get_char_width_and_height (GtkWidget *widget,
+				       int *width,
+				       int *height)
 {
     PangoLayout *pl;
     PangoContext *pc;
     GtkRcStyle *style;
-    int width;
+    int w = 0, h = 0;
 
     pc = gtk_widget_get_pango_context(widget);
     style = gtk_widget_get_modifier_style(widget);
@@ -104,10 +107,23 @@ gint get_char_width (GtkWidget *widget)
 
     pl = pango_layout_new(pc);
     pango_layout_set_text(pl, "X", -1);
-    pango_layout_get_pixel_size(pl, &width, NULL);
-
+    pango_layout_get_pixel_size(pl, &w, &h);
     g_object_unref(G_OBJECT(pl));
 
+    if (width != NULL) {
+	*width = w;
+    }
+
+    if (height != NULL) {
+	*height = h;
+    }
+}
+
+gint get_char_width (GtkWidget *widget)
+{
+    int width;
+
+    get_char_width_and_height(widget, &width, NULL);
     return width;
 }
 
@@ -187,26 +203,26 @@ int viewer_char_count (windata_t *vwin)
 {
     GtkTextBuffer *tbuf;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
     return gtk_text_buffer_get_char_count(tbuf);
 }
 
-void text_paste (windata_t *vwin, guint u, GtkWidget *widget)
+void text_paste (GtkWidget *w, windata_t *vwin)
 {
-    gchar *undo_buf = textview_get_text(vwin->w);
+    gchar *undo_buf = textview_get_text(vwin->text);
     gchar *old;
 
-    old = g_object_get_data(G_OBJECT(vwin->w), "undo");
+    old = g_object_get_data(G_OBJECT(vwin->text), "undo");
     g_free(old);
 
-    g_object_set_data(G_OBJECT(vwin->w), "undo", undo_buf);
+    g_object_set_data(G_OBJECT(vwin->text), "undo", undo_buf);
 
-    gtk_text_buffer_paste_clipboard(gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w)),
+    gtk_text_buffer_paste_clipboard(gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text)),
 				    gtk_clipboard_get(GDK_NONE),
 				    NULL, TRUE);
 }
 
-void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
+void text_undo (GtkWidget *w, windata_t *vwin)
 {
     gchar *old = NULL;
 
@@ -219,7 +235,7 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 	return;
     }
     
-    old = g_object_steal_data(G_OBJECT(vwin->w), "undo");
+    old = g_object_steal_data(G_OBJECT(vwin->text), "undo");
 
     if (old == NULL) {
 	errbox(_("No undo information available"));
@@ -228,7 +244,7 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 	GtkTextIter start, end;
 	GtkTextMark *ins;
 
-	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	ins = gtk_text_buffer_get_insert(buf);
 
 	gtk_text_buffer_get_start_iter(buf, &start);
@@ -236,7 +252,7 @@ void text_undo (windata_t *vwin, guint u, GtkWidget *widget)
 	gtk_text_buffer_delete(buf, &start, &end);
 
 	gtk_text_buffer_insert(buf, &start, old, -1);
-	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->w), 
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(vwin->text), 
 				     ins, 0.0, TRUE, 0.1, 0.0);
 	g_free(old);
     }
@@ -247,7 +263,7 @@ int text_can_undo (windata_t *vwin)
     if (vwin->sbuf != NULL) {
 	return gtk_source_buffer_can_undo(vwin->sbuf);
     } else {
-	gchar *old = g_object_get_data(G_OBJECT(vwin->w), "undo");
+	gchar *old = g_object_get_data(G_OBJECT(vwin->text), "undo");
 
 	return old != NULL;
     }
@@ -457,6 +473,8 @@ static void set_source_tabs (GtkWidget *w, int cw)
 		   k == GDK_ISO_Left_Tab || \
 		   k == GDK_KP_Tab)
 
+#define script_editing(r) (r == EDIT_SCRIPT || r == EDIT_FUNC_CODE)
+
 /* Special keystrokes in script window: Ctrl-Return sends the current
    line for execution; Ctrl-R sends the whole script for execution
    (i.e. is the keyboard equivalent of the "execute" icon).
@@ -489,7 +507,7 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	if (key->keyval == GDK_F1) {
 	    set_window_help_active(vwin);
 	    interactive_script_help(NULL, NULL, vwin);
-	} else if (vwin->role == EDIT_SCRIPT) {    
+	} else if (script_editing(vwin->role)) {    
 	    if (key->keyval == GDK_Return) {
 		ret = script_electric_enter(vwin);
 	    } else if (tabkey(key->keyval)) {
@@ -502,7 +520,8 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 }
 
 #define gretl_script_role(r) (r == EDIT_SCRIPT || \
-			      r == VIEW_SCRIPT)
+			      r == VIEW_SCRIPT || \
+			      r == EDIT_FUNC_CODE)
 
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable)
@@ -527,43 +546,70 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     gtk_source_buffer_set_check_brackets(sbuf, TRUE);
 #endif
 
-    vwin->w = gtk_source_view_new_with_buffer(sbuf);
+    vwin->text = gtk_source_view_new_with_buffer(sbuf);
     vwin->sbuf = sbuf;
 
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->w), GTK_WRAP_NONE);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(vwin->w), 4);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(vwin->w), 4);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(vwin->text), GTK_WRAP_NONE);
+    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(vwin->text), 4);
+    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(vwin->text), 4);
 
-    gtk_widget_modify_font(GTK_WIDGET(vwin->w), fixed_font);
+    gtk_widget_modify_font(GTK_WIDGET(vwin->text), fixed_font);
 
-    cw = get_char_width(vwin->w);
+    cw = get_char_width(vwin->text);
     hsize *= cw;
     hsize += 48;
 
-    set_source_tabs(vwin->w, cw);
+    set_source_tabs(vwin->text, cw);
 
-    gtk_window_set_default_size(GTK_WINDOW(vwin->dialog), hsize, vsize); 
-    gtk_text_view_set_editable(GTK_TEXT_VIEW(vwin->w), editable);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(vwin->w), editable);
+    gtk_window_set_default_size(GTK_WINDOW(vwin->main), hsize, vsize); 
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(vwin->text), editable);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(vwin->text), editable);
 
     if (gretl_script_role(vwin->role)) {
-	g_signal_connect(G_OBJECT(vwin->w), "key_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "key_press_event",
 			 G_CALLBACK(script_key_handler), vwin);
-	g_signal_connect(G_OBJECT(vwin->w), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button_press_event",
 			 G_CALLBACK(script_popup_handler), 
 			 vwin);
-	g_signal_connect(G_OBJECT(vwin->w), "button_release_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button_release_event",
 			 G_CALLBACK(interactive_script_help), vwin);
     } else if (foreign_script_role(vwin->role)) {
-	g_signal_connect(G_OBJECT(vwin->w), "button_press_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button_press_event",
 			 G_CALLBACK(script_popup_handler), 
 			 vwin);
     } else if (vwin->role == VIEW_LOG) {
-	g_signal_connect(G_OBJECT(vwin->w), "button_release_event",
+	g_signal_connect(G_OBJECT(vwin->text), "button_release_event",
 			 G_CALLBACK(interactive_script_help), vwin);
     }	
 }
 
+void text_zoom (GtkAction *action, gpointer data)
+{
+    const gchar *s = gtk_action_get_name(action);
+    windata_t *vwin = (windata_t *) data;
+    GtkTextBuffer *tbuf;
+    GtkTextTagTable *table;
+    static PangoFontDescription *hpf;
+    static gint fsize;
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+    table = gtk_text_buffer_get_tag_table(tbuf);
+
+    if (hpf == NULL) {
+	hpf = pango_font_description_copy(fixed_font);
+	fsize = pango_font_description_get_size(hpf) / PANGO_SCALE;
+    }
+
+    if (!strcmp(s, "ZoomIn")) {
+	fsize++;
+    } else if (!strcmp(s, "ZoomOut")) {
+	fsize--;
+    } 
+
+    pango_font_description_set_size(hpf, fsize * PANGO_SCALE);
+    gtk_widget_modify_font(vwin->text, hpf);
+}
+
 static GtkTextTagTable *gretl_tags_new (void)
 {
     GtkTextTagTable *table;
@@ -602,6 +648,7 @@ static GtkTextTagTable *gretl_tags_new (void)
     g_object_set(tag, "family", "sans",
 		 "style", PANGO_STYLE_ITALIC, 
 		 NULL);
+
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("superscript");
@@ -732,7 +779,7 @@ void textview_insert_file (windata_t *vwin, const char *fname)
     char fline[MAXSTR], *chunk;
     int i = 0;
 
-    g_return_if_fail(GTK_IS_TEXT_VIEW(vwin->w));
+    g_return_if_fail(GTK_IS_TEXT_VIEW(vwin->text));
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
@@ -742,7 +789,7 @@ void textview_insert_file (windata_t *vwin, const char *fname)
 
     thiscolor = nextcolor = PLAIN_TEXT;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
 
     memset(fline, 0, sizeof fline);
@@ -815,7 +862,7 @@ void textview_insert_from_tempfile (windata_t *vwin, PRN *prn)
     fp = gretl_print_read_tempfile(prn);
     if (fp == NULL) return;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, -1);
     memset(readbuf, 0, sizeof readbuf);
@@ -877,10 +924,12 @@ static void insert_xlink (GtkTextBuffer *tbuf, GtkTextIter *iter,
 {
     GtkTextTagTable *tab = gtk_text_buffer_get_tag_table(tbuf);
     GtkTextTag *tag;
+    int gfr = 0;
     gchar tagname[32];
 
     if (page == GFR_PAGE) {
 	strcpy(tagname, "tag:gfr");
+	gfr = 1;
 	page = 0;
     } else {
 	sprintf(tagname, "xtag:p%d", page);
@@ -889,7 +938,8 @@ static void insert_xlink (GtkTextBuffer *tbuf, GtkTextIter *iter,
     tag = gtk_text_tag_table_lookup(tab, tagname);
 
     if (tag == NULL) {
-	if (page == GFR_PAGE) {
+	/* the required tag is not already in the table */
+	if (gfr) {
 	    tag = gtk_text_buffer_create_tag(tbuf, tagname, "foreground", "blue", 
 					     "family", "sans", NULL);
 	} else if (indent != NULL) {
@@ -930,10 +980,11 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 	GtkTextTag *tag = tagp->data;
 	gint page = object_get_int(tag, "page");
 	gint xref = object_get_int(tag, "xref");
+	gint en = GPOINTER_TO_INT(p);
 
 	if (page != 0 || xref != 0) {
 	    if (page == GUIDE_PAGE) {
-		display_pdf_help(NULL, 1, NULL);
+		display_pdf_help(NULL);
 	    } else if (page == SCRIPT_PAGE) {
 		link_open_script(tag);
 	    } else {
@@ -941,15 +992,15 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 
 		if (role == FUNCS_HELP) {
 		    if (xref) {
-			plain_text_cmdref(p, page, NULL);
+			command_help_callback(page, en); 
 		    } else {
-			genr_funcs_ref(p, page, NULL);
+			function_help_callback(page);
 		    }
 		} else {
 		    if (xref) {
-			genr_funcs_ref(p, page, NULL);
+			function_help_callback(page);
 		    } else {
-			plain_text_cmdref(p, page, NULL);
+			command_help_callback(page, en); 
 		    }
 		}
 	    }
@@ -1097,7 +1148,7 @@ cmdref_visibility_notify (GtkWidget *tview,  GdkEventVisibility *e)
 
 static void maybe_connect_help_signals (windata_t *hwin, int en)
 {
-    int done = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->w), 
+    int done = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->text), 
 						 "sigs_connected"));
 
     if (hand_cursor == NULL) {
@@ -1111,22 +1162,22 @@ static void maybe_connect_help_signals (windata_t *hwin, int en)
     if (!done) {
 	gpointer en_ptr = GINT_TO_POINTER(en);
 
-	g_signal_connect(hwin->w, "key-press-event", 
+	g_signal_connect(hwin->text, "key-press-event", 
 			 G_CALLBACK(cmdref_key_press), en_ptr);
-	g_signal_connect(hwin->w, "event-after", 
+	g_signal_connect(hwin->text, "event-after", 
 			 G_CALLBACK(cmdref_event_after), en_ptr);
-	g_signal_connect(hwin->w, "motion-notify-event", 
+	g_signal_connect(hwin->text, "motion-notify-event", 
 			 G_CALLBACK(cmdref_motion_notify), NULL);
-	g_signal_connect(hwin->w, "visibility-notify-event", 
+	g_signal_connect(hwin->text, "visibility-notify-event", 
 			 G_CALLBACK(cmdref_visibility_notify), NULL);
-	g_object_set_data(G_OBJECT(hwin->w), "sigs_connected", 
+	g_object_set_data(G_OBJECT(hwin->text), "sigs_connected", 
 			  GINT_TO_POINTER(1));
     }
 }
 
 static void maybe_set_help_tabs (windata_t *hwin)
 {
-    int done = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->w), 
+    int done = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->text), 
 						 "tabs_set"));
 
     if (!done) {
@@ -1134,9 +1185,9 @@ static void maybe_set_help_tabs (windata_t *hwin)
 	
 	tabs = pango_tab_array_new(1, TRUE);
 	pango_tab_array_set_tab(tabs, 0, PANGO_TAB_LEFT, 50);
-	gtk_text_view_set_tabs(GTK_TEXT_VIEW(hwin->w), tabs);
+	gtk_text_view_set_tabs(GTK_TEXT_VIEW(hwin->text), tabs);
 	pango_tab_array_free(tabs);
-	g_object_set_data(G_OBJECT(hwin->w), "tabs_set", GINT_TO_POINTER(1));
+	g_object_set_data(G_OBJECT(hwin->text), "tabs_set", GINT_TO_POINTER(1));
     }
 }
 
@@ -1174,7 +1225,7 @@ static void cmdref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 	}
     }
 
-    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->w), tbuf);
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->text), tbuf);
 
     maybe_connect_help_signals(hwin, en);
     maybe_set_help_tabs(hwin);
@@ -1229,7 +1280,7 @@ static void funcref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 	s++;
     }
 
-    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->w), tbuf);
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->text), tbuf);
 
     maybe_connect_help_signals(hwin, en);
     maybe_set_help_tabs(hwin);
@@ -1239,22 +1290,18 @@ static gint help_popup_click (GtkWidget *w, gpointer p)
 {
     windata_t *hwin = (windata_t *) p;
     int action = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
-    gpointer enp = NULL;
+    int en = (hwin->role == CLI_HELP_EN);
     int page = 0;
 
-    if (hwin->role == CLI_HELP_EN) {
-	enp = GINT_TO_POINTER(1);
-    }
-
     if (action == 2) {
-	page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->w), 
+	page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->text), 
 						 "backpage"));
     }
 
     if (hwin->role == FUNCS_HELP) {
-	genr_funcs_ref(NULL, page, NULL);
+	function_help_callback(page);
     } else {
-	plain_text_cmdref(enp, page, NULL);
+	command_help_callback(page, en);
     }
 
     return FALSE;
@@ -1307,9 +1354,9 @@ help_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 	if (hwin->popup != NULL) {
 	    gtk_menu_popup(GTK_MENU(hwin->popup), NULL, NULL, NULL, NULL,
 			   event->button, event->time);
-	    gtk_signal_connect(GTK_OBJECT(hwin->popup), "destroy",
-			       GTK_SIGNAL_FUNC(gtk_widget_destroyed), 
-			       &hwin->popup);
+	    g_signal_connect(G_OBJECT(hwin->popup), "destroy",
+			     G_CALLBACK(gtk_widget_destroyed), 
+			     &hwin->popup);
 	}
 
 	return TRUE;
@@ -1604,7 +1651,7 @@ static void auto_indent_script (GtkWidget *w, windata_t *vwin)
     GtkTextIter start, end;
     gchar *buf;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
     gtk_text_buffer_get_start_iter(tbuf, &start);
     gtk_text_buffer_get_end_iter(tbuf, &end);
     buf = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
@@ -1700,7 +1747,7 @@ static struct textbit *vwin_get_textbit (windata_t *vwin, int mode)
     int selected = 0;
     struct textbit *tb;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
     if (gtk_text_buffer_get_selection_bounds(tbuf, &start, &end)) {
 	selected = 1;
     }
@@ -1922,7 +1969,7 @@ static int maybe_insert_smart_tab (windata_t *vwin)
     gchar *chunk = NULL;
     int pos = 0, ret = 0;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 
     if (gtk_text_buffer_get_selection_bounds(tbuf, &start, &end)) {
 	return 0;
@@ -1949,7 +1996,7 @@ static int maybe_insert_smart_tab (windata_t *vwin)
 
 	*prevword = *thisword = '\0';
 
-	s = textview_get_current_line(vwin->w);
+	s = textview_get_current_line(vwin->text);
 	if (s != NULL) {
 	    sscanf(s, "%8s", thisword);
 	    g_free(s);
@@ -1995,7 +2042,7 @@ static gboolean script_electric_enter (windata_t *vwin)
 	return FALSE;
     }
 
-    s = textview_get_current_line(vwin->w);
+    s = textview_get_current_line(vwin->text);
 
     if (s == NULL) {
 	return FALSE;
@@ -2021,7 +2068,7 @@ static gboolean script_electric_enter (windata_t *vwin)
 	    return FALSE;
 	}
 
-	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 	mark = gtk_text_buffer_get_insert(tbuf);
 	gtk_text_buffer_get_iter_at_mark(tbuf, &start, mark);
 	gtk_text_iter_set_line_offset(&start, 0);
@@ -2084,7 +2131,7 @@ static gboolean script_tab_handler (windata_t *vwin, GdkModifierType mods)
     struct textbit *tb;
     gboolean ret = FALSE;
 
-    g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->w), FALSE);
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->text), FALSE);
 
     if (smarttab && !(mods & GDK_SHIFT_MASK)) {
 	if (maybe_insert_smart_tab(vwin)) {
@@ -2116,11 +2163,13 @@ static gboolean script_tab_handler (windata_t *vwin, GdkModifierType mods)
 
 static void line_numbers_cb (GtkWidget *w, windata_t *vwin)
 {
-    int s = gtk_source_view_get_show_line_numbers(GTK_SOURCE_VIEW(vwin->w));
+    int s = gtk_source_view_get_show_line_numbers(GTK_SOURCE_VIEW(vwin->text));
 
-    gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(vwin->w), !s);
+    gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(vwin->text), !s);
 }
 
+#define editing_code(r) (r == EDIT_SCRIPT || r == EDIT_FUNC_CODE)
+
 static GtkWidget *
 build_script_popup (windata_t *vwin, struct textbit **ptb)
 {
@@ -2134,7 +2183,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
     struct textbit *tb = NULL;
     GtkWidget *item;
 
-    g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->w), NULL);
+    g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->text), NULL);
 
     /* "generic" text window menu -- we may add to this */
     pmenu = build_text_popup(vwin);
@@ -2152,7 +2201,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 
     tb->commented = text_is_commented(tb->chunk);
 
-    if (tb->commented > 0 && vwin->role != EDIT_SCRIPT) {
+    if (tb->commented > 0 && !editing_code(vwin->role)) {
 	g_free(tb->chunk);
 	free(tb);
 	*ptb = NULL;
@@ -2161,7 +2210,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 
     *ptb = tb;
 
-    if (tb->commented <= 0) {
+    if (tb->commented <= 0 && vwin->role != EDIT_FUNC_CODE) {
 	/* we have some uncommented material: allow exec option */
 	if (tb->selected) {
 	    item = gtk_menu_item_new_with_label(_("Execute region"));
@@ -2175,7 +2224,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
-    if (vwin->role == EDIT_SCRIPT && tb->commented >= 0) {
+    if (editing_code(vwin->role) && tb->commented >= 0) {
 	/* material is either all commented or all uncommented:
 	   allow comment/uncomment option */
 	int i = (tb->selected && !tb->commented)? 2 : 
@@ -2190,7 +2239,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
-    if (vwin->role == EDIT_SCRIPT) {
+    if (editing_code(vwin->role)) {
 	if (tb->selected) {
 	    item = gtk_menu_item_new_with_label(smarttab? 
 						_("Auto-indent region") :
@@ -2221,7 +2270,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 
  line_nums:	
 
-    if (GTK_IS_SOURCE_VIEW(vwin->w)) {
+    if (GTK_IS_SOURCE_VIEW(vwin->text)) {
 	item = gtk_menu_item_new_with_label(_("Toggle line numbers"));
 	g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(line_numbers_cb),
@@ -2574,21 +2623,36 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
     insert_text_with_markup(textb, &iter, buf, hwin->role);
     free(buf);
 
-    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->w), textb);
-    g_object_set_data(G_OBJECT(hwin->w), "backpage", 
+    gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->text), textb);
+    g_object_set_data(G_OBJECT(hwin->text), "backpage", 
 		      GINT_TO_POINTER(hwin->active_var));
     maybe_connect_help_signals(hwin, en);
     cursor_to_top(hwin);
     hwin->active_var = hcode;
 }
 
+static int get_screen_height (void)
+{
+    static int screen_height;
+
+    if (screen_height == 0) {
+	GdkScreen *s = gdk_screen_get_default();
+
+	if (s != NULL) {
+	    screen_height = gdk_screen_get_height(s);
+	}
+    }
+
+    return screen_height;
+}
+
 void create_text (windata_t *vwin, int hsize, int vsize, 
-		  gboolean editable)
+		  int nlines, gboolean editable)
 {
     GtkTextBuffer *tbuf = gretl_text_buf_new();
     GtkWidget *w = gtk_text_view_new_with_buffer(tbuf);
 
-    vwin->w = w;
+    vwin->text = w;
 
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
     gtk_text_view_set_left_margin(GTK_TEXT_VIEW(w), 4);
@@ -2596,13 +2660,27 @@ void create_text (windata_t *vwin, int hsize, int vsize,
 
     gtk_widget_modify_font(GTK_WIDGET(w), fixed_font);
 
-    if (hsize > 0) {
-	hsize *= get_char_width(w);
-	hsize += 48;
+    if (hsize > 0 || nlines > 0) {
+	int px, py;
+
+	get_char_width_and_height(w, &px, &py);
+
+	if (hsize > 0) {
+	    hsize *= px;
+	    hsize += 48;
+	}
+
+	if (nlines > 0) {
+	    double v1 = (nlines + 2) * py;
+	    int sv = get_screen_height();
+
+	    if (v1 > vsize / 1.2 && v1 < vsize * 1.2 && v1 <= .9 * sv) {
+		vsize = v1;
+	    }
+	}
     }
 
-    gtk_window_set_default_size(GTK_WINDOW(vwin->dialog), hsize, vsize); 
-
+    gtk_window_set_default_size(GTK_WINDOW(vwin->main), hsize, vsize); 
     gtk_text_view_set_editable(GTK_TEXT_VIEW(w), editable);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), editable);
 }
