@@ -23,7 +23,8 @@
 #define help_role(r) (r == CLI_HELP || \
                       r == GUI_HELP || \
                       r == CLI_HELP_EN || \
-                      r == GUI_HELP_EN)
+                      r == GUI_HELP_EN || \
+                      r == FUNCS_HELP)
 
 extern int tabwidth;
 extern int smarttab;
@@ -51,12 +52,14 @@ int textview_set_cursor_at_line (GtkWidget *view, int line);
 
 int viewer_char_count (windata_t *vwin);
 
-void text_paste (windata_t *vwin, guint u, GtkWidget *widget);
+void text_paste (GtkWidget *w, windata_t *vwin);
 
-void text_undo (windata_t *vwin, guint u, GtkWidget *widget);
+void text_undo (GtkWidget *w, windata_t *vwin);
 
 int text_can_undo (windata_t *vwin);
 
+void text_zoom (GtkAction *action, gpointer data);
+
 void textview_set_text_colorized (GtkWidget *view, const char *buf);
 
 void textview_append_text_colorized (GtkWidget *view, const char *buf,
@@ -67,7 +70,7 @@ void textview_insert_file (windata_t *vwin, const char *fname);
 void textview_insert_from_tempfile (windata_t *vwin, PRN *prn);
 
 void create_text (windata_t *vwin, int hsize, int vsize, 
-		  gboolean editable);
+		  int nlines, gboolean editable);
 
 void text_set_word_wrap (GtkWidget *w, gboolean wrap);
 
