@@ -175,6 +175,10 @@ gboolean main_varclick (GtkWidget *widget, GdkEventButton *event,
     GtkTreePath *path;
     gint row = 0;
 
+    if (datainfo == NULL || datainfo->n == 0) {
+	return TRUE;
+    }
+
     if (gtk_tree_view_get_path_at_pos(view, event->x, event->y, &path, 
 				      NULL, NULL, NULL)) {
 	row = tree_path_get_row_number(path);
@@ -208,7 +212,7 @@ bool_col_toggled (GtkCellRendererToggle *cell, gchar *path_str, windata_t *vwin)
     gboolean val;
     gint col;
 
-    col = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->w), "boolcol"));
+    col = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(vwin->main), "boolcol"));
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, col, &val, -1);
 
@@ -254,8 +258,14 @@ static gint tree_store_go_to (windata_t *vwin, int k)
 	gtk_tree_view_get_visible_rect(view, &r);
 
 	if (k == GDK_Page_Down) {
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12)
 	    gtk_tree_view_tree_to_widget_coords(view, r.x, r.y + r.height,
 						&wx, &wy);
+#else
+	    gtk_tree_view_convert_bin_window_to_widget_coords(view, 
+							      r.x, r.y + r.height,
+							      &wx, &wy);
+#endif
 	    gtk_tree_view_get_path_at_pos(view, wx, wy, &path, 
 					  NULL, NULL, NULL);
 	    if (path == NULL) {
@@ -265,8 +275,13 @@ static gint tree_store_go_to (windata_t *vwin, int k)
 	    gtk_tree_view_scroll_to_cell(view, path, NULL,
 					 TRUE, 0.0, 0.0);
 	} else {
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12)
 	    gtk_tree_view_tree_to_widget_coords(view, r.x, r.y,
 						&wx, &wy);
+#else
+	    gtk_tree_view_convert_bin_window_to_widget_coords(view, r.x, r.y,
+							      &wx, &wy);
+#endif
 	    gtk_tree_view_get_path_at_pos(view, wx, wy, &path, 
 					  NULL, NULL, NULL);
 	    gtk_tree_view_scroll_to_cell(view, path, NULL,
@@ -293,7 +308,7 @@ static gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     if (k == GDK_q) { 
 	/* Q = quit */
 	if (vwin != mdata) {
-	    gtk_widget_destroy(vwin->w);
+	    gtk_widget_destroy(vwin->main);
 	}
 	return TRUE;
     } else if (k == GDK_f) {
@@ -306,7 +321,7 @@ static gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 
 	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
 	if (mods & GDK_CONTROL_MASK) {
-	    menu_find(vwin, 1, NULL);
+	    listbox_find(NULL, vwin);
 	    return TRUE;
 	}	
     } else if (k == GDK_Home || k == GDK_End || 
@@ -409,7 +424,7 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
     for (i=0; i<viscols; i++) {
 	if (types[i] == G_TYPE_BOOLEAN) {
 	    bool_renderer = gtk_cell_renderer_toggle_new();
-	    g_object_set_data(G_OBJECT(vwin->w), "boolcol", GINT_TO_POINTER(i));
+	    g_object_set_data(G_OBJECT(vwin->main), "boolcol", GINT_TO_POINTER(i));
 	    g_signal_connect(bool_renderer, "toggled",
 			     G_CALLBACK(bool_col_toggled), vwin);
 	    column = gtk_tree_view_column_new_with_attributes(_(titles[i]),
