@@ -2069,20 +2069,45 @@ static void QLR_print_result (MODEL *pmod,
     }
 }
 
+static double robust_chow_test (MODEL *pmod, const int *list,
+				int *err)
+{
+    double test = NADBL;
+    int *tlist;
+
+    tlist = gretl_list_diff_new(pmod->list, list, 2);
+
+    if (tlist == NULL) {
+	*err = E_ALLOC;
+    } else {
+	test = robust_omit_F(tlist, pmod);
+	if (!na(test)) {
+	    test *= tlist[0]; /* chi-square form */
+	}
+	free(tlist);
+    }
+
+    return test;
+}
+
 static void save_chow_test (MODEL *pmod, char *chowdate,
-			    double F, double pval,
+			    double test, double pval,
 			    int dfn, int dfd)
 {
-    ModelTest *test = model_test_new(GRETL_TEST_CHOW);
+    ModelTest *mt = model_test_new(GRETL_TEST_CHOW);
 
-    if (test != NULL) {
-	model_test_set_teststat(test, GRETL_STAT_F);
-	model_test_set_param(test, chowdate);
-	model_test_set_value(test, F);
-	model_test_set_pvalue(test, pval);
-	model_test_set_dfn(test, dfn);
-	model_test_set_dfd(test, dfd);
-	maybe_add_test_to_model(pmod, test);
+    if (mt != NULL) {
+	if (dfd == 0) {
+	    model_test_set_teststat(mt, GRETL_STAT_WALD_CHISQ);
+	} else {
+	    model_test_set_teststat(mt, GRETL_STAT_F);
+	}
+	model_test_set_param(mt, chowdate);
+	model_test_set_value(mt, test);
+	model_test_set_pvalue(mt, pval);
+	model_test_set_dfn(mt, dfn);
+	model_test_set_dfd(mt, dfd);
+	maybe_add_test_to_model(pmod, mt);
     }	  
 }
 
@@ -2109,7 +2134,6 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     int origv = pdinfo->v;
     char chowdate[OBSLEN];
     MODEL chow_mod;
-    double F;
     int QLR = 0;
     int split, smax;
     int err = 0;
@@ -2138,17 +2162,16 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	if (split <= 0 || split >= pdinfo->n) { 
 	    strcpy(gretl_errmsg, _("Invalid sample split for Chow test"));
 	    err = E_DATA;
+	    goto bailout;
 	}
 	smax = split;
     }
 
-    if (!err) {
-	chowlist = make_chow_list(pmod, pZ, pdinfo, split, &err);
-    }
+    chowlist = make_chow_list(pmod, pZ, pdinfo, split, &err);
 
-    if (!err && QLR) {
+    if (QLR) {
 	/* Quandt likelihood ratio */
-	double Fmax = 0.0;
+	double F, Fmax = 0.0;
 	double *Ft = NULL;
 	int dfn = 0, dfd = 0;
 	int tmax = 0;
@@ -2198,36 +2221,67 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	if (Ft != NULL) {
 	    free(Ft);
 	}
+    } else {
+	/* regular (or robust) Chow test */
+	int robust = gretl_model_get_int(pmod, "robust");
+	gretlopt lsqopt = OPT_A;
 
-    } else if (!err) {
-	/* regular Chow test */
-	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, OPT_A);
+	if (robust) {
+	    lsqopt |= OPT_R;
+	}
+
+	chow_mod = lsq(chowlist, pZ, pdinfo, OLS, lsqopt);
 
 	if (chow_mod.errcode) {
 	    err = chow_mod.errcode;
 	    errmsg(err, prn);
 	} else {
+	    int dfd = (robust)? 0 : chow_mod.dfd;
 	    int dfn = chow_mod.ncoeff - pmod->ncoeff;
-	    double pval;
+	    double test, pval;
 
-	    chow_mod.aux = AUX_CHOW;
-	    printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
-	    F = (pmod->ess - chow_mod.ess) * chow_mod.dfd / 
-		(chow_mod.ess * dfn);
-	    pval = snedecor_cdf_comp(dfn, chow_mod.dfd, F);
-	    pprintf(prn, _("\nChow test for structural break at observation %s:\n"
-		    "  F(%d, %d) = %f with p-value %f\n\n"), chowdate,
-		    dfn, chow_mod.dfd, F, pval);
-	    
-	    if (opt & OPT_S) {
-		save_chow_test(pmod, chowdate, F, pval, dfn, chow_mod.dfd);
+	    if (!(opt & OPT_Q)) {
+		chow_mod.aux = AUX_CHOW;
+		printmodel(&chow_mod, pdinfo, OPT_NONE, prn);
 	    }
 
-	    record_test_result(F, pval, "Chow");
+	    if (robust) {
+		test = robust_chow_test(&chow_mod, pmod->list, &err);
+	    } else {
+		test = (pmod->ess - chow_mod.ess) * dfd / 
+		    (chow_mod.ess * dfn);
+	    }
+
+	    if (!na(test)) {
+		if (opt & OPT_Q) {
+		    pputc(prn, '\n');
+		}
+		pprintf(prn, _("Chow test for structural break at observation %s"),
+			       chowdate); 
+		pputc(prn, '\n');
+		if (robust) {
+		    pval = chisq_cdf_comp(dfn, test);
+		    pprintf(prn, "  %s(%d) = %g %s %.4f\n", _("Chi-square"),
+			    dfn, test, _("with p-value"), pval);
+		    pprintf(prn, "  %s: F(%d, %d) = %g %s %.4f\n\n", _("F-form"), 
+			    dfn, chow_mod.dfd, test / dfn, _("with p-value"),
+			    snedecor_cdf_comp(dfn, chow_mod.dfd, test / dfn));
+		} else {
+		    pval = snedecor_cdf_comp(dfn, dfd, test);
+		    pprintf(prn, "  F(%d, %d) = %g %s %.4f\n\n", 
+			    dfn, dfd, test, _("with p-value"), pval);
+		}
+		if (opt & OPT_S) {
+		    save_chow_test(pmod, chowdate, test, pval, dfn, dfd);
+		}
+		record_test_result(test, pval, "Chow");
+	    } 
 	}
 	clear_model(&chow_mod);
     }
 
+ bailout:
+
     /* clean up extra variables */
     dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
     free(chowlist);
