@@ -125,7 +125,7 @@ int import_other (const char *fname, int ftype,
 
 int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname);
 
-GretlFileType detect_filetype (char *fname, PATHS *ppaths, PRN *prn);
+GretlFileType detect_filetype (char *fname, PATHS *ppaths);
 
 gretlopt data_save_opt_from_suffix (const char *fname);
 
