@@ -19,10 +19,27 @@
 
 #include "libgretl.h"
 #include "gretl_func.h"
+#include "gretl_scalar.h"
 #include "libset.h"
 #include "dbread.h"
 
 #define DDEBUG 0
+#define FULLDEBUG 0
+
+static int dataset_changed;
+
+int check_dataset_is_changed (void)
+{
+    int ret = dataset_changed;
+
+    dataset_changed = 0;
+    return ret;
+}
+
+void set_dataset_is_changed (void)
+{
+    dataset_changed = 1;
+}
 
 /**
  * free_Z:
@@ -481,7 +498,6 @@ static void gretl_varinfo_init (VARINFO *vinfo)
     vinfo->transform = 0;
     vinfo->lag = 0;
     vinfo->compact_method = COMPACT_NONE;
-    vinfo->stack_level = 0;
     vinfo->line_width = 1;
     vinfo->sorted_markers = NULL;
     vinfo->stack_level = gretl_function_depth();
@@ -1026,16 +1042,14 @@ int dataset_add_observations (int newobs, double ***pZ, DATAINFO *pdinfo,
     bign = pdinfo->n + newobs;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    x = realloc((*pZ)[i], bign * sizeof *x);
-	    if (x == NULL) {
-		return E_ALLOC;
-	    }
-	    (*pZ)[i] = x;
-	    for (t=pdinfo->n; t<bign; t++) {
-		(*pZ)[i][t] = (i == 0)? 1.0 : NADBL;
-	    }	    
+	x = realloc((*pZ)[i], bign * sizeof *x);
+	if (x == NULL) {
+	    return E_ALLOC;
 	}
+	(*pZ)[i] = x;
+	for (t=pdinfo->n; t<bign; t++) {
+	    (*pZ)[i][t] = (i == 0)? 1.0 : NADBL;
+	}	    
     }
     
     if (pdinfo->markers && pdinfo->S != NULL) {
@@ -1102,13 +1116,11 @@ int dataset_drop_observations (int n, double ***pZ, DATAINFO *pdinfo)
     newn = pdinfo->n - n;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    x = realloc((*pZ)[i], newn * sizeof *x);
-	    if (x == NULL) {
-		return E_ALLOC;
-	    }
-	    (*pZ)[i] = x;
+	x = realloc((*pZ)[i], newn * sizeof *x);
+	if (x == NULL) {
+	    return E_ALLOC;
 	}
+	(*pZ)[i] = x;
     }
     
     if (pdinfo->markers && pdinfo->S != NULL) {
@@ -1159,9 +1171,7 @@ int dataset_shrink_obs_range (double ***pZ, DATAINFO *pdinfo)
 
 	mvsize = rem * sizeof **Z;
 	for (i=0; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i)) {
-		memmove(Z[i], Z[i] + head, mvsize);
-	    }
+	    memmove(Z[i], Z[i] + head, mvsize);
 	}
 
 	if (pdinfo->markers && pdinfo->S != NULL) {
@@ -1335,117 +1345,6 @@ dataset_add_allocated_series (double *x, double ***pZ, DATAINFO *pdinfo)
     return real_add_series(1, x, pZ, pdinfo);
 }
 
-/**
- * dataset_add_scalars:
- * @n: number of scalars to add.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- * Allocates space for @n new scalar members of the dataset.
- * The added variables are initialized to zero.
- *
- * Returns: 0 on success, %E_ALLOC on error.
- */
-
-int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo)
-{
-    double **newZ;
-    int v = pdinfo->v;
-    int i, err = 0;
-
-    if (v == 0) {
-	pdinfo->n = pdinfo->v = 1;
-	err = start_new_Z(pZ, pdinfo, 0);
-	if (err) {
-	    pdinfo->n = pdinfo->v = 0;
-	    return err;
-	}
-	v = 1;
-    }
-
-    newZ = realloc(*pZ, (v + n) * sizeof *newZ);  
-
-    if (newZ == NULL) {
-	err = E_ALLOC;
-    } else {
-	*pZ = newZ;
-    }
-
-    if (!err) {
-	for (i=0; i<n; i++) {
-	    newZ[v+i] = NULL;
-	}
-	for (i=0; i<n; i++) {
-	    newZ[v+i] = malloc(sizeof **newZ);
-	    if (newZ[v+i] == NULL) {
-		err = E_ALLOC;
-		break;
-	    }
-	    newZ[v+i][0] = 0.0;
-	}
-    }
-
-    if (!err) {
-	err = dataset_expand_varinfo(n, pdinfo);
-    }
-
-    if (!err) {
-	for (i=0; i<n; i++) {
-	    set_var_scalar(pdinfo, v + i, 1);
-	}
-    }
-
-    return err;
-}
-
-/**
- * dataset_add_scalar:
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- * Allocates space for a new scalar member of the dataset.
- * The added variable is initialized to zero.
- *
- * Returns: 0 on success, %E_ALLOC on error.
- */
-
-int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo)
-{
-    return dataset_add_scalars(1, pZ, pdinfo);
-}
-
-/**
- * dataset_add_scalar_as:
- * @x: scalar value.
- * @newname: name to give the new variable.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- * Adds to the dataset a new scalar with name @newname and
- * value given by @x.  The new variable is added at one
- * level "deeper" (in terms of function execution) than the
- * current level.  This is for use with user-defined functions.
- *
- * Returns: 0 on success, %E_ALLOC on error.
- */
-
-int dataset_add_scalar_as (double x, const char *newname,
-			   double ***pZ, DATAINFO *pdinfo)
-{
-    int v, err = 0;
-
-    err = dataset_add_scalar(pZ, pdinfo);
-
-    if (!err) {
-	v = pdinfo->v - 1;
-	(*pZ)[v][0] = x;
-	strcpy(pdinfo->varname[v], newname);
-	STACK_LEVEL(pdinfo, v) += 1;
-    }
-
-    return err;
-}
-
 /**
  * dataset_add_series_as:
  * @x: array to be added.
@@ -1513,25 +1412,17 @@ int dataset_copy_variable_as (int v, const char *newname,
 {
     int t, err;
 
-    if (var_is_series(pdinfo, v)) {
-	err = real_add_series(1, NULL, pZ, pdinfo);
-    } else {
-	err = dataset_add_scalar(pZ, pdinfo);
-    }
+    err = real_add_series(1, NULL, pZ, pdinfo);
 
     if (!err) {
 	int vnew = pdinfo->v - 1;
 
-	if (var_is_series(pdinfo, v)) {
-	    for (t=0; t<pdinfo->n; t++) {
-		(*pZ)[vnew][t] = (*pZ)[v][t];
-	    }
-	} else {
-	    (*pZ)[vnew][0] = (*pZ)[v][0];
+	for (t=0; t<pdinfo->n; t++) {
+	    (*pZ)[vnew][t] = (*pZ)[v][t];
 	}
 	strcpy(pdinfo->varname[vnew], newname);
 	STACK_LEVEL(pdinfo, vnew) += 1;
-	 /* FIXME other varinfo stuff?? */
+	/* FIXME other varinfo stuff?? */
     }
 
     return err;
@@ -1604,14 +1495,42 @@ static int vars_renumbered (const int *list, DATAINFO *pdinfo,
     return 0;
 }
 
-int overwrite_err (const DATAINFO *pdinfo, int v)
+int overwrite_err (const char *name)
 {
-    sprintf(gretl_errmsg, "The variable %s is read-only", 
-	    pdinfo->varname[v]);
+    sprintf(gretl_errmsg, "The variable %s is read-only", name);
 
     return E_DATA;
 }
 
+/**
+ * rename_var_by_id:
+ * @idstr: string representation of the ID number of the
+ * variable to be renamed.
+ * @vname: new name to give the variable.
+ * @pdinfo: dataset information.
+ * 
+ * Returns: 0 on sucess, %E_DATA on error.
+ */
+
+int dataset_rename_variable (DATAINFO *pdinfo, int v, 
+			     const char *name)
+{
+    if (v < 0 || v >= pdinfo->v) {
+	return E_DATA;
+    }
+
+    if (object_is_const(pdinfo->varname[v])) {
+	return overwrite_err(pdinfo->varname[v]);
+    }
+
+    if (strcmp(pdinfo->varname[v], name)) {
+	strcpy(pdinfo->varname[v], name);
+	dataset_changed = 1;
+    }
+
+    return 0;
+}
+
 static int real_drop_listed_vars (int *list, double ***pZ, 
 				  DATAINFO *pdinfo, int *renumber,
 				  int drop, PRN *prn)
@@ -1656,8 +1575,8 @@ static int real_drop_listed_vars (int *list, double ***pZ,
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 	if (v > 0 && v < oldv) {
-	    if (var_is_const(pdinfo, v)) {
-		return overwrite_err(pdinfo, v);
+	    if (object_is_const(pdinfo->varname[v])) {
+		return overwrite_err(pdinfo->varname[v]);
 	    }
 	    if (v < delmin) {
 		delmin = v;
@@ -1955,7 +1874,7 @@ int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
     int i, t;
     int err = 0;
 
-    if (v < 1 || v >= pdinfo->v || var_is_scalar(pdinfo, v)) {
+    if (v < 1 || v >= pdinfo->v) {
 	return E_DATA;
     }
 
@@ -1991,9 +1910,6 @@ int dataset_sort_by (int v, double **Z, DATAINFO *pdinfo, gretlopt opt)
     }
 
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
 	for (t=0; t<pdinfo->n; t++) {
 	    x[t] = Z[i][sv[t].obsnum];
 	}
@@ -2039,12 +1955,12 @@ static int dataset_sort (const char *s, double **Z, DATAINFO *pdinfo,
 	return E_DATA;
     }
 
-    v = varindex(pdinfo, vname);
+    v = series_index(pdinfo, vname);
     if (v == pdinfo->v) {
 	return E_UNKVAR;
     }
 
-    if (v < 1 || var_is_scalar(pdinfo, v)) {
+    if (v < 1) {
 	return E_DATA;
     }
     
@@ -2065,14 +1981,17 @@ static int dataset_sort (const char *s, double **Z, DATAINFO *pdinfo,
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 {
-    int newv, v = pdinfo->v;
+    int newv = pdinfo->v - delvars;
     int i, err = 0;
 
     if (delvars <= 0) {
 	return 0;
     }
 
-    newv = pdinfo->v - delvars;
+#if FULLDEBUG
+    fprintf(stderr, "*** dataset_drop_last_variables: dropping %d, newv = %d\n",
+	    delvars, newv);
+#endif
 
     if (newv < 1) {
 	fprintf(stderr, "dataset_drop_last_vars: pdinfo->v = %d, delvars = %d "
@@ -2081,6 +2000,13 @@ int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 	return E_DATA;
     }
 
+#if FULLDEBUG
+    for (i=0; i<pdinfo->v; i++) {
+	fprintf(stderr, "var %d (%s, level %d) %s\n", i, pdinfo->varname[i],
+		STACK_LEVEL(pdinfo, i), (i >= newv)? "deleting" : "");
+    }
+#endif
+
     for (i=newv; i<pdinfo->v; i++) {
 	free(pdinfo->varname[i]);
 	free_varinfo(pdinfo, i);
@@ -2098,24 +2024,32 @@ int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo)
 	double ***fZ = fetch_full_Z();
 	DATAINFO *fdinfo = fetch_full_datainfo();
 
-	/* The following is required for the special case of undoing
-	   sub-sampling inside a function, when the dataset is already
-	   subsampled on entry to the function.  Note that we actually
-	   _do_ something here only if the number of variables in the
-	   full dataset equals that in the subsetted dataset (before
-	   the deletions in the latter).  Therefore, specifically, we
-	   do _not_ do anything if the vars deleted (above) from the
-	   subsetted dataset were not present in the full version.
+	/* 
+	   Context: we're deleting @delvars variables at the end of Z,
+	   leaving @newv variables.  The dataset is currently
+	   subsampled.
+
+	   Question: should we delete any variables at the end of
+	   fullZ to keep the two arrays in sync?
+
+	   If @newv < fdinfo->v, this must mean that at least some of
+	   the extra vars we're deleting from the current sub-sampled
+	   Z have already been synced to fullZ, so we should do the
+	   deletion from fullZ.
 	*/
 
-	if (fdinfo->v == v) {
+	if (newv < fdinfo->v) { 
+#if FULLDEBUG
+	    fprintf(stderr, "prior fdinfo->v = %d: shrinking fullZ to %d vars\n", 
+		    fdinfo->v, newv);
+#endif
 	    for (i=newv; i<fdinfo->v; i++) {
 		free((*fZ)[i]);
 		(*fZ)[i] = NULL;
 	    }
 	    err = shrink_dataset_to_size(fZ, fdinfo, newv, DROP_SPECIAL);
 	    reset_full_Z(fZ);
-	}
+	} 
     }
 
     return err;
@@ -2169,12 +2103,18 @@ static int get_stack_param_val (const char *s, const double **Z,
 	char vname[VNAMELEN];
 	int i, len = strcspn(s, " -");
 
-	if (len > VNAMELEN - 1) len = VNAMELEN - 1;
+	if (len > VNAMELEN - 1) {
+	    len = VNAMELEN - 1;
+	}
 	*vname = '\0';
 	strncat(vname, s, len);
-	i = varindex(pdinfo, vname);
-	if (i < pdinfo->v) {
-	    val = (int) Z[i][0];
+	if (gretl_is_scalar(vname)) {
+	    val = gretl_scalar_get_value(vname);
+	} else {
+	    i = series_index(pdinfo, vname);
+	    if (i < pdinfo->v) {
+		val = (int) Z[i][0];
+	    }
 	}
     }
 
@@ -2295,7 +2235,7 @@ int dataset_stack_variables (const char *vname, const char *line,
     /* end of active portion of line */
     *p = '\0';
 
-    genv = varindex(pdinfo, vname);
+    genv = series_index(pdinfo, vname);
 
     /* do we have a range of vars? */
     sprintf(format, "%%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1);
@@ -2304,8 +2244,8 @@ int dataset_stack_variables (const char *vname, const char *line,
 	    v1 = atoi(vn1);
 	    v2 = atoi(vn2);
 	} else {
-	    v1 = varindex(pdinfo, vn1);
-	    v2 = varindex(pdinfo, vn2);
+	    v1 = series_index(pdinfo, vn1);
+	    v2 = series_index(pdinfo, vn2);
 	}
 	if (v1 >= 0 && v2 > v1 && v2 < pdinfo->v) {
 	    nv = v2 - v1 + 1;
@@ -2338,7 +2278,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 	    if (isdigit(*p)) {
 		v1 = atoi(p);
 	    } else {
-		v1 = varindex(pdinfo, p);
+		v1 = series_index(pdinfo, p);
 	    }
 	    if (v1 < 0 || v1 >= pdinfo->v) {
 		err = E_UNKVAR;
@@ -2388,11 +2328,8 @@ int dataset_stack_variables (const char *vname, const char *line,
 
 	    j = (vnum == NULL)? i + v1 : vnum[i];
 
-	    if (var_is_series(pdinfo, j)) {
-		ok = pdinfo->n - missing_tail((*pZ)[j], pdinfo->n);
-	    } else {
-		ok = 1;
-	    }
+	    ok = pdinfo->n - missing_tail((*pZ)[j], pdinfo->n);
+
 	    if (ok > maxok) {
 		maxok = ok;
 	    }
@@ -2445,11 +2382,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 	}
 
 	for (t=offset; t<tmax; t++) {
-	    if (var_is_series(pdinfo, j)) {
-		bigx[bigt] = (*pZ)[j][t];
-	    } else {
-		bigx[bigt] = (*pZ)[j][0];
-	    }
+	    bigx[bigt] = (*pZ)[j][t];
 	    if (pdinfo->S != NULL && bigt != t) {
 		strcpy(pdinfo->S[bigt], pdinfo->S[t]);
 	    }
@@ -2498,7 +2431,7 @@ int dataset_stack_variables (const char *vname, const char *line,
 
 static int found_log_parent (const char *s, char *targ)
 {
-    int len = gretl_varchar_spn(s);
+    int len = gretl_namechar_spn(s);
 
     if (len < VNAMELEN && s[len] == ')') {
 	sscanf(s, "%15[^)]", targ);
@@ -2556,31 +2489,14 @@ int is_log_variable (int i, const DATAINFO *pdinfo, char *parent)
 void set_var_discrete (DATAINFO *pdinfo, int i, int s) 
 {
     if (i > 0 && i < pdinfo->v) {
-	if (s) {
+	int flags = pdinfo->varinfo[i]->flags;
+
+	if (s && !(flags & VAR_DISCRETE)) {
 	    pdinfo->varinfo[i]->flags |= VAR_DISCRETE;
-	} else {
+	    dataset_changed = 1;
+	} else if (!s && (flags & VAR_DISCRETE)) {
 	    pdinfo->varinfo[i]->flags &= ~VAR_DISCRETE;
-	}
-    }
-}
-
-/**
- * set_var_scalar:
- * @pdinfo: pointer to data information struct.
- * @i: index number of variable.
- * @s: non-zero to mark variable as a scalar, zero to 
- * mark as not scalar (i.e. a series).
- *
- * Mark a variable as being a scalar or not.
- */
-
-void set_var_scalar (DATAINFO *pdinfo, int i, int s) 
-{
-    if (i > 0 && i < pdinfo->v) {
-	if (s) {
-	    pdinfo->varinfo[i]->flags |= VAR_SCALAR;
-	} else {
-	    pdinfo->varinfo[i]->flags &= ~VAR_SCALAR;
+	    dataset_changed = 1;
 	}
     }
 }
@@ -2637,13 +2553,30 @@ int var_get_linewidth (const DATAINFO *pdinfo, int i)
     }
 }
 
+int var_set_description (DATAINFO *pdinfo, int i,
+			 const char *s) 
+{
+    char *targ = pdinfo->varinfo[i]->label;
+
+    if (strcmp(targ, s)) {
+	*targ = 0;
+	strncat(targ, s, MAXLABEL - 1);
+	dataset_changed = 1;
+    }
+
+    return 0;
+}
+
 int var_set_display_name (DATAINFO *pdinfo, int i,
 			  const char *s) 
 {
     char *targ = pdinfo->varinfo[i]->display_name;
 
-    *targ = 0;
-    strncat(targ, s, MAXDISP - 1);
+    if (strcmp(targ, s)) {
+	*targ = 0;
+	strncat(targ, s, MAXDISP - 1);
+	dataset_changed = 1;
+    }
 
     return 0;
 }
@@ -2731,7 +2664,7 @@ static int dataset_int_param (const char **ps, int op,
     sscanf(s, "%31s", test);
     *ps += strlen(test);
 
-    k = gretl_int_from_string(test, (const double **) Z, pdinfo, err);
+    k = gretl_int_from_string(test, err);
     if (*err) {
 	return 0;
     }
@@ -2838,17 +2771,10 @@ int dataset_resample (int n, unsigned int seed,
 
     j = 0;
     for (i=0; i<pdinfo->v && !err; i++) {
-	if (i > 0 && var_is_scalar(pdinfo, i)) {
-	    RZ[j] = malloc(sizeof **RZ);
-	    if (RZ[j] != NULL) {
-		RZ[j][0] = (*pZ)[i][0];
-	    }
-	} else {
-	    RZ[j] = malloc(n * sizeof **RZ);
-	    if (RZ[j] != NULL && i == 0) {
-		for (t=0; t<n; t++) {
-		    RZ[j][t] = 1.0;
-		}
+	RZ[j] = malloc(n * sizeof **RZ);
+	if (RZ[j] != NULL && i == 0) {
+	    for (t=0; t<n; t++) {
+		RZ[j][t] = 1.0;
 	    }
 	}
 	if (RZ[j] == NULL) {
@@ -2876,9 +2802,7 @@ int dataset_resample (int n, unsigned int seed,
 	s = gretl_rand_int_max(T) + pdinfo->t1;
 	j = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i)) {
-		RZ[j][t] = (*pZ)[i][s];
-	    } 
+	    RZ[j][t] = (*pZ)[i][s];
 	    j++;
 	}
 	if (S != NULL) {
@@ -2999,6 +2923,19 @@ int modify_dataset (int op, const int *list, const char *s,
     return err;
 }
 
+int dataset_get_structure (const DATAINFO *pdinfo)
+{
+    if (pdinfo == NULL || pdinfo->n == 0) {
+	return DATA_NONE;
+    } else if (dataset_is_panel(pdinfo)) {
+	return DATA_PANEL;
+    } else if (dataset_is_time_series(pdinfo)) {
+	return DATA_TS;
+    } else {
+	return DATA_XSECT;
+    }
+}
+
 /**
  * dataset_purge_missing_rows:
  * @Z: data array.
@@ -3025,7 +2962,7 @@ int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo)
     for (t=0; t<pdinfo->n; t++) {
 	missrow = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (!var_is_scalar(pdinfo, i) && !na(Z[i][t])) {
+	    if (!na(Z[i][t])) {
 		missrow = 0;
 		break;
 	    }
@@ -3058,7 +2995,7 @@ int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo)
     for (t=0; t<pdinfo->n; t++) {
 	missrow = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (!var_is_scalar(pdinfo, i) && !na(Z[i][t])) {
+	    if (!na(Z[i][t])) {
 		missrow = 0;
 		break;
 	    }
@@ -3066,9 +3003,7 @@ int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo)
 	if (missrow) {
 	    sz = (pdinfo->n - t) * sizeof **Z;
 	    for (i=1; i<pdinfo->v; i++) {
-		if (!var_is_scalar(pdinfo, i)) {
-		    memmove(Z[i] + t, Z[i] + t + 1, sz);
-		}
+		memmove(Z[i] + t, Z[i] + t + 1, sz);
 	    }
 	    if (pdinfo->S != NULL) {
 		free(pdinfo->S[t]);
@@ -3082,13 +3017,11 @@ int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo)
     new_n = pdinfo->n - totmiss;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (!var_is_scalar(pdinfo, i)) {
-	    Zi = realloc(Z[i], new_n * sizeof *Zi);
-	    if (Zi == NULL) {
-		err = E_ALLOC;
-	    } else {
-		Z[i] = Zi;
-	    }
+	Zi = realloc(Z[i], new_n * sizeof *Zi);
+	if (Zi == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    Z[i] = Zi;
 	}
     }
 
