@@ -20,6 +20,13 @@
 #ifndef DATASET_H
 #define DATASET_H
 
+typedef enum {
+    DATA_NONE,
+    DATA_XSECT,
+    DATA_TS,
+    DATA_PANEL
+} DatasetStructure;
+
 typedef enum {
     NO_MARKERS = 0,
     REGULAR_MARKERS,
@@ -28,10 +35,9 @@ typedef enum {
 
 typedef enum {
     VAR_DISCRETE   = 1 << 0,
-    VAR_SCALAR     = 1 << 1,
-    VAR_HIDDEN     = 1 << 2,
-    VAR_GENERATED  = 1 << 3,
-    VAR_SETCONST   = 1 << 4
+    VAR_HIDDEN     = 1 << 1,
+    VAR_GENERATED  = 1 << 2,
+    VAR_LISTARG    = 1 << 3
 } VarinfoFlags;
 
 typedef enum {
@@ -201,25 +207,6 @@ typedef enum {
  */
 #define var_is_discrete(p, i) ((p)->varinfo[i]->flags & VAR_DISCRETE)
 
-/**
- * var_is_scalar:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether or not a variable is a scalar.
- */
-#define var_is_scalar(p, i) ((p)->varinfo[i]->flags & VAR_SCALAR)
-
-/**
- * var_is_series:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether or not a variable is a series (as opposed
- * to a scalar).
- */
-#define var_is_series(p, i) (!((p)->varinfo[i]->flags & VAR_SCALAR))
-
 /**
  * var_is_hidden:
  * @p: pointer to data information struct.
@@ -240,33 +227,32 @@ typedef enum {
 #define var_is_generated(p, i) ((p)->varinfo[i]->flags & VAR_GENERATED)
 
 /**
- * var_is_const:
+ * var_is_listarg:
  * @p: pointer to data information struct.
  * @i: index number of variable.
  *
  * Determine whether or not a variable has been marked as
- * "const".
+ * belonging to a list argument to a function.
  */
-#define var_is_const(p, i) (i == 0 || ((p)->varinfo[i]->flags & VAR_SETCONST))
+#define var_is_listarg(p, i) ((p)->varinfo[i]->flags & VAR_LISTARG)
 
 /**
- * set_var_const:
+ * set_var_listarg:
  * @p: pointer to data information struct.
  * @i: index number of variable.
  *
- * Set the "const" flag on the given variable.
+ * Set the "listarg" flag on the given variable.
  */
-#define set_var_const(p, i) ((p)->varinfo[i]->flags |= VAR_SETCONST)
+#define set_var_listarg(p, i) ((p)->varinfo[i]->flags |= VAR_LISTARG)
 
 /**
- * unset_var_const:
+ * unset_var_listarg:
  * @p: pointer to data information struct.
  * @i: index number of variable.
  *
- * Remove the "const" flag from the given variable.
+ * Remove the "listarg" flag from the given variable.
  */
-#define unset_var_const(p, i) ((p)->varinfo[i]->flags &= ~VAR_SETCONST)
-
+#define unset_var_listarg(p, i) ((p)->varinfo[i]->flags &= ~VAR_LISTARG)
 
 void free_Z (double **Z, DATAINFO *pdinfo);
 
@@ -329,20 +315,16 @@ int dataset_add_series (int newvars, double ***pZ, DATAINFO *pdinfo);
 int dataset_add_allocated_series (double *x, double ***pZ, 
 				  DATAINFO *pdinfo);
 
-int dataset_add_scalars (int n, double ***pZ, DATAINFO *pdinfo);
-
-int dataset_add_scalar (double ***pZ, DATAINFO *pdinfo);
-
-int dataset_add_scalar_as (double x, const char *newname,
-			   double ***pZ, DATAINFO *pdinfo);
-
 int dataset_add_series_as (double *x, const char *newname,
 			   double ***pZ, DATAINFO *pdinfo);
 
 int dataset_copy_variable_as (int v, const char *newname,
 			      double ***pZ, DATAINFO *pdinfo);
 
-int overwrite_err (const DATAINFO *pdinfo, int v);
+int overwrite_err (const char *name);
+
+int dataset_rename_variable (DATAINFO *pdinfo, int v, 
+			     const char *name);
 
 int dataset_drop_listed_variables (int *list, double ***pZ, 
 				   DATAINFO *pdinfo, int *renumber,
@@ -376,6 +358,9 @@ int var_get_linewidth (const DATAINFO *pdinfo, int i);
 int var_set_display_name (DATAINFO *pdinfo, int i,
 			  const char *s); 
 
+int var_set_description (DATAINFO *pdinfo, int i,
+			 const char *s); 
+
 const char *var_get_graph_name (const DATAINFO *pdinfo, int i);
 
 unsigned int get_resampling_seed (void);
@@ -389,6 +374,12 @@ int modify_dataset (int op, const int *list, const char *s,
 		    double ***pZ, DATAINFO *pdinfo, 
 		    PRN *prn);
 
+int dataset_get_structure (const DATAINFO *pdinfo);
+
 int dataset_purge_missing_rows (double **Z, DATAINFO *pdinfo);
 
+int check_dataset_is_changed (void);
+
+void set_dataset_is_changed (void);
+
 #endif /* DATASET_H */
