@@ -1895,15 +1895,14 @@ static char *odbc_get_query (char *s, int *err)
 
 static int odbc_get_varname (char *s, char *vname)
 {
-    int n = gretl_varchar_spn(s);
-    int err = 0;
+    int n, err;
 
-    if (n == 0 || n > VNAMELEN - 1) {
+    err = extract_varname(vname, s, &n);
+
+    if (err || n == 0) {
 	gretl_errmsg_set(_("Expected a valid variable name"));
 	err = E_PARSE;
     } else {
-	*vname = '\0';
-	strncat(vname, s, n);
 	err = check_varname(vname);
     }
 
@@ -2057,7 +2056,7 @@ int db_get_series (char *line, double ***pZ, DATAINFO *pdinfo,
 	series_info_init(&sinfo);
 
 	/* see if the series is already in the dataset */
-	v = varindex(pdinfo, series);
+	v = series_index(pdinfo, series);
 	if (v < pdinfo->v && method == COMPACT_NONE) {
 	    this_var_method = COMPACT_METHOD(pdinfo, v);
 	}
@@ -2448,11 +2447,11 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	pdinfo->v = 2;
 	dbv = 1;
 	if (start_new_Z(pZ, pdinfo, 0)) {
-	    strcpy(gretl_errmsg, _("Out of memory adding series"));
+	    strcpy(gretl_errmsg, _("Out of memory!"));
 	    return 1;
 	}
     } else if (new && dataset_add_series(1, pZ, pdinfo)) {
-	strcpy(gretl_errmsg, _("Out of memory adding series"));
+	strcpy(gretl_errmsg, _("Out of memory!"));
 	return 1;
     }
 
@@ -2480,7 +2479,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	}
 	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, method);
 	if (xvec == NULL) {
-	    strcpy(gretl_errmsg, _("Out of memory adding series"));
+	    strcpy(gretl_errmsg, _("Out of memory!"));
 	    if (new) {
 		dataset_drop_last_variables(1, pZ, pdinfo);
 	    }
@@ -2557,14 +2556,12 @@ static double *compact_series (const double *src, int i, int n, int oldn,
 #endif
 
     x = malloc(n * sizeof *x);
-    if (x == NULL) return NULL;
-
-    for (t=0; t<n; t++) {
-	x[t] = (i == 0)? 1.0 : NADBL;
+    if (x == NULL) {
+	return NULL;
     }
 
-    if (i == 0) {
-	return x;
+    for (t=0; t<n; t++) {
+	x[t] = NADBL;
     }
 
     idx = startskip;
@@ -2647,13 +2644,8 @@ daily_series_to_monthly (const double *src, DATAINFO *pdinfo, int i,
     int sopt, eopt;
 
     x = malloc(nm * sizeof *x);
-    if (x == NULL) return NULL;
-
-    if (i == 0) {
-	for (t=0; t<nm; t++) {
-	    x[t] = 1.0;
-	}
-	return x;
+    if (x == NULL) {
+	return NULL;
     }
 
     if (offset < 0) {
@@ -2997,10 +2989,8 @@ weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo,
 
     for (i=1; i<pdinfo->v; i++) {
 	/* initialize all series, first obs */
-	if (var_is_series(pdinfo, i)) {
-	    mZ[i][0] = NADBL;
-	    mn[i] = 0;
-	}
+	mZ[i][0] = NADBL;
+	mn[i] = 0;
     }    
 
     for (s=0; s<pdinfo->n; s++) {
@@ -3010,15 +3000,13 @@ weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo,
 	if (monbak > 0 && mon != monbak) {
 	    /* new month: finalize the previous one */
 	    for (i=1; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i)) {
-		    if (method == COMPACT_EOP) {
-			if (s > 0) {
-			    mZ[i][t] = Z[i][s-1];
-			}
-		    } else if (method == COMPACT_AVG) {
-			if (mn[i] > 0) {
-			    mZ[i][t] /= (double) mn[i];
-			}
+		if (method == COMPACT_EOP) {
+		    if (s > 0) {
+			mZ[i][t] = Z[i][s-1];
+		    }
+		} else if (method == COMPACT_AVG) {
+		    if (mn[i] > 0) {
+			mZ[i][t] /= (double) mn[i];
 		    }
 		}
 	    }
@@ -3027,42 +3015,38 @@ weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo,
 		t++;
 		for (i=1; i<pdinfo->v; i++) {
 		    /* initialize all series, current obs */
-		    if (var_is_series(pdinfo, i)) {
-			if (method == COMPACT_SOP) {
-			    mZ[i][t] = Z[i][s];
-			} else {
-			    mZ[i][t] = NADBL;
-			}
-			mn[i] = 0;
+		    if (method == COMPACT_SOP) {
+			mZ[i][t] = Z[i][s];
+		    } else {
+			mZ[i][t] = NADBL;
 		    }
-		}  		
+		    mn[i] = 0;
+		}
 	    }
 	} 
 
 	/* cumulate non-missing weekly observations? */
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i)) {
-		if (method == COMPACT_SOP) {
-		    ; /* handled above */
-		} else if (method == COMPACT_EOP) {
+	    if (method == COMPACT_SOP) {
+		; /* handled above */
+	    } else if (method == COMPACT_EOP) {
+		mZ[i][t] = Z[i][s];
+	    } else if (!na(Z[i][s])) {
+		if (na(mZ[i][t])) {
 		    mZ[i][t] = Z[i][s];
-		} else if (!na(Z[i][s])) {
-		    if (na(mZ[i][t])) {
-			mZ[i][t] = Z[i][s];
-		    } else {
-			mZ[i][t] += Z[i][s];
-		    }
-		    mn[i] += 1;
+		} else {
+		    mZ[i][t] += Z[i][s];
 		}
-		if (mon == monbak && s == pdinfo->n - 1) {
-		    /* reached the end: ship out last obs */
-		    if (method == COMPACT_EOP) {
-			mZ[i][t] = NADBL;
-		    } else if (method == COMPACT_AVG && mn[i] > 0) {
-			mZ[i][t] /= (double) mn[i];
-		    }
-		}		    
+		mn[i] += 1;
 	    }
+	    if (mon == monbak && s == pdinfo->n - 1) {
+		/* reached the end: ship out last obs */
+		if (method == COMPACT_EOP) {
+		    mZ[i][t] = NADBL;
+		} else if (method == COMPACT_AVG && mn[i] > 0) {
+		    mZ[i][t] /= (double) mn[i];
+		}
+	    }		    
 	}
 	monbak = mon;
     }
@@ -3128,9 +3112,7 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
     DATAINFO minfo;
     int startyr = 1, endyr;
     int startmon = 1, endmon;
-    double *x;
-    int nseries = 0;
-    int i, err = 0;
+    int err = 0;
 
     minfo.n = weeks_to_months_check(pdinfo, &startyr, &endyr, &startmon, &endmon);
     fprintf(stderr, "Weekly data: found %d months\n", minfo.n);
@@ -3144,23 +3126,8 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 	return err;
     }
 
-    /* handle scalars */
-    for (i=1; i<pdinfo->v && !err; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    x = realloc(mZ[i], sizeof *x);
-	    if (x == NULL) {
-		err = E_ALLOC;
-	    } else {
-		mZ[i] = x;
-		mZ[i][0] = (*pZ)[i][0];
-	    }
-	} else {
-	    nseries++;
-	}
-    }
-
     /* compact series */
-    if (!err && nseries > 0) {
+    if (!err && pdinfo->v > 1) {
 	err = weeks_to_months_exec(mZ, (const double **) *pZ, pdinfo, method);
     }
 
@@ -3182,6 +3149,18 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static int shorten_the_constant (double **Z, int n)
+{
+    double *tmp = realloc(Z[0], n * sizeof **Z);
+
+    if (tmp == NULL) {
+	return E_ALLOC;
+    } else {
+	Z[0] = tmp;
+	return 0;
+    }
+}
+
 /* conversion to weekly using a "representative day", e.g. use
    each Wednesday value.  repday is 0-based on Sunday.
 */
@@ -3192,6 +3171,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
     int y1, m1, d1;
     char obs[OBSLEN];
     double *x = NULL;
+    double *tmp;
     int n = 0, n_ok = 0;
     int wday, ok;
     int i, t, err = 0;
@@ -3204,7 +3184,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
 	if (wday == repday) {
 	    ok = 0;
 	    for (i=1; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i) && !na(Z[i][t])) {
+		if (!na(Z[i][t])) {
 		    ok = 1;
 		    break;
 		}
@@ -3232,20 +3212,16 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
 	return E_ALLOC;
     }
 
+    err = shorten_the_constant(Z, n);
+
     for (i=1; i<pdinfo->v && !err; i++) {
-	double *tmp;
 	int s = 0;
 
-	if (i > 0 && var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
-	if (i > 0) {
-	    for (t=0; t<pdinfo->n; t++) {
-		ntodate_full(obs, t, pdinfo);
-		wday = get_day_of_week(obs);
-		if (wday == repday) {
-		    x[s++] = Z[i][t];
-		}
+	for (t=0; t<pdinfo->n; t++) {
+	    ntodate_full(obs, t, pdinfo);
+	    wday = get_day_of_week(obs);
+	    if (wday == repday) {
+		x[s++] = Z[i][t];
 	    }
 	}
 	tmp = realloc(Z[i], n * sizeof **Z);
@@ -3254,7 +3230,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
 	} else {
 	    Z[i] = tmp;
 	    for (t=0; t<n; t++) { 
-		Z[i][t] = (i == 0)? 1.0 : x[t];
+		Z[i][t] = x[t];
 	    }
 	}
     }
@@ -3277,11 +3253,13 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
     return err;
 }
 
-static int daily_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
+static int daily_dataset_to_monthly (double **Z, DATAINFO *pdinfo,
 				     CompactMethod default_method)
 {
     int nm, startyr, startmon, endyr, endmon;
     int offset, any_eop;
+    CompactMethod method;
+    double *x;
     int i, err = 0;
 
     nm = get_n_ok_months(pdinfo, default_method, &startyr, &startmon,
@@ -3289,30 +3267,25 @@ static int daily_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 
     if (nm <= 0) {
 	gretl_errmsg_set(_("Compacted dataset would be empty"));
-	err = 1;
-    } else {
-	for (i=0; i<pdinfo->v && !err; i++) {
-	    CompactMethod method;
-	    double *x;
+	return E_DATA;
+    }
 
-	    if (i > 0 && var_is_scalar(pdinfo, i)) {
-		continue;
-	    }
+    err = shorten_the_constant(Z, nm);
 
-	    method = COMPACT_METHOD(pdinfo, i);
-	    if (method == COMPACT_NONE) {
-		method = default_method;
-	    }
+    for (i=1; i<pdinfo->v && !err; i++) {
+	method = COMPACT_METHOD(pdinfo, i);
+	if (method == COMPACT_NONE) {
+	    method = default_method;
+	}
 
-	    x = daily_series_to_monthly((*pZ)[i], pdinfo, i, nm,
-					startyr, startmon, 
-					offset, any_eop, method);
-	    if (x == NULL) {
-		err = E_ALLOC;
-	    } else {
-		free((*pZ)[i]);
-		(*pZ)[i] = x;
-	    }
+	x = daily_series_to_monthly(Z[i], pdinfo, i, nm,
+				    startyr, startmon, 
+				    offset, any_eop, method);
+	if (x == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    free(Z[i]);
+	    Z[i] = x;
 	}
     }
 
@@ -3431,6 +3404,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 		      int repday)
 {
     int newn, oldn = pdinfo->n, oldpd = pdinfo->pd;
+    double **Z = *pZ;
     int compfac;
     int startmaj, startmin;
     int endmaj, endmin;
@@ -3460,10 +3434,10 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     if (newpd == 52 && oldpd >= 5 && oldpd <= 7 && 
 	default_method == COMPACT_WDAY) {
 	/* daily to weekly, using "representative day" */
-	return daily_dataset_to_weekly(*pZ, pdinfo, repday);
+	return daily_dataset_to_weekly(Z, pdinfo, repday);
     } else if (newpd == 12 && oldpd >= 5 && oldpd <= 7) {
 	/* daily to monthly: special */
-	return daily_dataset_to_monthly(pZ, pdinfo, default_method);
+	return daily_dataset_to_monthly(Z, pdinfo, default_method);
     } else if (oldpd >= 5 && oldpd <= 7) {
 	/* daily to weekly */
 	compfac = oldpd;
@@ -3535,38 +3509,38 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 	dataset_destroy_obs_markers(pdinfo);
     }
 
+    err = shorten_the_constant(Z, pdinfo->n);
+
     /* compact the individual data series */
-    for (i=0; i<pdinfo->v && err == 0; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    CompactMethod this_method = default_method;
-	    int startskip = min_startskip;
-	    double *x;
+    for (i=1; i<pdinfo->v && !err; i++) {
+	CompactMethod this_method = default_method;
+	int startskip = min_startskip;
+	double *x;
 
-	    if (!all_same) {
-		if (COMPACT_METHOD(pdinfo, i) != COMPACT_NONE) {
-		    this_method = COMPACT_METHOD(pdinfo, i);
-		}
-
-		startskip = compfac - (startmin % compfac) + 1;
-		startskip = startskip % compfac;
-		if (this_method == COMPACT_EOP) {
-		    if (startskip > 0) {
-			startskip--;
-		    } else {
-			startskip = compfac - 1;
-		    }
-		}
+	if (!all_same) {
+	    if (COMPACT_METHOD(pdinfo, i) != COMPACT_NONE) {
+		this_method = COMPACT_METHOD(pdinfo, i);
 	    }
 
-	    x = compact_series((*pZ)[i], i, pdinfo->n, oldn, startskip, 
-			       min_startskip, compfac, this_method);
-	    if (x == NULL) {
-		err = E_ALLOC;
-	    } else {
-		free((*pZ)[i]);
-		(*pZ)[i] = x;
+	    startskip = compfac - (startmin % compfac) + 1;
+	    startskip = startskip % compfac;
+	    if (this_method == COMPACT_EOP) {
+		if (startskip > 0) {
+		    startskip--;
+		} else {
+		    startskip = compfac - 1;
+		}
 	    }
 	}
+
+	x = compact_series(Z[i], i, pdinfo->n, oldn, startskip, 
+			   min_startskip, compfac, this_method);
+	if (x == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    free(Z[i]);
+	    Z[i] = x;
+	}
     }
 
     return err;
@@ -3620,9 +3594,6 @@ int expand_data_set (double ***pZ, DATAINFO *pdinfo, int newpd)
     }
 
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
 	for (t=0; t<oldn; t++) {
 	    x[t] = (*pZ)[i][t];
 	}
