@@ -51,7 +51,7 @@
 #define XPX_DEBUG 0
 
 static void regress (MODEL *pmod, double *xpy, const double **Z, 
-		     double rho);
+		     double rho, gretlopt opt);
 static int cholbeta (MODEL *pmod, double *xpy,  double *rss);
 static void diaginv (double *xpx, double *xpy, double *diag, int nv);
 
@@ -663,12 +663,58 @@ static int gretl_choleski_regress (MODEL *pmod, const double **Z,
     fputc('\n', stderr);
 #endif
 
-    regress(pmod, xpy, Z, rho);
+    regress(pmod, xpy, Z, rho, opt);
     free(xpy);
 
     return pmod->errcode;
 }
 
+static int gretl_null_regress (MODEL *pmod, const double **Z)
+{
+    double yt;
+    int t;
+
+    if (pmod->yhat == NULL) {
+	pmod->yhat = malloc(pmod->full_n * sizeof *pmod->yhat);
+    } 
+
+    if (pmod->uhat == NULL) {
+	pmod->uhat = malloc(pmod->full_n * sizeof *pmod->uhat);
+    }
+
+    if (pmod->yhat == NULL || pmod->uhat == NULL) {
+	pmod->errcode = E_ALLOC;
+	return pmod->errcode;
+    }
+
+    pmod->nobs = 0;
+    pmod->ifc = 0;
+    pmod->ess = 0.0;
+    pmod->rsq = pmod->adjrsq = 0.0;
+
+    for (t=0; t<pmod->full_n; t++) {
+	yt = Z[pmod->list[1]][t];
+	if (t < pmod->t1 || t > pmod->t2 || na(yt)) {
+	    pmod->uhat[t] = pmod->yhat[t] = NADBL;
+	} else {
+	    pmod->uhat[t] = yt;
+	    pmod->yhat[t] = 0.0;
+	    pmod->ess += yt * yt;
+	    pmod->nobs += 1;
+	}
+    }
+
+    if (pmod->ess == 0) {
+	pmod->sigma = 0.0;
+    } else if (pmod->nobs > 1) {
+	pmod->sigma = sqrt(pmod->ess / (pmod->nobs - 1));
+    } else {
+	pmod->errcode = E_DATA;
+    }
+
+    return pmod->errcode;
+}
+
 /* limited freeing of elements before passing a model
    on for QR estimation in the case of singularity 
 */
@@ -698,6 +744,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
     int jackknife = 0;
     int use_qr = libset_get_bool(USE_QR);
     int pwe = (opt & OPT_P);
+    int nullmod = 0;
     int yno, i;
 
     gretl_error_clear();
@@ -725,7 +772,10 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     } 
 
-    if (list[0] == 1 || pdinfo->v == 1) {
+    if (list[0] == 1 && ci == OLS && (opt & OPT_U)) {
+	/* null model OK */
+	nullmod = 1;
+    } else if (list[0] == 1 || pdinfo->v == 1) {
 	fprintf(stderr, "E_DATA: lsq: list[0] = %d, pdinfo->v = %d\n",
 		list[0], pdinfo->v);
 	mdl.errcode = E_DATA;
@@ -851,7 +901,9 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
 	jackknife = 1;
     }
 
-    if (!jackknife && ((opt & OPT_R) || use_qr)) { 
+    if (nullmod) {
+	gretl_null_regress(&mdl, (const double **) *pZ);
+    } else if (!jackknife && ((opt & OPT_R) || use_qr)) { 
 	mdl.rho = rho;
 	gretl_qr_regress(&mdl, (const double **) *pZ, pdinfo, opt);
     } else {
@@ -956,6 +1008,7 @@ MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo,
  *   %OPT_Z (internal use) suppress the automatic elimination of 
  *      perfectly collinear variables.
  *   %OPT_X: compute "variance matrix" as just (X'X)^{-1}
+ *   %OPT_B: don't compute R^2.
  *
  * Computes least squares estimates of the model specified by @list,
  * using an estimator determined by the value of @ci.
@@ -1229,7 +1282,13 @@ static void compute_r_squared (MODEL *pmod, const double *y, int *ifc)
 		    den += y[t] * y[t];
 		}
 	    }
-	    pmod->rsq = 1 - pmod->ess / den; /* NIST method */
+
+	    /* make the centered R^2 available for output */
+	    gretl_model_set_double(pmod, "centered-R2", pmod->rsq);
+
+	    /* but make the "official" figure the uncentered R^2,
+	       as per NIST, R, Stata, SPSS... */
+	    pmod->rsq = 1 - pmod->ess / den;
 	    pmod->adjrsq = 
 		1.0 - ((1.0 - pmod->rsq) * (pmod->nobs - 1.0) / pmod->dfd);
 	} 
@@ -1255,7 +1314,7 @@ static void compute_r_squared (MODEL *pmod, const double *y, int *ifc)
 */
 
 static void regress (MODEL *pmod, double *xpy, const double **Z, 
-		     double rho)
+		     double rho, gretlopt opt)
 {
     int v, yno = pmod->list[1];
     int ifc = pmod->ifc;
@@ -1315,10 +1374,13 @@ static void regress (MODEL *pmod, double *xpy, const double **Z,
     hatvar(pmod, n, Z); 
     if (pmod->errcode) return;
 
-    if (pmod->tss > 0.0) {
-	compute_r_squared(pmod, Z[yno], &ifc);
-    } else if (pmod->tss == 0.0) {
-	uncentered_r_squared(pmod, Z[yno]);
+    if (!(opt & OPT_B)) {
+	/* changed 2008-09-25 */
+	if (pmod->tss > 0.0) {
+	    compute_r_squared(pmod, Z[yno], &ifc);
+	} else if (pmod->tss == 0.0) {
+	    uncentered_r_squared(pmod, Z[yno]);
+	}
     }
 
 #if 0
@@ -3330,14 +3392,26 @@ arch_test_save_or_print (const gretl_matrix *b, const gretl_matrix *V,
     if (V != NULL) {
 	int i, k = order + 1;
 	double *se = malloc(k * sizeof *se);
+	char **names;
+
+	names = strings_array_new_with_length(k, 16);
+
+	if (se != NULL && names != NULL) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Test for ARCH of order %d"), order);
+	    pputs(prn, "\n\n");
 
-	if (se != NULL) {
 	    for (i=0; i<k; i++) {
 		se[i] = sqrt(gretl_matrix_get(V, i, i));
+		sprintf(names[i], "alpha(%d)", i);
 	    }
-	    print_arch_coeffs(b->val, se, T, order, prn, AUX_ARCH);
-	    free(se);
+
+	    plain_print_aux_coeffs(b->val, se, (const char **) names, 
+				   k, T - k, ARCH, prn);
 	}
+
+	free(se);
+	free_strings_array(names, k);
     }
 
     if (test != NULL) {
