@@ -594,10 +594,13 @@ int filter_series (const double *x, double *y, const DATAINFO *pdinfo,
     return err;
 }
 
-int panel_mean_series (const double *x, double *y, const DATAINFO *pdinfo)
+int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo, 
+		     int k)
 {
     const int *unit;
+    double ret = NADBL;
     double xbar = NADBL;
+    double aux = NADBL;
     int smin = 0;
     int s, t, Ti = 0;
 
@@ -607,83 +610,126 @@ int panel_mean_series (const double *x, double *y, const DATAINFO *pdinfo)
 
     unit = pdinfo->paninfo->unit;
 
-    for (t=0; t<=pdinfo->n; t++) {
-	if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
-	    if (!na(xbar)) {
-		xbar /= Ti;
-	    }
-	    /* got a new unit (or reached the end): 
-	       ship out current mean */
-	    for (s=smin; s<t; s++) {
-		y[s] = xbar;
-	    }
-	    if (t == pdinfo->n) {
-		break;
-	    }
-	    Ti = 0;
-	    xbar = NADBL;
-	    smin = t;
-	}
-	if (!na(x[t])) {
-	    if (na(xbar)) {
-		xbar = x[t];
-	    } else {
-		xbar += x[t];
-	    }
-	    Ti++;
-	}
-    }
-
-    return 0;
-}
-
-int panel_sd_series (const double *x, double *y, const DATAINFO *pdinfo)
-{
-    const int *unit;
-    double sd, xbar = NADBL;
-    int smin = 0;
-    int s, t, Ti = 0;
-
-    if (pdinfo->paninfo == NULL) {
-	return E_DATA;
-    }
-
-    unit = pdinfo->paninfo->unit;
-
-    for (t=0; t<=pdinfo->n; t++) {
-	if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
-	    if (na(xbar)) {
-		sd = NADBL;
-	    } else if (Ti == 1) {
-		sd = 0.0;
-	    } else {
-		xbar /= Ti;
-		sd = 0.0;
+    switch (k) {
+    case F_PNOBS:
+	for (t=0; t<=pdinfo->n; t++) {
+	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
 		for (s=smin; s<t; s++) {
-		    if (!na(x[s])) {
-			sd += (x[s] - xbar) * (x[s] - xbar);
-		    }
+		    y[s] = Ti;
 		}
-		sd = sqrt(sd / (Ti - 1));
+		if (t == pdinfo->n) {
+		    break;
+		}
+		Ti = 0;
+		smin = t;
 	    }
-	    for (s=smin; s<t; s++) {
-		y[s] = sd;
+	    if (!na(x[t])) {
+		Ti++;
 	    }
-	    if (t == pdinfo->n) {
-		break;
-	    }
-	    Ti = 0;
-	    xbar = NADBL;
-	    smin = t;
 	}
-	if (!na(x[t])) {
-	    if (na(xbar)) {
-		xbar = x[t];
-	    } else {
-		xbar += x[t];
+	break;
+    case F_PMIN:
+	for (t=0; t<=pdinfo->n; t++) {
+	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+		for (s=smin; s<t; s++) {
+		    y[s] = aux;
+		}
+		if (t == pdinfo->n) {
+		    break;
+		}
+		aux = NADBL;
+		smin = t;
+	    }
+	    if (!na(x[t])) {
+		if (na(aux) || x[t]<aux) {
+		    aux = x[t];
+		}
 	    }
-	    Ti++;
 	}
+	break;
+    case F_PMAX:
+	for (t=0; t<=pdinfo->n; t++) {
+	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+		for (s=smin; s<t; s++) {
+		    y[s] = aux;
+		}
+		if (t == pdinfo->n) {
+		    break;
+		}
+		aux = NADBL;
+		smin = t;
+	    }
+	    if (!na(x[t])) {
+		if (na(aux) || x[t]>aux) {
+		    aux = x[t];
+		}
+	    }
+	}
+	break;
+    case F_PMEAN:
+	for (t=0; t<=pdinfo->n; t++) {
+	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+		if (!na(xbar)) {
+		    xbar /= Ti;
+		}
+		/* got a new unit (or reached the end): 
+		   ship out current mean */
+		for (s=smin; s<t; s++) {
+		    y[s] = xbar;
+		}
+		if (t == pdinfo->n) {
+		    break;
+		}
+		Ti = 0;
+		xbar = NADBL;
+		smin = t;
+	    }
+	    if (!na(x[t])) {
+		if (na(xbar)) {
+		    xbar = x[t];
+		} else {
+		    xbar += x[t];
+		}
+		Ti++;
+	    }
+	}
+	break;
+    case F_PSD:
+	for (t=0; t<=pdinfo->n; t++) {
+	    if (t == pdinfo->n || (t > 0 && unit[t] != unit[t-1])) {
+		if (na(xbar)) {
+		    ret = NADBL;
+		} else if (Ti == 1) {
+		    ret = 0.0;
+		} else {
+		    xbar /= Ti;
+		    ret = sqrt((aux/Ti - xbar*xbar) * Ti/(Ti-1));
+		}
+		for (s=smin; s<t; s++) {
+		    y[s] = ret;
+		}
+		if (t == pdinfo->n) {
+		    break;
+		}
+		Ti = 0;
+		xbar = NADBL;
+		aux = NADBL;
+		smin = t;
+	    }
+	    if (!na(x[t])) {
+		if (na(xbar)) {
+		    xbar = x[t];
+		    aux = x[t]*x[t];
+		} else {
+		    xbar += x[t];
+		    aux += x[t]*x[t];
+		}
+		Ti++;
+	    }
+	}
+	break;
+    default:
+	break;
     }
 
     return 0;
@@ -975,24 +1021,6 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
     return err;
 }
 
-static int panel_x_offset (const DATAINFO *pdinfo, int *bad)
-{
-    char *p = strchr(pdinfo->stobs, ':');
-    int offset = 0;
-
-    if (p == NULL) {
-	p = strchr(pdinfo->stobs, '.');
-    }
-
-    if (p == NULL) {
-	*bad = 1;
-    } else {
-	offset = atoi(p + 1) - 1;
-    }
-
-    return offset;
-}
-
 static int n_new_dummies (const DATAINFO *pdinfo,
 			  int nunits, int nperiods)
 {
@@ -1001,14 +1029,14 @@ static int n_new_dummies (const DATAINFO *pdinfo,
 
     for (i=0; i<nunits; i++) {
 	sprintf(dname, "du_%d", i + 1);
-	if (varindex(pdinfo, dname) < pdinfo->v) {
+	if (gretl_is_series(dname, pdinfo)) {
 	    nnew--;
 	}
     }
 
     for (i=0; i<nperiods; i++) {
 	sprintf(dname, "dt_%d", i + 1);
-	if (varindex(pdinfo, dname) < pdinfo->v) {
+	if (gretl_is_series(dname, pdinfo)) {
 	    nnew--;
 	}
     }
@@ -1028,14 +1056,10 @@ make_dummy_name_and_label (int vi, const DATAINFO *pdinfo, int center,
 	strcpy(vlabel, "uncentered periodic dummy");
     } else if (pdinfo->pd == 4 && pdinfo->structure == TIME_SERIES) {
 	sprintf(vname, "dq%d", vi);
-	sprintf(vlabel, 
-		_("= 1 if quarter = %d, 0 otherwise"), vi);
+	sprintf(vlabel, _("= 1 if quarter = %d, 0 otherwise"), vi);
     } else if (pdinfo->pd == 12 && pdinfo->structure == TIME_SERIES) {
-	char mname[8];
-
-	get_month_name(mname, vi);
-	sprintf(vname, "d%s", mname);
-	sprintf(vlabel, _("= 1 if month is %s, 0 otherwise"), mname);
+	sprintf(vname, "dm%d", vi);
+	sprintf(vlabel, _("= 1 if month = %d, 0 otherwise"), vi);
     } else {
 	char dumstr[8] = "dummy_";
 	char numstr[8];
@@ -1082,7 +1106,7 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
 
     for (vi=0; vi<ndums; vi++) {
 	make_dummy_name_and_label(vi + 1, pdinfo, center, vname, vlabel);
-	di = varindex(pdinfo, vname);
+	di = series_index(pdinfo, vname);
 	if (di >= pdinfo->v || strcmp(vlabel, VARLABEL(pdinfo, di))) {
 	    nnew++;
 	} else if (vi == 0) {
@@ -1103,7 +1127,7 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     }
 
     if (dataset_add_series(ndums, pZ, pdinfo)) {
-	strcpy(gretl_errmsg, _("Out of memory error"));
+	strcpy(gretl_errmsg, _("Out of memory!"));
 	return 0;
     }
 
@@ -1177,7 +1201,7 @@ int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt)
     int ndum, nnew;
     int n_unitdum = 0;
     int n_timedum = 0;
-    int newvnum, offset, bad = 0;
+    int newvnum;
     double xx;
 
     if (opt & OPT_T) {
@@ -1215,7 +1239,7 @@ int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt)
 
 	sprintf(vname, "dt_%d", vi);
 
-	dnum = varindex(pdinfo, vname);
+	dnum = series_index(pdinfo, vname);
 	if (dnum >= orig_v) {
 	    dnum = newvnum++;
 	}
@@ -1233,20 +1257,16 @@ int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt)
 	}
     }
 
-    offset = panel_x_offset(pdinfo, &bad);
-
     /* generate unit-based dummies, if wanted */
 
     for (vi=1; vi<=n_unitdum; vi++) {
-	int dmin = (vi-1) * pdinfo->pd;
-	int dmax = vi * pdinfo->pd - offset;
+	int dmin = (vi - 1) * pdinfo->pd;
+	int dmax = vi * pdinfo->pd;
 	int dnum;
 
-	if (vi > 1) dmin -= offset;
-
 	sprintf(vname, "du_%d", vi);
 
-	dnum = varindex(pdinfo, vname);
+	dnum = series_index(pdinfo, vname);
 	if (dnum >= orig_v) {
 	    dnum = newvnum++;
 	}	
@@ -1257,13 +1277,7 @@ int panel_dummies (double ***pZ, DATAINFO *pdinfo, gretlopt opt)
 		_("unit"), vi);
 
 	for (t=0; t<pdinfo->n; t++) {
-	    if (bad) {
-		(*pZ)[dnum][t] = NADBL;
-	    } else if (t >= dmin && t < dmax) {
-		(*pZ)[dnum][t] = 1.0;
-	    } else {
-		(*pZ)[dnum][t] = 0.0;
-	    }
+	    (*pZ)[dnum][t] = (t >= dmin && t < dmax)? 1 : 0;
 	}
     }
 
@@ -1292,7 +1306,7 @@ int gen_unit (double ***pZ, DATAINFO *pdinfo)
 	return 1;
     }
 
-    i = varindex(pdinfo, "unit");
+    i = series_index(pdinfo, "unit");
 
     if (i == pdinfo->v && dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -1372,7 +1386,7 @@ int gen_time (double ***pZ, DATAINFO *pdinfo, int tm)
 {
     int i, t;
 
-    i = varindex(pdinfo, (tm)? "time" : "index");
+    i = series_index(pdinfo, (tm)? "time" : "index");
 
     if (i == pdinfo->v && dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -1420,7 +1434,7 @@ int gen_wkday (double ***pZ, DATAINFO *pdinfo)
 	return E_PDWRONG;
     }
 
-    i = varindex(pdinfo, "weekday");
+    i = series_index(pdinfo, "weekday");
 
     if (i == pdinfo->v && dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
@@ -1773,17 +1787,12 @@ int get_t_from_obs_string (const char *s, const double **Z,
 	    fprintf(stderr, " plain_obs_number gives t = %d\n", t);
 #endif
 	} else {
-	    int v = varindex(pdinfo, s);
-
-	    if (v == pdinfo->v && strlen(s) == 1) {
+	    if (gretl_is_scalar(s)) {
+		t = gretl_scalar_get_value(s);
+	    } else if (strlen(s) == 1) {
 		t = loop_scalar_read(s[0]);
 #if OBS_DEBUG
 		fprintf(stderr, " loop_scalar_read gave t = %d\n", t);
-#endif
-	    } else if (v < pdinfo->v) {
-		t = (int) Z[v][0];
-#if OBS_DEBUG
-		fprintf(stderr, " based on var %d: t = %d\n", v, t);
 #endif
 	    }
 
@@ -1859,7 +1868,8 @@ int check_declarations (char ***pS, parser *p)
     }
 
     for (i=0; i<n && !p->err; i++) {
-	if (varindex(p->dinfo, S[i]) < p->dinfo->v || 
+	if (gretl_is_series(S[i], p->dinfo) ||
+	    gretl_is_scalar(S[i]) ||
 	    get_matrix_by_name(S[i]) ||
 	    get_list_by_name(S[i]) ||
 	    get_string_by_name(S[i])) {
