@@ -71,6 +71,7 @@ struct str_table dvars[] = {
     { R_NOBS,      "$nobs" },
     { R_NVARS,     "$nvars" },
     { R_PD,        "$pd" },
+    { R_DATATYPE,  "$datatype" },
     { R_TEST_STAT, "$test" },
     { R_TEST_PVAL, "$pvalue" },
     { R_TEST_LNL,  "$rlnl" },
@@ -81,6 +82,8 @@ struct str_table dvars[] = {
     { R_STOPWATCH, "$stopwatch" },
     { R_NSCAN,     "$nscan" },
     { R_PUNIT,     "$unit" },
+    { R_WINDOWS,   "$windows" },
+    { R_VERSION,   "$version" },
     { 0,           NULL },
 };
 
@@ -120,7 +123,8 @@ struct str_table mvars[] = {
     { M_SYSB,    "$sysB" },
     { M_FCAST,   "$fcast" },
     { M_FCERR,   "$fcerr" },
-    { M_COEFF_CI, "$coeff_ci" },
+    { M_COEFF_CI,"$coeff_ci" },
+    { M_XLIST,   "$xlist" },
     { 0,         NULL }
 };
 
@@ -179,6 +183,9 @@ struct str_table funcs[] = {
     { F_GAMMA,    "gammafun" },
     { F_LNGAMMA,  "lngamma" },
     { F_RESAMPLE, "resample" },
+    { F_PNOBS,    "pnobs" },     /* per-unit nobs in panels */
+    { F_PMIN,     "pmin" },      /* panel min */
+    { F_PMAX,     "pmax" },       /* panel max */
     { F_PMEAN,    "pmean" },     /* panel mean */
     { F_PSD,      "psd" },       /* panel std dev */
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
@@ -246,6 +253,7 @@ struct str_table funcs[] = {
     { F_BFGSMAX,  "BFGSmax" },
     { F_OBSNUM,   "obsnum" },
     { F_ISSERIES, "isseries" },
+    { F_ISSCALAR, "isscalar" },
     { F_ISLIST,   "islist" },
     { F_ISSTRING, "isstring" },
     { F_ISNULL,   "isnull" },
@@ -560,8 +568,11 @@ int genr_function_word (const char *s)
 static void undefined_symbol_error (const char *s, parser *p)
 {
     parser_print_input(p);
-    pprintf(p->prn, _("The symbol '%s' is undefined\n"), s);
-    sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
+    if (p->ch == '.') {
+	sprintf(gretl_errmsg, _("%s: no such object\n"), s);
+    } else {
+	sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
+    }
     p->err = E_UNKVAR;
 }
 
@@ -746,13 +757,15 @@ static void look_up_word (const char *s, parser *p)
 	    if (p->idnum > 0) {
 		p->sym = DUM;
 	    } else {
-		p->idnum = varindex(p->dinfo, s);
+		p->idnum = series_index(p->dinfo, s);
 		if (p->idnum < p->dinfo->v) {
-		    p->sym = (var_is_scalar(p->dinfo, p->idnum))?
-			USCALAR : USERIES;
+		    p->sym = USERIES;
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
 		    p->idnum = DUM_TREND;
+		} else if (gretl_is_scalar(s)) {
+		    p->sym = USCALAR;
+		    p->idstr = gretl_strdup(s);
 		} else if (get_matrix_by_name(s)) {
 		    p->sym = UMAT;
 		    p->idstr = gretl_strdup(s);
@@ -835,6 +848,12 @@ static void word_check_next_char (const char *s, parser *p)
 	} else {
 	    p->err = 1;
 	}	    
+    } else if (p->ch == '.' && isalpha(*p->point)) {
+	if (p->sym == LIST) {
+	    p->sym = LISTVAR;
+	} else {
+	    p->err = 1;
+	}
     }
 
     if (p->err) {
@@ -1303,13 +1322,17 @@ const char *getsymb (int t, const parser *p)
 	return "VSTR";
     } else if (t == EMPTY) {
 	return "EMPTY";
+    } else if (t == LISTVAR) {
+	return "LISTVAR";
     }
 
     if (p != NULL) {
 	if (t == NUM) {
 	    return fromdbl(p->xval); 
-	} else if (t == USCALAR || t == USERIES) {
+	} else if (t == USERIES) {
 	    return p->dinfo->varname[p->idnum];
+	} else if (t == USCALAR) {
+	    return p->idstr;
 	} else if (t == UMAT || t == UOBJ ||
 		   t == LOOPIDX) {
 	    return p->idstr;
@@ -1402,8 +1425,9 @@ const char *getsymb (int t, const parser *p)
     case B_HCAT: 
 	return "~";
     case B_VCAT: 
-    case B_LCAT:
 	return "|";
+    case B_LCAT:
+	return " ";
     case P_COM: 
 	return ",";
     case P_DOT: 
