@@ -29,6 +29,25 @@
 # define GDEBUG 0
 #endif
 
+# define settings_obs_value(p) (p->lh.obs >= 0)
+
+static void write_scalar_message (const parser *p, int oldv, PRN *prn)
+{
+    double x = gretl_scalar_get_value(p->lh.name);
+
+    if (p->flags & P_LHSCAL) {
+	pprintf(prn, _("Replaced scalar %s"), p->lh.name);
+    } else {
+	pprintf(prn, _("Generated scalar %s"), p->lh.name);
+    }
+
+    if (na(x)) {
+	pputs(prn, " = NA");
+    } else {
+	pprintf(prn, " = %g", x);
+    }
+}
+
 static void gen_write_message (const parser *p, int oldv, PRN *prn)
 {
     if (prn == NULL || !gretl_messages_on()) {
@@ -36,25 +55,12 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
     }
 
     if (p->targ == NUM) {
-	if (var_is_series(p->dinfo, p->lh.v)) {
+	if (settings_obs_value(p)) {
+	    /* set specific observation in series */
 	    pprintf(prn, _("Modified series %s (ID %d)"),
 		    p->lh.name, p->lh.v);
 	} else {
-	    int t = lh_obs(p);
-	    double x = (*p->Z)[p->lh.v][t];
-
-	    if (p->lh.v < oldv) {
-		pprintf(prn, _("Replaced scalar %s (ID %d)"),
-			p->lh.name, p->lh.v);
-	    } else {
-		pprintf(prn, _("Generated scalar %s (ID %d)"),
-			p->lh.name, p->lh.v);
-	    }
-	    if (na(x)) {
-		pputs(prn, " = NA");
-	    } else {
-		pprintf(prn, " = %g", x);
-	    }
+	    write_scalar_message(p, oldv, prn);
 	}
     } else if (p->targ == VEC) {
 	if (p->lh.v < oldv) {
@@ -94,7 +100,7 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 
 static void gen_write_warning (const parser *p, PRN *prn)
 {
-    if (prn != NULL && !repeating_function_exec()) {
+    if (prn != NULL) {
 	if (*p->warning != '\0') {
 	    pprintf(prn, "%s: %s\n", 
 		    p->warning, _("missing values were generated"));
@@ -130,7 +136,7 @@ static int maybe_record_lag_info (parser *p)
     if (sscanf(s, "%15[^ ()](%d)", vname, &lag) == 2) {
 	s = strchr(s, ')');
 	if (s != NULL && string_is_blank(s + 1)) {
-	    int pv = varindex(p->dinfo, vname);
+	    int pv = series_index(p->dinfo, vname);
 
 	    if (pv < p->dinfo->v) {
 		strcpy(p->dinfo->varinfo[p->lh.v]->parent, 
@@ -150,8 +156,8 @@ static void gen_write_label (parser *p, int oldv)
     const char *src = "";
     size_t len = 0;
 
-    if (p->targ != NUM && p->targ != VEC) {
-	/* not relevant for matrices, lists */
+    if (p->targ != VEC) {
+	/* this is relevant only for series */
 	return;
     }
 
@@ -308,10 +314,62 @@ int gretl_reserved_word (const char *str)
     return ret;
 }
 
+/**
+ * extract_varname:
+ * @targ: target string into which to write name.
+ * @src: source string.
+ * @len: location to receive the length of the extracted portion.
+ * 
+ * Writes up to #VNAMELEN - 1 characters from @s into @vname.
+ * 
+ * Returns: 0 on success, non-zero if the number of valid varname
+ * characters in @s is greater than #VNAMELEN - 1.
+ */
+
+int extract_varname (char *targ, const char *src, int *len)
+{
+    int err = 0;
+
+    *targ = '\0';
+    *len = gretl_namechar_spn(src);
+
+    if (*len >= VNAMELEN) {
+	/* too long to be a valid variable name */
+	err = E_UNKVAR;
+    } else {
+	strncat(targ, src, *len);
+    }
+
+    return err;
+}
+
+static int try_for_listvar (const DATAINFO *pdinfo, const char *s)
+{
+    char vname[VNAMELEN];
+    char lname[VNAMELEN];
+
+    if (sscanf(s, "%15[^.].%15s", lname, vname) == 2) {
+	int *list = get_list_by_name(lname);
+
+	if (list != NULL) {
+	    int i, vi;
+
+	    for (i=1; i<=list[0]; i++) {
+		vi = list[i];
+		if (!strcmp(vname, pdinfo->varname[vi])) {
+		    return vi;
+		}
+	    }
+	}
+    }
+
+    return pdinfo->v;
+}
+
 #define GEN_LEVEL_DEBUG 0
 
 /**
- * varindex:
+ * series_index:
  * @pdinfo: data information struct.
  * @varname: name of variable to test.
  *
@@ -320,17 +378,13 @@ int gretl_reserved_word (const char *str)
  * that name.
  */
 
-int varindex (const DATAINFO *pdinfo, const char *varname)
+int series_index (const DATAINFO *pdinfo, const char *varname)
 {
     const char *s = varname;
-    int fsd = 0;
-    int i, ret = pdinfo->v;
+    int i, fd;
+    int ret = pdinfo->v;
 
-    if (s == NULL || *s == 0) {
-	return ret;
-    }
-
-    if (isdigit(*s)) {
+    if (s == NULL || *s == 0 || isdigit(*s)) {
 	return ret;
     }
 
@@ -338,35 +392,45 @@ int varindex (const DATAINFO *pdinfo, const char *varname)
 	return 0;
     }
 
-    fsd = gretl_function_depth();
+    if (strchr(s, '.')) {
+	return try_for_listvar(pdinfo, s);
+    }
 
-    if (fsd > 0) {
-	/* inside a function: see only vars at that level */
-	for (i=1; i<pdinfo->v; i++) { 
-	    if (STACK_LEVEL(pdinfo, i) == fsd && 
-		!strcmp(pdinfo->varname[i], s)) {
-		ret = i;
-		break;
-	    }
-	}
-    } else {
-	/* see all vars */
-	for (i=1; i<pdinfo->v; i++) { 
-	    if (!strcmp(pdinfo->varname[i], s)) { 
-		ret = i;
-		break;
+    fd = gretl_function_depth();
+
+    for (i=1; i<pdinfo->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+	    if (!strcmp(pdinfo->varname[i], s)) {
+		if (lists_protected() && var_is_listarg(pdinfo, i)) {
+		    /* variable is not visible by name in context */
+		    fprintf(stderr, "var %d (%s) is invisible\n", i, s);
+		} else {
+		    ret = i;
+		    break;
+		}
 	    }
 	}
     }
 
 #if GEN_LEVEL_DEBUG
-    fprintf(stderr, "varindex for '%s', fsd = %d: got %d (pdinfo->v = %d)\n", 
-	    s, fsd, ret, pdinfo->v);
+    fprintf(stderr, "series_index for '%s', fd = %d: got %d (pdinfo->v = %d)\n", 
+	    s, fd, ret, pdinfo->v);
 #endif 
 
     return ret;
 }
 
+int gretl_is_series (const char *name, const DATAINFO *pdinfo)
+{
+    if (pdinfo == NULL) {
+	return 0;
+    } else {
+	int v = series_index(pdinfo, name);
+
+	return (v >= 0 && v < pdinfo->v);
+    }
+}
+
 int genr_special_word (const char *s)
 {
     if (!strcmp(s, "dummy") ||
@@ -435,7 +499,7 @@ static int gen_special (const char *s, const char *line,
 
     if (!err && write_label) {
 	strcpy(p->lh.name, s);
-	p->lh.v = varindex(pdinfo, s);
+	p->lh.v = series_index(pdinfo, s);
 	p->Z = pZ;
 	p->dinfo = pdinfo;
 	p->targ = VEC;
@@ -804,14 +868,22 @@ int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
     if (opt & OPT_L) {
 	flags |= P_LOOP;
     } else if (opt & OPT_S) {
+	/* context is NLS/MLE or similar */
 	flags |= P_SLAVE;
     }
 
     realgen(NULL, p, pZ, pdinfo, prn, flags);
 
+    if (flags & P_SLAVE) {
+	if (p->warn != 0 && p->err == 0) {
+	    /* warnings re. NAs become errors */
+	    p->err = p->warn;
+	}
+    }
+
     if (p->err == 0) {
 	gen_save_or_print(p, prn);
-    }
+    } 
 
     gen_cleanup(p);
 
@@ -855,6 +927,23 @@ int genr_get_loopline (parser *p)
     }
 }
 
+int genr_get_output_type (const parser *p)
+{
+    int t = GRETL_TYPE_NONE;
+
+    if (!p->err) {
+	if (p->targ == NUM) {
+	    t = GRETL_TYPE_DOUBLE;
+	} else if (p->targ == VEC) {
+	    t = GRETL_TYPE_SERIES;
+	} else if (p->targ == MAT) {
+	    t = GRETL_TYPE_MATRIX;
+	} 
+    }
+
+    return t;
+}
+
 int genr_get_output_varnum (const parser *p)
 {
     return p->lh.v;
@@ -869,6 +958,15 @@ gretl_matrix *genr_get_output_matrix (const parser *p)
     }
 }
 
+double genr_get_output_scalar (const parser *p)
+{
+    if (p->targ == NUM) {
+	return gretl_scalar_get_value(p->lh.name);
+    } else {
+	return NADBL;
+    }
+}
+
 const char *genr_get_formula (const GENERATOR *genr)
 {
     return genr->input;
@@ -889,5 +987,11 @@ void genr_unset_na_check (parser *p)
     p->flags &= ~P_NATEST;
 }
 
+int genr_get_series_max (parser *p)
+{
+    /* dummy: to be added later */
+    return 0;
+}
+
 
 
