@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "usermat.h"
 #include "gretl_func.h"
+#include "gretl_scalar.h"
 
 #define GENDEBUG 0
 
@@ -99,6 +100,7 @@ enum {
 	      OVAR,	  /* object variable: variable "under" an object */
   /* 70 */    LOOPIDX,	  /* loop index variable */
 	      LIST,	  /* reference to named list */
+	      LISTVAR,    /* variable in list */
 	      STR,	  /* string */
 	      FARGS,	  /* set of n function arguments */
 	      EMPTY,      /* "null" */
@@ -106,8 +108,8 @@ enum {
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
 	      EROOT,	  /* dummy root for (...) expression */
               UFUN,	  /* user-defined function */
-	      VSTR,       /* string variable */
-  /* 80 */    INC,   
+  /* 80 */    VSTR,       /* string variable */
+	      INC,   
 	      DEC,
 	      QUERY,
 	      EOT,	  /* end of transmission */
@@ -166,6 +168,9 @@ enum {
     F_HPFILT,
     F_BKFILT,
     F_RESAMPLE,
+    F_PNOBS,
+    F_PMIN,
+    F_PMAX,
     F_PMEAN,
     F_PSD,
     F_IMAT,
@@ -197,6 +202,7 @@ enum {
     F_RANK, 
     F_OBSNUM,
     F_ISSERIES,
+    F_ISSCALAR,
     F_ISLIST,	 
     F_ISSTRING,
     F_ISNULL,
@@ -226,7 +232,6 @@ enum {
     F_LOWER,
     F_MREAD,
     F_POLROOTS,
-    F_DUMIFY,
     F_XPX,
     F_GETENV,
     F_ARGNAME,
@@ -239,6 +244,7 @@ enum {
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
+    F_DUMIFY,
     F_SORTBY,
     F_RUNIFORM,
     F_RNORMAL,
@@ -286,7 +292,8 @@ enum {
 enum {
     CONST_PI = 1,
     CONST_NA,
-    CONST_WIN32
+    CONST_WIN32,
+    CONST_VERSION
 };
 
 enum {
@@ -326,13 +333,13 @@ enum {
                    !func2_symb(s) && s != EROOT)
 
 #define b2sym(s) (evalb2(s) || s == DMSTR || s == DMSL || \
-                  s == OVAR || s == UFUN)
+                  s == OVAR || s == UFUN || s == LISTVAR)
 
 #define b3sym(s) (s == QUERY)
 
 #define bnsym(s) (s == MDEF || s == FARGS)
 
-#define freestr(s) (s == STR || s == UMAT || s == UOBJ || \
+#define freestr(s) (s == STR || s == USCALAR || s == UMAT || s == UOBJ || \
                     s == LOOPIDX || s == LIST || s == VSTR)
 
 #define bare_data_type(s) (s > PUNCT_MAX && s < DTYPE_MAX)
@@ -414,13 +421,14 @@ enum {
     P_SORT    = 1 << 15, /* doing a simple sort of a variable */
     P_NATEST  = 1 << 16, /* testing for NAs in expression */
     P_UFRET   = 1 << 17, /* returning value generated by user function */
-    P_LHLIST  = 1 << 18, /* there was a pre-existing LHS list */
-    P_LHSTR   = 1 << 19, /* there was a pre-existing LHS string */
-    P_LHMAT   = 1 << 20, /* there was a pre-existing LHS matrix */
-    P_QUIET   = 1 << 21, /* don't print any messages or labels */
-    P_GETSTR  = 1 << 22, /* state: flag acceptance of plain strings */
-    P_LOOP    = 1 << 23, /* executing pre-built genr in loop context */
-    P_SLAVE   = 1 << 24  /* running as "slave" of NLS/MLE/GMM */
+    P_LHSCAL  = 1 << 18, /* there was a pre-existing LHS scalar */
+    P_LHLIST  = 1 << 19, /* there was a pre-existing LHS list */
+    P_LHSTR   = 1 << 20, /* there was a pre-existing LHS string */
+    P_LHMAT   = 1 << 21, /* there was a pre-existing LHS matrix */
+    P_QUIET   = 1 << 22, /* don't print any messages or labels */
+    P_GETSTR  = 1 << 23, /* state: flag acceptance of plain strings */
+    P_LOOP    = 1 << 24, /* executing pre-built genr in loop context */
+    P_SLAVE   = 1 << 25  /* running as "slave" of NLS/MLE/GMM */
 };
 
 struct lhinfo {
@@ -469,7 +477,6 @@ struct parser_ {
 #define starting(p) (p->flags & P_START)
 #define autoreg(p) (p->flags & P_AUTOREG)
 #define simple_ufun_call(p) (p->flags & P_UFUN)
-#define lh_obs(p) ((p->lh.obs > 0)? p->lh.obs : 0)
 
 int parser_getc (parser *p);
 void parser_ungetc (parser *p);
