@@ -69,17 +69,14 @@ static NODE *newref (parser *p, int t)
 #endif
 
     if (n != NULL) {
-	if (t == USCALAR) {
-	    n->vnum = p->idnum;
-	    n->t = NUM;
-	    n->v.xval = (*p->Z)[n->vnum][0];
-	} else if (t == USERIES) {
+	if (t == USERIES) {
 	    n->vnum = p->idnum;
 	    n->t = VEC;
 	    n->v.xvec = (*p->Z)[n->vnum];
 	} else {
 	    n->t = t;
-	    if (t == UMAT || t == UOBJ || t == LOOPIDX || t == LIST) {
+	    if (t == USCALAR || t == UMAT || t == UOBJ || 
+		t == LOOPIDX || t == LIST) {
 		n->v.str = p->idstr;
 	    } else {
 		n->v.idnum = p->idnum;
@@ -395,6 +392,65 @@ static NODE *base (parser *p, NODE *up)
     return t;
 }
 
+/* construct a node that contains a reference to a data series, based
+   on the name of a list (already captured in p->idstr) and the name
+   of a variable that is supposed to be in the list, which is separated
+   from the listname by '.'.
+*/
+
+static NODE *listvar_node (parser *p)
+{
+    int *list = get_list_by_name(p->idstr);
+    NODE *ret = NULL;
+    char vname[VNAMELEN] = {0};
+    int i, n, v, ok = 0;
+
+    if (list == NULL) {
+	p->err = E_UNKVAR;
+	return NULL;
+    }
+
+    n = gretl_namechar_spn(p->point);
+    if (n >= VNAMELEN) {
+	/* too long -- can't be a valid varname */
+	p->err = E_UNKVAR;
+	return NULL;
+    }
+
+    for (i=0; i<n; i++) {
+	parser_getc(p);
+	vname[i] = p->ch;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (!strcmp(vname, p->dinfo->varname[v])) {
+	    /* found the variable */
+	    free(p->idstr);
+	    p->idstr = NULL;
+	    p->idnum = v;
+	    ret = newref(p, USERIES);
+	    if (ret == NULL) {
+		p->err = E_ALLOC;
+	    } else {
+		ok = 1;
+	    }
+	    break;
+	}
+    }
+
+    if (!ok && !p->err) {
+	p->err = E_UNKVAR;
+    }
+
+    if (!p->err) {
+	parser_getc(p);
+	lex(p);
+    }
+
+    return ret;    
+}
+
 /* Grab a string argument.  Note: we have a mechanism in genlex.c for
    retrieving arguments that take the form of quoted string literals
    or names of string variables.  The special use of this function is
@@ -608,10 +664,15 @@ static void get_slice_parts (NODE *t, parser *p)
 	    t->v.b2.l = expr(p);
 	}
 	if (p->sym == P_COL) {
-	    /* need second part of colon-separated range */
+	    /* second part of colon-separated range */
 	    t->v.b2.l = newb2(SUBSL, t->v.b2.l, NULL);
 	    lex(p);
-	    t->v.b2.l->v.b2.r = expr(p);
+	    if (p->sym == P_COM || p->sym == G_RBR) {
+		/* reached end: second part implicitly empty */
+		t->v.b2.l->v.b2.r = newempty(EMPTY);
+	    } else {
+		t->v.b2.l->v.b2.r = expr(p);
+	    }
 	}
 	if (p->sym == G_RBR) {
 	    /* no comma, no second arg string: may be OK */
@@ -629,10 +690,15 @@ static void get_slice_parts (NODE *t, parser *p)
 		t->v.b2.r = expr(p);
 	    }
 	    if (p->sym == P_COL) {
-		/* need second part of colon-separated range */
+		/* second part of colon-separated range */
 		t->v.b2.r = newb2(SUBSL, t->v.b2.r, NULL);
 		lex(p);
-		t->v.b2.r->v.b2.r = expr(p);
+		if (p->sym == G_RBR) {
+		    /* reached end: second part implicitly empty */
+		    t->v.b2.r->v.b2.r = newempty(EMPTY);
+		} else {
+		    t->v.b2.r->v.b2.r = expr(p);
+		}
 	    }
 	    if (p->sym == G_RBR) {
 		if (p->ch == '\'') {
@@ -740,8 +806,12 @@ static void get_ovar_ref (NODE *t, parser *p)
 }
 
 static NODE *powterm (parser *p)
-{  
-    int opt = 0;
+{ 
+    /* watch out for unary operators */
+    int sym = p->sym == B_SUB ? U_NEG : 
+	p->sym == B_ADD ? U_POS :
+	p->sym;
+    int opt = OPT_NONE;
     NODE *t;
 
     if (p->err) {
@@ -753,20 +823,32 @@ static NODE *powterm (parser *p)
 	    p->sym, p->ch? p->ch : '0', p->ch);
 #endif
 
-    if (p->sym == F_RUNIFORM || p->sym == F_RNORMAL) {
+    if (sym == F_RUNIFORM || sym == F_RNORMAL) {
 	opt = BOTH_OPT;
-    } else if (p->sym == F_FDJAC || p->sym == F_BFGSMAX) {
+    } else if (sym == F_FDJAC || sym == F_BFGSMAX) {
 	opt = RIGHT_STR;
+    } else if (sym == F_DUMIFY) {
+	opt = RIGHT_OPT;
     }
 
-    if (func2_symb(p->sym)) {
-	t = newb2(p->sym, NULL, NULL);
+    if (unary_op(sym)) {
+	if (p->ch == 0) {
+	    context_error(0, p);
+	    return NULL;
+	}
+        t = newb1(sym, NULL);
+        if (t != NULL) {
+            lex(p);
+            t->v.b1.b = powterm(p);
+        }
+    } else if (func2_symb(sym)) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    get_args(t, p, opt);
 	}
-    } else if (string0_func(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (string0_func(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
@@ -775,33 +857,33 @@ static NODE *powterm (parser *p)
 		get_multi_args(t->v.b1.b, p);
 	    }
 	}	
-    } else if (string_arg_func(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (string_arg_func(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = get_string_arg(p);
 	}	
-    } else if (func1_symb(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (func1_symb(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = base(p, t);
 	}
-    } else if (p->sym == LAG || p->sym == OBS) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == LAG || sym == OBS) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, USERIES); 
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
-    } else if (func2_symb(p->sym)) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (func2_symb(sym)) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    get_args(t, p, opt);
 	}
-    } else if (p->sym == MSL || p->sym == DMSL) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == MSL || sym == DMSL) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    if (p->sym == MSL) {
 		t->v.b2.l = newstr(p, STR);
@@ -814,27 +896,39 @@ static NODE *powterm (parser *p)
 		get_slice_parts(t->v.b2.r, p);
 	    }
 	}
-    } else if (p->sym == DMSTR) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == DMSTR) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, MVAR);
 	    lex(p);
 	    t->v.b2.r = get_string_arg(p);
 	}
-    } else if (p->sym == OVAR) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == OVAR) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newstr(p, STR);
 	    get_ovar_ref(t, p);
 	}
-    } else if (p->sym == G_LPR) {
+    } else if (sym == LISTVAR) {
+	t = listvar_node(p);
+	if (t != NULL && (p->sym == G_LPR || p->sym == G_LBR)) {
+	    /* list.series node may be "inflected" as lag or obs */
+	    p->sym = (p->sym == G_LPR)? LAG : OBS;
+	    t = newb2(p->sym, t, NULL);
+	    if (t != NULL) {
+		parser_ungetc(p);
+		lex(p);
+		t->v.b2.r = base(p, t);
+	    }
+	}
+    } else if (sym == G_LPR) {
 	/* dummy root for parenthesized expressions, to facilitate
 	   taking the transpose of matrix stuff, e.g. (A*B)' */
 	t = newb1(EROOT, NULL);
 	if (t != NULL) {
 	    t->v.b1.b = base(p, t);
 	}
-    } else if (p->sym == G_LCB) {
+    } else if (sym == G_LCB) {
 	/* explicit matrix definition, possibly followed by
 	   a "subslice" specification */
 	int sub = 0;
@@ -853,8 +947,8 @@ static NODE *powterm (parser *p)
 		}		
 	    }
 	}
-    } else if (p->sym == UFUN) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == UFUN) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newstr(p, STR);
 	    lex(p);
@@ -863,8 +957,8 @@ static NODE *powterm (parser *p)
 		get_multi_args(t->v.b2.r, p);
 	    }
 	}
-    } else if (funcn_symb(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (funcn_symb(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
@@ -872,8 +966,8 @@ static NODE *powterm (parser *p)
 		get_multi_args(t->v.b1.b, p);
 	    }
 	}
-    } else if (p->sym == STR || p->sym == VSTR) {
-	t = newstr(p, p->sym);
+    } else if (sym == STR || sym == VSTR) {
+	t = newstr(p, sym);
 	lex(p);
     } else {
 	t = base(p, NULL);
@@ -890,7 +984,6 @@ static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
 	p->sym == B_ADD ? U_POS : 
-	p->sym == B_AND ? U_ADDR :
 	p->sym;
     NODE *t;
 
