@@ -239,6 +239,7 @@ static GptFlags get_gp_flags (gretlopt opt, int k, FitType *f)
 	    flags |= GPT_FIT_OMIT;
 	}
 	if (opt & OPT_T) {
+	    fprintf(stderr, "setting GPT_IDX\n");
 	    flags |= GPT_IDX;
 	}
     }
@@ -269,13 +270,6 @@ static void printvars (FILE *fp, int t, const int *list, const double **Z,
     double xt;
     int i;
 
-    if (x == NULL && list[0] == 2) {
-	/* skip missing obs for simple scatterplot */
-	if (na(Z[list[1]][t]) || na(Z[list[2]][t])) {
-	    return;
-	}
-    }
-
     if (x != NULL) {
 	xt = x[t] + offset;
 	fprintf(fp, "%.8g ", xt);
@@ -659,14 +653,14 @@ static const gretlRGB default_color[N_GP_COLORS] = {
     { 0xff, 0x00, 0x00 },
     { 0x00, 0x00, 0xff },
     { 0x00, 0xcc, 0x00 }, /* full-intensity green is not very legible */
-    { 0x9b, 0xa6, 0xbb }  /* color for box fill */
+    { 0x5f, 0x6b, 0x84 }  /* color for box fill */
 };
 
 static gretlRGB user_color[N_GP_COLORS] = {
     { 0xff, 0x00, 0x00 },
     { 0x00, 0x00, 0xff },
     { 0x00, 0xcc, 0x00 },
-    { 0x9b, 0xa6, 0xbb }
+    { 0x5f, 0x6b, 0x84 }
 };
 
 static void print_rgb_x (char *s, gretlRGB color)
@@ -1358,14 +1352,14 @@ loess_plot (gnuplot_info *gi, const double **Z, const DATAINFO *pdinfo)
 	goto bailout;
     }
 
-    s1 = var_get_graph_name(pdinfo, gi->list[1]);
-    s2 = var_get_graph_name(pdinfo, gi->list[2]);
+    s1 = var_get_graph_name(pdinfo, yno);
+    s2 = var_get_graph_name(pdinfo, xno);
 
     sprintf(title, G_("%s versus %s (with loess fit)"), s1, s2);
     fputs("set key top left\n", fp);
     fprintf(fp, "set title \"%s\"\n", title);
-    print_axis_label('x', s1, fp);
-    print_axis_label('y', s2, fp);
+    print_axis_label('y', s1, fp);
+    print_axis_label('x', s2, fp);
     print_auto_fit_string(PLOT_FIT_LOESS, fp);
 
     fputs("plot \\\n", fp);
@@ -1503,6 +1497,70 @@ static int get_fitted_line (gnuplot_info *gi,
     return err;
 }
 
+/* Find the minimum and maximum x-axis values and construct the gnuplot
+   x-range.  We have to be a bit careful here to include only values
+   that will actually display on the plot, i.e. x-values that are
+   accompanied by at least one non-missing y-axis value.
+*/
+
+static void 
+print_x_range_from_list (gnuplot_info *gi, const double **Z, const int *list)
+{
+    int vx, k, dk = -1;
+    const double *x;
+
+    if (gi->flags & GPT_DUMMY) {
+	dk = list[0];
+	k = dk - 1;
+    } else {
+	k = list[0];
+    }
+
+    vx = list[k];
+    x = Z[vx];
+    
+    if (gretl_isdummy(gi->t1, gi->t2, x)) {
+	fputs("set xrange [-1:2]\n", gi->fp);	
+	fputs("set xtics (\"0\" 0, \"1\" 1)\n", gi->fp);
+	gi->xrange = 3;
+    } else {
+	double xmin, xmin0 = NADBL;
+	double xmax = NADBL;
+	int t, i, vy, obs_ok;
+
+	for (t=gi->t1; t<=gi->t2; t++) {
+	    obs_ok = 0;
+	    if (!na(x[t]) && (dk < 0 || !na(Z[dk][t]))) {
+		for (i=1; i<k; i++) {
+		    vy = list[i];
+		    if (!na(Z[vy][t])) {
+			/* got x obs and at least one y obs */
+			obs_ok = 1;
+			break;
+		    }
+		}
+	    }
+	    if (obs_ok) {
+		if (na(xmin0) || x[t] < xmin0) {
+		    xmin0 = x[t];
+		}
+		if (na(xmax) || x[t] > xmax) {
+		    xmax = x[t];
+		}
+	    }
+	}
+		    
+	gi->xrange = xmax - xmin0;
+	xmin = xmin0 - gi->xrange * .025;
+	if (xmin0 >= 0.0 && xmin < 0.0) {
+	    xmin = 0.0;
+	}
+	xmax += gi->xrange * .025;
+	fprintf(gi->fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
+	gi->xrange = xmax - xmin;
+    }
+}
+
 static void 
 print_x_range (gnuplot_info *gi, const double *x)
 {
@@ -1627,6 +1685,19 @@ maybe_print_panel_jot (int t, const DATAINFO *pdinfo, FILE *fp)
     }
 }
 
+static int all_graph_data_missing (const int *list, int t, const double **Z)
+{
+    int i;
+
+    for (i=1; i<=list[0]; i++) {
+	if (!na(Z[list[i]][t])) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
 static void
 print_gp_data (gnuplot_info *gi, const double **Z, 
 	       const DATAINFO *pdinfo)
@@ -1661,6 +1732,10 @@ print_gp_data (gnuplot_info *gi, const double **Z,
 	    const char *label = NULL;
 	    char obs[OBSLEN];
 
+	    if (gi->x == NULL && all_graph_data_missing(gi->list, t, Z)) {
+		continue;
+	    }
+
 	    if (!(gi->flags & GPT_TS) && i == 1) {
 		if (pdinfo->markers) {
 		    label = pdinfo->S[t];
@@ -1688,9 +1763,11 @@ gpinfo_init (gnuplot_info *gi, gretlopt opt, const int *list,
     int l0 = list[0];
 
     gi->fit = PLOT_FIT_NONE;
-
     gi->flags = get_gp_flags(opt, l0, &gi->fit);
-    gi->flags |= GPT_TS; /* may be renounced later */
+
+    if (gi->fit == PLOT_FIT_NONE) {
+	gi->flags |= GPT_TS; /* may be renounced later */
+    }
 
     gi->t1 = t1;
     gi->t2 = t2;
@@ -2124,20 +2201,20 @@ int gnuplot (const int *plotlist, const char *literal,
     char fit_line[128] = {0};
     int oddman = 0;
     int many = 0;
-    int i, err = 0;
-
     gnuplot_info gi;
+    int i, err;
 
     gretl_error_clear();
 
+    err = incompatible_options(opt, OPT_T | OPT_I | OPT_L | OPT_Q | OPT_N);
+    if (err) {
+	return err;
+    }
+
 #if GP_DEBUG
     printlist(plotlist, "gnuplot: plotlist");
 #endif
 
-    /* below: did have "height 1 width 1 box" for win32,
-       "width 1 box" otherwise */
-    strcpy(keystr, "set key left top\n");
-
     err = gpinfo_init(&gi, opt, plotlist, literal, 
 		      pdinfo->t1, pdinfo->t2);
     if (err) {
@@ -2160,6 +2237,10 @@ int gnuplot (const int *plotlist, const char *literal,
     /* convenience pointer */
     list = gi.list;
 
+    /* below: did have "height 1 width 1 box" for win32,
+       "width 1 box" otherwise */
+    strcpy(keystr, "set key left top\n");
+
     if (gi.flags & GPT_IMPULSES) {
 	strcpy(withstr, "w i");
     }
@@ -2294,10 +2375,7 @@ int gnuplot (const int *plotlist, const char *literal,
     if (gi.x != NULL) {
 	print_x_range(&gi, gi.x);
     } else {
-	int k = list[0];
-	int v = (gi.flags & GPT_DUMMY)? list[k - 1] : list[k];
-
-	print_x_range(&gi, Z[v]);
+	print_x_range_from_list(&gi, Z, list);
     }
 
     if (gi.flags & GPT_Y2AXIS) { 
@@ -2485,7 +2563,16 @@ int multi_scatters (const int *list, const double **Z,
 
     if (obs != NULL) {
 	double startdate = obs[pdinfo->t1];
-	int jump = (pdinfo->t2 - pdinfo->t1 + 1) / (2 * pdinfo->pd);
+	double enddate = obs[pdinfo->t2];
+	int jump;
+
+	fprintf(fp, "set xrange [%g:%g]\n", floor(startdate), ceil(enddate));
+
+	if (pdinfo->pd == 1) {
+	    jump = (pdinfo->t2 - pdinfo->t1 + 1) / 6;
+	} else {
+	    jump = (pdinfo->t2 - pdinfo->t1 + 1) / (4 * pdinfo->pd);
+	}
 
 	fprintf(fp, "set xtics %g, %d\n", ceil(startdate), jump);
     } else {
