@@ -76,8 +76,6 @@ int gretl_compiling_function (void);
 
 int gretl_function_depth (void);
 
-int repeating_function_exec (void);
-
 int gretl_start_compiling_function (const char *line, PRN *prn);
 
 int gretl_function_append_line (const char *line);
@@ -91,7 +89,9 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 
 char *gretl_func_get_arg_name (const char *argvar, int *err);
 
-void set_drop_function_vars (int s);
+int object_is_const (const char *name);
+
+void sample_range_get_extrema (const DATAINFO *pdinfo, int *t1, int *t2);
 
 const char *get_funcerr_message (void);
 
@@ -111,8 +111,9 @@ int write_function_package (fnpkg *pkg,
 			    const char *version,
 			    const char *date,
 			    const char *descrip,
+			    char *sample,
 			    FuncDataReq dreq,
-			    float minver);
+			    int minver);
 
 int function_package_get_info (const char *fname,
 			       fnpkg **ppkg,
@@ -122,11 +123,12 @@ int function_package_get_info (const char *fname,
 			       char **version,
 			       char **date,
 			       char **descrip,
+			       char **sample,
 			       FuncDataReq *dreq,
-			       float *minver);
+			       int *minver);
 
 int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
-			  float minver);
+			  int minver);
 
 int write_user_function_file (const char *fname);
 
