@@ -100,7 +100,7 @@ int doing_nls (void)
     static int called, nls;
 
     if (!called) {
-	nls = (strcmp("/File/_Open data", _("/File/_Open data")) ||
+	nls = (strcmp("_Open data", _("_Open data")) ||
 	       strcmp("Test statistic", _("Test statistic")) ||
 	       strcmp("annual", _("annual")));
 	called = 1;
@@ -262,6 +262,7 @@ int iso_latin_version (void)
     if (gretl_cset_maj == 8859 &&
 	(gretl_cset_min == 1 || 
 	 gretl_cset_min == 2 ||
+	 gretl_cset_min == 5 ||
 	 gretl_cset_min == 9 ||
 	 gretl_cset_min == 15)) {
 	return gretl_cset_min;
@@ -272,16 +273,20 @@ int iso_latin_version (void)
 	return 1;
     } else if (gretl_cpage == 1250) {
 	return 2;
+    } else if (gretl_cpage == 1251) {
+	return 5;
     } else if (gretl_cpage == 1254) {
 	return 9;
     }
 # endif
 
-    /* Polish, Turkish: UTF-8 locale? */
+    /* Polish, Russian, Turkish: UTF-8 locale? */
     lang = getenv("LANG");
     if (lang != NULL) {
 	if (!strncmp(lang, "pl", 2)) {
 	    return 2;
+	} else if (!strncmp(lang, "ru", 2)) {
+	    return 5;
 	} else if (!strncmp(lang, "tr", 2)) {
 	    return 9;
 	}
@@ -597,27 +602,6 @@ char *iso_to_ascii (char *s)
     return real_iso_to_ascii(s, 1);
 }
 
-char *get_month_name (char *mname, int m)
-{
-    struct tm mt;
-
-    mt.tm_sec = 0;
-    mt.tm_min = 0;
-    mt.tm_hour = 0;
-    mt.tm_mday = 1;
-    mt.tm_mon = m - 1;
-    mt.tm_year = 100;
-
-    strftime(mname, 7, "%b", &mt);
-    *mname = tolower(*mname);
-
-#ifdef ENABLE_NLS
-    real_iso_to_ascii(mname, gretl_cset_min);
-#endif
-
-    return mname;
-}
-
 #ifdef ENABLE_NLS
 
 char *sprint_l2_to_ascii (char *targ, const char *s, size_t len)
@@ -648,6 +632,8 @@ static char *get_gp_encoding_set (char *s, int targ)
 	strcpy(s, "ISO-8859-");
 	if (latin == 2) {
 	    strcat(s, "2");
+	} else if (latin == 5) {
+	    strcat(s, "5");
 	} else if (latin == 9) {
 	    strcat(s, "9");
 	} else if (latin == 15) {
@@ -660,6 +646,8 @@ static char *get_gp_encoding_set (char *s, int targ)
 	strcpy(s, "CP125");
 	if (latin == 2) {
 	    strcat(s, "0");
+	} else if (latin == 5) {
+	    strcat(s, "1");
 	} else if (latin == 9) {
 	    strcat(s, "4");
 	} else {
