@@ -25,12 +25,10 @@
 
 #define LDEBUG 0
 
-#define LNAMELEN 32
-
 typedef struct saved_list_ saved_list;
 
 struct saved_list_ {
-    char name[LNAMELEN];
+    char name[VNAMELEN];
     int *list;
     int level;
 };
@@ -62,7 +60,7 @@ static saved_list *saved_list_new (const int *list, const char *name)
 	    sl = NULL;
 	} else {
 	    *sl->name = 0;
-	    strncat(sl->name, name, LNAMELEN - 1);
+	    strncat(sl->name, name, VNAMELEN - 1);
 	}
     }
 
@@ -79,17 +77,19 @@ static void free_saved_list (saved_list *sl)
 
 static saved_list *get_saved_list_by_name (const char *name)
 {
+    saved_list *sl = NULL;
     int fsd = gretl_function_depth();
     int i;
 
     for (i=0; i<n_lists; i++) {
 	if (!strcmp(name, list_stack[i]->name) && 
 	    fsd == list_stack[i]->level) {
-	    return list_stack[i];
+	    sl = list_stack[i];
+	    break;
 	}
     }
 
-    return NULL;
+    return sl;
 }
 
 /**
@@ -148,8 +148,8 @@ const char *get_list_name_by_index (int idx)
  * get_list_by_name:
  * @name: the name of the list to be found.
  *
- * Looks up @name in the stack of saved lists (if any) and
- * retrieves the associated list.
+ * Looks up @name in the stack of saved lists, at the current level
+ * of function execution, and retrieves the associated list.
  *
  * Returns: the list, or %NULL if the lookup fails. 
  */
@@ -322,7 +322,7 @@ static int real_remember_list (const int *list, const char *name,
 		}
 		if (strlen(realname) < strlen(name)) {
 		    pprintf(prn, _("Warning: the name was truncated to %d characters\n"), 
-			    LNAMELEN - 1);
+			    VNAMELEN - 1);
 		}
 	    }
 	    n_lists++;
@@ -425,7 +425,7 @@ int rename_saved_list (const char *orig, const char *new)
 	    err = destroy_saved_list(sl1);
 	}
 	*sl0->name = '\0';
-	strncat(sl0->name, new, LNAMELEN - 1);
+	strncat(sl0->name, new, VNAMELEN - 1);
     } 
 
     return err;
@@ -622,6 +622,11 @@ int gretl_lists_revise (const int *dlist, int dmin)
     int lmax = 0;
     int i, j, k;
 
+#if 0
+    fprintf(stderr, "gretl_lists_revise: dlist = %p, dmin = %d\n", 
+	    (void *) dlist, dmin);
+#endif
+
     if (dlist != NULL) {
 	/* determine lowest deleted ID */
 	dmin = dlist[1];
@@ -667,11 +672,11 @@ int gretl_lists_revise (const int *dlist, int dmin)
     /* use mapping to revise saved lists */
     for (j=0; j<n_lists; j++) {
 	list = list_stack[j]->list;
-	for (i=1; i<=list[0]; i++) {
+	for (i=list[0]; i>0; i--) {
 	    k = list[i] - dmin + 1;
 	    if (k >= 1) {
 		if (maplist[k] == -1) {
-		    gretl_list_delete_at_pos(list, i--);
+		    gretl_list_delete_at_pos(list, i);
 		} else {
 		    list[i] = maplist[k];
 		}
@@ -761,10 +766,9 @@ int *gretl_consecutive_list_new (int lmin, int lmax)
 	return NULL;
     }
 
-    list = malloc((n + 1) * sizeof *list);
+    list = gretl_list_new(n);
 
     if (list != NULL) {
-	list[0] = n;
 	for (i=0; i<n; i++) {
 	    list[i+1] = lmin + i;
 	}
@@ -2093,7 +2097,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     int *list = NULL;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+	if (!var_is_hidden(pdinfo, i) &&
 	    STACK_LEVEL(pdinfo, i) == fsd) {
 	    nv++;
 	}
@@ -2110,7 +2114,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     if (list != NULL) {
 	j = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+	    if (!var_is_hidden(pdinfo, i) &&
 		STACK_LEVEL(pdinfo, i) == fsd) {
 		list[j++] = i;
 	    }
@@ -2221,7 +2225,7 @@ int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
 		    *err = E_PARSE;
 		}
 	    } else {
-		v = varindex(pdinfo, test);
+		v = series_index(pdinfo, test);
 		if (v < pdinfo->v) {
 		    list = gretl_list_append_term(&list, v);
 		} else {
@@ -2300,7 +2304,7 @@ int load_user_lists_file (const char *fname)
 		if (!gretl_xml_get_prop_as_string(cur, "name", &lname)) {
 		    err = E_DATA;
 		} else {
-		    strncat(list_stack[i]->name, lname, LNAMELEN - 1);
+		    strncat(list_stack[i]->name, lname, VNAMELEN - 1);
 		    free(lname);
 		    list_stack[i]->list = 
 			gretl_xml_node_get_list(cur, doc, &err);
