@@ -870,6 +870,8 @@ within_groups_dataset (const double **Z, const DATAINFO *pdinfo,
 	}
     }
 
+    free(vlist);
+
     return winfo;
 }
 
@@ -2556,7 +2558,7 @@ add_dummies_to_list (const int *list, DATAINFO *pdinfo, int **pbiglist)
     j = list[0] + 1;
     for (i=2; i<=pdinfo->pd; i++) {
 	sprintf(dname, "dt_%d", i);
-	v = varindex(pdinfo, dname);
+	v = series_index(pdinfo, dname);
 	if (v == pdinfo->v) {
 	    err = E_DATA;
 	    break;
@@ -2761,10 +2763,12 @@ MODEL real_panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 	if (err) { 
 	    pputs(prn, _("Couldn't estimate group means regression\n"));
 	} else {
-	    random_effects(&pan, (const double **) *pZ, pdinfo, 
-			   (const double **) gZ, ginfo, prn);
-	    save_breusch_pagan_result(&pan);
-	    finalize_hausman_test(&pan, prn);
+	    err = random_effects(&pan, (const double **) *pZ, pdinfo, 
+				 (const double **) gZ, ginfo, prn);
+	    if (!err) {
+		save_breusch_pagan_result(&pan);
+		finalize_hausman_test(&pan, prn);
+	    }
 	}
 
 	if (ginfo != NULL) {
@@ -3473,9 +3477,6 @@ int switch_panel_orientation (double **Z, DATAINFO *pdinfo)
 
     /* copy the data series across in transformed order */
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
 	for (t=0; t<pdinfo->n; t++) {
 	    /* transcribe to tmp in original order */
 	    tmp[t] = Z[i][t];
@@ -3655,8 +3656,8 @@ static void sorter_fill_points (sorter *s, const double **Z,
 
 static int compare_obs (const void *a, const void *b)
 {
-    const s_point *pa = (const s_point *) a;
-    const s_point *pb = (const s_point *) b;
+    s_point *pa = (s_point *) a;
+    s_point *pb = (s_point *) b;
     int ret;
 
     ret = (pa->val1 > pb->val1) - (pa->val1 < pb->val1);
@@ -3664,6 +3665,11 @@ static int compare_obs (const void *a, const void *b)
 	ret = (pa->val2 > pb->val2) - (pa->val2 < pb->val2);
     }
 
+    if (ret == 0) {
+	/* mark an error: got a duplicated value */
+	pa->obsnum = pb->obsnum = -1;
+    }
+
     return ret;
 }
 
@@ -3704,6 +3710,22 @@ static int compose_panel_indices (DATAINFO *pdinfo, int T,
     return err;
 }
 
+static int check_indices (sorter *s, int n)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	if (s->points[i].obsnum < 0) {
+	    sprintf(gretl_errmsg, 
+		    _("Error: unit %g, period %g: duplicated observation"),
+		    s->points[i].val1, s->points[i].val2);
+	    return E_DATA;
+	}
+    }
+
+    return 0;
+}
+
 static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
 			       int uv, int tv, int *ustrs)
 {
@@ -3712,6 +3734,7 @@ static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
     double *tmp = NULL;
     int i, t;
     sorter s;
+    int err = 0;
 
     tmp = malloc(n * sizeof *tmp);
     if (tmp == NULL) {
@@ -3741,15 +3764,18 @@ static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
     qsort((void *) s.points, (size_t) n, sizeof s.points[0], 
 	  compare_obs);
 
+    err = check_indices(&s, n);
+    if (err) {
+	goto bailout;
+    }
+
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    for (t=0; t<n; t++) {
-		tmp[t] = Z[i][t];
-	    }
-	    for (t=0; t<n; t++) {
-		Z[i][t] = tmp[s.points[t].obsnum];
-	    }
-	}	
+	for (t=0; t<n; t++) {
+	    tmp[t] = Z[i][t];
+	}
+	for (t=0; t<n; t++) {
+	    Z[i][t] = tmp[s.points[t].obsnum];
+	}
     }
 
     if (S != NULL) {
@@ -3770,12 +3796,14 @@ static int panel_data_sort_by (double **Z, DATAINFO *pdinfo,
 	    }
 	}
 	free_strings_array(S, n);
-    }	
+    }
+
+ bailout:	
 
     free(s.points);
     free(tmp);
 
-    return 0;
+    return err;
 }
 
 /* Given the variables coding for panel unit and time period,
@@ -3833,28 +3861,19 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
     char **S = NULL;
     int *nuid = NULL;
     int *ntid = NULL;
-    int n_scalars = 0;
     int n_orig = pdinfo->n;
     int t2_orig = pdinfo->t2;
     int i, j, s, t;
     int err = 0;
 
-    for (i=1; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    n_scalars++;
-	}
-    }
-
     err = normalize_uid_tid(tid, nperiods, (const double **) Z,
 			    pdinfo, uv, tv, &nuid, &ntid);
 
     if (!err) {
-	/* allocate temporary storage, skipping any scalars */
-	pdinfo->v -= n_scalars;
+	/* allocate temporary storage */
 	pdinfo->n = nunits * nperiods;
 	pdinfo->t2 = pdinfo->n - 1;
 	err = allocate_Z(&bigZ, pdinfo);
-	pdinfo->v += n_scalars;
     }
 
     if (!err && pdinfo->S != NULL && ustrs) {
@@ -3873,9 +3892,7 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
 	    j = 1;
 	    s = nuid[t] * nperiods + ntid[t];
 	    for (i=1; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i)) {
-		    bigZ[j++][s] = Z[i][t];
-		}
+		bigZ[j++][s] = Z[i][t];
 	    }
 	    if (S != NULL) {
 		strcpy(S[s], pdinfo->S[t]);
@@ -3898,9 +3915,7 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
 	    } else if (i == tv) {
 		btv = j;
 	    }
-	    if (var_is_series(pdinfo, i)) {
-		j++;
-	    }
+	    j++;
 	}
 
 	/* complete the index info in slots uv and tv */
@@ -3916,10 +3931,8 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
 
 	/* swap the padded arrays into Z */
 	for (i=0, j=0; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i)) {
-		free(Z[i]);
-		Z[i] = bigZ[j++];
-	    }
+	    free(Z[i]);
+	    Z[i] = bigZ[j++];
 	}
 
 	free(bigZ);
@@ -3931,7 +3944,9 @@ static int pad_panel_dataset (const double *uid, int uv, int nunits,
     if (pdinfo->S != NULL) {
 	/* expand the obs (unit) marker strings appropriately */
 	if (S == NULL) {
-	    dataset_destroy_obs_markers(pdinfo);
+	    free_strings_array(pdinfo->S, n_orig);
+	    pdinfo->S = NULL;
+	    pdinfo->markers = NO_MARKERS;
 	} else {
 	    char si[OBSLEN];
 
@@ -3983,25 +3998,22 @@ static int uv_tv_from_line (const char *line, const DATAINFO *pdinfo,
 	return E_PARSE;
     }
 
-    *uv = varindex(pdinfo, uvname);
+    *uv = series_index(pdinfo, uvname);
     if (*uv == pdinfo->v) {
+	/* FIXME "not a series" */
 	sprintf(gretl_errmsg, _("Unknown variable '%s'"), uvname);
 	err = E_UNKVAR;
-    } else if (!var_is_series(pdinfo, *uv)) {
-	err = E_DATATYPE;
-    }
+    } 
 
     if (err) {
 	return err;
     }
 
-    *tv = varindex(pdinfo, tvname);
+    *tv = series_index(pdinfo, tvname);
     if (*tv == pdinfo->v) {
 	sprintf(gretl_errmsg, _("Unknown variable '%s'"), tvname);
 	err = E_UNKVAR;
-    } else if (!var_is_series(pdinfo, *tv)) {
-	err = E_DATATYPE;
-    }
+    } 
 
     return err;
 }
@@ -4166,8 +4178,8 @@ static int find_time_var (const DATAINFO *pdinfo)
     int i, v;
 
     for (i=0; tnames[i] != NULL; i++) {
-	v = varindex(pdinfo, tnames[i]);
-	if (v < pdinfo->v && var_is_series(pdinfo, v)) {
+	v = series_index(pdinfo, tnames[i]);
+	if (v < pdinfo->v) {
 	    return v;
 	}
     }
