@@ -23,6 +23,7 @@
 #include "libset.h"
 #include "gretl_string_table.h"
 #include "gretl_www.h"
+#include "texprint.h"
 
 #include <unistd.h>
 
@@ -30,6 +31,7 @@
 # include <windows.h>
 #else
 # include <sys/stat.h>
+# include <sys/types.h>
 # include <dirent.h>
 # include <errno.h>
 #endif
@@ -65,7 +67,7 @@ static int add_suffix (char *fname, const char *sfx)
 /* Heuristic: filename contains non-ascii characters, and
    validates as UTF-8 */
 
-static int fname_is_utf8 (const unsigned char *s)
+int fname_is_utf8 (const unsigned char *s)
 {
     const unsigned char *p = s;
     int sevenbit = 1;
@@ -100,6 +102,18 @@ void set_fopen_use_utf8 (void)
     fopen_use_utf8 = 1;
 }
 
+/**
+ * get_fopen_use_utf8:
+ *
+ * Returns: 1 if fienames should be in UTF-8 when passed to the C
+ * library's fopen(), otherwise 0.
+ */
+
+int get_fopen_use_utf8 (void)
+{
+    return fopen_use_utf8;
+}
+
 /**
  * gretl_fopen:
  * @fname: name of file to be opened.
@@ -1392,6 +1406,7 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	gretl_paths.plotfile[0] = '\0';
 	strcpy(ppaths->pngfont, "verdana 8");
     } else {
+	/* not defaults: after reading from registry */
 	ensure_slash(ppaths->gretldir);
 	if (*ppaths->dotdir == '\0') {
 	    correct_blank_dotdir(ppaths);
@@ -1403,11 +1418,9 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	if (strcmp(ppaths->dotdir, ppaths->workdir)) { 
 	    err += validate_writedir(ppaths->workdir);
 	}
-#if defined(HAVE_X12A) || defined(HAVE_TRAMO)
 	if (!err) {
 	    err = set_tramo_x12a_dirs(ppaths, err);
 	}
-#endif
     }
 
     if (opt & OPT_X) {
@@ -1428,6 +1441,8 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	strcpy(ppaths->cli_helpfile, ppaths->helpfile);
     }
 
+    sprintf(ppaths->gnuplot, "%swgnuplot.exe", ppaths->gretldir);
+
     sprintf(envstr, "GTKSOURCEVIEW_LANGUAGE_DIR=%sshare\\gtksourceview-1.0"
 	    "\\language-specs", ppaths->gretldir);
     putenv(envstr);
@@ -1441,6 +1456,8 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 	shelldir_init(ppaths->workdir);
     }
 
+    set_gretl_tex_preamble();
+
     return err;
 }
 
@@ -1449,22 +1466,59 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
 static void check_gretldir (PATHS *ppaths)
 {
     char *epath = getenv("GRETL_HOME");
+    char buf[FILENAME_MAX];
+    FILE *fp;
+    int gotit = 0;
 
     ensure_slash(ppaths->gretldir);
 
     if (epath != NULL && strcmp(epath, ppaths->gretldir)) {
-	/* which one is right? */
-	char test[FILENAME_MAX];
-	FILE *fp;
-
-	sprintf(test, "%sCOPYING", epath);
-	fp = gretl_fopen(test, "r");
+	/* environment vs rc file: is the env version OK? */
+	sprintf(buf, "%sCOPYING", epath);
+	fp = gretl_fopen(buf, "r");
 	if (fp != NULL) {
+	    fclose(fp);
 	    *ppaths->gretldir = '\0';
 	    strncat(ppaths->gretldir, epath, MAXLEN - 2);
 	    ensure_slash(ppaths->gretldir);
-	    fclose(fp);
+	    gotit = 1;
 	}
+    } else {
+	/* no env setting: check what the rc file says */
+	sprintf(buf, "%sCOPYING", ppaths->gretldir);
+	fp = gretl_fopen(buf, "r");
+	if (fp != NULL) {
+	    fclose(fp);
+	    gotit = 1;
+	}
+    }	
+
+    if (!gotit) {
+	/* we're messed up; try to recover */
+	gchar *proc_exe;
+	const char *s;
+	pid_t pid;
+	ssize_t nr;
+
+	pid = getpid();
+	proc_exe = g_strdup_printf("/proc/%d/exe", pid);
+	nr = readlink(proc_exe, buf, FILENAME_MAX - 1);
+	if (nr > 0) {
+	    buf[nr] = '\0';
+	    fprintf(stderr, "gretl is process %d, '%s'\n", (int) pid, buf);
+	    /* should be something like /foo/bar/bin/gretl; we
+	       want the /foo/bar bit to append to
+	    */
+	    s = strstr(buf, "bin/gretl");
+	    if (s != NULL) {
+		*ppaths->gretldir = '\0';
+		strncat(ppaths->gretldir, buf, s - buf);
+		strcat(ppaths->gretldir, "share/gretl/");
+		fprintf(stderr, "gretldir is really '%s'?\n", 
+			ppaths->gretldir);
+	    }
+	}
+	g_free(proc_exe);
     }
 }
 
@@ -1576,6 +1630,8 @@ int gretl_set_paths (PATHS *ppaths, gretlopt opt)
     }
 #endif
 
+    set_gretl_tex_preamble();
+
     return err;
 }
 
@@ -1754,9 +1810,6 @@ int cli_read_rc (PATHS *paths)
 	    } else if (!strcmp(key, "usecwd")) {
 		usecwd = rc_bool(val);
 		libset_set_bool(USE_CWD, usecwd);
-	    } else if (!strcmp(key, "gnuplot")) {
-		*paths->gnuplot = '\0';
-		strncat(paths->gnuplot, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "binbase")) {
 		*paths->binbase = '\0';
 		strncat(paths->binbase, val, MAXLEN - 1);
