@@ -29,7 +29,7 @@ struct PRN_ {
     char *buf;         /* buffer to which to print, or NULL */
     size_t bufsize;    /* allocated size of buffer */
     size_t blen;       /* string length of buffer */
-    int savepos;       /* saved poistion in stream or buffer */
+    int savepos;       /* saved position in stream or buffer */
     PrnFormat format;  /* plain, TeX, RTF */
     int fixed;         /* = 1 for fixed-size buffer */
     char delim;        /* CSV field delimiter */
@@ -340,6 +340,83 @@ const char *gretl_print_get_buffer (PRN *prn)
     return (prn != NULL)? prn->buf : NULL;
 }
 
+/**
+ * gretl_print_get_trimmed_buffer:
+ * @prn: printing struct.
+ * 
+ * Obtain a pointer to the buffer associated
+ * with @prn, if any.  This pointer must not be
+ * modified in any way.  An opening newline
+ * is skipped.
+ *
+ * Returns: the buffer, or %NULL on failure.
+ */
+
+const char *gretl_print_get_trimmed_buffer (PRN *prn)
+{
+    char *buf = (prn != NULL)? prn->buf : NULL;
+
+    if (buf != NULL) {
+	int i, n;
+
+	if (*buf == '\n') {
+	    buf++;
+	}
+	n = strlen(buf);
+	for (i=n-1; i>0; i--) {
+	    if (buf[i] == '\n' && buf[i-1] == '\n') {
+		buf[i] = '\0';
+	    } else {
+		break;
+	    }
+	}
+    }
+
+    return buf;
+}
+
+/**
+ * gretl_print_get_size:
+ * @prn: printing struct.
+ * @width: location to receive width, or %NULL.
+ * @height: location to receive height, or %NULL.
+ * 
+ * If @prn has a non-null buffer attached, provide
+ * the width and/or height of the buffer, the width in
+ * characters and the height in lines.
+ */
+
+void gretl_print_get_size (PRN *prn, int *width, int *height)
+{
+    int w = 0, h = 0;
+
+    if (prn != NULL && prn->buf != NULL) {
+	const char *s = prn->buf;
+	int lw = 0;
+
+	while (*s) {
+	    if (*s == '\n') {
+		h++;
+		if (lw > w) {
+		    w = lw;
+		}
+		lw = 0;
+	    } else {
+		lw++;
+	    }
+	    s++;
+	}
+    }
+
+    if (width != NULL) {
+	*width = w;
+    }
+
+    if (height != NULL) {
+	*height = h;
+    }    
+}
+
 /**
  * gretl_print_steal_buffer:
  * @prn: printing struct.
@@ -515,7 +592,7 @@ void gretl_print_set_delim (PRN *prn, char delim)
 }
 
 /**
- * gretl_print_unset_doc_format:
+ * gretl_print_toggle_doc_flag:
  * @prn: printing struct.
  * 
  * Toggles the %GRETL_FORMAT_DOC flag on @prn.
@@ -528,6 +605,36 @@ void gretl_print_toggle_doc_flag (PRN *prn)
     }
 }
 
+/**
+ * gretl_print_set_utf_flag:
+ * @prn: printing struct.
+ * 
+ * Sets the %GRETL_FORMAT_UTF flag on @prn.
+ */
+
+void gretl_print_set_utf_flag (PRN *prn)
+{
+    if (prn != NULL) {
+	prn->format |= GRETL_FORMAT_UTF;
+    }
+}
+
+/**
+ * gretl_print_supports_utf:
+ * @prn: printing struct.
+ * 
+ * Returns: 1 if @prn supports UTF-8 characters, else 0.
+ */
+
+int gretl_print_supports_utf (PRN *prn)
+{
+    if (prn != NULL) {
+	return (prn->format & GRETL_FORMAT_UTF)? 1 : 0;
+    } else {
+	return 0;
+    }
+}
+
 static int realloc_prn_buffer (PRN *prn)
 {
     size_t newlen;
@@ -882,7 +989,7 @@ int print_end_redirection (PRN *prn)
 
 int plain_format (PRN *prn)
 {
-    return (prn != NULL && prn->format == GRETL_FORMAT_TXT);
+    return (prn != NULL && (prn->format & GRETL_FORMAT_TXT));
 }
 
 /**
