@@ -39,7 +39,8 @@ typedef enum {
     GRETL_FORMAT_CSV       = 1 << 7,
     GRETL_FORMAT_TAB       = 1 << 8,
     GRETL_FORMAT_MODELTAB  = 1 << 9,
-    GRETL_FORMAT_LANDSCAPE = 1 << 10
+    GRETL_FORMAT_LANDSCAPE = 1 << 10,
+    GRETL_FORMAT_UTF       = 1 << 11
 } PrnFormat;
 
 /* functions follow */
@@ -60,8 +61,12 @@ int gretl_print_reset_buffer (PRN *prn);
 
 const char *gretl_print_get_buffer (PRN *prn);
 
+const char *gretl_print_get_trimmed_buffer (PRN *prn);
+
 char *gretl_print_steal_buffer (PRN *prn);
 
+void gretl_print_get_size (PRN *prn, int *width, int *height);
+
 FILE *gretl_print_read_tempfile (PRN *prn);
 
 int gretl_print_stop_tempfile_read (PRN *prn, FILE *fp);
@@ -76,6 +81,10 @@ void gretl_print_set_format (PRN *prn, PrnFormat format);
 
 void gretl_print_toggle_doc_flag (PRN *prn);
 
+void gretl_print_set_utf_flag (PRN *prn);
+
+int gretl_print_supports_utf (PRN *prn);
+
 void gretl_print_set_delim (PRN *prn, char delim);
 
 int pprintf (PRN *prn, const char *template, ...);
