@@ -1658,41 +1658,49 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
     return rset;
 }
 
-static int 
-print_restricted_coeff (const MODEL *pmod, int i,
-			double coeff, double sderr, int k,
-			const DATAINFO *pdinfo, 
-			PRN *prn)
+static int print_restricted_estimates (MODEL *pmod,
+				       const DATAINFO *pdinfo,
+				       gretl_matrix *vb,
+				       gretl_matrix *S,
+				       double s2,
+				       int nc, int rk, 
+				       PRN *prn)
 {
-    model_coeff mc;
-    int gotnan = 0;
+    const double *b = vb->val;
+    char **names;
+    double v, *se;
+    int i;
 
-    model_coeff_init(&mc);
-
-    if (xna(coeff)) {
-	gotnan = 1;
+    names = strings_array_new_with_length(nc, VNAMELEN);
+    if (names == NULL) {
+	return E_ALLOC;
     }
 
-    mc.b = coeff;
-    mc.se = sderr;
-
-    if (!xna(coeff) && !xna(sderr) && sderr > 0) {
-	mc.tval = coeff / sderr;
-	mc.pval = coeff_pval(pmod->ci, mc.tval, pmod->dfd + k);
+    se = malloc(nc * sizeof *se);
+    if (se == NULL) {
+	free_strings_array(names, nc);
+	return E_ALLOC;
     }
 
-    gretl_model_get_param_name(pmod, pdinfo, i, mc.name);
+    pprintf(prn, "%s:\n\n", _("Restricted estimates"));
 
-    print_coeff(&mc, prn);
+    for (i=0; i<nc; i++) {
+	v = gretl_matrix_get(S, i, i);
+	se[i] = (v > 1.0e-16)? sqrt(v) : 0.0;
+	gretl_model_get_param_name(pmod, pdinfo, i, names[i]);
+    }
 
-    return gotnan;
-}
+    plain_print_aux_coeffs(b, se, (const char **) names, nc, pmod->dfd + rk, 
+			   pmod->ci, prn);
 
-static void coeff_header (const MODEL *pmod, PRN *prn)
-{
-    int use_param = pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM;
+    pputc(prn, '\n');
+    pprintf(prn, "  %s = %.*g\n", _("Standard error of the regression"), 
+	    GRETL_DIGITS, sqrt(s2));
+    
+    free_strings_array(names, nc);
+    free(se);
 
-    print_coeff_heading(use_param, prn);
+    return 0;
 }
 
 /* generate full restricted estimates: this function is used
@@ -1758,20 +1766,9 @@ do_restricted_estimates (gretl_restriction *rset,
 				      b, S, NULL, &s2);
 
     if (!err) {
-	double v, coeff, se;
-
-	pprintf(prn, "%s:\n\n", _("Restricted estimates"));
-	coeff_header(pmod, prn);
-	for (i=0; i<k; i++) {
-	    coeff = gretl_vector_get(b, i);
-	    v = gretl_matrix_get(S, i, i);
-	    se = (v > 1.0e-16)? sqrt(v) : 0.0;
-	    print_restricted_coeff(pmod, i, coeff, se, rset->k, pdinfo, prn);
-	}
-	pputc(prn, '\n');
-	pprintf(prn, "  %s = %.*g\n", _("Standard error of residuals"), 
-		GRETL_DIGITS, sqrt(s2));
-    } 
+	print_restricted_estimates(pmod, pdinfo, b, S, s2,
+				   k, rset->k, prn);
+    }
 
  bailout:
     
