@@ -26,7 +26,7 @@
 #include "usermat.h"
 #include "gretl_panel.h"
 #include "loop_private.h"
-#include "texprint.h"
+#include "gretl_scalar.h"
 #include "gretl_string_table.h"
 #include "forecast.h"
 
@@ -121,11 +121,6 @@ int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo)
     int t, m = 0;
     double xx;
 
-    if (var_is_scalar(pdinfo, i)) {
-	px[0] = Z[i][0];
-	return 1;
-    }
-    
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	xx = Z[i][t];
 	if (na(xx)) {
@@ -417,7 +412,7 @@ int gretl_is_oprobit_ok (int t1, int t2, const double *x)
  * @pdinfo: dataset information. 
  * 
  * Check whether variable Z[v] equals 1 over the sample
- * range given in @pdinfo, (aside from any missing values).
+ * range given in @pdinfo (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
  */
@@ -843,6 +838,7 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 int set_obs (const char *line, double **Z, DATAINFO *pdinfo, 
 	     gretlopt opt)
 {
+    char pdstr[VNAMELEN];
     char stobs[OBSLEN];
     int structure = STRUCTURE_UNKNOWN;
     double sd0 = pdinfo->sd0;
@@ -867,11 +863,16 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 
     /* now we get down to business */
 
-    if (sscanf(line, "%*s %d %10s", &pd, stobs) != 2) {
+    if (sscanf(line, "%*s %15s %10s", pdstr, stobs) != 2) {
 	strcpy(gretl_errmsg, _("Failed to parse line as frequency, startobs"));
 	return 1;
     }
 
+    pd = gretl_int_from_string(pdstr, &err);
+    if (err) {
+	return err;
+    }
+
     /* truncate stobs if not a calendar date */
     if (strchr(stobs, '/') != NULL) {
 	dated = 1;
@@ -993,11 +994,56 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * gretl_double_from_string:
+ * @s: string to examine.
+ * @err: location to receive error code.
+ * 
+ * If @s is a valid string representation of a double,
+ * return that integer, otherwise if @s is the name of a
+ * scalar variable, return the value of that variable,
+ * otherwise set the content of @err to a non-zero value.
+ *
+ * Returns: double value.
+ */
+
+double gretl_double_from_string (const char *s, int *err)
+{
+    char *test;
+    double x;
+
+    if (s == NULL || *s == 0) {
+	*err = E_DATA;
+	return NADBL;
+    }
+
+    errno = 0;
+
+    x = strtod(s, &test);
+
+    if (errno == ERANGE) {
+	*err = E_DATA;
+	errno = 0;
+	return NADBL;
+    }
+
+    if (*test == '\0') {
+	return x;
+    }
+
+    if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
+    } else {
+	*err = E_DATA;
+	x = NADBL;
+    } 
+
+    return x;    
+}
+
 /**
  * gretl_int_from_string:
  * @s: string to examine.
- * @Z: data array.
- * @pdinfo: data information struct.
  * @err: location to receive error code.
  * 
  * If @s is a valid string representation of an integer,
@@ -1008,10 +1054,10 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
  * Returns: integer value.
  */
 
-int gretl_int_from_string (const char *s, const double **Z, 
-			   const DATAINFO *pdinfo, int *err)
+int gretl_int_from_string (const char *s, int *err)
 {
     char *test;
+    double x;
     int n = 0;
 
     if (s == NULL || *s == 0) {
@@ -1031,27 +1077,18 @@ int gretl_int_from_string (const char *s, const double **Z,
 
     if (*test == '\0') {
 	return n;
-    } else if (Z == NULL || pdinfo == NULL) {
-	*err = E_DATA;
-    } else if (test[1] == '\0' && is_active_index_loop_char(*test)) {
+    } else if (s[1] == '\0' && is_active_index_loop_char(*s)) {
 	n = loop_scalar_read(*s);
-    } else {
-	int v = varindex(pdinfo, s);
-	double x;
-
-	if (v >= pdinfo->v) {
-	    *err = E_UNKVAR;
-	} else if (var_is_series(pdinfo, v)) {
-	    *err = E_TYPES;
+    } else if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
+	if (na(x)) {
+	    *err = E_MISSDATA;
 	} else {
-	    x = Z[v][0];
-	    if (na(x)) {
-		*err = E_MISSDATA;
-	    } else {
-		n = (int) x;
-	    }
-	} 
-    }
+	    n = (int) x;
+	}	
+    } else {
+	*err = E_DATA;
+    } 
 
     return n;    
 }
@@ -1104,35 +1141,30 @@ int varnum_from_string (const char *str, DATAINFO *pdinfo)
 }
 
 /**
- * rename_var_by_id:
- * @idstr: string representation of the ID number of the
- * variable to be renamed.
- * @vname: new name to give the variable.
+ * gretl_type_from_name:
+ * @s: the name to check.
  * @pdinfo: dataset information.
  * 
- * Returns: 0 on sucess, %E_DATA on error.
+ * Returns: non-zero if @s is the name of an existing
+ * series, matrix, scalar, list or string variable,
+ * otherwise 0.
  */
 
-int rename_var_by_id (const char *idstr, const char *vname, 
-		      DATAINFO *pdinfo)
+GretlType gretl_type_from_name (const char *s, const DATAINFO *pdinfo)
 {
-    int v = varnum_from_string(idstr, pdinfo);
-
-    if (v < 0) {
-	return E_DATA;
+    if (gretl_is_series(s, pdinfo)) {
+	return GRETL_TYPE_SERIES;
+    } else if (get_matrix_by_name(s) != NULL) {
+	return GRETL_TYPE_MATRIX;
+    } else if (gretl_is_scalar(s)) {
+	return GRETL_TYPE_DOUBLE;
+    } else if (get_list_by_name(s)) {
+	return GRETL_TYPE_LIST;
+    } else if (get_string_by_name(s)) {
+	return GRETL_TYPE_STRING;
+    } else {
+	return GRETL_TYPE_NONE;
     }
-
-    if (var_is_const(pdinfo, v)) {
-	return overwrite_err(pdinfo, v);
-    }
-
-    /* should be pre-checked for validity of varname and
-       non-duplication (see interact.c under RENAME)
-    */
-
-    strcpy(pdinfo->varname[v], vname);
-
-    return 0;
 }
 
 /**
@@ -1202,8 +1234,11 @@ double **doubles_array_new (int m, int n)
     double **X;
     int i;
 
-    X = malloc(m * sizeof *X);
+    if (m == 0) {
+	return NULL;
+    }
 
+    X = malloc(m * sizeof *X);
     if (X == NULL) {
 	return X;
     }
@@ -1212,12 +1247,14 @@ double **doubles_array_new (int m, int n)
 	X[i] = NULL;
     }
 
-    for (i=0; i<m; i++) {
-	X[i] = malloc(n * sizeof **X);
-	if (X[i] == NULL) {
-	    doubles_array_free(X, m);
-	    X = NULL;
-	    break;
+    if (n > 0) {
+	for (i=0; i<m; i++) {
+	    X[i] = malloc(n * sizeof **X);
+	    if (X[i] == NULL) {
+		doubles_array_free(X, m);
+		X = NULL;
+		break;
+	    }
 	}
     }
 
@@ -1369,11 +1406,7 @@ int re_estimate (char *model_spec, MODEL *tmpmod,
 
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 {
-    if (var_is_series(pdinfo, i)) {
-	return Z[i][pdinfo->t1];
-    } else {
-	return Z[i][0];
-    }	
+    return Z[i][pdinfo->t1];
 }
 
 #ifndef WIN32
@@ -1481,6 +1514,23 @@ int gretl_copy_file (const char *src, const char *dest)
     return 0;
 }  
 
+int gretl_delete_var_by_name (const char *s, PRN *prn)
+{
+    int err;
+
+    if (gretl_is_scalar(s)) {
+	err = gretl_scalar_delete(s, prn);
+    } else if (get_matrix_by_name(s)) {
+	err = user_matrix_destroy_by_name(s, prn);
+    } else if (get_string_by_name(s)) {
+	err = delete_saved_string(s, prn);
+    } else {
+	err = E_UNKVAR;
+    }
+
+    return err;
+}
+
 /* timer */  
 
 static clock_t tim0;
@@ -1507,7 +1557,6 @@ void libgretl_init (void)
     libset_init();
     gretl_rand_init();
     gretl_stopwatch_init();
-    set_gretl_tex_preamble(); 
 }
 
 void libgretl_session_cleanup (int mode)
@@ -1518,6 +1567,7 @@ void libgretl_session_cleanup (int mode)
     gretl_plotx(NULL);
 
     if (mode != SESSION_PRESERVE_MATRICES) {
+	destroy_user_scalars();
 	destroy_user_matrices();
     }
 }
