@@ -88,15 +88,15 @@ int gretl_compare_ints (const void *a, const void *b);
 
 void printlist (const int *list, const char *msg);
 
-int gretl_int_from_string (const char *s, const double **Z, 
-			   const DATAINFO *pdinfo, int *err);
+double gretl_double_from_string (const char *s, int *err);
+
+int gretl_int_from_string (const char *s, int *err);
 
 int positive_int_from_string (const char *s);
 
 int varnum_from_string (const char *str, DATAINFO *pdinfo);
 
-int rename_var_by_id (const char *idstr, const char *vname, 
-		      DATAINFO *pdinfo);
+GretlType gretl_type_from_name (const char *s, const DATAINFO *pdinfo);
 
 int re_estimate (char *model_spec, MODEL *tmpmod, 
 		 double ***pZ, DATAINFO *pdinfo);
@@ -118,6 +118,8 @@ double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo);
 
 int gretl_copy_file (const char *src, const char *dest);
 
+int gretl_delete_var_by_name (const char *s, PRN *prn);
+
 #ifndef WIN32
 int gretl_spawn (char *cmdline);
 #endif
