@@ -26,6 +26,11 @@
 #include <dirent.h>
 #include <windows.h>
 
+enum {
+    TO_BACKSLASH,
+    FROM_BACKSLASH
+};
+
 DIR *win32_opendir (const char *dname);
 
 int read_reg_val (HKEY tree, const char *base,
@@ -49,6 +54,8 @@ int gretl_spawn (char *cmdline);
 
 int gretl_shell (const char *arg, PRN *prn);
 
+char *slash_convert (char *str, int which);
+
 char *desktop_path (void);
 
 char *appdata_path (void);
@@ -61,6 +68,8 @@ int win32_rename (const char *oldpath, const char *newpath);
 
 int win32_delete_dir (const char *path);
 
+char *R_path_from_registry (void);
+
 #endif /* WIN32 */
 
 #endif /* GRETL_WIN32_H */
