@@ -22,15 +22,17 @@
 #include "gretl_panel.h"
 #include "gretl_func.h"
 #include "usermat.h"
+#include "gretl_scalar.h"
 #include "dbread.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 
 #undef XML_DEBUG
 
-#define GRETLDATA_VERSION "1.1"
+#define GRETLDATA_VERSION "1.2"
 
 #ifdef WIN32
 # include <glib.h>
@@ -409,8 +411,9 @@ void gretl_xml_put_matrix (const gretl_matrix *m, const char *name,
 	fprintf(fp, "<gretl-matrix rows=\"%d\" cols=\"%d\">\n", 
 		m->rows, m->cols);
     } else {
-	fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\">\n", 
-		name, m->rows, m->cols);
+	fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\" "
+		"t1=\"%d\" t2=\"%d\">\n", 
+		name, m->rows, m->cols, m->t1, m->t2);
     }
 
     for (i=0; i<m->rows; i++) {
@@ -1021,6 +1024,7 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
     const char *p;
     double x;
     int rows, cols;
+    int t1 = 0, t2 = 0;
     int i, j;
 
     tmp = xmlGetProp(node, (XUC) "rows");
@@ -1056,6 +1060,18 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 	return NULL;
     }
 
+    tmp = xmlGetProp(node, (XUC) "t1");
+    if (tmp != NULL) {
+	t1 = atoi((char *) tmp);
+	free(tmp);
+    }
+
+    tmp = xmlGetProp(node, (XUC) "t2");
+    if (tmp != NULL) {
+	t2 = atoi((char *) tmp);
+	free(tmp);
+    }
+
     if (colnames != NULL) {
 	*colnames = (char *) xmlGetProp(node, (XUC) "colnames");
     }
@@ -1094,6 +1110,9 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
     if (*err) {
 	gretl_matrix_free(m);
 	m = NULL;
+    } else {
+	m->t1 = t1;
+	m->t2 = t2;
     }
 
     return m;
@@ -1401,8 +1420,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     pmax = malloc(nvars * sizeof *pmax);
     if (pmax == NULL) {
-	sprintf(gretl_errmsg, _("Out of memory"));
-	err = 1;
+	err = E_ALLOC;
 	goto cleanup;
     } 
 
@@ -1427,11 +1445,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     for (i=1; i<=nvars; i++) {
 	v = savenum(list, i);
-	if (var_is_series(pdinfo, v)) {
-	    pmax[i-1] = get_precision(&Z[v][pdinfo->t1], tsamp, 10);
-	} else {
-	    pmax[i-1] = GRETL_SCALAR_DIGITS;
-	}
+	pmax[i-1] = get_precision(&Z[v][pdinfo->t1], tsamp, 10);
     }
 
     ntodate_full(startdate, pdinfo->t1, pdinfo);
@@ -1507,17 +1521,6 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    fprintf(fp, "<variable name=\"%s\"", xmlbuf);
 	}
 
-	if (var_is_scalar(pdinfo, v) && !na(Z[v][0])) {
-	    if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
-		sprintf(numstr, "\n role=\"scalar\" value=\"%.12g\"",
-			Z[v][0]);
-	    } else {
-		sprintf(numstr, "\n role=\"scalar\" value=\"%.*f\"",
-			pmax[i-1], Z[v][0]);
-	    }
-	    alt_puts(numstr, fp, fz);
-	}
-
 	if (*VARLABEL(pdinfo, v)) {
 	    uerr = gretl_xml_encode_to_buf(xmlbuf, VARLABEL(pdinfo, v), sizeof xmlbuf);
 	    if (!uerr) {
@@ -1630,9 +1633,6 @@ int gretl_write_gdt (const char *fname, const int *list,
 	alt_puts(">", fp, fz);
 	for (i=1; i<=nvars; i++) {
 	    v = savenum(list, i);
-	    if (var_is_scalar(pdinfo, v)) {
-		continue;
-	    }
 	    if (na(Z[v][t])) {
 		strcpy(numstr, "NA ");
 	    } else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
@@ -1691,14 +1691,12 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	    err = 1;
 	}
 	if (!err && dataset_allocate_varnames(pdinfo)) {
-	    sprintf(gretl_errmsg, _("Out of memory reading data file"));
-	    err = 1;
+	    err = E_ALLOC;
 	}
 	if (!err) {
 	    *pZ = malloc(pdinfo->v * sizeof **pZ);
 	    if (*pZ == NULL) {
-		sprintf(gretl_errmsg, _("Out of memory reading data file"));
-		err = 1;
+		err = E_ALLOC;
 	    }
 	}		
 	free(tmp);
@@ -1755,7 +1753,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "lag");
 	    if (tmp != NULL) {
-		pdinfo->varinfo[i]->transform = atoi((char *) tmp); 
+		pdinfo->varinfo[i]->lag = atoi((char *) tmp); 
 		free(tmp);
 	    }
 
@@ -1774,6 +1772,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "role");
 	    if (tmp != NULL) {
+#if 0 /* FIXME old datafiles? */
 		if (!strcmp((char *) tmp, "scalar")) {
 		    char *val = (char *) xmlGetProp(cur, (XUC) "value");
 		    
@@ -1786,6 +1785,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 			set_var_scalar(pdinfo, i, 1);
 		    }
 		}
+#endif
 		free(tmp);
 	    }
 	    i++;
@@ -1810,9 +1810,6 @@ static int process_values (double **Z, DATAINFO *pdinfo, int t, char *s)
     gretl_error_clear();
 
     for (i=1; i<pdinfo->v && !err; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
 	s = strpbrk(s, "01234567890+-NA");
 	if (s == NULL) {
 	    fprintf(stderr, "i = %d: s == NULL in process_values()\n", i);
@@ -1856,10 +1853,11 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     int n, i, t;
     void *handle;
     int (*show_progress) (long, long, int) = NULL;
+    int err = 0;
 
     tmp = xmlGetProp(node, (XUC) "count");
     if (tmp == NULL) {
-	return 1;
+	return E_DATA;
     } 
 
     if (sscanf((char *) tmp, "%d", &n) == 1) {
@@ -1868,7 +1866,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     } else {
 	sprintf(gretl_errmsg, _("Failed to parse number of observations"));
 	free(tmp);
-	return 1;
+	return E_DATA;
     }
 
     if (progress > 0) {
@@ -1882,25 +1880,24 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     if (tmp) {
 	if (!strcmp((char *) tmp, "true")) {
 	    if (dataset_allocate_obs_markers(pdinfo)) {
-		sprintf(gretl_errmsg, "Out of memory");
-		return 1;
+		return E_ALLOC;
 	    }
 	} else if (strcmp((char *) tmp, "false")) {
 	    sprintf(gretl_errmsg, _("labels attribute for observations must be "
 		    "'true' or 'false'"));
-	    return 1;
+	    return E_DATA;
 	}
 	free(tmp);
     } else {
-	return 1;
+	return E_DATA;
     }
 
     tmp = xmlGetProp(node, (XUC) "panel-info");
     if (tmp) {
 	if (!strcmp((char *) tmp, "true")) {
-	    if (dataset_allocate_panel_info(pdinfo)) {
-		sprintf(gretl_errmsg, "Out of memory");
-		return 1;
+	    err = dataset_allocate_panel_info(pdinfo);
+	    if (err) {
+		return err;
 	    }
 	    panelobs = 1;
 	} 
@@ -1914,12 +1911,9 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     pdinfo->t2 = pdinfo->n - 1;
 
     for (i=0; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    continue;
-	}
 	(*pZ)[i] = malloc(pdinfo->n * sizeof ***pZ);
 	if ((*pZ)[i] == NULL) {
-	    return 1;
+	    return E_ALLOC;
 	}
     }
 
@@ -1935,7 +1929,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 
     if (cur == NULL) {
 	sprintf(gretl_errmsg, _("Got no observations\n"));
-	return 1;
+	return E_DATA;
     }
 
     if (progress) {
@@ -1953,7 +1947,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		    free(tmp);
 		} else {
 		    sprintf(gretl_errmsg, _("Case marker missing at obs %d"), t+1);
-		    return 1;
+		    return E_DATA;
 		}
 	    }
 
@@ -1972,7 +1966,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		} 
 		if (ok < 2) {
 		    sprintf(gretl_errmsg, "Panel index missing at obs %d", t+1);
-		    return 1;
+		    return E_DATA;
 		}
 		pdinfo->paninfo->unit[t] = j;
 		pdinfo->paninfo->period[t] = s;
@@ -1988,35 +1982,49 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		t++;
 	    } else {
 		sprintf(gretl_errmsg, _("Values missing at observation %d"), t+1);
-		return 1;
+		err = E_DATA;
+		goto bailout;
 	    }
-	}	    
+	}	   
+ 
 	cur = cur->next;
+
+	if (cur != NULL && t == pdinfo->n) {
+	    /* got too many observations */
+	    t = pdinfo->n + 1;
+	    goto bailout;
+	}
+
 	if (progress && t > 0 && t % 50 == 0) {
 	    (*show_progress) (50L, (long) pdinfo->n, SP_NONE);
 	}
     }
 
+ bailout:
+
     if (progress) {
 	(*show_progress)(0L, (long) pdinfo->n, SP_FINISH);
 	close_plugin(handle);
     }
 
-    if (t != pdinfo->n) {
+    if (!err && t != pdinfo->n) {
 	sprintf(gretl_errmsg, _("Number of observations does not match declaration"));
-	return 1;
+	err = E_DATA;
     }
 
-    else return 0;
+    return err;
 }
 
 static long get_filesize (const char *fname)
 {
     struct stat buf;
 
+    errno = 0;
+
     if (stat(fname, &buf) == 0) {
         return buf.st_size;
     } else {
+	gretl_errmsg_set_from_errno(NULL);
         return -1;
     }
 }
@@ -2171,7 +2179,7 @@ static int lag_from_label (int v, const DATAINFO *pdinfo, int *lag)
     int pv = 0;
 
     if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, lag) == 3) {
-	pv = varindex(pdinfo, vname);
+	pv = series_index(pdinfo, vname);
 	pv = (pv < pdinfo->v)? pv : 0;
     }
 
@@ -2187,28 +2195,34 @@ static int dummy_child_from_label (int v, const DATAINFO *pdinfo)
 
     if (sscanf(test, _("dummy for %s = %lf"), vname, &val) == 2 ||
 	sscanf(test, "dummy for %s = %lf", vname, &val) == 2) {
-	pv = varindex(pdinfo, vname);
+	pv = series_index(pdinfo, vname);
 	pv = (pv < pdinfo->v)? pv : 0;
     }
 
     return pv;
 }
 
-static void record_transform_info (double **Z, DATAINFO *pdinfo)
+static void record_transform_info (double **Z, DATAINFO *pdinfo, double version)
 {
+    VARINFO *vinfo;
     int i, p, pv;
 
     for (i=1; i<pdinfo->v; i++) {
+	vinfo = pdinfo->varinfo[i];
+	if (vinfo->transform == LAGS) {
+	    /* already handled */
+	    continue;
+	}
 	pv = lag_from_label(i, pdinfo, &p);
 	if (pv > 0) {
-	    strcpy(pdinfo->varinfo[i]->parent, pdinfo->varname[pv]);
-	    pdinfo->varinfo[i]->transform = LAGS;
-	    pdinfo->varinfo[i]->lag = p;
-	} else {
+	    strcpy(vinfo->parent, pdinfo->varname[pv]);
+	    vinfo->transform = LAGS;
+	    vinfo->lag = p;
+	} else if (version < 1.1) {
 	    pv = dummy_child_from_label(i, pdinfo);
 	    if (pv > 0) {
-		strcpy(pdinfo->varinfo[i]->parent, pdinfo->varname[pv]);
-		pdinfo->varinfo[i]->transform = DUMMIFY;
+		strcpy(vinfo->parent, pdinfo->varname[pv]);
+		vinfo->transform = DUMMIFY;
 	    }
 	}
     }
@@ -2256,6 +2270,21 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 
     gretl_error_clear();
 
+    /* COMPAT: Do not generate nodes for formatting spaces */
+    LIBXML_TEST_VERSION
+	xmlKeepBlanksDefault(0);
+
+    fsz = get_filesize(fname);
+
+    if (fsz < 0) {
+	return E_FOPEN;
+    } else if (fsz > 100000) {
+	fprintf(stderr, "%s %ld bytes %s...\n", 
+		(is_gzipped(fname))? I_("Uncompressing") : I_("Reading"),
+		fsz, I_("of data"));
+	if (opt & OPT_P) progress = fsz;
+    }
+
 #ifdef ENABLE_NLS
     check_for_console(prn);
 #endif
@@ -2266,18 +2295,6 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	goto bailout;
     }
 
-    /* COMPAT: Do not generate nodes for formatting spaces */
-    LIBXML_TEST_VERSION
-	xmlKeepBlanksDefault(0);
-
-    fsz = get_filesize(fname);
-    if (fsz > 100000) {
-	fprintf(stderr, "%s %ld bytes %s...\n", 
-		(is_gzipped(fname))? I_("Uncompressing") : I_("Reading"),
-		fsz, I_("of data"));
-	if (opt & OPT_P) progress = fsz;
-    }
-
     doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
 	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
@@ -2406,8 +2423,8 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	}
     }
 
-    if (!err && gdtversion < 1.1) {
-	record_transform_info(*pZ, pdinfo);
+    if (!err && gdtversion < 1.2) {
+	record_transform_info(*pZ, pdinfo, gdtversion);
     }
 
     if (err && tmpdinfo != NULL) {
@@ -2553,6 +2570,44 @@ int load_user_matrix_file (const char *fname)
     return err;
 }
 
+int load_user_scalars_file (const char *fname) 
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    char *name, *val;
+    int err = 0;
+
+    xmlKeepBlanksDefault(0);
+
+    err = gretl_xml_open_doc_root(fname, "gretl-scalars", &doc, &cur);
+    if (err) {
+	return err;
+    }
+
+    cur = cur->xmlChildrenNode;
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "gretl-scalar")) {
+	    name = (char *) xmlGetProp(cur, (XUC) "name");
+	    val = (char *) xmlGetProp(cur, (XUC) "value");
+	    if (name == NULL || val == NULL) {
+		err = 1;
+	    } else {
+		err = gretl_scalar_add(name, dot_atof(val));
+	    }
+	    free(name);
+	    free(val);
+	}
+	cur = cur->next;
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+	xmlCleanupParser();
+    }
+
+    return err;
+}
+
 int load_user_XML_file (const char *fname)
 {
     char *rootname = NULL;
