@@ -112,7 +112,8 @@ typedef enum {
     GRETL_TYPE_SCALAR_REF,
     GRETL_TYPE_SERIES_REF,
     GRETL_TYPE_MATRIX_REF,
-    GRETL_TYPE_UVAR,
+    GRETL_TYPE_USCALAR,
+    GRETL_TYPE_USERIES,
     GRETL_TYPE_DATE
 } GretlType;
 
@@ -260,7 +261,7 @@ struct VARINFO_ {
     int transform;
     int lag;
     char compact_method;
-    char stack_level;
+    char stack_level; /* FIXME should be int? */
     char line_width;
     char **sorted_markers;
 };
