@@ -25,6 +25,7 @@
 #include "matrix_extra.h"
 #include "gretl_string_table.h"
 #include "gretl_func.h"
+#include "gretl_scalar.h"
 
 #include <unistd.h>
 #include <errno.h>
@@ -100,6 +101,7 @@ struct set_vars_ {
 #define WARNINGS "warnings"
 #define GRETL_DEBUG "debug"
 #define BLAS_NMK_MIN "blas_nmk_min"
+#define PROTECT_LISTS "protect_lists"
 
 #define libset_boolvar(s) (!strcmp(s, ECHO) || \
                            !strcmp(s, MESSAGES) || \
@@ -114,7 +116,8 @@ struct set_vars_ {
 			   !strcmp(s, USE_QR) || \
 			   !strcmp(s, SHELL_OK) || \
 			   !strcmp(s, USE_CWD) || \
-			   !strcmp(s, USE_FCP))
+			   !strcmp(s, USE_FCP) || \
+                           !strcmp(s, PROTECT_LISTS))
 
 #define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
 			  !strcmp(s, BHHH_TOLER) || \
@@ -138,7 +141,8 @@ struct set_vars_ {
 
 /* global state */
 set_vars *state;
-int gretl_debug;
+static int gretl_debug;
+static int protect_lists = 1;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -350,7 +354,7 @@ static void state_vars_init (set_vars *sv)
     sv->vecm_norm = NORM_PHILLIPS;
     sv->initvals = NULL;
 
-    sv->bfgs_maxiter = 500;
+    sv->bfgs_maxiter = -1;
     sv->bfgs_toler = NADBL;
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
@@ -502,6 +506,11 @@ int gretl_debugging_on (void)
     return gretl_debug;
 }
 
+int lists_protected (void)
+{
+    return protect_lists;
+}
+
 char get_csv_delim (const DATAINFO *pdinfo)
 {
     check_for_state();
@@ -676,7 +685,7 @@ static int libset_get_scalar (const char *var, const char *arg,
 			      int *pi, double *px)
 {
     double x = NADBL;
-    int v, err = 0;
+    int err = 0;
 
     if (libset_numeric_string(arg, pi, px, &err)) {
 	if (err) {
@@ -689,15 +698,11 @@ static int libset_get_scalar (const char *var, const char *arg,
 	return err;
     }
 
-    v = varindex(pdinfo, arg);
-
-    if (v >= pdinfo->v) {
-	sprintf(gretl_errmsg, "'%s': unrecognized name", arg);
-	return E_UNKVAR;
-    } else if (var_is_series(pdinfo, v)) {
-	return E_DATATYPE;
+    if (gretl_is_scalar(arg)) {
+	x = gretl_scalar_get_value(arg);
     } else {
-	x = Z[v][0];
+	sprintf(gretl_errmsg, "'%s': not a scalar", arg);
+	return E_UNKVAR;
     }
 
     if (negval_invalid(var) && x < 0.0) {
@@ -861,7 +866,7 @@ static int set_line_width (const char *s0, const char *s1,
     if (isdigit((unsigned char) *s0)) {
 	v = atoi(s0);
     } else {
-	v = varindex(pdinfo, s0);
+	v = series_index(pdinfo, s0);
     }
 
     if (v < 1 || v >= pdinfo->v) {
@@ -1171,6 +1176,7 @@ static int display_settings (PRN *prn)
     }
 
     libset_print_bool(USE_CWD, prn);
+    libset_print_bool(PROTECT_LISTS, prn);
 
     libset_header(_("Numerical methods"), prn);
 
@@ -1703,6 +1709,11 @@ int libset_get_bool (const char *s)
 {
     int flag, ret = 0;
 
+    if (!strcmp(s, PROTECT_LISTS)) {
+	/* global special */
+	return protect_lists;
+    }
+
     if (!strcmp(s, MAX_VERBOSE) && gretl_debug > 1) {
 	/* strong debugging turns on max_verbose */
 	return 1;
@@ -1750,6 +1761,12 @@ void libset_set_bool (const char *s, int set)
 	return;
     }
 
+    if (!strcmp(s, PROTECT_LISTS)) {
+	/* global special */
+	protect_lists = set;
+	return;
+    }
+
     flag = boolvar_get_flag(s);
 
     if (flag == 0) {
