@@ -28,6 +28,7 @@
 #include "var.h"
 #include "objstack.h"
 #include "gretl_func.h"
+#include "gretl_scalar.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -60,11 +61,6 @@ enum loop_types {
     EACH_LOOP
 };
 
-enum loop_val_codes {
-    LOOP_VAL_BAD = 0,
-    LOOP_VAL_UNDEF = -9999
-};
-
 #define DEFAULT_NOBS 512
 
 #define indexed_loop(l) (l->type == INDEX_LOOP || \
@@ -116,7 +112,6 @@ enum loop_flags {
 
 struct controller_ {
     double val;
-    int vnum;
     char vname[VNAMELEN];
     int vsign;
     char *expr;
@@ -212,108 +207,105 @@ int gretl_execute_loop (void)
     return loop_execute;
 }
 
-static double controller_evaluate_expr (const char *expr, 
-					const char *vname,
-					int vnum,
-					double ***pZ,
-					DATAINFO *pdinfo)
+/* get a value from a loop "limit" element, either a specified
+   scalar variable or a numeric constant */
+
+static double 
+controller_get_value (controller *clr, double ***pZ, DATAINFO *pdinfo)
 {
     double x = NADBL;
-    int v, err = 0;
 
-    gretl_error_clear();
-
-    if (!strcmp(vname, "iftest")) {
-	x = generate_scalar(expr, pZ, pdinfo, &err);
+    if (clr->vname[0] != '\0') {
+	if (gretl_is_scalar(clr->vname)) {
+	    x = gretl_scalar_get_value(clr->vname) * clr->vsign;
+	} else {
+	   gretl_errmsg_sprintf(_("'%s': not a scalar"), clr->vname);
+	} 
     } else {
-	err = generate(expr, pZ, pdinfo, OPT_Q, NULL);
-	if (!err) {
-	    v = (vnum > 0)? vnum : varindex(pdinfo, vname);
-	    if (v < pdinfo->v) {
-		x = (*pZ)[v][0];
-	    } else {
-		err = 1;
-	    }
-	}
-    }
-
-    if (!err && na(x)) {
-	err = 1;
+	/* numerical constant? */
+	x = clr->val;
     }
 
 #if LOOP_DEBUG
-    fprintf(stderr, "controller_evaluate_expr: expr='%s', x=%g, err=%d\n", 
-	    expr, x, err);
+    fprintf(stderr, "controller_get_value: vname='%s', returning %g\n", 
+	    clr->vname, x);
 #endif
 
-    if (err) {
-	gretl_errmsg_sprintf("%s: '%s'", _("error evaluating loop condition"),
-			     expr);
-    }
-
     return x;
 }
 
-/* get a value from a loop controller element, either via a
-   "genr"-type expression, or from a specified scalar variable, or
-   from a numeric constant */
-
-static double 
-loop_controller_get_value (controller *clr, double ***pZ, DATAINFO *pdinfo)
+static double loop_get_initval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 {
-    double ret = NADBL;
-
-    if (clr->expr != NULL) {
-	ret = controller_evaluate_expr(clr->expr, clr->vname, clr->vnum,
-				       pZ, pdinfo);
-	clr->val = ret;
-    } else if (clr->vnum > 0) {
-	ret = (*pZ)[clr->vnum][0] * clr->vsign;
-	clr->val = ret;
-    } else {
-	ret = clr->val;
-    } 
-
 #if LOOP_DEBUG
-    fprintf(stderr, "loop_controller_get_value: vnum = %d, returning %g\n", 
-	    clr->vnum, ret);
+    fprintf(stderr, "getting loop initval...\n");
 #endif
+    return controller_get_value(&loop->init, pZ, pdinfo);
+}
+
+static double loop_get_finalval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
+{
+#if LOOP_DEBUG
+    fprintf(stderr, "getting loop finalval...\n");
+#endif
+    return controller_get_value(&loop->final, pZ, pdinfo);
+}
+
+/* apply initialization in case of for-loop */
+
+static void
+forloop_init (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
+{
+    const char *expr = loop->init.expr;
+
+    if (expr != NULL) {
+	*err = generate(expr, pZ, pdinfo, OPT_Q, NULL);
+	if (*err) {
+	    gretl_errmsg_sprintf("%s: '%s'", _("error evaluating loop condition"),
+				 expr);
+	}
+    }
+}
+
+/* evaluate boolean condition in for-loop or while-loop */
+
+static int 
+loop_testval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
+{
+    const char *expr = loop->test.expr;
+    int ret = 1;
+
+    if (expr != NULL) {
+	double x = generate_scalar(expr, pZ, pdinfo, err);
+
+	if (!*err && na(x)) {
+	    *err = E_DATA;
+	    ret = 0;
+	} else {
+	    ret = x;
+	}
+	if (*err) {
+	    gretl_errmsg_sprintf("%s: '%s'", _("error evaluating loop condition"),
+				 expr);
+	}
+    }
 
     return ret;
 }
 
-static double loop_initval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
-{
-#if LOOP_DEBUG
-    fprintf(stderr, "getting loop_initval...\n");
-#endif
-    return loop_controller_get_value(&loop->init, pZ, pdinfo);
-}
+/* evaluate third expression in for-loop, if any */
 
-static double loop_finalval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
+static void 
+loop_delta (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
 {
-#if LOOP_DEBUG
-    fprintf(stderr, "getting loop_finalval...\n");
-#endif
-    return loop_controller_get_value(&loop->final, pZ, pdinfo);
-}
+    const char *expr = loop->delta.expr;
 
-static double loop_testval (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
-{
-#if LOOP_DEBUG
-    fprintf(stderr, "getting loop_testval...\n");
-#endif
-    return loop_controller_get_value(&loop->test, pZ, pdinfo);
-}
-
-static double loop_delta (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
-{
-    double x = loop_controller_get_value(&loop->delta, pZ, pdinfo);
-
-#if LOOP_DEBUG
-    fprintf(stderr, "loop_incrval, returning %g\n", x);
-#endif
-    return x;
+    if (expr != NULL) {
+	*err = generate(expr, pZ, pdinfo, OPT_Q, NULL);
+	if (*err) {
+	    gretl_errmsg_sprintf("%s: '%s'", _("error evaluating loop condition"),
+				 expr);
+	}
+    }
 }
 
 static gretlopt get_loop_opts (ExecState *s, int *err)
@@ -581,92 +573,25 @@ static void gretl_loop_destroy (LOOPSET *loop)
     free(loop);
 }
 
-static int 
-ok_loop_var (const LOOPSET *loop, const DATAINFO *pdinfo, const char *vname)
-{
-    int v = varindex(pdinfo, vname);
-
-    if (v >= pdinfo->v) {
-	if (loop->parent == NULL) {
-	    sprintf(gretl_errmsg, 
-		    _("Undefined variable '%s' in loop condition."), vname);
-	    v = LOOP_VAL_BAD;
-	} else {
-	    /* the variable may be defined in an outer loop,
-	       and will become available at runtime? */
-	    v = LOOP_VAL_UNDEF;
-	}
-    } else if (var_is_series(pdinfo, v)) {
-	strcpy(gretl_errmsg, _("The loop control variable "
-	       "must be a scalar"));
-	v = LOOP_VAL_BAD;
-    }
-	
-    return v;
-}
-
-static int 
-controller_set_var (controller *clr, LOOPSET *loop, const DATAINFO *pdinfo,
-		    const char *s)
-{
-    int v, err = 0;
-    int vsign = 1;
-
-    if (*s == '-' && isalpha((unsigned char) *(s + 1))) {
-	/* negative sign in front of varname? */
-	vsign = -1;
-	v = ok_loop_var(loop, pdinfo, s + 1);
-    } else {
-	v = ok_loop_var(loop, pdinfo, s);
-    }
-
-    if (v == LOOP_VAL_BAD) {
-	err = 1;
-    } else if (v == LOOP_VAL_UNDEF) {
-	clr->vnum = LOOP_VAL_UNDEF;
-	*clr->vname = '\0';
-	strncat(clr->vname, s, VNAMELEN - 1);
-    } else {
-	clr->vnum = v;
-	clr->vsign = vsign;
-    }
-
-#if LOOP_DEBUG
-    fprintf(stderr, " controller set var: vnum=%d, vsign=%d\n", 
-	    clr->vnum, clr->vsign);
-#endif
-
-    return err;
-}
-
 static int parse_as_while_loop (LOOPSET *loop,
 				double ***pZ, DATAINFO *pdinfo,
 				const char *s)
 {
-    char *expr = NULL;
-    double x = NADBL;
-    int err = 0;
-
-    expr = gretl_strdup(s);
-    x = controller_evaluate_expr(expr, "iftest", 0, pZ, pdinfo);
-
-    if (na(x)) {
-	err = E_DATA;
-	free(expr);
-    }
-
-    if (!err) {
-	loop->test.expr = expr;
-	strcpy(loop->test.vname, "iftest");
-	loop->type = WHILE_LOOP;
-    }
-
 #if LOOP_DEBUG
-    fprintf(stderr, "parse_as_while_loop: cond = '%s', x = %g, err = %d\n", 
-	    s, x, err);
+    fprintf(stderr, "parse_as_while_loop: cond = '%s'\n", s);
 #endif
 
-    return err;
+    if (s == NULL || *s == '\0') {
+	return E_PARSE;
+    }
+
+    loop->type = WHILE_LOOP;
+    loop->test.expr = gretl_strdup(s);
+    if (loop->test.expr == NULL) {
+	return E_ALLOC;
+    }
+
+    return 0;
 }
 
 static const char *ichars = "ijklmpqrs"; 
@@ -698,35 +623,55 @@ static int bad_ichar (char c)
     return err;
 }
 
-static int index_get_int (controller *clr, const char *s, 
-			  double ***pZ, DATAINFO *pdinfo,
-			  int *err)
-{
-    int v = varindex(pdinfo, s);
-    int k = 0;
+/* for a loop control expression such as "j=start..end", get the
+   initial or final value from the string @s (we also use this to get
+   the count for a simple count loop).
+*/
 
-    if (v < pdinfo->v) {
-	clr->vnum = v;
-	k = (int) (*pZ)[v][0];
+static int index_get_limit (LOOPSET *loop, controller *clr, const char *s, 
+			    const double **Z, const DATAINFO *pdinfo)
+{
+    int v, err = 0;
+
+    if (integer_string(s)) {
+	/* plain numerical value */
+	clr->val = atoi(s);
     } else {
-	k = (int) generate_scalar(s, pZ, pdinfo, err);
+	if (*s == '-') {
+	    /* negative of variable? */
+	    clr->vsign = -1;
+	    s++;
+	}
+	if (gretl_is_scalar(s)) {
+	    *clr->vname = '\0';
+	    strncat(clr->vname, s, VNAMELEN - 1);
+	    clr->val = (int) gretl_scalar_get_value(s);
+	} else if ((v = series_index(pdinfo, s)) < pdinfo->v) {
+	    /* found a series by the name of s */
+	    gretl_errmsg_sprintf(_("'%s': not a scalar"), s);
+	} else if (loop->parent != NULL && strlen(s) == gretl_namechar_spn(s)) {
+	    /* potentially valid varname, but unknown at present */
+	    *clr->vname = '\0';
+	    strncat(clr->vname, s, VNAMELEN - 1);
+	} else {
+	    gretl_errmsg_sprintf(_("Undefined variable '%s' in loop condition."), s);
+	    err = E_UNKVAR;
+	}
     }
 
-    return k;
+    return err;
 }
 
 #define maybe_date(s) (strchr(s, ':') || strchr(s, '/'))
 
 static int parse_as_indexed_loop (LOOPSET *loop,
-				  double ***pZ,
-				  DATAINFO *pdinfo,
+				  const double **Z,
+				  const DATAINFO *pdinfo,
 				  char ichar,
 				  const char *lvar, 
 				  const char *start,
 				  const char *end)
 {
-    int nstart = -1, nend = -1;
-    int dated = 0;
     int err;
 
     if (lvar != NULL) {
@@ -744,62 +689,38 @@ static int parse_as_indexed_loop (LOOPSET *loop,
        numeric constants, then variables */
 
 #if LOOP_DEBUG
-    fprintf(stderr, "parse_as_indexed_loop: start='%s'\n", start);
+    fprintf(stderr, "parse_as_indexed_loop: start='%s', end='%s'\n", start, end);
 #endif
 
-    if (!err) {
-	if (maybe_date(start)) {
-	    nstart = dateton(start, pdinfo);
-	    if (nstart >= 0) {
-		dated = 1;
+    if (!err && maybe_date(start)) {
+	loop->init.val = dateton(start, pdinfo);
+	if (loop->init.val < 0) {
+	    err = E_DATA;
+	} else {
+	    loop->final.val = dateton(end, pdinfo);
+	    if (loop->final.val < 0) {
+		err = E_DATA;
 	    } else {
-		err = E_DATA;
+		loop->type = DATED_LOOP;
 	    }
-	} else if (integer_string(start)) {
-	    nstart = atoi(start);
-	    /* FIXME check for annual date here? */
-#if LOOP_DEBUG
-	    fprintf(stderr, "integer string: nstart = %d\n", nstart);
-#endif
-	} else {
-	    nstart = index_get_int(&loop->init, start, pZ, pdinfo, &err);
 	}
-    }
-
-#if LOOP_DEBUG
-    fprintf(stderr, "parse_as_indexed_loop: end='%s'\n", end);
-#endif
-
-    if (!err) {
-	if (dated) {
-	    nend = dateton(end, pdinfo);
-	    if (nend < 0) {
-		err = E_DATA;
-	    }
-	} else if (integer_string(end)) {
-	    nend = atoi(end);
-#if LOOP_DEBUG
-	    fprintf(stderr, "integer string: nend = %d\n", nend);
-#endif
-	} else {
-	    nend = index_get_int(&loop->final, end, pZ, pdinfo, &err);
+    } else if (!err) { 
+	err = index_get_limit(loop, &loop->init, start, Z, pdinfo);
+	if (!err) {
+	    err = index_get_limit(loop, &loop->final, end, Z, pdinfo);
 	}
-    }
-
-    if (!err) {
-	loop->init.val = nstart;
-	loop->final.val = nend;
-	if (dated) {
-	    loop->type = DATED_LOOP;
-	} else {
+	if (!err) {
 	    loop->type = INDEX_LOOP;
 	}
+    }
+
+    if (!err) {
 	loop->ichar = ichar;
     }
 
 #if LOOP_DEBUG
-    fprintf(stderr, "parse_as_indexed_loop: init.val=%g, final.val=%g\n",
-	    loop->init.val, loop->final.val);
+    fprintf(stderr, "indexed_loop: init.val=%g, final.val=%g, err=%d\n",
+	    loop->init.val, loop->final.val, err);
 #endif
 
     return err;
@@ -808,141 +729,63 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 static int parse_as_count_loop (LOOPSET *loop, 
 				const double **Z,
 				const DATAINFO *pdinfo,
-				const char *count)
+				const char *s)
 {
-    int nt = -1;
-    int err = 0;
+    int err;
 
-    /* try for a numeric value, or failing that, a variable */
-
-    if (numeric_string(count)) {
-	nt = atoi(count); 
-    } else { 
-	err = controller_set_var(&loop->final, loop, pdinfo, count);
-    }
+    err = index_get_limit(loop, &loop->final, s, Z, pdinfo);
 
     if (!err) {
 	loop->init.val = 1;
-	if (loop->final.vnum == 0) {
-	    loop->final.val = nt;
-	}
 	loop->type = COUNT_LOOP;
     }
 
 #if LOOP_DEBUG
-    fprintf(stderr, "parse_as_count_loop: init.val=%g, final.val=%g, "
-	    "final.vnum=%d\n", loop->init.val, loop->final.val,
-	    loop->final.vnum);
+    fprintf(stderr, "parse_as_count_loop: init.val=%g, final.val=%g\n",
+	    loop->init.val, loop->final.val);
 #endif
 
     return err;
 }
 
-/* before testing the "delta" element in a putative "for" loop, save
-   the state of the variable in question so we can restore it after
-   running the test */
-
-static double *save_delta_state (int v, double **Z, DATAINFO *pdinfo)
-{
-    double *x0;
-    int i, n;
-
-    n = (var_is_series(pdinfo, v))? pdinfo->n : 1;
-
-    x0 = malloc(n * sizeof *x0);
-    if (x0 != NULL) {
-	for (i=0; i<n; i++) {
-	    x0[i] = Z[v][i];
-	}
-    }
-
-    return x0;
-}
-
-static void 
-restore_delta_state (int v, double *x0, double **Z, DATAINFO *pdinfo)
-{
-    int i, n;
-
-    n = (var_is_series(pdinfo, v))? pdinfo->n : 1;
-
-    for (i=0; i<n; i++) {
-	Z[v][i] = x0[i];
-    }
-
-    free(x0);
-}
-
 static int 
-test_forloop_element (char *s, LOOPSET *loop,
-		      double ***pZ, DATAINFO *pdinfo,
-		      int i)
+set_forloop_element (char *s, LOOPSET *loop,
+		     double ***pZ, DATAINFO *pdinfo,
+		     int i)
 {
-    char vname[VNAMELEN] = {0};
-    double x = NADBL;
+    controller *clr = (i == 0)? &loop->init :
+	(i == 1)? &loop->test : &loop->delta;
     int len, err = 0;
 
-    if (s == NULL) {
-	/* FIXME: allow empty "for" fields? */
-	return E_PARSE;
-    }
-
-    if (i == 1) {
-	/* middle term: Boolean test */
-	strcpy(vname, "iftest");
-	x = controller_evaluate_expr(s, vname, 0, pZ, pdinfo);
-    } else {
-	/* treat as regular "genr" expression */
-	len = gretl_varchar_spn(s);
-	if (len < VNAMELEN) {
-	    strncat(vname, s, len);
-	    if (i == 2) {
-		/* "increment" expression: we'll have to undo whatever
-		   the test evaluation does */
-		int v = varindex(pdinfo, vname);
-
-		if (v < pdinfo->v) {
-		    double *x0 = save_delta_state(v, *pZ, pdinfo);
-
-		    if (x0 == NULL) {
-			err = E_ALLOC;
-		    } else {
-			x = controller_evaluate_expr(s, vname, v, pZ, pdinfo);
-			restore_delta_state(v, x0, *pZ, pdinfo);
-		    }
-		} else {
-		    err = E_UNKVAR;
-		}
-	    } else {
-		x = controller_evaluate_expr(s, vname, 0, pZ, pdinfo);
-	    }	
-	} else {
-	    err = E_UNKVAR;
-	}
-    }
-
 #if LOOP_DEBUG
-    fprintf(stderr, "test_forloop_element: i=%d: '%s', x = %g\n", i, s, x);
+    fprintf(stderr, "set_forloop_element: i=%d: '%s'\n", i, s);
 #endif
 
-    if (na(x)) {
-	err = 1;
+    if (s == NULL || *s == '\0') {
+	/* an empty "for" field */
+	if (i == 1) {
+	    /* test is implicitly always true */
+	    clr->val = 1;
+	} else {
+	    /* no-op */
+	    clr->val = 0;
+	}
+	return 0;
     }
 
-    if (!err) {
-	controller *clr = (i == 0)? &loop->init :
-	    (i == 1)? &loop->test : &loop->delta;
+    clr->expr = gretl_strdup(s);
+    if (clr->expr == NULL) {
+	err = E_ALLOC;
+    }
 
-	clr->expr = gretl_strdup(s);
-	if (clr->expr == NULL) {
-	    err = 1;
-	} else {
-	    strcpy(clr->vname, vname);
-	    if (i != 1) {
-		clr->vnum = varindex(pdinfo, vname);
-	    }
-	}	
-    }	
+    if (!err && i == 0) {
+	/* initialization: look for varname for possible substitution */
+	err = extract_varname(clr->vname, s, &len);
+    } 
+
+#if LOOP_DEBUG
+    fprintf(stderr, " expr='%s', vname='%s'\n", clr->expr, clr->vname);
+#endif
 
     return err;
 }
@@ -957,12 +800,16 @@ static int allocate_each_strings (LOOPSET *loop, int n)
 static int list_vars_to_strings (LOOPSET *loop, const int *list,
 				 const DATAINFO *pdinfo)
 {
-    int i, vi, n = list[0];
+    int i, vi;
     int err;
 
-    err = allocate_each_strings(loop, n);
+#if LOOP_DEBUG
+    fprintf(stderr, "list_vars_to_strings: adding %d strings\n", list[0]);
+#endif
 
-    for (i=0; i<n && !err; i++) {
+    err = allocate_each_strings(loop, list[0]);
+
+    for (i=0; i<list[0] && !err; i++) {
 	vi = list[i+1];
 	if (vi < 0 || vi >= pdinfo->v) {
 	    err = E_DATA;
@@ -977,8 +824,8 @@ static int list_vars_to_strings (LOOPSET *loop, const int *list,
     return err;
 }
 
-/* At loop runtime, check the named list and insert the names
-   of the variables as "eachstrs"; flag an error if the list
+/* At loop runtime, check the named list and insert the names (or
+   numbers) of the variables as "eachstrs"; flag an error if the list
    has disappeared.
 */
 
@@ -1007,8 +854,9 @@ static int loop_list_refresh (LOOPSET *loop, const DATAINFO *pdinfo)
 }
 
 /* Identify the named list, if any, but do not yet fill out the
-   variable-name strings: these will be set when the loop is
-   actually run, since the list may have changed in the meantime.
+   variable-name (or variable-number) strings: these will be set when
+   the loop is actually run, since the list may have changed in the
+   meantime.
 */
 
 static int list_loop_setup (LOOPSET *loop, char *s, int *nf)
@@ -1021,6 +869,10 @@ static int list_loop_setup (LOOPSET *loop, char *s, int *nf)
 
     list = get_list_by_name(s);
 
+#if LOOP_DEBUG
+    printlist(list, "list, in list_loop_setup");
+#endif
+
     if (list == NULL) {
 	err = E_UNKVAR;
     } else {
@@ -1047,8 +899,8 @@ each_strings_from_list_of_vars (LOOPSET *loop, const DATAINFO *pdinfo,
     if (sscanf(s, "%15[^.]..%15s", vn1, vn2) != 2) {
 	err = E_PARSE;
     } else {
-	v1 = varindex(pdinfo, vn1);
-	v2 = varindex(pdinfo, vn2);
+	v1 = series_index(pdinfo, vn1);
+	v2 = series_index(pdinfo, vn2);
 
 	if (v1 < 0 || v2 < 0 || v1 >= pdinfo->v || v2 >= pdinfo->v) {
 	    err = 1;
@@ -1169,44 +1021,62 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
     return err;
 }
 
+/* try to parse out (expr1; expr2; expr3) */
+
 static int 
 parse_as_for_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, char *s)
 {
-    char *forstr = NULL;
-    char *forbits[3];
-    char *p = strchr(s, '(');
-    int i, len, err = 0;
+    char *tmp, *q;
+    int i, j, len;
+    int sc = 0;
+    int err = 0;
 
-    if (p == NULL) {
-	err = 1;
-    } else {
-	len = strcspn(p, ")");
-	if (len < 4) {
-	    err = E_PARSE;
-	} else if ((forstr = malloc(len + 1)) == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    i = 0;
-	    /* make compressed copy of string */
-	    p++;
-	    while (*p) {
-		if (*p == ')') break;
-		if (*p != ' ') {
-		    forstr[i++] = *p;
-		}
-		p++;
-	    }
-	    forstr[i] = '\0';
-	    /* split terms separated by ';' */
-	    for (i=0; i<3 && !err; i++) {
-		forbits[i] = strtok((i == 0)? forstr : NULL, ";");
-		err = test_forloop_element(forbits[i], loop, 
-					   pZ, pdinfo, i);
-	    }
-	    free(forstr);
-	}
+    s += strcspn(s, "(");
+    if (*s != '(') {
+	return E_PARSE;
     }
 
+    s++;
+    q = strrchr(s, ')');
+    if (q == NULL) {
+	return E_PARSE;
+    }
+
+    len = q - s;
+    if (len < 2) { /* minimal OK string is ";;" */
+	return E_PARSE;
+    }
+
+    tmp = malloc(len + 1);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    for (j=0; j<3 && s!=q && !err; j++) {
+	/* make a compressed copy of field j */
+	i = 0;
+	while (s != q) {
+	    if (*s == ';') {
+		sc++;
+		s++;
+		break; /* onto next field */
+	    }
+	    if (*s != ' ') {
+		tmp[i++] = *s;
+	    }
+	    s++;
+	}
+	tmp[i] = '\0';
+	err = set_forloop_element(tmp, loop, pZ, pdinfo, j);
+    }  
+
+    if (!err && (sc != 2 || s != q)) {
+	/* must have two semi-colons; must have reached righmost ')' */
+	err = E_PARSE;
+    }
+
+    free(tmp);
+
     if (!err) {
 	loop->type = FOR_LOOP;
     }
@@ -1233,9 +1103,11 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 #endif
 
     if (sscanf(s, "%c = %15[^.]..%15s", &ichar, op, rvar) == 3) {
-	err = parse_as_indexed_loop(loop, pZ, pdinfo, ichar, NULL, op, rvar);
+	err = parse_as_indexed_loop(loop, (const double **) *pZ, pdinfo, 
+				    ichar, NULL, op, rvar);
     } else if (sscanf(s, "for %15[^= ] = %15[^.]..%15s", lvar, op, rvar) == 3) {
-	err = parse_as_indexed_loop(loop, pZ, pdinfo, 0, lvar, op, rvar);
+	err = parse_as_indexed_loop(loop, (const double **) *pZ, pdinfo, 
+				    0, lvar, op, rvar);
     } else if (!strncmp(s, "foreach", 7)) {
 	err = parse_as_each_loop(loop, pdinfo, s + 7);
     } else if (!strncmp(s, "for", 3)) {
@@ -1355,6 +1227,7 @@ static int loop_count_too_high (LOOPSET *loop)
  * @loop: pointer to loop commands struct.
  * @pZ: pointer to data array.
  * @pdinfo: data information struct.
+ * @err: location to receive error code.
  *
  * Check whether a looping condition is still satisfied.
  *
@@ -1362,7 +1235,7 @@ static int loop_count_too_high (LOOPSET *loop)
  */
 
 static int 
-loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
+loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
 {
     int ok = 0;
 
@@ -1372,7 +1245,7 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 	ok = 0;
     } else if (loop->type == COUNT_LOOP || indexed_loop(loop)) {
 #if LOOP_DEBUG
-	fprintf(stderr, "** COUNT_LOOP: iter = %d, itermax = %d\n",
+	fprintf(stderr, "** count or indexed loop: iter = %d, itermax = %d\n",
 		loop->iter, loop->itermax);
 #endif
 	/* simple count loop or indexed loop */
@@ -1380,17 +1253,17 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 	    ok = 1;
 	}
     } else {
-	/* more complex forms of control... */
+	/* more complex forms of control (for, while) */
 	if (loop_count_too_high(loop)) {
 	    /* safeguard against infinite loops */
 	    ok = 0;
 	} else if (loop->type == FOR_LOOP) {
 	    if (loop->iter > 0) {
-		loop_delta(loop, pZ, pdinfo);
+		loop_delta(loop, pZ, pdinfo, err);
 	    }
-	    ok = loop_testval(loop, pZ, pdinfo);
+	    ok = loop_testval(loop, pZ, pdinfo, err);
 	} else if (loop->type == WHILE_LOOP) {
-	    ok = loop_testval(loop, pZ, pdinfo);
+	    ok = loop_testval(loop, pZ, pdinfo, err);
 	}
     }
 
@@ -1400,7 +1273,6 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 static void controller_init (controller *clr)
 {
     clr->val = NADBL;
-    clr->vnum = 0;
     clr->vname[0] = 0;
     clr->vsign = 1;
     clr->expr = NULL;
@@ -1719,18 +1591,17 @@ static int loop_store_start (LOOPSET *loop, const int *list,
 	    loop->store.dinfo->v, loop->store.dinfo->n);
 #endif
 
-    for (i=1; i<=list[0]; i++) {
-	int vi = list[i];
-	char *p;
+    for (i=1; i<=list[0] && !err; i++) {
+	const char *s = gretl_scalar_get_name(list[i]);
 
-	strcpy(loop->store.dinfo->varname[i], pdinfo->varname[vi]);
-	strcpy(VARLABEL(loop->store.dinfo, i), VARLABEL(pdinfo, vi));
-	if ((p = strstr(VARLABEL(loop->store.dinfo, i), "(scalar)"))) {
-	    *p = 0;
+	if (s == NULL) {
+	    err = E_DATA;
+	} else {
+	    strcpy(loop->store.dinfo->varname[i], s);
 	}
     }
 
-    return 0;
+    return err;
 }
 
 static int loop_store_update (LOOPSET *loop, int lno,
@@ -1738,8 +1609,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
 			      const double **Z, DATAINFO *pdinfo,
 			      gretlopt opt)
 {
-    int i, vi, t;
-    int err = 0;
+    int i, t, err = 0;
 
     if (loop->store.loopline >= 0 && loop->store.loopline != lno) {
 	strcpy(gretl_errmsg, "Only one 'store' command is allowed in a "
@@ -1756,7 +1626,6 @@ static int loop_store_update (LOOPSET *loop, int lno,
     }
 
     loop->store.loopline = lno;
-
     t = loop->store.n;
 
     if (t >= loop->store.dinfo->n) {
@@ -1766,12 +1635,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
     }
 
     for (i=1; i<=list[0]; i++) {
-	vi = list[i];
-	if (var_is_series(pdinfo, vi)) { 
-	    loop->store.Z[i][t] = Z[vi][pdinfo->t1];
-	} else {
-	    loop->store.Z[i][t] = Z[vi][0];
-	}
+	loop->store.Z[i][t] = gretl_scalar_get_value_by_index(list[i]);
     }
 
     loop->store.n += 1;
@@ -1956,7 +1820,7 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 }
 
 /* Update the LOOP_PRINT struct lprn using the current values from the
-   variables in list.  If this is the fist use we ned to do some
+   variables in list.  If this is the fist use we need to do some
    allocation first.
 */
 
@@ -1967,7 +1831,8 @@ static int loop_print_update (LOOP_PRINT *lprn,
 #ifdef ENABLE_GMP
     mpf_t m;
 #endif
-    int j, vj, t;
+    int j, vj;
+    double x;
     int err = 0;
 
     if (lprn->list == NULL) {
@@ -1984,20 +1849,20 @@ static int loop_print_update (LOOP_PRINT *lprn,
     
     for (j=0; j<list[0]; j++) {
 	vj = list[j+1];
-	t = (var_is_series(pdinfo, vj))? pdinfo->t1 : 0;
+	x = gretl_scalar_get_value_by_index(vj);
 #ifdef ENABLE_GMP
-	mpf_set_d(m, Z[vj][t]); 
+	mpf_set_d(m, x); 
 	mpf_add(lprn->sum[j], lprn->sum[j], m);
 	mpf_mul(m, m, m);
 	mpf_add(lprn->ssq[j], lprn->ssq[j], m);
 #else
-	lprn->sum[j] += Z[vj][t];
-	lprn->ssq[j] += Z[vj][t] * Z[vj][t];
+	lprn->sum[j] += x;
+	lprn->ssq[j] += x * x;
 #endif
-	if (!na(lprn->xbak[j]) && realdiff(Z[vj][t], lprn->xbak[j])) {
+	if (!na(lprn->xbak[j]) && realdiff(x, lprn->xbak[j])) {
 	    lprn->diff[j] = 1;
 	}
-	lprn->xbak[j] = Z[vj][t];
+	lprn->xbak[j] = x;
     }
 
 #ifdef ENABLE_GMP
@@ -2284,6 +2149,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
 {
     bigval mean, m, sd;
     int i, vi, n;
+    const char *s;
 
     if (lprn == NULL) {
 	return;
@@ -2302,6 +2168,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
     
     for (i=0; i<lprn->list[0]; i++) {
 	vi = lprn->list[i+1];
+	s = gretl_scalar_get_name(vi);
 	mpf_div_ui(mean, lprn->sum[i], (unsigned long) n);
 	if (lprn->diff[i] == 0) {
 	    mpf_set_d(sd, 0.0);
@@ -2316,7 +2183,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
 		mpf_set_d(sd, 0.0);
 	    }
 	}
-	pprintf(prn, "%*s", VNAMELEN - 1, pdinfo->varname[vi]);
+	pprintf(prn, "%*s", VNAMELEN - 1, s);
 	pprintf(prn, "%#14g %#14g\n", mpf_get_d(mean), mpf_get_d(sd));
     }
 
@@ -2326,6 +2193,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
 #else
     for (i=0; i<lprn->list[0]; i++) {
 	vi = lprn->list[i+1];
+	s = gretl_scalar_get_name(vi);
 	mean = lprn->sum[i] / n;
 	if (lprn->diff[i] == 0) {
 	    sd = 0.0;
@@ -2333,7 +2201,7 @@ static void print_loop_prn (LOOP_PRINT *lprn, const DATAINFO *pdinfo,
 	    m = (lprn->ssq[i] - n * mean * mean) / n;
 	    sd = (m < 0)? 0 : sqrt((double) m);
 	}
-	pprintf(prn, "%*s", VNAMELEN - 1, pdinfo->varname[vi]);
+	pprintf(prn, "%*s", VNAMELEN - 1, s);
 	pprintf(prn, "%#14g %#14g\n", (double) mean, (double) sd);
     }
 #endif
@@ -2435,8 +2303,10 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 			int *subst)
 {
     char targ[VNAMELEN + 3] = {0};
-    char *p;
+    char ins[32];
+    char *p, *pins;
     int targlen;
+    double forval = 0;
     int idx = 0;
     int err = 0;
 
@@ -2445,7 +2315,11 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 #endif
 
     if (loop->type == FOR_LOOP) {
-	sprintf(targ, "$%s", pdinfo->varname[loop->init.vnum]);
+	if (!gretl_is_scalar(loop->init.vname)) {
+	    /* nothing to substitute */
+	    return 0;
+	} 
+	sprintf(targ, "$%s", loop->init.vname);
 	targlen = strlen(targ);
     } else if (indexed_loop(loop)) {
 	targ[0] = '$';
@@ -2457,15 +2331,31 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
     }
 
 #if SUBST_DEBUG
-    fprintf(stderr, " target = '%s'\n", targ);
+    fprintf(stderr, " target = '%s', idx = %d\n", targ, idx);
 #endif
 
+    if (strstr(str, targ) == NULL) {
+	/* nothing to be done */
+	return 0;
+    }
+
+    pins = ins;
+
+    /* prepare substitute */
+    if (loop->type == FOR_LOOP) {
+	forval = gretl_scalar_get_value(loop->init.vname);
+	/* the rest is handled below */
+    } else if (loop->type == INDEX_LOOP) {
+	sprintf(ins, "%d", idx);
+    } else if (loop->type == DATED_LOOP) {
+	ntodate(ins, idx, pdinfo);
+    } else if (loop->type == EACH_LOOP) {
+	pins = loop->eachstrs[idx - 1];
+    }  
+
     while ((p = strstr(str, targ)) != NULL) {
-	char ins[32];
-	char *pins = ins;
-	char *q;
+	char *q = malloc(strlen(p));
 
-	q = malloc(strlen(p));
 	if (q == NULL) {
 	    err = 1;
 	    break;
@@ -2474,8 +2364,7 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 	strcpy(q, p + targlen);
 
 	if (loop->type == FOR_LOOP) {
-	    sprintf(ins, "%g", Z[loop->init.vnum][0]); /* scalar */
-
+	    sprintf(ins, "%g", forval);
 	    if (p - str > 0 && *(p - 1) == '[' && *(p + targlen) == ']') {
 		/* got an obs-type string, on the pattern [$lvar] */
 		int t = dateton(ins, pdinfo);
@@ -2485,12 +2374,6 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
 		}
 		sprintf(ins, "%d", t);
 	    } 
-	} else if (loop->type == INDEX_LOOP) {
-	    sprintf(ins, "%d", idx);
-	} else if (loop->type == DATED_LOOP) {
-	    ntodate(ins, idx, pdinfo);
-	} else if (loop->type == EACH_LOOP) {
-	    pins = loop->eachstrs[idx - 1];
 	} 
 
 	strcpy(p, pins);
@@ -2533,55 +2416,6 @@ static int extend_loop_dataset (LOOP_STORE *lstore)
     return 0;
 }
 
-static int clr_attach_var (controller *clr, const DATAINFO *pdinfo)
-{
-    int v = varindex(pdinfo, clr->vname);
-    int err = 0;
-
-    if (v >= pdinfo->v) {
-	sprintf(gretl_errmsg, 
-		_("Undefined variable '%s' in loop condition."), clr->vname);
-	err = 1;
-    } else {
-	clr->vnum = v;
-    } 
-
-    return err;
-}
-
-/* Connect up any loop control variables that were undefined at
-   compile time, and flag an error if any are still undefined
-   at run time.
-*/
-
-static int 
-connect_loop_control_vars (LOOPSET *loop, const DATAINFO *pdinfo)
-{
-    int err = 0;
-
-    if (loop->init.vnum == LOOP_VAL_UNDEF) {
-	err = clr_attach_var(&loop->init, pdinfo);
-    }
-
-    if (!err && loop->test.vnum == LOOP_VAL_UNDEF) {
-	err = clr_attach_var(&loop->test, pdinfo);
-    }
-
-    if (!err && loop->delta.vnum == LOOP_VAL_UNDEF) {
-	err = clr_attach_var(&loop->delta, pdinfo);
-    }
-
-    if (!err && loop->final.vnum == LOOP_VAL_UNDEF) {
-	err = clr_attach_var(&loop->final, pdinfo);
-    }
-
-    if (!err && is_list_loop(loop)) {
-	loop_list_refresh(loop, pdinfo);
-    }
-
-    return err;
-}
-
 static void progressive_loop_zero (LOOPSET *loop)
 {
     int i;
@@ -2606,31 +2440,31 @@ static void progressive_loop_zero (LOOPSET *loop)
 static int 
 top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 {
-    int err;
+    int err = 0;
 
     loop->iter = 0;
 
-    err = connect_loop_control_vars(loop, pdinfo);
+    if (is_list_loop(loop)) {
+	err = loop_list_refresh(loop, pdinfo);
+    } else if (loop->type == INDEX_LOOP) {
+	loop->init.val = loop_get_initval(loop, pZ, pdinfo);
+    } else if (loop->type == FOR_LOOP) {
+	forloop_init(loop, pZ, pdinfo, &err);
+    }
 
-    if (!err) {
-	if (loop->init.vnum > 0) {
-	    loop_initval(loop, pZ, pdinfo);
-	} 
+    if (!err && (loop->type == COUNT_LOOP || indexed_loop(loop))) {
+	double maxval = loop_get_finalval(loop, pZ, pdinfo);
 
-	if (loop->type == COUNT_LOOP || indexed_loop(loop)) {
-	    double maxval = loop_finalval(loop, pZ, pdinfo);
-
-	    if (na(loop->init.val) || na(maxval)) {
-		err = 1;
-	    } else {
-		loop->itermax = maxval - loop->init.val + 1;
+	if (na(loop->init.val) || na(maxval)) {
+	    err = 1;
+	} else {
+	    loop->itermax = maxval - loop->init.val + 1;
 #if LOOP_DEBUG
-		fprintf(stderr, "*** itermax = %g - %g + 1 = %d\n",
-			maxval, loop->init.val, loop->itermax);
+	    fprintf(stderr, "*** itermax = %g - %g + 1 = %d\n",
+		    maxval, loop->init.val, loop->itermax);
 #endif
-	    }
 	}
-    } 
+    }
 
     if (!err) {
 	if (indexed_loop(loop)) {
@@ -2858,7 +2692,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     err = top_of_loop(loop, pZ, pdinfo);
     
-    while (!err && loop_condition(loop, pZ, pdinfo)) {
+    while (!err && loop_condition(loop, pZ, pdinfo, &err)) {
 #if LOOP_DEBUG
 	fprintf(stderr, "top of loop: iter = %d\n", loop->iter);
 #endif
@@ -2888,6 +2722,9 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    /* We already have the "ci" index recorded, but here
 	       we do some further parsing. 
 	    */
+	    if (loop_is_progressive(loop)) {
+		cmd->flags |= CMD_PROG;
+	    }
 	    err = parse_command_line(line, cmd, pZ, pdinfo);
 
 	    if (cmd->ci < 0) {
