@@ -29,6 +29,7 @@
 #include "gretl_func.h"
 #include "nlspec.h"
 #include "cmd_private.h"
+#include "gretl_scalar.h"
 
 #include "gretl_f2c.h"
 #include "../../minpack/minpack.h"  
@@ -45,19 +46,21 @@ enum {
 } nls_modes;
 
 struct parm_ {
-    char name[VNAMELEN];  /* name of parameter (scalar or vector) */
+    char name[VNAMELEN];  /* name of parameter */
+    int type;             /* type of parameter (scalar or vector) */
     char *deriv;          /* string representation of derivative of regression
 			     function with respect to param (or NULL) */
-    int vnum;             /* ID number of scalar variable in dataset */
     int dnum;             /* ID number of variable holding the derivative */
     int nc;               /* number of individual coefficients associated
 			     with the parameter */
+    char dname[VNAMELEN]; /* name of variable holding the derivative */
     gretl_matrix *vec;    /* pointer to vector parameter */
     gretl_matrix *dmat;   /* pointer to matrix derivative */
 };
 
-#define scalar_param(s,i) (s->params[i].vnum > 0)
+#define scalar_param(s,i) (s->params[i].type == GRETL_TYPE_DOUBLE)
 #define matrix_deriv(s,i) (s->params[i].dmat != NULL)
+#define scalar_deriv(s,i) (gretl_is_scalar(s->params[i].dname))
 
 /* file-scope global variables */
 
@@ -146,15 +149,11 @@ static int nls_genr_setup (nlspec *s)
 	genrs[i] = NULL;
     }
 
-#if 0 
-    /* namespace problems when two or more non-nested functions
-       are called in an NLS/MLE/GMM block! */
-    set_drop_function_vars(0);
-#endif
-
     j = 0;
 
     for (i=0; i<ngen && !err; i++) {
+	char *dname = NULL;
+
 	if (i < s->naux) {
 	    /* auxiliary variables */
 	    strcpy(formula, s->aux[i]);
@@ -167,12 +166,15 @@ static int nls_genr_setup (nlspec *s)
 	    }
 	} else {
 	    /* derivatives/gradients */
+	    sprintf(s->params[j].dname, "$nl_x%d", i);
 	    if (scalar_param(s, j)) {
-		sprintf(formula, "$nl_x%d = %s", i, s->params[j].deriv);
+		sprintf(formula, "%s = %s", s->params[j].dname, 
+			s->params[j].deriv);
 	    } else {
-		sprintf(formula, "matrix $nl_x%d = %s", i, 
+		sprintf(formula, "matrix %s = %s",  s->params[j].dname,
 			s->params[j].deriv);
 	    }
+	    dname = s->params[j].dname;
 	    j++;
 	}
 	
@@ -203,9 +205,10 @@ static int nls_genr_setup (nlspec *s)
 	m = genr_get_output_matrix(genrs[i]);
 
 	if (v == 0 && m == NULL) {
+	    /* not a series, not a matrix: should be scalar */
 	    if (genr_is_print(genrs[i])) {
 		continue;
-	    } else {
+	    } else if (i >= s->naux && (dname == NULL || !gretl_is_scalar(dname))) {
 		fprintf(stderr, "nls_genr_setup: bad type: %s\n", formula);
 		err = E_TYPES;
 		break;
@@ -246,10 +249,6 @@ static int nls_genr_setup (nlspec *s)
 	s->genrs = genrs;
     }
 
-    if (err) {
-	set_drop_function_vars(1);
-    }
-
     return err;
 }
 
@@ -262,6 +261,8 @@ static int nls_auto_genr (nlspec *s, int i)
 {
     int j;
 
+    s->generr = 0;
+
 #if NLS_DEBUG
     fprintf(stderr, "nls_auto_genr: input i = %d\n", i);
 #endif
@@ -279,6 +280,9 @@ static int nls_auto_genr (nlspec *s, int i)
 	fprintf(stderr, " generating aux var %d:\n %s\n", j, s->aux[j]);
 #endif
 	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
+	if (s->generr) {
+	    return s->generr;
+	}
     }
 
     if (i == 0 && s->nlfunc == NULL) {
@@ -288,9 +292,10 @@ static int nls_auto_genr (nlspec *s, int i)
 
     j = s->naux + i;
 #if NLS_DEBUG
-    fprintf(stderr, " j = naux+i = %d+%d = %d: executing genr[%d]:\n", 
-	    s->naux, i, j, j);
+    fprintf(stderr, " j = naux+i = %d+%d = %d: executing genr[%d] at %p:\n", 
+	    s->naux, i, j, j, (void *) s->genrs[j]);
     fprintf(stderr, " %s\n", genr_get_formula(s->genrs[j]));
+    fprintf(stderr, " Z[%d] = %p\n", s->dinfo->v-1, (void *) (*s->Z)[s->dinfo->v-1]);
 #endif
     s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
 
@@ -388,11 +393,9 @@ static int push_scalar_coeff (nlspec *s, double x)
     return 0;
 }
 
-static int nlspec_push_param (nlspec *s, const char *name,
-			      int v, const double **Z,
-			      char *deriv)
+static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
 {
-    parm *params;
+    parm *params, *p;
     int np = s->nparam;
     int err;
     
@@ -401,25 +404,27 @@ static int nlspec_push_param (nlspec *s, const char *name,
 	return E_ALLOC;
     }
 
-    params[np].name[0] = '\0';
-    strncat(params[np].name, name, VNAMELEN - 1);
-    params[np].deriv = deriv;
-    params[np].vnum = v;
-    params[np].dnum = 0;
-    params[np].nc = 0;
-    params[np].vec = NULL;
-    params[np].dmat = NULL;
+    p = &params[np];
+
+    p->name[0] = '\0';
+    strncat(p->name, name, VNAMELEN - 1);
+    p->type = GRETL_TYPE_DOUBLE;
+    p->deriv = deriv;
+    p->dnum = 0;
+    p->nc = 1;
+    p->dname[0] = '\0';
+    p->vec = NULL;
+    p->dmat = NULL;
 
 #if NLS_DEBUG
-    fprintf(stderr, "added param[%d] = '%s' (v = %d)\n", np, params[np].name, v);
+    fprintf(stderr, "added param[%d] = '%s'\n", np, p->name);
 #endif
 
     s->params = params;
     s->nparam = np + 1;
 
-    if (v > 0) {
-	s->params[np].nc = 1;
-	err = push_scalar_coeff(s, Z[v][0]);
+    if (gretl_is_scalar(name)) {
+	err = push_scalar_coeff(s, gretl_scalar_get_value(name));
     } else {
 	gretl_matrix *m = get_matrix_by_name(name);
 	int k = gretl_vector_get_length(m);
@@ -428,8 +433,9 @@ static int nlspec_push_param (nlspec *s, const char *name,
 	fprintf(stderr, "vector param: m = %p, k = %d\n", (void *) m, k);
 #endif
 
-	s->params[np].vec = m;
-	s->params[np].nc = k;
+	p->type = GRETL_TYPE_MATRIX;
+	p->vec = m;
+	p->nc = k;
 	err = push_vec_coeffs(s, m, k);
 	if (!err) {
 	    s->nvec += 1;
@@ -439,32 +445,19 @@ static int nlspec_push_param (nlspec *s, const char *name,
     return err;
 }
 
-static int 
-check_param_name (const char *name, const DATAINFO *pdinfo, int *pv)
+static int check_param_name (const char *name)
 {
-    int v = varindex(pdinfo, name);
-
-    if (v == 0) {
-	return E_DATA;
-    } else if (v >= pdinfo->v) {
-	gretl_matrix *m = get_matrix_by_name(name);
-	
-	if (m != NULL) {
-	    if (gretl_vector_get_length(m) > 0) {
-		*pv = 0;
-	    } else {
-		return E_DATATYPE;
-	    }
-	} else {
-	    return E_UNKVAR;
-	}
-    } else if (!var_is_scalar(pdinfo, v)) {
-	return E_DATATYPE;
+    if (gretl_is_scalar(name)) {
+	return 0;
     } else {
-	*pv = v;
-    }
+	gretl_matrix *m = get_matrix_by_name(name);
 
-    return 0;
+	if (gretl_vector_get_length(m) == 0) {
+	    return E_DATATYPE;
+	} else {
+	    return 0;
+	}
+    }
 }
 
 /* Scrutinize word and see if it's a new scalar that should be added
@@ -473,30 +466,18 @@ check_param_name (const char *name, const DATAINFO *pdinfo, int *pv)
    if no "deriv" or "params" lines have been provided.
 */
 
-static int 
-maybe_add_param_to_spec (nlspec *s, const char *word, 
-			 const double **Z, const DATAINFO *pdinfo)
+static int maybe_add_param_to_spec (nlspec *s, const char *word)
 {
-    int i, v, err;
+    int i, err = 0;
 
 #if NLS_DEBUG
     fprintf(stderr, "maybe_add_param: looking at '%s'\n", word);
 #endif
 
-    if (function_from_string(word) || const_lookup(word)) {
-	return 0;
-    }
+    /* FIXME ? ref back to CVS? */
 
-    v = varindex(pdinfo, word);
-
-    if (v < pdinfo->v) {
-	if (var_is_series(pdinfo, v)) {
-	    /* only scalars are wanted here */
-	    return 0;
-	}
-    } else {
-	sprintf(gretl_errmsg, _("Unknown variable '%s'"), word);
-	return E_UNKVAR;
+    if (!gretl_is_scalar(word)) {
+	return 0; /* ?? */
     }
 
     /* if this term is already present in the spec, skip it */
@@ -510,7 +491,7 @@ maybe_add_param_to_spec (nlspec *s, const char *word,
     fprintf(stderr, "maybe_add_param: adding '%s'\n", word);
 #endif
 
-    err = nlspec_push_param(s, word, v, Z, NULL);
+    err = nlspec_push_param(s, word, NULL);
 
     return err;
 }
@@ -521,9 +502,7 @@ maybe_add_param_to_spec (nlspec *s, const char *word,
    line.
 */
 
-static int 
-get_params_from_nlfunc (nlspec *s, const double **Z,
-			const DATAINFO *pdinfo)
+static int get_params_from_nlfunc (nlspec *s)
 {
     const char *f = s->nlfunc;
     const char *p;
@@ -543,17 +522,14 @@ get_params_from_nlfunc (nlspec *s, const double **Z,
 	p = f;
 	if (isalpha(*f) && *(f + 1)) { 
 	    /* find a variable name */
-	    n = gretl_varchar_spn(f);
-	    p += n;
-	    if (n > VNAMELEN - 1) {
-		/* variable name is too long */
-		return 1;
+	    err = extract_varname(name, f, &n);
+	    if (err) {
+		return err;
 	    }
-	    *name = 0;
-	    strncat(name, f, n);
+	    p += n;
 	    if (np > 0) {
 		/* right-hand side term */
-		err = maybe_add_param_to_spec(s, name, Z, pdinfo);
+		err = maybe_add_param_to_spec(s, name);
 	    }
 	    np++;
 	} else {
@@ -581,7 +557,7 @@ nlspec_add_params_from_line (nlspec *s, const char *str,
 			     const double **Z, const DATAINFO *pdinfo)
 {
     int i, nf = count_fields(str);
-    int v, err = 0;
+    int err = 0;
 
     if (s->params != NULL || nf == 0) {
 	return E_DATA;
@@ -597,14 +573,12 @@ nlspec_add_params_from_line (nlspec *s, const char *str,
 
 	if (name == NULL) {
 	    err = E_ALLOC;
+	} else {
+	    err = check_param_name(name);
 	}
 
 	if (!err) {
-	    err = check_param_name(name, pdinfo, &v);
-	}
-
-	if (!err) {
-	    err = nlspec_push_param(s, name, v, Z, NULL);
+	    err = nlspec_push_param(s, name, NULL);
 	}
 
 	free(name);
@@ -620,33 +594,30 @@ nlspec_add_params_from_line (nlspec *s, const char *str,
 /**
  * nlspec_add_param_list:
  * @spec: nls specification.
- * @list: list of variables by ID number.
- * @Z: data array.
- * @pdinfo: information on dataset.
+ * @np: number of parameters.
+ * @vals: array of initial parameter values.
+ * @names: array of parameter names.
  *
- * Adds to @spec a list of (scalar) parameters to be estimated, as
- * given in @list.  For an example of use see arma.c in the
- * gretl plugin directory.
+ * Adds to @spec a list of (scalar) parameters to be estimated.
+ * For an example of use see arma.c in the gretl plugin directory.
  *
  * Returns: 0 on success, non-zero error code on error.
  */
 
-int nlspec_add_param_list (nlspec *spec, const int *list,
-			   const double **Z, const DATAINFO *pdinfo)
+int nlspec_add_param_list (nlspec *spec, int np, double *vals,
+			   char **names, double ***pZ,
+			   DATAINFO *pdinfo)
 {
-    int i, v, np = list[0];
-    int err = 0;
+    int i, err = 0;
 
     if (spec->params != NULL || np == 0) {
 	return E_DATA;
     }
 
     for (i=0; i<np && !err; i++) {
-	v = list[i+1];
-	if (v > 0 && v < pdinfo->v && var_is_scalar(pdinfo, v)) {
-	    err = nlspec_push_param(spec, pdinfo->varname[v], v, Z, NULL);
-	} else {
-	    err = E_DATA;
+	err = gretl_scalar_add(names[i], vals[i]);
+	if (!err) {
+	    err = nlspec_push_param(spec, names[i], NULL);
 	}
     }
 
@@ -657,69 +628,114 @@ int nlspec_add_param_list (nlspec *spec, const int *list,
     return err;
 }
 
-/* return the address in the dataset, or in a user matrix, that
-   holds the "external" value of a given coefficient */
+/* update the 'external' values of scalars or matrices using
+   the values produced by the optimizer.
+*/
 
-static double *coeff_address (nlspec *s, int i)
+int update_coeff_values (const double *b, nlspec *s)
 {
-    double **Z = *(s->Z);
-    int j, k, pos = 0;
+    int i, j, k = 0;
+    parm *p;
 
-    if (s->nvec == 0) {
-	/* the mapping is simple */
-	return &(Z[s->params[i].vnum][0]);
-    }
-
-    for (j=0; j<s->nparam; j++) {
-	if (scalar_param(s, j)) {
-	    if (i == pos) {
-		return &(Z[s->params[j].vnum][0]);
-	    }
-	    pos++;
+    for (i=0; i<s->nparam; i++) {
+	p = &s->params[i];
+	if (scalar_param(s, i)) {
+	    gretl_scalar_set_value(p->name, b[k++]);
 	} else {
-	    gretl_matrix *m = get_matrix_by_name(s->params[j].name);
+	    gretl_matrix *m = get_matrix_by_name(p->name);
 
-	    if (m != s->params[j].vec) {
-		fprintf(stderr, "*** coeff_address: by name, '%s' is at %p; "
-			"stored addr = %p\n", s->params[j].name,
-			(void *) m, (void *) s->params[j].vec);
-		s->params[j].vec = m;
+	    if (m == NULL) {
+		fprintf(stderr, "Couldn't find location for coeff %d\n", k);
+		return E_DATA;
+	    } else {
+		if (m != p->vec) {
+		    fprintf(stderr, "*** coeff_address: by name, '%s' is at %p; "
+			    "stored addr = %p\n", p->name,
+			    (void *) m, (void *) p->vec);
+		    p->vec = m;
+		}
+		for (j=0; j<p->nc; j++) {
+		    gretl_vector_set(m, j, b[k++]);
+		}
 	    }
-
-	    k = s->params[j].nc;
-	    if (i >= pos && i < pos + k) {
-		return &(s->params[j].vec->val[i - pos]);
-	    }
-	    pos += k;
 	}
     }
 
-    fprintf(stderr, "Couldn't find location for coeff %d\n", i);
-	
-    return NULL;
-}
-
-int update_coeff_values (const double *x, nlspec *s)
-{
-    double *d;
-    int i;
-
-    /* write the values produced by the optimizer into the dataset */
-
-    for (i=0; i<s->ncoeff; i++) {
-	d = coeff_address(s, i);
-	if (d == NULL) {
-	    return 1;
-	}
-	*d = x[i];
-#if NLS_DEBUG
-	fprintf(stderr, "update params: revised coeff[%d] = %.14g\n", i, x[i]);
-#endif
-    }
-
     return 0;
 }
 
+#define NLS_SKIP_MISSING 0 /* not properly tested yet */
+
+#if NLS_SKIP_MISSING
+
+static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
+{
+    DATAINFO *dinfo = NULL;
+    double ***pZ = NULL;
+    double **origZ = *spec->Z;
+    int nvar = spec->dinfo->v;
+    int nobs = 0;
+    int i, t, s;
+
+    spec->missmask = malloc(spec->dinfo->n + 1);
+    if (spec->missmask == NULL) {
+	return E_ALLOC;
+    }
+
+    spec->missmask[spec->dinfo->n] = '\0';
+    memset(spec->missmask, '0', spec->dinfo->n);
+
+    for (t=t1; t<=t2; t++) {
+	if (na(origZ[spec->lhv][t])) {
+	    spec->missmask[t] = '1';
+	} else {
+	    nobs++;
+	}
+    }
+
+    if (nobs < spec->ncoeff) {
+	return E_DF;
+    }
+
+    pZ = malloc(sizeof *pZ);
+    if (pZ == NULL) {
+	return E_ALLOC;
+    }
+
+    dinfo = create_auxiliary_dataset(pZ, nvar, nobs);
+    if (dinfo == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<nvar; i++) {
+	strcpy(dinfo->varname[i], spec->dinfo->varname[i]);
+	s = 0;
+	for (t=t1; t<=t2; t++) {
+	    if (!na(origZ[spec->lhv][t])) {
+		(*pZ)[i][s++] = origZ[i][t];
+	    }
+	}
+    }
+
+    spec->real_t1 = t1;
+    spec->real_t2 = t2;
+
+    spec->Z = pZ;
+    spec->dinfo = dinfo;
+    spec->t1 = 0;
+    spec->t2 = nobs - 1;
+    spec->nobs = nobs;
+
+#if NLS_DEBUG
+    fprintf(stderr, "s->t1 = %d, s->t2 = %d, s->nobs = %d, nvar = %d\n",
+	    spec->t1, spec->t2, spec->nobs, nvar);
+#endif
+
+    return 0;
+}
+
+#endif
+
 /* Adjust starting and ending points of sample if need be, to avoid
    missing values; abort if there are missing values within the
    (possibly reduced) sample range.  For this purpose we generate the
@@ -782,14 +798,18 @@ static int nl_missval_check (nlspec *s)
 	if (na((*s->Z)[v][t])) {
 	    fprintf(stderr, "  after setting t1=%d, t2=%d, "
 		    "got NA for var %d at obs %d\n", t1, t2, v, t);
+#if NLS_SKIP_MISSING
+	    return nls_make_trimmed_dataset(s, t1, t2);
+#else
 	    return E_MISSDATA;
+#endif
 	}
     }  
 
  nl_miss_exit:
 
-    s->t1 = t1;
-    s->t2 = t2;
+    s->t1 = s->real_t1 = t1;
+    s->t2 = s->real_t2 = t2;
     s->nobs = t2 - t1 + 1;
 
 #if NLS_DEBUG
@@ -797,7 +817,7 @@ static int nl_missval_check (nlspec *s)
 	    s->t1, s->t2, s->nobs);
 #endif
 
-    return 0;
+    return err;
 }
 
 /* the next two functions are used in the context of BFGS */
@@ -1023,23 +1043,36 @@ static int get_nls_derivs (int k, int T, int offset, double *g, double **G,
 		    gi = *(++G) + offset;
 		}
 	    }
+	} else if (scalar_deriv(spec, j)) {
+	    /* transcribe from scalar var to array g */
+	    x = gretl_scalar_get_value(spec->params[j].dname);
+	    for (t=0; t<T; t++) {
+		gi[t] = (spec->ci == MLE)? x : -x;
+	    }
+	    if (j < spec->nparam - 1) {
+		/* advance the writing position */
+		if (g != NULL) {
+		    gi += T;
+		} else {
+		    gi = *(++G) + offset;
+		}
+	    }	    
 	} else {
-	    /* derivative is scalar or series */
-	    int s, ser, v = spec->params[j].dnum;
+	    /* derivative is series */
+	    int s, v = spec->params[j].dnum;
 
 	    if (v == 0) {
 		/* FIXME? */
 		v = spec->params[j].dnum = spec->dinfo->v - 1;
 	    }
-	    ser = var_is_series(spec->dinfo, v);
 
 #if NLS_DEBUG
-	    fprintf(stderr, "param[%d]: dnum = %d, series = %d\n", j, v, ser);
+	    fprintf(stderr, "param[%d]: dnum = %d\n", j, v);
 #endif
 
 	    /* transcribe from dataset to array g */
 	    for (t=0; t<T; t++) {
-		s = (ser)? (t + spec->t1) : 0;
+		s = t + spec->t1;
 		x = (*spec->Z)[v][s];
 		gi[t] = (spec->ci == MLE)? x : -x;
 #if NLS_DEBUG > 1
@@ -1047,7 +1080,6 @@ static int get_nls_derivs (int k, int T, int offset, double *g, double **G,
 			t, v, s, gi[t]);
 #endif
 	    }
-
 	    if (j < spec->nparam - 1) {
 		/* advance the writing position */
 		if (g != NULL) {
@@ -1071,7 +1103,6 @@ static int get_mle_gradient (double *b, double *g, int n,
     nlspec *spec = (nlspec *) p;
     gretl_matrix *m;
     double x;
-    int t1, t2;
     int i, j, k, t;
     int err = 0;
 
@@ -1115,24 +1146,25 @@ static int get_mle_gradient (double *b, double *g, int n,
 #endif
 		i++;
 	    }
+	} else if (scalar_deriv(spec, j)) {
+	    x = gretl_scalar_get_value(spec->params[j].dname);
+	    if (na(x)) {
+		fprintf(stderr, "NA in gradient calculation\n");
+		err = 1;
+	    } else {
+		g[i] = x;
+	    }
 	} else {
-	    /* derivative may be series or scalar */
+	    /* derivative must be series */
 	    int v = spec->params[j].dnum;
 
 #if ML_DEBUG > 1
 	    fprintf(stderr, "param[%d], dnum = %d\n", j, v);
 #endif
 
-	    if (var_is_series(spec->dinfo, v)) {
-		t1 = spec->t1;
-		t2 = spec->t2;
-	    } else {
-		t1 = t2 = 0;
-	    }
-
 	    g[i] = 0.0;
 
-	    for (t=t1; t<=t2; t++) {
+	    for (t=spec->t1; t<=spec->t2; t++) {
 		x = (*spec->Z)[v][t];
 #if ML_DEBUG > 1
 		fprintf(stderr, "s->Z[%d][%d] = %g\n", v, t, x);
@@ -1163,7 +1195,7 @@ static void add_stats_to_model (MODEL *pmod, nlspec *spec,
 {
     int dv = spec->dv;
     double d, tss;
-    int t;
+    int s, t;
 
     pmod->ess = spec->crit;
     pmod->sigma = sqrt(pmod->ess / (pmod->nobs - spec->ncoeff));
@@ -1171,11 +1203,13 @@ static void add_stats_to_model (MODEL *pmod, nlspec *spec,
     pmod->ybar = gretl_mean(pmod->t1, pmod->t2, Z[dv]);
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, Z[dv]);
 
+    s = (spec->missmask != NULL)? 0 : pmod->t1;
+
     tss = 0.0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	d = Z[dv][t] - pmod->ybar;
+	d = Z[dv][s++] - pmod->ybar;
 	tss += d * d;
-    }  
+    } 
 
     if (tss == 0.0) {
 	pmod->rsq = NADBL;
@@ -1356,15 +1390,16 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
 		    }
 		    j++;
 		}
+	    } else if (scalar_deriv(spec, i)) {
+		x = gretl_scalar_get_value(spec->params[i].dname);
+		for (t=0; t<T; t++) {
+		    gretl_matrix_set(G, j, t, x);
+		}
+		j++;
 	    } else {
 		v = spec->params[i].dnum;
-		if (var_is_scalar(spec->dinfo, v)) {
-		    x = (*spec->Z)[v][0];
-		}
 		for (t=0; t<T; t++) {
-		    if (var_is_series(spec->dinfo, v)) {
-			x = (*spec->Z)[v][t + spec->t1];
-		    }
+		    x = (*spec->Z)[v][t + spec->t1];
 		    gretl_matrix_set(G, j, t, x);
 		}
 		j++;
@@ -1580,9 +1615,8 @@ print_GNR_dataset (const int *list, double **gZ, DATAINFO *gdinfo)
    which is why we make the artificial dataset, gZ, full length.
 */
 
-static MODEL GNR (double *uhat, double *jac, nlspec *spec,
-		  double ***pZ, const DATAINFO *pdinfo, 
-		  PRN *prn)
+static MODEL GNR (double *uhat, double *jac, nlspec *spec, 
+		  const double **Z, DATAINFO *pdinfo, PRN *prn)
 {
     double **gZ = NULL;
     DATAINFO *gdinfo;
@@ -1595,11 +1629,19 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
     int perfect = 0;
     int err = 0;
 
+#if NLS_DEBUG
+    fprintf(stderr, "GNR: T = %d\n", T);
+    v = pdinfo->v;
+    fprintf(stderr, "dinfo->v = %d\n", v);
+    fprintf(stderr, "Z = %p\n", (void *) Z);
+    fprintf(stderr, "Z[%d] = %p\n", v-1, (void *) Z[v-1]);
+#endif
+
     if (gretl_iszero(0, spec->nobs - 1, uhat)) {
 	pputs(prn, _("Perfect fit achieved\n"));
 	perfect = 1;
 	for (t=0; t<spec->nobs; t++) {
-	    uhat[t] = 1.0;
+	    uhat[t] = 1.0; /* will be adjusted later */
 	}
 	spec->crit = 0.0;
     }
@@ -1617,11 +1659,6 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
     gdinfo->t1 = spec->t1;
     gdinfo->t2 = spec->t2;
 
-#if 0
-    fprintf(stderr, "pdinfo->n = %d, gdinfo->t1 = %d, gdinfo->t2 = %d\n",
-	    pdinfo->n, gdinfo->t1, gdinfo->t2);
-#endif
-    
     glist = gretl_list_new(spec->ncoeff + 1);
 
     if (glist == NULL) {
@@ -1661,6 +1698,10 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
 		gZ[v][t] = NADBL;
 	    }
 	}
+#if NLS_DEBUG
+	fprintf(stderr, "GNR: calling get_nls_derivs\n");
+	fprintf(stderr, "Z[%d] = %p\n", pdinfo->v-1, (void *) Z[pdinfo->v-1]);
+#endif
 	get_nls_derivs(spec->nparam, T, spec->t1, NULL, gZ + 2, spec);
     } else {
 	for (i=0; i<spec->ncoeff; i++) {
@@ -1680,7 +1721,7 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
     print_GNR_dataset(glist, gZ, gdinfo);
 #endif
 
-    lsqopt = OPT_A;
+    lsqopt = OPT_A | OPT_B; /* OPT_B: don't calculate R^2 */
     if (spec->opt & OPT_R) {
 	/* robust variance matrix, if wanted */
 	lsqopt |= OPT_R;
@@ -1708,7 +1749,7 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
 
     if (gnr.errcode == 0) {
 	gnr.ci = spec->ci;
-	add_stats_to_model(&gnr, spec, (const double **) *pZ);
+	add_stats_to_model(&gnr, spec, Z);
 	if (add_nls_std_errs_to_model(&gnr)) {
 	    gnr.errcode = E_ALLOC;
 	}
@@ -1718,8 +1759,7 @@ static MODEL GNR (double *uhat, double *jac, nlspec *spec,
 	ls_criteria(&gnr);
 	add_coeffs_to_model(&gnr, spec->coeff);
 	add_param_names_to_model(&gnr, spec, pdinfo);
-	add_fit_resid_to_model(&gnr, spec, uhat, (const double **) *pZ, 
-			       perfect);
+	add_fit_resid_to_model(&gnr, spec, uhat, Z, perfect);
 	gnr.list[1] = spec->dv;
 
 	/* set relevant data on model to be shipped out */
@@ -1878,6 +1918,7 @@ static void clear_nlspec (nlspec *spec)
     spec->grcount = 0;
 
     spec->t1 = spec->t2 = 0;
+    spec->real_t1 = spec->real_t2 = 0;
     spec->nobs = 0;
 
     spec->Z = NULL;
@@ -1888,6 +1929,9 @@ static void clear_nlspec (nlspec *spec)
 	oc_set_destroy(spec->oc);
 	spec->oc = NULL;
     }
+
+    free(spec->missmask);
+    spec->missmask = NULL;
 }
 
 /* 
@@ -2069,7 +2113,7 @@ static int mle_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
     if (!err) {
 	BFGS_GRAD_FUNC gradfun = (s->mode == ANALYTIC_DERIVS)?
 	    get_mle_gradient : NULL;
-	int maxit = libset_get_int(BFGS_MAXITER);
+	int maxit = 500;
 
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
 		       &s->fncount, &s->grcount, 
@@ -2311,7 +2355,7 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
     const char *p = dstr;
     char *name = NULL;
     char *deriv = NULL;
-    int v, err = 0;
+    int err = 0;
 
     if (spec->ci == GMM) {
 	strcpy(gretl_errmsg, _("Analytical derivatives cannot be used with GMM"));
@@ -2329,10 +2373,10 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
 	return E_PARSE;
     }
 
-    err = check_param_name(name, pdinfo, &v);
+    err = check_param_name(name);
     
     if (!err) {
-	err = nlspec_push_param(spec, name, v, Z, deriv);
+	err = nlspec_push_param(spec, name, deriv);
 	if (err) {
 	    free(deriv);
 	    deriv = NULL;
@@ -2345,20 +2389,12 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
 	spec->mode = ANALYTIC_DERIVS;
     }
 
-#if NLS_DEBUG
-    if (!err) {
-	fprintf(stderr, "add_param_with_deriv: '%s'\n"
-		" set vnum = %d, initial value = %.14g\n", dstr, 
-		v, Z[v][0]);
-    }
-#endif
-
     return err;
 }
 
 static void get_aux_command_word (char *word, const char *line)
 {
-    int n = gretl_varchar_spn(line);
+    int n = gretl_namechar_spn(line);
 
     *word = '\0';
 
@@ -2494,7 +2530,7 @@ nlspec_set_regression_function (nlspec *spec, const char *fnstr,
 	sprintf(gretl_errmsg, _("parse error in '%s'\n"), fnstr);
 	err =  E_PARSE;
     } else if (spec->ci == NLS) {
-	spec->dv = varindex(pdinfo, vname);
+	spec->dv = series_index(pdinfo, vname);
 	if (spec->dv == pdinfo->v) {
 	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), vname);
 	    err = E_UNKVAR;
@@ -2672,6 +2708,68 @@ static int check_spec_requirements (nlspec *spec)
     return 0;
 }
 
+/* remedial treatment for an NLS model estimated using
+   a trimmed dataset, to avoid missing observations
+*/
+
+static int nls_model_fix_sample (MODEL *pmod,
+				 nlspec *spec, 
+				 DATAINFO *pdinfo)
+{
+    double *uhat = NULL;
+    double *yhat = NULL;
+
+    if (pmod->uhat != NULL) {
+	uhat = malloc(pdinfo->n * sizeof *uhat);
+	if (uhat == NULL) {
+	    return E_ALLOC;
+	}
+    }
+
+    if (pmod->yhat != NULL) {
+	yhat = malloc(pdinfo->n * sizeof *yhat);
+	if (yhat == NULL) {
+	    free(uhat);
+	    return E_ALLOC;
+	}
+    }    
+
+    if (uhat != NULL || yhat != NULL) {
+	int t, s = 0;
+
+	for (t=0; t<pdinfo->n; t++) {
+	    if (t < spec->real_t1 || t > spec->real_t2 ||
+		spec->missmask[t] == '1') {
+		if (uhat != NULL) uhat[t] = NADBL;
+		if (yhat != NULL) yhat[t] = NADBL;
+	    } else {
+		if (uhat != NULL) uhat[t] = pmod->uhat[s];
+		if (yhat != NULL) yhat[t] = pmod->yhat[s];
+		s++;
+	    }
+	}
+
+	if (uhat != NULL) {
+	    free(pmod->uhat);
+	    pmod->uhat = uhat;
+	}
+
+	if (yhat != NULL) {
+	    free(pmod->yhat);
+	    pmod->yhat = yhat;
+	}
+    }
+
+    pmod->full_n = pdinfo->n;
+    pmod->t1 = spec->real_t1;
+    pmod->t2 = spec->real_t2;
+
+    pmod->missmask = spec->missmask;
+    spec->missmask = NULL;
+
+    return 0;
+}
+
 /* static function providing the real content for the two public
    wrapper functions below */
 
@@ -2685,7 +2783,7 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     int i, t, err = 0;
 
 #if NLS_DEBUG
-    fprintf(stderr, "real_nls: starting\n");
+    fprintf(stderr, "real_nls: starting, pdinfo->v = %d\n", origv);
 #endif
 
     gretl_model_init(&nlsmod);
@@ -2711,7 +2809,7 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 
     if (spec->mode == NUMERIC_DERIVS && spec->nparam == 0 &&
 	spec->ci != GMM) {
-	err = get_params_from_nlfunc(spec, (const double **) *pZ, pdinfo);
+	err = get_params_from_nlfunc(spec);
 	if (err) {
 	    if (err == 1) {
 		nlsmod.errcode = E_PARSE;
@@ -2795,7 +2893,8 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	    } else {
 		/* Use Gauss-Newton Regression for covariance matrix,
 		   standard errors */
-		nlsmod = GNR(fvec, jac, spec, pZ, pdinfo, prn);
+		nlsmod = GNR(fvec, jac, spec, (const double **) *spec->Z, 
+			     spec->dinfo, prn);
 	    }
 	} else {
 	    make_nl_model(&nlsmod, spec, pdinfo);
@@ -2819,16 +2918,29 @@ static MODEL real_nls (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	destroy_private_matrices();
     }
 
+    destroy_private_scalars();
+
+    if (spec->Z != pZ) {
+	if (!nlsmod.errcode) {
+	    nls_model_fix_sample(&nlsmod, spec, pdinfo);
+	}
+	destroy_dataset(*spec->Z, spec->dinfo);
+	free(spec->Z);
+    }
+
     clear_nlspec(spec);
 
+#if NLS_DEBUG
+    fprintf(stderr, "real_nls: finishing, dropping %d series\n",
+	    pdinfo->v - origv);
+#endif
+
     dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
 
     if (nlsmod.errcode == 0 && !(opt & OPT_A)) {
 	set_model_id(&nlsmod);
     }
 
-    set_drop_function_vars(1);
-
     return nlsmod;
 }
 
@@ -2929,8 +3041,8 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->fncount = 0;
     spec->grcount = 0;
 
-    spec->t1 = pdinfo->t1;
-    spec->t2 = pdinfo->t2;
+    spec->t1 = spec->real_t1 = pdinfo->t1;
+    spec->t2 = spec->real_t2 = pdinfo->t2;
     spec->nobs = spec->t2 - spec->t1 + 1;
 
     spec->Z = NULL;
@@ -2938,6 +3050,7 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->prn = NULL;
 
     spec->oc = NULL;
+    spec->missmask = NULL;
 
     return spec;
 }
@@ -3190,7 +3303,7 @@ double *numerical_hessian (double *b, int n, BFGS_CRIT_FUNC func, void *data,
  bailout:
 
     if (*err == E_NAN) {
-	fprintf(stderr, "Got E_NAN in numerical_hessian()\n");
+	gretl_errmsg_set(_("Failed to compute numerical Hessian"));
     }
 
     free(c);
@@ -3275,6 +3388,8 @@ static void BFGS_get_user_values (double *b, int n, int *maxit,
     umaxit = libset_get_int(BFGS_MAXITER);
     if (umaxit > 0) {
 	*maxit = umaxit;
+    } else if (*maxit < 0) {
+	*maxit = 500;
     }
     
     utol = libset_get_double(BFGS_TOLER);
@@ -3322,7 +3437,7 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     f = cfunc(b, data);
 
     if (na(f)) {
-	fprintf(stderr, "initial value of f is not finite\n");
+	gretl_errmsg_set(_("BFGS: initial value of objective function is not finite"));
 	err = E_DATA;
 	goto bailout;
     }
@@ -3758,8 +3873,7 @@ static double user_get_criterion (const double *b, void *p)
 {
     umax *u = (umax *) p;
     double x = NADBL;
-    int i, v;
-    int err;
+    int i, t, err;
 
     for (i=0; i<u->ncoeff; i++) {
 	u->b->val[i] = b[i];
@@ -3771,15 +3885,18 @@ static double user_get_criterion (const double *b, void *p)
 	return NADBL;
     }
 
-    v = genr_get_output_varnum(u->g);
-    if (v > 0) {
-	x = (*u->Z)[v][0];
-    } else {
+    t = genr_get_output_type(u->g);
+
+    if (t == GRETL_TYPE_DOUBLE) {
+	x = genr_get_output_scalar(u->g);
+    } else if (t == GRETL_TYPE_MATRIX) {
 	gretl_matrix *m = genr_get_output_matrix(u->g);
 
 	if (m != NULL && m->rows == 1 && m->cols == 1) {
 	    x = m->val[0];
 	}
+    } else {
+	err = E_TYPES;
     }
 
     u->x_out = x;
@@ -3828,7 +3945,7 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
     umax u;
     double ret = NADBL;
     gretlopt opt = OPT_NONE;
-    int maxit;
+    int maxit = 500;
     int fcount = 0, gcount = 0;
     double tol;
 
@@ -3847,7 +3964,6 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
 	return NADBL;
     }
 
-    maxit = libset_get_int(BFGS_MAXITER);
     tol = libset_get_double(BFGS_TOLER);
 
     if (libset_get_bool(MAX_VERBOSE)) {
