@@ -104,17 +104,21 @@ static void gretl_object_unstack (void *ptr)
     }
     
     if (pos >= 0) {
-	stacker *new_stack;
-
-	for (i=pos; i<n_obj-1; i++) {
-	    ostack[i] = ostack[i+1];
-	}
-
 	n_obj--;
+	if (n_obj == 0) {
+	    free(ostack);
+	    ostack = NULL;
+	} else {
+	    stacker *new_stack;
 
-	new_stack = realloc(ostack, (n_obj - 1) * sizeof *new_stack);
-	if (new_stack != NULL) {
-	    ostack = new_stack;
+	    for (i=pos; i<n_obj; i++) {
+		ostack[i] = ostack[i+1];
+	    }
+
+	    new_stack = realloc(ostack, n_obj * sizeof *new_stack);
+	    if (new_stack != NULL) {
+		ostack = new_stack;
+	    }
 	}
     }
 }
@@ -376,21 +380,32 @@ void *get_last_model (GretlObjType *type)
  *
  * Returns: the name of the object of type @type with
  * location @p, or %NULL if the object is not found.
+ * The return value may be ovewritten (up to
+ * MAXSAVENAME-1 characters), but must not be freed.
  */
 
-const char *gretl_object_get_name (void *p, GretlObjType type)
+char *gretl_object_get_name (void *p, GretlObjType type)
 {
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = p;
 
+	if (pmod->name == NULL) {
+	    pmod->name = calloc(MAXSAVENAME, 1);
+	}
 	return pmod->name;
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = p;
 
+	if (var->name == NULL) {
+	    var->name = calloc(MAXSAVENAME, 1);
+	}
 	return var->name;
     } else if (type == GRETL_OBJ_SYS) {
 	equation_system *sys = p;
 
+	if (sys->name == NULL) {
+	    sys->name = calloc(MAXSAVENAME, 1);
+	}
 	return sys->name;
     }
 
@@ -597,7 +612,7 @@ int gretl_object_rename (void *p, GretlObjType type, const char *oname)
 
 /* safety measure prior to freeing models on exit */
 
-void remove_model_from_stack (MODEL *pmod)
+void remove_model_from_stack_on_exit (MODEL *pmod)
 {
     int i;
 
@@ -617,6 +632,12 @@ void remove_model_from_stack (MODEL *pmod)
     gretl_model_unprotect(pmod);
 }
 
+void gretl_object_remove_from_stack (void *ptr, GretlObjType type)
+{
+    gretl_object_unstack(ptr);
+    gretl_object_unref(ptr, type);
+}
+
 static int object_stack_index (const void *p)
 {
     int i, ret = -1;
@@ -662,7 +683,7 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 	return 0;
     }
 
-    if (name == NULL) {
+    if (name == NULL || *name == '\0') {
 	err = gretl_object_compose_name(p, type);
     } else {
 	err = gretl_object_rename(p, type, name);
@@ -710,7 +731,9 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 
 int gretl_stack_object (void *ptr, GretlObjType type)
 {
-    return real_stack_object(ptr, type, NULL, NULL);
+    char *name = gretl_object_get_name(ptr, type);
+
+    return real_stack_object(ptr, type, name, NULL);
 }
 
 int gretl_stack_object_as (void *ptr, GretlObjType type, const char *name)
@@ -903,6 +926,27 @@ real_get_obj_matrix (void *p, GretlObjType type, int idx, int *err)
     return M;
 }
 
+static int *
+real_get_obj_list (void *p, GretlObjType type, int idx, int *err)
+{
+    int *list = NULL;
+    
+    if (idx <= 0) {
+	*err = 1;
+	return list;
+    }
+
+    if (type == GRETL_OBJ_EQN && idx == M_XLIST) {
+	MODEL *pmod = (MODEL *) p;
+
+	list = gretl_model_get_x_list(pmod);
+    } else {
+	*err = E_BADSTAT;
+    }
+
+    return list;
+}
+
 static stacker genr_model;
 
 void set_genr_model (MODEL *pmod)
@@ -975,6 +1019,20 @@ GretlObjType gretl_model_get_type_and_ci (const char *name,
     return ret;
 }
 
+int *saved_object_get_list (const char *oname, int idx, int *err)
+{
+    int *ret = NULL;
+    stacker *smatch;
+
+    smatch = find_smatch(oname);
+
+    if (smatch != NULL) {
+	ret = real_get_obj_list(smatch->ptr, smatch->type, idx, err);
+    }
+
+    return ret;
+}
+
 double saved_object_get_scalar (const char *oname, int idx, int *err)
 {
     double ret = INVALID_STAT;
@@ -1042,15 +1100,15 @@ saved_object_get_matrix (const char *oname, int idx, int *err)
     return M;
 }
 
-static int varchar_spn_with_space (const char *s)
+static int namechar_spn_with_space (const char *s)
 {
-    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+    const char *ok = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"0123456789_ ";
     int ret = 0;
 
     if (isalpha(*s)) {
-	ret = strspn(s, varchars);
+	ret = strspn(s, ok);
     }
 
     return ret;
@@ -1082,9 +1140,9 @@ int parse_object_command (const char *s, char *name, char **cmd)
     }
 
     if (quoted) {
-	len = varchar_spn_with_space(s);
+	len = namechar_spn_with_space(s);
     } else {
-	len = gretl_varchar_spn(s);
+	len = gretl_namechar_spn(s);
     }
 
     if (len == 0) {
@@ -1306,9 +1364,9 @@ int check_variable_deletion_list (int *list, const DATAINFO *pdinfo)
 
     vsave = highest_numbered_var_in_saved_object(pdinfo);
 
-    for (i=1; i<=list[0]; i++) {
+    for (i=list[0]; i>0; i--) {
 	if (list[i] <= vsave) {
-	    gretl_list_delete_at_pos(list, i--);
+	    gretl_list_delete_at_pos(list, i);
 	    pruned = 1;
 	}
     }
@@ -1354,3 +1412,4 @@ void gretl_saved_objects_cleanup (void)
 	last_model.type = 0;
     }
 }
+
