@@ -83,6 +83,7 @@ struct gretl_option gretl_opts[] = {
     { AR1,      OPT_P, "pwe"},
     { AR1,      OPT_Q, "quiet"},
     { APPEND,   OPT_T, "time-series" },
+    { APPEND,   OPT_Q, "quiet" },
     { ARBOND,   OPT_A, "asymptotic" },
     { ARBOND,   OPT_D, "time-dummies" },
     { ARBOND,   OPT_H, "orthdev" },
@@ -95,6 +96,7 @@ struct gretl_option gretl_opts[] = {
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
+    { CHOW,     OPT_Q, "quiet" },
     { COINT,    OPT_N, "nc" },
     { COINT,    OPT_R, "ctt" },     
     { COINT,    OPT_S, "skip-df" },
@@ -142,7 +144,9 @@ struct gretl_option gretl_opts[] = {
     { FREQ,     OPT_Q, "quiet" },
     { FREQ,     OPT_S, "silent" },
     { FREQ,     OPT_Z, "normal" },
-    { GARCH,    OPT_A, "arma-init" },    
+    { GARCH,    OPT_A, "arma-init" },
+    { GARCH,    OPT_F, "fcp" }, 
+    { GARCH,    OPT_N, "nc" }, 
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
     { GMM,      OPT_I, "iterate" },
@@ -181,6 +185,7 @@ struct gretl_option gretl_opts[] = {
     { LMTEST,   OPT_W, "white" },
     { LMTEST,   OPT_X, "white-nocross" },
     { LOGIT,    OPT_P, "p-values" },
+    { LOGIT,    OPT_Q, "quiet" },
     { LOGIT,    OPT_R, "robust" },
     { LOGIT,    OPT_V, "verbose" },
     { LOOP,     OPT_P, "progressive" },
@@ -224,6 +229,7 @@ struct gretl_option gretl_opts[] = {
     { OPEN,     OPT_O, "odbc" },
     { OPEN,     OPT_P, "preserve" },
     { OPEN,     OPT_W, "www" },
+    { OPEN,     OPT_Q, "quiet" },
     { OUTFILE,  OPT_A, "append" },
     { OUTFILE,  OPT_C, "close" },
     { OUTFILE,  OPT_W, "write" },
@@ -252,6 +258,7 @@ struct gretl_option gretl_opts[] = {
     { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
     { PROBIT,   OPT_P, "p-values" },
+    { PROBIT,   OPT_Q, "quiet" },
     { PROBIT,   OPT_R, "robust" },
     { PROBIT,   OPT_V, "verbose" },
     { QUANTREG, OPT_I, "intervals" },
@@ -453,7 +460,7 @@ static const char *ok_flags = "abcdefghijklmnopqrstuvwxz";
 
 #define isflag(c) (c && (strchr(ok_flags, c) != NULL))
 
-static gretlopt opt_from_flag (unsigned char c)
+gretlopt opt_from_flag (unsigned char c)
 {
     int i;
 
