@@ -25,6 +25,7 @@
 #include "forecast.h"
 #include "gretl_func.h"
 #include "matrix_extra.h"
+#include "gretl_scalar.h"
 
 #include <time.h>
 
@@ -612,7 +613,7 @@ static void print_var_smpl (int v, const double **Z,
  * when using scientific notation, e.g. "1.45E-002".  This function
  * checks for this and cuts it out if need be.
  *
- * Returns: the fixed numeric string.
+ * Returns: the corrected numeric string.
  */
 
 char *gretl_fix_exponent (char *s)
@@ -627,7 +628,7 @@ char *gretl_fix_exponent (char *s)
     }
 
     n = strlen(s);
-    if (s[n-1] == '.') {
+    if (s[n-1] == '.' || s[n-1] == ',') {
 	s[n-1] = 0;
     }
 
@@ -650,6 +651,57 @@ static void cut_extra_zero (char *numstr, int digits)
     }
 }
 
+/* below: targ should be 36 bytes long */
+
+void gretl_sprint_fullwidth_double (double x, int digits, char *targ,
+				    PRN *prn)
+{
+    char decpoint = '.';
+    int n;
+
+    if (na(x)) {
+	strcpy(targ, "NA");
+	return;
+    }
+
+#ifdef ENABLE_NLS
+    decpoint = get_local_decpoint();
+#endif
+
+    if (digits == -4) {
+	if (x < .0001) {
+	    sprintf(targ, "%#.3G", x);
+	    digits = 3;
+	} else {
+	    sprintf(targ, "%.4f", x);
+	    return;
+	}
+    } else {
+	/* let's not print non-zero values for numbers smaller than
+	   machine zero */
+	x = screen_zero(x);
+	sprintf(targ, "%#.*G", digits, x);
+    }
+
+    gretl_fix_exponent(targ);
+
+    n = strlen(targ) - 1;
+    if (targ[n] == decpoint) {
+	targ[n] = 0;
+    }
+
+    cut_extra_zero(targ, digits);
+
+    if (*targ == '-' && gretl_print_supports_utf(prn)) {
+	char tmp[36];
+
+	strcpy(tmp, targ + 1);
+	*targ = '\0';
+	strcat(targ, "−"); /* U+2212: minus */
+	strcat(targ, tmp);
+    }
+}
+
 /* The following function formats a double in such a way that the
    decimal point will be printed in the same position for all
    numbers printed this way.  The total width of the number
@@ -1032,7 +1084,8 @@ static void fit_resid_head (const FITRESID *fr,
 	pputc(prn, '\n');
 
 	if (!na(fr->sigma)) {
-	    pprintf(prn, _("Standard error of residuals = %g\n"), fr->sigma);
+	    pprintf(prn, "%s = %.*g\n", _("Standard error of the regression"), 
+		    GRETL_DIGITS, fr->sigma);
 	}
     }
     
@@ -1578,56 +1631,28 @@ static void print_varlist (const char *name, const int *list,
 }
 
 static void 
-print_listed_objects (const char *s, const DATAINFO *pdinfo, PRN *prn)
+print_listed_objects (const char *s, const DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
 {
     const gretl_matrix *m;
     const int *list;
     char *name;
 
     while ((name = gretl_word_strdup(s, &s)) != NULL) {
-	m = get_matrix_by_name(name);
-	if (m != NULL) {
+	if (gretl_is_scalar(name)) {
+	    print_scalar_by_name(name, opt, prn);
+	} else if ((m = get_matrix_by_name(name)) != NULL) {
 	    gretl_matrix_print_to_prn(m, name, prn);
-	} else {
-	    list = get_list_by_name(name);
-	    if (list != NULL) {
-		print_varlist(name, list, pdinfo, prn);
-	    }
+	} else if ((list = get_list_by_name(name)) != NULL) {
+	    print_varlist(name, list, pdinfo, prn);
+	} else if (!strcmp(name, "scalars")) {
+	    print_all_scalars(opt, prn);
 	}
+	    
 	free(name);
     }
 }
 
-static void print_scalar (double x, const char *vname, 
-			  gretlopt opt, int allconst,
-			  PRN *prn)
-{
-    if (!allconst) {
-	pputc(prn, '\n');
-    }
-
-    pprintf(prn, "%15s = ", vname);
-
-    if (na(x)) {
-	pputs(prn, "NA");
-    } else {
-	if (x >= 0.0) {
-	    pputc(prn, ' ');
-	}
-	if (opt & OPT_L) {
-	    pprintf(prn, "%#.*E", libset_get_int(LONGDIGITS), x);
-	} else if (opt & OPT_T) {
-	    pprintf(prn, "%#.10E", x);
-	} else {
-	    pprintf(prn, "%#.6g", x);
-	}
-    }
-
-    if (allconst) {
-	pputc(prn, '\n');
-    }
-}
-
 static int printdata_blocks;
 
 int get_printdata_blocks (void)
@@ -1681,7 +1706,6 @@ int printdata (const int *list, const char *mstr,
     int pause = gretl_get_text_pause();
     int j, v, v1, v2, jc, nvjc, lineno, ncol;
     int screenvar = 0;
-    int scalars = 0;
     int sortvar = 0;
     int maxlen = 0, bplen = 13;
     int *plist = NULL;
@@ -1731,30 +1755,8 @@ int printdata (const int *list, const char *mstr,
 
     lineno = 1;
 
-    /* screen out any scalars and print them first */
-
-    for (j=1; j<=plist[0]; j++) {
-	int len, v = plist[j];
-
-	if (var_is_scalar(pdinfo, v)) {
-	    print_scalar(Z[v][0], pdinfo->varname[v], opt, 0, prn);
-	    scalars = 1;
-	    gretl_list_delete_at_pos(plist, j);
-	    j--;
-	} else {
-	    len = strlen(pdinfo->varname[v]);
-	    if (len > maxlen) {
-		maxlen = len;
-	    }
-	}
-    }
-    
-    if (scalars) {
-	pputc(prn, '\n');
-    }
-
     if (plist[0] == 0) {
-	/* no series left after elimination of scalars */
+	/* no series */
 	pputc(prn, '\n');
 	goto endprint;
     }
@@ -1889,7 +1891,7 @@ int printdata (const int *list, const char *mstr,
  endprint:
 
     if (mstr != NULL) {
-	print_listed_objects(mstr, pdinfo, prn);
+	print_listed_objects(mstr, pdinfo, opt, prn);
     }
 
     free(plist);
@@ -1935,9 +1937,9 @@ int print_data_sorted (const int *list, const int *obsvec,
 	return E_DATA;
     }
 
-    /* ...with no scalars or bad variable numbers */
+    /* ...with no bad variable numbers */
     for (i=1; i<=list[0]; i++) {
-	if (list[i] >= pdinfo->v || var_is_scalar(pdinfo, list[i])) {
+	if (list[i] >= pdinfo->v) {
 	    return E_DATA;
 	}
     }
