@@ -23,6 +23,7 @@
 #include "libset.h"
 #include "gretl_func.h"
 #include "gretl_string_table.h"
+#include "gretl_scalar.h"
 #include "matrix_extra.h"
 
 #include <errno.h>
@@ -110,18 +111,20 @@ static double printf_get_scalar (char *s, double ***pZ,
 #endif
 
     if (numeric_string(s)) {
-	return atof(s);
-    }
-
-    v = varindex(pdinfo, s);
-
-    if (v < pdinfo->v && var_is_series(pdinfo, v)) {
-	char genstr[32];
-
-	sprintf(genstr, "%s[%d]", s, t + 1);
-	x = generate_scalar(genstr, pZ, pdinfo, err);
+	x = atof(s);
+    } else if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
     } else {
-	x = generate_scalar(s, pZ, pdinfo, err);
+	v = series_index(pdinfo, s);
+
+	if (v < pdinfo->v) {
+	    char genstr[32];
+
+	    sprintf(genstr, "%s[%d]", s, t + 1);
+	    x = generate_scalar(genstr, pZ, pdinfo, err);
+	} else {
+	    x = generate_scalar(s, pZ, pdinfo, err);
+	}
     }
 
 #if PSDEBUG
@@ -422,7 +425,7 @@ static int split_printf_line (const char *s, char *targ, int *sp,
 			      char **format, char **args)
 {
     const char *p;
-    int n;
+    int n, err = 0;
 
     *sp = 0;
 
@@ -436,14 +439,14 @@ static int split_printf_line (const char *s, char *targ, int *sp,
     if (*sp) {
 	/* need a target name */
 	s += strspn(s, " ");
-	n = gretl_varchar_spn(s);
-	if (n == 0 || n >= VNAMELEN) {
-	    return E_PARSE;
-	} else {
-	    *targ = '\0';
-	    strncat(targ, s, n);
-	    s += n;
+	err = extract_varname(targ, s, &n);
+	if (!err && n == 0) {
+	    err = E_PARSE;
 	}
+	if (err) {
+	    return err;
+	}
+	s += n;
 	/* allow comma after target */
 	s += strspn(s, " ");
 	if (*s == ',') {
@@ -599,16 +602,13 @@ int do_printf (const char *line, double ***pZ,
 static int 
 sscanf_target_var (const char *vname, DATAINFO *pdinfo, int *err)
 {
-    int v = varindex(pdinfo, vname);
-
-    if (v <= 0 || v >= pdinfo->v) {
-	*err = E_UNKVAR;
-    } else if (!var_is_scalar(pdinfo, v)) {
+    if (gretl_is_scalar(vname)) {
+	return 0;
+    } else {
 	strcpy(gretl_errmsg, _("sscanf: numerical target must be scalar"));
 	*err = E_DATA;
+	return 0;
     }
-
-    return v;
 }
 
 struct bracket_scan {
@@ -822,8 +822,12 @@ static int scan_scalar (const char *targ, char **psrc,
 	}
     }
 
-    if (v > 0 && !errno && endp != *psrc) {
-	Z[v][0] = (fc == 'd')? k : x;
+    if (!errno && endp != *psrc) {
+	if (v == 0) {
+	    gretl_scalar_set_value(targ, (fc == 'd')? k : x);
+	} else if (v > 0) {
+	    Z[v][0] = (fc == 'd')? k : x;
+	}
 	*ns += 1;
     }
 
@@ -1012,7 +1016,7 @@ split_scanf_line (const char *s, char **src, char **format,
 	n = get_literal_length(s);
     } else {
 	if (*s == '@') s++;
-	n = gretl_varchar_spn(s);
+	n = gretl_namechar_spn(s);
     } 
 
     if (n == 0) {
