@@ -710,52 +710,6 @@ char **gretl_string_split (const char *s, int *n)
     return S;
 }
 
-/**
- * gretl_double_from_string:
- * @s: the source string.
- * @ptr: location to receive end of field pointer, or %NULL.
- *
- * Tries to parse the first 'word' found in @s as a 
- * floating-point number, and returns either the number or
- * #NADBL on failure.  If the parsing was successful and if
- * @ptr is not %NULL, on exit it points at the next position 
- * in @s after the just-parsed numeric portion.
- *
- * Returns: floating-point value or #NADBL.
- */
-
-double gretl_double_from_string (const char *s, const char **ptr)
-{
-    double x = NADBL;
-
-    if (s == NULL) {
-	if (ptr != NULL) {
-	    *ptr = NULL;
-	}
-    } else if (*s == '\0') {
-	if (ptr != NULL) {
-	    *ptr = s;
-	}
-    } else {
-	int n = strspn(s, " ");
-
-	s += n;
-	n = strcspn(s, " ");
-	if (n <= 31) {
-	    if (sscanf(s, "%lf", &x) != 1) {
-		x = NADBL;
-		n = 0;
-	    } 
-	}
-
-	if (n > 0 && ptr != NULL) {
-	    *ptr = s + n;
-	}
-    }
-
-    return x;
-}
-
 /**
  * gretl_trunc:
  * @str: the string to truncate.
@@ -776,7 +730,7 @@ char *gretl_trunc (char *str, size_t n)
 }
 
 /**
- * gretl_varchar_spn:
+ * gretl_namechar_spn:
  * @s: the string to examine.
  *
  * Returns: the length of the intial segment of @s which
@@ -785,15 +739,15 @@ char *gretl_trunc (char *str, size_t n)
  * starting with a letter.
  */
 
-int gretl_varchar_spn (const char *s)
+int gretl_namechar_spn (const char *s)
 {
-    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+    const char *ok = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"0123456789_";
     int ret = 0;
 
     if (isalpha(*s)) {
-	ret = strspn(s, varchars);
+	ret = strspn(s, ok);
     }
 
     return ret;
@@ -1998,8 +1952,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
     pspec = g_pattern_spec_new(pattern);
 
     for (i=1; i<pdinfo->v; i++) { 
-	if (var_is_series(pdinfo, i) &&
-	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
+	if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
 	    n++;
 	}
     }
@@ -2011,8 +1964,7 @@ int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
 
 	    list[0] = n;
 	    for (i=1; i<pdinfo->v; i++) { 
-		if (var_is_series(pdinfo, i) &&
-		    g_pattern_match_string(pspec, pdinfo->varname[i])) {
+		if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
 		    list[j++] = i;
 		}
 	    }
@@ -2032,8 +1984,7 @@ int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
     pspec = g_pattern_spec_new(pattern);
 
     for (i=1; i<pdinfo->v; i++) { 
-	if (var_is_series(pdinfo, i) &&
-	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
+	if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
 	    ret = 1;
 	    break;
 	}
