@@ -25,8 +25,10 @@
 #include "gretl_panel.h"
 #include "cmd_private.h"
 #include "dbread.h"
+#include "gretl_scalar.h"
 
 #define SUBDEBUG 0
+#define FULLDEBUG 0
 
 /*
   The purpose of the static pointers below: When the user subsamples
@@ -236,6 +238,7 @@ relink_to_full_dataset (double ***pZ, DATAINFO *pdinfo)
 #if SUBDEBUG
     fprintf(stderr, "relink_to_full_dataset: fullZ = %p, fullinfo = %p\n",
 	    (void *) fullZ, (void *) fullinfo);
+    fprintf(stderr, "fullinfo->v = %d\n", fullinfo->v);
 #endif
 
     fullZ = NULL;
@@ -250,6 +253,20 @@ relink_to_full_dataset (double ***pZ, DATAINFO *pdinfo)
 
 static void sync_dataset_elements (const DATAINFO *pdinfo)
 {
+    if (fullinfo->v > pdinfo->v) {
+	int i;
+
+#if FULLDEBUG
+	fprintf(stderr, "*** sync_dataset_elements: fullinfo->v = %d but pdinfo->v = %d\n",
+		fullinfo->v, pdinfo->v);
+	fprintf(stderr, " deleting the last %d element(s) of fullZ\n", fullinfo->v - pdinfo->v);
+#endif
+	for (i=pdinfo->v; i<fullinfo->v; i++) {
+	    free(fullZ[i]);
+	    fullZ[i] = NULL;
+	}
+	fullinfo->v = pdinfo->v;
+    }
     fullinfo->varname = pdinfo->varname;
     fullinfo->varinfo = pdinfo->varinfo;
     fullinfo->descrip = pdinfo->descrip;
@@ -292,79 +309,13 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
-static int resample_update_dataset (double ***RZ, DATAINFO *pdinfo)
+static int resample_trim_dataset (double ***RZ, DATAINFO *pdinfo)
 {
-    int newv = fullinfo->v;
-    double **newZ = NULL;
-    int drop = 0;
-    int i, j, err = 0;
-
-    /* carry back values of pre-existing scalars */
-
-    for (i=1; i<fullinfo->v && i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    fullZ[i][0] = (*RZ)[i][0];
-	} 
-    }
-
-    /* figure how many scalars and how many series have been added */
-
-    for (i=fullinfo->v; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    newv++;
-	} else {
-	    drop++;
-	}
-    }
-
-    /* if new series have been added, drop them all */
+    int drop = pdinfo->v - fullinfo->v;
+    int err = 0;
 
     if (drop > 0) {
-	int *list = gretl_list_new(drop);
-
-	if (list == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    j = 1;
-	    for (i=fullinfo->v; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i)) {
-		    list[j++] = i;
-		}
-	    }	    
-	}
-	
-	if (list != NULL) {
-	    err = dataset_drop_listed_variables(list, RZ, pdinfo, NULL, NULL);
-	    free(list);
-	}
-    }
-
-    /* if new scalars have been added, add them to fullZ */
-
-    if (!err && newv > fullinfo->v) {
-	newZ = realloc(fullZ, newv * sizeof *fullZ);
-	if (newZ == NULL) {
-	    newv = fullinfo->v;
-	    err = E_ALLOC;
-	} else {
-	    fullZ = newZ;
-	    j = fullinfo->v;
-	    for (i=fullinfo->v; i<pdinfo->v && !err; i++) {
-		if (var_is_scalar(pdinfo, i)) {
-		    fullZ[j] = malloc(sizeof **newZ);
-		    if (fullZ[j] == NULL) {
-			err = E_ALLOC;
-		    } else {
-			fullZ[j][0] = (*RZ)[i][0];
-			j++;
-		    }
-		}
-	    }
-	}
-    }
-
-    if (!err) {
-	fullinfo->v = newv;
+	err = dataset_drop_last_variables(drop, RZ, pdinfo);
     }
 
     return err;
@@ -386,16 +337,12 @@ update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
 
     for (i=1; i<fullinfo->v && i<pdinfo->v; i++) {
 	s = 0;
-	if (var_is_scalar(pdinfo, i)) {
-	    fullZ[i][0] = subZ[i][0];
-	} else {
-	    for (t=0; t<fullinfo->n; t++) {
-		if (pdinfo->submask[t] == 1) {
-		    fullZ[i][t] = subZ[i][s++];
-		} else if (pdinfo->submask[t] == 'p') {
-		    /* skip panel padding (?) */
-		    s++;
-		}
+	for (t=0; t<fullinfo->n; t++) {
+	    if (pdinfo->submask[t] == 1) {
+		fullZ[i][t] = subZ[i][s++];
+	    } else if (pdinfo->submask[t] == 'p') {
+		/* skip panel padding (?) */
+		s++;
 	    }
 	}
     }
@@ -459,15 +406,11 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
     fullZ = newZ;
 
     for (i=V0; i<pdinfo->v && !err; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    fullZ[i] = malloc(N * sizeof **newZ);
-	} else {
-#if SUBDEBUG
-	    fprintf(stderr, " new var %d (%s) is a scalar\n", i, 
-		    pdinfo->varname[i]);
+#if FULLDEBUG
+	fprintf(stderr, "adding to full: var %d (%s, level %d)\n",
+		i, pdinfo->varname[i], STACK_LEVEL(pdinfo, i));
 #endif
-	    fullZ[i] = malloc(sizeof **newZ);
-	}
+	fullZ[i] = malloc(N * sizeof **newZ);
 	if (fullZ[i] == NULL) {
 	    err = E_ALLOC;
 	}
@@ -483,13 +426,9 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 #endif
 
     for (i=V0; i<pdinfo->v; i++) {
-	if (var_is_scalar(pdinfo, i)) {
-	    fullZ[i][0] = Z[i][0]; 
-	} else {
-	    s = 0;
-	    for (t=0; t<N; t++) {
-		fullZ[i][t] = (pdinfo->submask[t])? Z[i][s++] : NADBL;
-	    }
+	s = 0;
+	for (t=0; t<N; t++) {
+	    fullZ[i][t] = (pdinfo->submask[t])? Z[i][s++] : NADBL;
 	}
     }
 
@@ -551,17 +490,28 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
     gretl_error_clear();
 
     if (!complex_subsampled()) {
-	if (pdinfo->t1 != 0 || pdinfo->t2 != pdinfo->n - 1) {
-	    pdinfo->t1 = 0;
-	    pdinfo->t2 = pdinfo->n - 1;
-#if SUBDEBUG
-	    fprintf(stderr, "restore_full_sample: reset t1 and t2\n");
-#endif
+	int t1min, t2max;
+
+	if (state == NULL) {
+	    /* not inside a function */
+	    t1min = 0;
+	    t2max = pdinfo->n - 1;
+	} else {
+	    /* don't go outside the bounds set on entry to
+	       a function */
+	    sample_range_get_extrema(pdinfo, &t1min, &t2max);
 	}
+
+	pdinfo->t1 = t1min;
+	pdinfo->t2 = t2max;
+#if SUBDEBUG
+	fprintf(stderr, "restore_full_sample: just set t1=%d and t2=%d\n",
+		t1min, t2max);
+#endif
 	return 0;
     }
 
-#if SUBDEBUG
+#if FULLDEBUG || SUBDEBUG
     fprintf(stderr, "\nrestore_full_sample: pZ=%p, *pZ=%p, pdinfo=%p\n",
 	    (void *) pZ, (void *) *pZ, (void *) pdinfo);
 #endif
@@ -577,7 +527,7 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
 
     if (pdinfo->submask == RESAMPLED) {
 	/* not regular subsample but resampled dataset */
-	resample_update_dataset(pZ, pdinfo);
+	resample_trim_dataset(pZ, pdinfo);
     } else {
 	/* update values for pre-existing series, which may have been
 	   modified via genr etc */
@@ -615,9 +565,21 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
 
     relink_to_full_dataset(pZ, pdinfo);
 
-    if (!err && state != NULL && state->submask != NULL) {
+    if (!err && state != NULL) {
 	/* "full" really means, relative to the original state */
-	err = restrict_sample_from_mask(state->submask, pZ, pdinfo);
+	if (state->submask != NULL) {
+	    err = restrict_sample_from_mask(state->submask, pZ, pdinfo);
+	} else {
+	    int t1min, t2max;
+
+	    sample_range_get_extrema(pdinfo, &t1min, &t2max);
+	    if (pdinfo->t1 < t1min) {
+		pdinfo->t1 = t1min;
+	    }
+	    if (pdinfo->t2 > t2max) {
+		pdinfo->t2 = t2max;
+	    }
+	}
     }
 
     return err;
@@ -651,7 +613,7 @@ make_missing_mask (const int *list, const double **Z, const DATAINFO *pdinfo,
 	    mask[t] = 1;
 	    for (i=1; i<=list[0]; i++) {
 		vi = list[i];
-		if (var_is_series(pdinfo, vi) && na(Z[vi][t])) {
+		if (na(Z[vi][t])) {
 		    mask[t] = 0;
 		    break;
 		}
@@ -662,7 +624,7 @@ make_missing_mask (const int *list, const double **Z, const DATAINFO *pdinfo,
 	for (t=0; t<pdinfo->n; t++) {
 	    mask[t] = 1;
 	    for (i=1; i<pdinfo->v; i++) {
-		if (var_is_series(pdinfo, i) && na(Z[i][t])) {
+		if (na(Z[i][t])) {
 		    mask[t] = 0;
 		    break;
 		}
@@ -719,7 +681,7 @@ static int mask_from_dummy (const char *s, const double **Z, const DATAINFO *pdi
     int dnum, err = 0;
 
     sscanf(s, "%15s", dname);
-    dnum = varindex(pdinfo, dname);
+    dnum = series_index(pdinfo, dname);
 
     if (dnum == pdinfo->v) {
 	sprintf(gretl_errmsg, _("Variable '%s' not defined"), dname);
@@ -938,20 +900,16 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
 
     /* copy data values */
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i)) {
-	    st = 0;
-	    for (t=0; t<pdinfo->n; t++) {
-		if (mask == NULL) {
-		    subZ[i][st++] = Z[i][t];
-		} else if (mask[t] == 1) {
-		    subZ[i][st++] = Z[i][t];
-		} else if (mask[t] == 'p') {
-		    /* panel padding */
-		    subZ[i][st++] = NADBL;
-		}
+	st = 0;
+	for (t=0; t<pdinfo->n; t++) {
+	    if (mask == NULL) {
+		subZ[i][st++] = Z[i][t];
+	    } else if (mask[t] == 1) {
+		subZ[i][st++] = Z[i][t];
+	    } else if (mask[t] == 'p') {
+		/* panel padding */
+		subZ[i][st++] = NADBL;
 	    }
-	} else {
-	    subZ[i][0] = Z[i][0];
 	}
     }
 
@@ -1252,7 +1210,7 @@ int restrict_sample (const char *line, const int *list,
 
     gretl_error_clear();
 
-#if SUBDEBUG
+#if FULLDEBUG || SUBDEBUG
     fprintf(stderr, "\nrestrict_sample: '%s'\n", line);
     fprintf(stderr, " pdinfo=%p, pZ=%p, *pZ=%p, state=%p\n", 
 	    (void *) pdinfo, (void *) pZ, (void *) *pZ,
@@ -1346,18 +1304,14 @@ static int panel_round (const DATAINFO *pdinfo, int t, int code)
 static int smpl_get_int (const char *s, double ***pZ, DATAINFO *pdinfo,
 			 int *err)
 {
-    int k = 0;
+    int k;
 
     if (integer_string(s)) {
 	k = atoi(s);
+    } else if (gretl_is_scalar(s)) {
+	k = gretl_scalar_get_value(s);
     } else {
-	int v = varindex(pdinfo, s);
-
-	if (v < pdinfo->v && var_is_scalar(pdinfo, v)) {
-	    k = (int) (*pZ)[v][0];
-	} else {
-	    k = (int) generate_scalar(s, pZ, pdinfo, err);
-	}
+	k = (int) generate_scalar(s, pZ, pdinfo, err);
     }
 
     return k;
@@ -1405,6 +1359,7 @@ static int get_sample_limit (char *s, double ***pZ, DATAINFO *pdinfo,
 int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 {
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
+    int tmin = 0, tmax = 0;
     char newstart[OBSLEN], newstop[OBSLEN];
 
     gretl_error_clear();
@@ -1432,6 +1387,8 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 	/* no-op, just print the current sample */
 	return 0;
     }
+
+    sample_range_get_extrema(pdinfo, &tmin, &tmax);
 	
     if (nf == 1) {
 	if (sscanf(line, "%10s", newstart) != 1) {
@@ -1439,7 +1396,7 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 	    return 1;
 	} else {
 	    new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
-	    if (new_t1 < 0 || new_t1 >= pdinfo->n) {
+	    if (new_t1 < tmin || new_t1 > tmax) {
 		strcpy(gretl_errmsg, _("error in new starting obs"));
 		return 1;
 	    }
@@ -1457,7 +1414,7 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 
     if (strcmp(newstart, ";")) {
 	new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
-	if (new_t1 < 0 || new_t1 >= pdinfo->n) {
+	if (new_t1 < tmin || new_t1 > tmax) {
 	    strcpy(gretl_errmsg, _("error in new starting obs"));
 	    return 1;
 	}	
@@ -1465,13 +1422,13 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 
     if (strcmp(newstop, ";")) {
 	new_t2 = get_sample_limit(newstop, pZ, pdinfo, SMPL_T2);
-	if (new_t2 < 0 || new_t2 >= pdinfo->n) {
+	if (new_t2 < tmin || new_t2 > tmax) {
 	    strcpy(gretl_errmsg, _("error in new ending obs"));
 	    return 1;
 	}
     }
 
-    if (new_t1 < 0 || new_t1 > new_t2) {
+    if (new_t1 < tmin || new_t1 > new_t2) {
 	strcpy(gretl_errmsg, _("Invalid null sample"));
 	return 1;
     }
@@ -1496,7 +1453,7 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	tmiss = 0;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_hidden(pdinfo, i) || var_is_scalar(pdinfo, i)) {
+	    if (var_is_hidden(pdinfo, i)) {
 		continue;
 	    }
 	    if (na((*pZ)[i][t])) {
