@@ -304,15 +304,10 @@ equation_system_new (int method, const char *name, int *err)
 	return NULL;
     }
 
+    sys->name = NULL;
+
     if (name != NULL) {
-	sys->name = gretl_strdup(name);
-	if (sys->name == NULL) {
-	    *err = E_ALLOC;
-	    free(sys);
-	    return NULL;
-	}
-    } else {
-	sys->name = NULL;
+	equation_system_set_name(sys, name);
     }
 
     sys->refcount = 0;
@@ -772,7 +767,7 @@ system_print_F_test (const equation_system *sys,
     RvR = gretl_matrix_alloc(Rrows, Rrows);
 
     if (Rbq == NULL || RvR == NULL) {
-	pputs(prn, "Out of memory in F test\n");
+	pputs(prn, "Out of memory!\n");
 	goto bailout;
     }
 
@@ -1336,7 +1331,7 @@ static int sys_check_lists (equation_system *sys, DATAINFO *pdinfo)
 	lag = pdinfo->varinfo[vj]->lag;
 	if (lag > 0) {
 	    vname = pdinfo->varinfo[vj]->parent;
-	    src = varindex(pdinfo, vname);
+	    src = series_index(pdinfo, vname);
 	    if (in_gretl_list(sys->ylist, src)) {
 		err = add_predet_to_sys(sys, pdinfo, vj, src, lag);
 		gretl_list_delete_at_pos(xplist, j--);
@@ -1404,7 +1399,7 @@ int equation_system_finalize (equation_system *sys,
 
     err = sys_check_lists(sys, pdinfo);
 
-    if (!err && sys->name != NULL) {
+    if (!err && sys->name != NULL && *sys->name != '\0') {
 	/* save the system for subsequent estimation */
 	err = gretl_stack_object_as(sys, GRETL_OBJ_SYS, sys->name);
     }
@@ -1525,15 +1520,18 @@ const char *system_short_string (const MODEL *pmod)
 
 void equation_system_set_name (equation_system *sys, const char *name)
 {
-    if (sys->name != NULL && !strcmp(sys->name, name)) {
+    if (name == sys->name) {
 	return;
     }
 
-    if (sys->name != NULL) {
-	free(sys->name);
+    if (sys->name == NULL) {
+	sys->name = malloc(MAXSAVENAME);
     }
 
-    sys->name = gretl_strdup(name);
+    if (sys->name != NULL) {
+	*sys->name = '\0';
+	strncat(sys->name, name, MAXSAVENAME - 1);
+    }
 }
 
 int system_adjust_t1t2 (equation_system *sys,
@@ -1925,15 +1923,9 @@ parse_identity (const char *str, double ***pZ, DATAINFO *pdinfo, int *err)
 	p += strspn(p, " ");
 	if (i == 0) {
 	    /* left-hand side variable */
-	    len = gretl_varchar_spn(p);
-	    if (len == 0) {
-		*err = E_PARSE;
-	    } else if (len >= VNAMELEN) {
-		*err = E_UNKVAR;
-	    } else {
-		*vname = '\0';
-		strncat(vname, p, len);
-		v = varindex(pdinfo, vname);
+	    *err = extract_varname(vname, p, &len);
+	    if (!*err) {
+		v = series_index(pdinfo, vname);
 		if (v == pdinfo->v) {
 		    *err = E_UNKVAR;
 		} else {
@@ -1956,16 +1948,13 @@ parse_identity (const char *str, double ***pZ, DATAINFO *pdinfo, int *err)
 	    *err = E_PARSE;
 	} else {
 	    /* right-hand size variable (may be lag) */
-	    len = gretl_varchar_spn(p);
-	    if (len >= VNAMELEN) {
-		*err = E_UNKVAR;
-	    } else if (gotop && len == 0) {
+	    *err = extract_varname(vname, p, &len);
+	    if (!*err && gotop && len == 0) {
 		/* dangling operator */
 		*err = E_PARSE;
-	    } else {
-		*vname = '\0';
-		strncat(vname, p, len);
-		v = varindex(pdinfo, vname);
+	    }
+	    if (!*err) {
+		v = series_index(pdinfo, vname);
 		if (v == pdinfo->v) {
 		    *err = E_UNKVAR;
 		} else {
@@ -2023,19 +2012,16 @@ parse_identity (const char *str, double ***pZ, DATAINFO *pdinfo, int *err)
     while (*p) {
 	p += strspn(p, " ");
 	if (i == 0) {
-	    *vname = '\0';
-	    strncat(vname, p, gretl_varchar_spn(p));
-	    ident->depvar = varindex(pdinfo, vname);
+	    extract_varname(vname, p, &len);
+	    ident->depvar = series_index(pdinfo, vname);
 	    p = strchr(p, '=') + 1;
 	    i++;
 	} else if (*p == '+' || *p == '-') {
 	    ident->atoms[i-1].op = (*p == '+')? OP_PLUS : OP_MINUS;
 	    p++;
 	} else {
-	    len = gretl_varchar_spn(p);
-	    *vname = '\0';
-	    strncat(vname, p, len);
-	    v = varindex(pdinfo, vname);
+	    extract_varname(vname, p, &len);
+	    v = series_index(pdinfo, vname);
 	    p += len;
 	    if (*p == '(') {
 		lag = strtol(p + 1, &test, 10);
@@ -2422,11 +2408,11 @@ equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 {
     equation_system *sys;
     xmlNodePtr cur;
-    char *name;
+    char *sname;
     int method = 0;
     int i, j, got = 0;
 
-    got += gretl_xml_get_prop_as_string(node, "name", &name);
+    got += gretl_xml_get_prop_as_string(node, "name", &sname);
     got += gretl_xml_get_prop_as_int(node, "method", &method);
 
     if (got < 2) {
@@ -2434,7 +2420,7 @@ equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 	return NULL;
     }     
 
-    sys = equation_system_new(method, name, err);
+    sys = equation_system_new(method, sname, err);
     if (*err) {
 	return NULL;
     }
@@ -2449,6 +2435,8 @@ equation_system_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 	goto bailout;
     } 
 
+    gretl_xml_get_prop_as_int(node, "order", &sys->order);
+
     sys->lists = malloc(sys->neqns * sizeof sys->lists);
     if (sys->lists == NULL) {
 	*err = E_ALLOC;
@@ -2519,8 +2507,8 @@ int equation_system_serialize (equation_system *sys,
     fprintf(fp, "<gretl-equation-system name=\"%s\" saveflags=\"%d\" method=\"%d\" ",  
 	    (sys->name != NULL)? sys->name : "none", flags, sys->method);
 
-    fprintf(fp, "n_equations=\"%d\" nidents=\"%d\" flags=\"%d\">\n",
-	    sys->neqns, sys->nidents, (int) sys->flags);
+    fprintf(fp, "n_equations=\"%d\" nidents=\"%d\" flags=\"%d\" order=\"%d\">\n",
+	    sys->neqns, sys->nidents, (int) sys->flags, sys->order);
 
     for (i=0; i<sys->neqns; i++) {
 	gretl_xml_put_tagged_list("eqnlist", sys->lists[i], fp);
