@@ -631,7 +631,6 @@ void tex_custom_coeff_table_start (const char **cols, PRN *prn)
 
 void tex_coeff_table_start (const char **cols, int binary, PRN *prn)
 {
-
     char pt;
 
     if (use_custom) {
@@ -964,30 +963,29 @@ static const char *get_gretltex_local (void)
 
 void set_gretl_tex_preamble (void)
 {
+    char test[MAXLEN];
     FILE *fp;
-    const char *gretltex = "gretlpre.tex";
-#ifdef ENABLE_NLS
-    const char *localtex = get_gretltex_local();
 
+#ifdef ENABLE_NLS
     /* first choice: localized preamble file */
-    sprintf(tex_preamble_file, "%s%s", gretl_work_dir(), localtex);
-    fp = gretl_fopen(tex_preamble_file, "r");
-    if (fp == NULL) {
-	tex_preamble_file[0] = '\0';
-    } else {
+    sprintf(test, "%s%s", gretl_work_dir(), get_gretltex_local());
+    fp = gretl_fopen(test, "r");
+    if (fp != NULL) {
+	strcpy(tex_preamble_file, test);
 	fclose(fp);
 	return;
     }    
 #endif
 
     /* preamble file on disk */
-    sprintf(tex_preamble_file, "%s%s", gretl_work_dir(), gretltex);
-    fp = gretl_fopen(tex_preamble_file, "r");
-    if (fp == NULL) {
-	tex_preamble_file[0] = '\0';
-    } else {
+    sprintf(test, "%sgretlpre.tex", gretl_work_dir());
+    fp = gretl_fopen(test, "r");
+    if (fp != NULL) {
+	strcpy(tex_preamble_file, test);
 	fclose(fp);
     }
+
+    gretl_error_clear();
 }
 
 static void landscape_modify_line (char *line)
@@ -1030,6 +1028,7 @@ void set_tex_use_utf (int s)
 
 void gretl_tex_preamble (PRN *prn, int fmt)
 {
+    char* lang = getenv("LANG");
     FILE *fp = NULL;
     int userfile = 0;
 
@@ -1074,6 +1073,9 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 	} else {
 	    pputs(prn, "\\usepackage[latin1]{inputenc}\n");
 	}
+	if (lang != NULL && !strncmp(lang, "ru", 2)) {
+	    pputs(prn, "\\usepackage[russian]{babel}\n");
+	}
 #endif
 	if (fmt & GRETL_FORMAT_EQN) {
 	    pputs(prn, "\\usepackage{amsmath}\n\n");
