@@ -217,8 +217,8 @@ real_get_matrix_by_name (const char *name, int slevel)
     }
 
     for (i=0; i<n_matrices; i++) {
-	if (!strcmp(name, matrices[i]->name) &&
-	    matrices[i]->level == level) {
+	if (matrices[i]->level == level && 
+	    !strcmp(name, matrices[i]->name)) {
 	    return matrices[i]->M;
 	}
     }
@@ -232,8 +232,8 @@ user_matrix *get_user_matrix_by_name (const char *name)
     int i;
 
     for (i=0; i<n_matrices; i++) {
-	if (!strcmp(name, matrices[i]->name) &&
-	    matrices[i]->level == level) {
+	if (matrices[i]->level == level && 
+	    !strcmp(name, matrices[i]->name)) {
 	    return matrices[i];
 	}
     }
@@ -488,33 +488,23 @@ int copy_matrix_as (const gretl_matrix *m, const char *new)
     return err;
 }
 
-/**
- * user_matrix_set_name_and_level:
- * @m: matrix to be reconfigured.
- * @name: new name to be given to matrix.
- * @level: function execution level to be assigned to matrix.
- *
- * If matrix @m is found on the stack of saved matrices, change
- * its name to @newname and change its level to @level.
- *
- * Returns: 0 on success, 1 if the matrix is not found.
- */
-
-int user_matrix_set_name_and_level (const gretl_matrix *M, char *name, 
-				    int level)
+static int msel_out_of_bounds (int *range, int n)
 {
-    user_matrix *u = get_user_matrix_by_data(M);
-    int err = 0;
+    int *bad = NULL;
 
-    if (u == NULL) {
-	err = 1;
-    } else {
-	*u->name = '\0';
-	strncat(u->name, name, VNAMELEN - 1);
-	u->level = level;
+    if (range[0] < 1 || range[0] > n) {
+	bad = &range[0];
+    } else if (range[1] < 1 || range[1] > n) {
+	bad = &range[1];
     }
 
-    return err;
+    if (bad != NULL) {
+	sprintf(gretl_errmsg, _("Index value %d is out of bounds"), 
+		*bad);
+	return 1;
+    } else {
+	return 0;
+    }
 }
 
 static int *mspec_to_list (int type, union msel *sel, int n,
@@ -528,13 +518,21 @@ static int *mspec_to_list (int type, union msel *sel, int n,
     }
 
     if (type == SEL_RANGE) {
+	if (sel->range[1] == MSEL_MAX) {
+	    sel->range[1] = n;
+	}
+	if (msel_out_of_bounds(sel->range, n)) {
+	    *err = 1;
+	    return NULL;
+	}
 	ns = sel->range[1] - sel->range[0] + 1;
     } else {
 	ns = gretl_vector_get_length(sel->m);
     }
 
     if (ns <= 0) {
-	*err = E_DATA;
+	strcpy(gretl_errmsg, _("Range is non-positive!")); 
+	*err = 1;
 	return NULL;
     }
 
@@ -1132,9 +1130,9 @@ gretl_matrix *user_matrix_matrix_func (gretl_matrix *m, int f,
 	} else if (f == F_INV) {
 	    *err = gretl_invert_matrix(R);
 	} else if (f == F_UPPER) {
-	    *err = gretl_matrix_zero_upper(R);
-	} else if (f == F_LOWER) {
 	    *err = gretl_matrix_zero_lower(R);
+	} else if (f == F_LOWER) {
+	    *err = gretl_matrix_zero_upper(R);
 	} else {
 	    *err = E_DATA;
 	}
