@@ -2753,11 +2753,18 @@ real_gretl_restricted_vecm (GRETL_VAR *orig,
 
 void gretl_VAR_set_name (GRETL_VAR *var, const char *name)
 {
-    if (var->name != NULL) {
-	free(var->name);
+    if (name == var->name) {
+	return;
     }
 
-    var->name = gretl_strdup(name);
+    if (var->name == NULL) {
+	var->name = malloc(MAXSAVENAME);
+    } 
+
+    if (var->name != NULL) {
+	*var->name = '\0';
+	strncat(var->name, name, MAXSAVENAME - 1);
+    }
 }
 
 const char *gretl_VAR_get_name (const GRETL_VAR *var)
@@ -3379,6 +3386,7 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
     MODEL *pmod;
     double x;
     int gotA = (var->A != NULL);
+    int gotX = (var->X != NULL);
     int gotV = (var->vcv != NULL);
     int j, i;
     int err = 0;
@@ -3426,6 +3434,10 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
 	VAR_write_A_matrix(var);
     }
 
+    if (!err && !gotX) {
+	fprintf(stderr, "Can't we rebuild VAR->X somehow?\n");
+    }    
+
     if (!err && !gotV) {
 	VAR_write_vcv_matrix(var);
     }
@@ -3438,6 +3450,7 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
     GRETL_VAR *var;
     MODEL *pmod;
     xmlNodePtr cur;
+    char *vname;
     int i, n, got = 0;
 
     var = gretl_VAR_rebuilder_new();
@@ -3457,6 +3470,12 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 
     var->ci = (var->ci == 0)? VAR : VECM;
 
+    gretl_xml_get_prop_as_string(node, "name", &vname);
+    if (vname != NULL) {
+	gretl_VAR_set_name(var, vname);
+	free(vname);
+    }
+
     /* these are not show-stoppers */
     gretl_xml_get_prop_as_int(node, "detflags", &var->detflags);
     gretl_xml_get_prop_as_int(node, "LBs", &var->LBs);
@@ -3500,6 +3519,10 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err)
 	    } else {
 		if (!strcmp(mname, "A")) {
 		    var->A = gretl_xml_get_matrix(cur, doc, err);
+		} else if (!strcmp(mname, "X")) {
+		    var->X = gretl_xml_get_matrix(cur, doc, err);
+		} else if (!strcmp(mname, "Y")) {
+		    var->Y = gretl_xml_get_matrix(cur, doc, err);
 		}
 		free(mname);
 	    }
@@ -3622,6 +3645,12 @@ int gretl_VAR_serialize (const GRETL_VAR *var, SavedObjectFlags flags,
 	gretl_xml_put_double_array("Ivals", var->Ivals, N_IVALS, fp);
     }
 
+    if (var->X != NULL && var->Y != NULL) {
+	/* could be fiddly to reconstruct, needed for IRF bootstrap */
+	gretl_xml_put_matrix(var->X, "X", fp);
+	gretl_xml_put_matrix(var->Y, "Y", fp);
+    }   
+
     if (var->ci == VECM) {
 	/* this is hard to reconstruct for VECMs */
 	gretl_xml_put_matrix(var->A, "A", fp);
