@@ -155,13 +155,18 @@ static int print_iterations (const char *path, PRN *prn)
     return err;
 }
 
+#define daily_frequency(f) (f == 5 || f == 6 || f == 7)
+
 static int x12_date_to_n (const char *s, const DATAINFO *pdinfo)
 {
     char date[12] = {0};
 
-    if (dated_daily_data(pdinfo)) {
-	/* FIXME? */
-	return atoi(s);
+    if (daily_frequency(pdinfo->pd)) {
+	int t, maj, min;
+
+	sscanf(s, "%1d%2d", &maj, &min);
+	t = (maj - 1) * pdinfo->pd + min - 1;
+	return t;
     }
 
     if (pdinfo->pd > 1) {
@@ -218,11 +223,9 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 
     fclose(fp);
 
-    if (nobs != nefobs && nefobs > 0) {
+    if (nefobs > 0) {
 	pmod->nobs = nefobs;
-	pmod->t1 += nobs - nefobs;
-    } else {
-	pmod->nobs = nobs;
+	pmod->t1 = pmod->t2 - nefobs + 1;
     }
 
     return 0;
@@ -575,10 +578,12 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     int yr, subper = 0;
     char *s;
 
-    /* daily data: for now we'll try just numbering the observations
-       consecutively */
-    if (dated_daily_data(pdinfo)) {
-	sprintf(str, "%d", t + 1);
+    /* daily data: FIXME when dated? */
+    if (daily_frequency(pdinfo->pd)) {
+	int maj = t / pdinfo->pd + 1;
+	int min = (t + 1) % pdinfo->pd;
+
+	sprintf(str, "%d.%d", maj, min);
 	return;
     } 
 
@@ -657,6 +662,7 @@ static int write_spc_file (const char *fname,
     FILE *fp;
     char datestr[12];
     int nfcast = 0;
+    int t1 = ainfo->t1;
     int tmax;
     int i;
 
@@ -675,16 +681,14 @@ static int write_spc_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    make_x12a_date_string(ainfo->t1, pdinfo, datestr);
+    fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
+	    pdinfo->varname[ainfo->yno]);
 
-    if (dated_daily_data(pdinfo)) {
-	/* FIXME? */
-	fprintf(fp, "series {\n period = 1\n title = \"%s\"\n",  
-		pdinfo->varname[ainfo->yno]);
-    } else {
-	fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
-		pdinfo->varname[ainfo->yno]);
-    }	
+    if (!arma_exact_ml(ainfo)) {
+	t1 -= ainfo->maxlag;
+    }
+    
+    make_x12a_date_string(t1, pdinfo, datestr);
     fprintf(fp, " start = %s\n", datestr);
 
     ylist[0] = 1;
@@ -698,7 +702,7 @@ static int write_spc_file (const char *fname,
 	/* FIXME ensure we don't print any NAs */
 
 	tmax = pdinfo->n - 1;
-	nobs = tmax - ainfo->t1 + 1;
+	nobs = tmax - ainfo->t1 + 1; /* t1? */
 	if (nobs > MAXOBS) {
 	    tmax -= nobs - MAXOBS;
 	}
@@ -712,7 +716,7 @@ static int write_spc_file (const char *fname,
 #endif
     } 
 
-    output_series_to_spc(ylist, Z, ainfo->t1, tmax, fp);
+    output_series_to_spc(ylist, Z, t1, tmax, fp);
 
     if (tmax > ainfo->t2) {
 	make_x12a_date_string(ainfo->t2, pdinfo, datestr);
@@ -732,7 +736,7 @@ static int write_spc_file (const char *fname,
 	    fprintf(fp, "%s ", pdinfo->varname[xlist[i]]);
 	}
 	fputs(")\n", fp);
-	output_series_to_spc(xlist, Z, ainfo->t1, tmax, fp);
+	output_series_to_spc(xlist, Z, t1, tmax, fp);
 	free(xlist);
     }
     fputs("}\n", fp);
@@ -748,6 +752,10 @@ static int write_spc_file (const char *fname,
 
     fputs("estimate {\n", fp);
 
+    /* could enable here: "tol = XX, maxiter = NN"  
+       the default is tol = 1.0e-5, maxiter = 200 
+    */
+
     if (opt & OPT_V) {
 	fputs(" print = (acm itr lkf lks mdl est rts rcm)\n", fp);
     } else {
@@ -803,6 +811,7 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     PRN *vprn = NULL;
     MODEL armod;
     struct arma_info ainfo;
+    char flags = ARMA_X12A;
 #ifdef WIN32
     char *cmd;
 #endif
@@ -815,7 +824,11 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	opt |= OPT_F;
     }
 
-    arma_info_init(&ainfo, ARMA_X12A, pqspec, pdinfo);
+    if (!(opt & OPT_C)) {
+	flags |= ARMA_EXACT;
+    }
+
+    arma_info_init(&ainfo, flags, pqspec, pdinfo);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
@@ -870,21 +883,15 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	sprintf(path, "%s%c%s", workdir, SLASH, yname); 
 	armod.t1 = ainfo.t1;
 	armod.t2 = ainfo.t2;
-	armod.nobs = armod.t2 - armod.t1 + 1;
 	populate_arma_model(&armod, alist, path, Z, pdinfo, &ainfo);
 	if (verbose && !armod.errcode) {
 	    print_iterations(path, vprn);
 	}
 	if (!armod.errcode) {
-	    int acode = ARMA_X12A;
-
 	    if (gretl_in_gui_mode()) {
 		add_unique_output_file(&armod, path);
 	    }
-	    if (!(opt & OPT_C)) {
-		acode |= ARMA_EXACT;
-	    }
-	    gretl_model_set_int(&armod, "arma_flags", acode);
+	    gretl_model_set_int(&armod, "arma_flags", (int) flags);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
