@@ -48,13 +48,9 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 	return;
     }
 
-    strcpy(pmod->params[0], pdinfo->varname[0]);
-
-    j = 1;
+    j = 0;
     for (i=0; i<r; i++) {
-	if (pmod->list[5+i] > 0) {
-	    strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]);
-	}
+	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]);
     }
 
     strcpy(pmod->params[j++], "alpha(0)");
@@ -83,7 +79,7 @@ static void rescale_results (double *theta, gretl_matrix *V,
     for (i=0; i<npar; i++) {
 	sfi = (i < nc)? scale : (i == nc)? sc2 : 1.0;
 	for (j=0; j<=i; j++) {
-	    sf = (j < nc)? scale*sfi : (j == nc)? sc2*sfi : sfi;
+	    sf = (j < nc)? scale * sfi : (j == nc)? sc2 * sfi : sfi;
 	    vij = gretl_matrix_get(V, i, j) * sf;
 	    gretl_matrix_set(V, i, j, vij);
 	    gretl_matrix_set(V, j, i, vij);
@@ -96,7 +92,7 @@ write_garch_stats (MODEL *pmod, const int *list,
 		   const double **Z, const DATAINFO *pdinfo,
 		   double *theta, gretl_matrix *V, double scale,
 		   const double *e, const double *h, 
-		   int npar, int nc, int pad, PRN *prn)
+		   int npar, int nc, int pad, int ifc, PRN *prn)
 {
     int err = 0;
     double *coeff, *sderr, *vcv, *garch_h;
@@ -163,7 +159,7 @@ write_garch_stats (MODEL *pmod, const int *list,
     mle_criteria(pmod, 1);
 
     pmod->ci = GARCH;
-    pmod->ifc = 1;
+    pmod->ifc = ifc;
     
     add_garch_varnames(pmod, pdinfo, list);
 
@@ -206,15 +202,18 @@ static int make_garch_dataset (const int *list, double **Z,
 	*py = y;
     } 
 
-    if (pad) {
-	X = doubles_array_new(nx, bign);
-    } else {
-	X = malloc(nx * sizeof *X);
-    }
+    if (nx > 0) {
+	if (pad) {
+	    X = doubles_array_new(nx, bign);
+	} else {
+	    X = malloc(nx * sizeof *X);
+	}
 
-    if (X == NULL) {
-	free(y);
-	return E_ALLOC;
+	if (X == NULL) {
+	    free(y);
+	    *py = NULL;
+	    return E_ALLOC;
+	}
     }
 
     if (pad > 0) {
@@ -295,10 +294,10 @@ static void garch_print_init (const double *theta, int k,
     pputs(prn, "\n Variance parameters:\n");
 
     pprintf(prn, "  alpha[0] = %g\n", theta[j++]);
-    for (i=0; i<p; i++) {
+    for (i=0; i<q; i++) {
 	pprintf(prn, "  alpha[%d] = %g\n", i+1, theta[j++]);
     }
-    for (i=0; i<q; i++) {
+    for (i=0; i<p; i++) {
 	pprintf(prn, "   beta[%d] = %g\n", i, theta[j++]);
     }
 
@@ -345,7 +344,8 @@ static int garch_manual_init (double *theta, int k, int p, int q,
 static int 
 garch_driver (const int *list, double **Z, double scale,
 	      const DATAINFO *pdinfo, MODEL *pmod,
-	      double *vparm, gretlopt opt, PRN *prn)
+	      double *vparm, int ifc, gretlopt opt, 
+	      PRN *prn)
 {
     int t1 = pmod->t1, t2 = pmod->t2;
     int nc = pmod->ncoeff;
@@ -425,7 +425,7 @@ garch_driver (const int *list, double **Z, double scale,
 	}
     }
 
-    if (libset_get_bool("fcp")) {
+    if ((opt & OPT_F) || libset_get_bool(USE_FCP)) {
 	err = garch_estimate(y, (const double **) X,
 			     t1 + pad, t2 + pad, bign, nc, 
 			     p, q, theta, V, e, e2, h,
@@ -437,13 +437,11 @@ garch_driver (const int *list, double **Z, double scale,
 				 scale, &ll, &fnc, &grc, vopt, prn);
     }
 
-    if (err != 0) {
-	pmod->errcode = err;
-    } else {
+    if (!err) {
 	pmod->lnL = ll;
 	write_garch_stats(pmod, list, (const double **) Z, pdinfo, 
 			  theta, V, scale, e, h, npar, nc, pad,
-			  prn);
+			  ifc, prn);
 	if (iters > 0) {
 	    gretl_model_set_int(pmod, "iters", iters);
 	} else if (fnc > 0) {
@@ -468,6 +466,10 @@ garch_driver (const int *list, double **Z, double scale,
 	free(X);
     }
 
+    if (err && !pmod->errcode) {
+	pmod->errcode = err;
+    }
+
     return err;
 }
 
@@ -538,7 +540,7 @@ garchpar_from_armapar (const double *armapar, int q, int p,
 
     for (i=1; i<=q; i++) {
 	x = armapar[p+i];
-	vparm[p+i] = (x > 0.0)? 0 : -x;
+	vparm[p+i] = (x > 0.0)? 0.0001 : -x;
 	sum_ab += vparm[p+i];
     }
 
@@ -561,7 +563,7 @@ garch_init_by_arma (const MODEL *pmod, const int *glist,
 		    double ***pZ, DATAINFO *pdinfo,
 		    double scale, double *vparm)
 {
-    int q = glist[1], p = glist[2];
+    int p = glist[1], q = glist[2];
     int v = pdinfo->v;
     int *list = NULL;
     int err = 0;
@@ -593,7 +595,7 @@ garch_init_by_arma (const MODEL *pmod, const int *glist,
 	err = amod.errcode;
 	if (!err) {
 	    model_count_minus();
-	    garchpar_from_armapar(amod.coeff, q, p, vparm);
+	    garchpar_from_armapar(amod.coeff, p, q, vparm);
 	    for (i=0; i<q+p+1; i++) {
 		fprintf(stderr, "from ARMA: vparm_init[%d] = %#12.6g\n", i, 
 			vparm[i]);
@@ -618,84 +620,75 @@ garch_init_by_arma (const MODEL *pmod, const int *glist,
 #define XPOS 5
 
 static int *get_garch_list (const int *list, const double **Z,
-			    const DATAINFO *pdinfo, int *err)
+			    const DATAINFO *pdinfo, 
+			    gretlopt opt, int *ifc, int *err)
 {
     int *glist = NULL;
     int i, p = list[1], q = list[2];
-    int add0 = 1;
+    int cpos = 0;
+    int add0 = 0;
 
     *err = 0;
 
     /* rule out pure AR in variance: the model is unidentified */
     if (p > 0 && q == 0) {
-	gretl_errmsg_set(_("Error in garch command"));
+	gretl_errmsg_set(_("GARCH: p > 0 and q = 0: the model is unidentified"));
 	*err = E_DATA;
 	return NULL;
     }
 
     /* rule out excessive total GARCH-iness */
     if (p + q > 5) {
-	gretl_errmsg_set(_("Error in garch command"));
+	gretl_errmsg_set(_("GARCH: p + q must be less than 6"));
 	*err = E_DATA;
 	return NULL;
     }
 
     /* check for presence of constant among regressors */
     for (i=XPOS; i<=list[0]; i++) {
-	if (list[i] == 0 || true_const(list[i], Z, pdinfo)) {
-	    /* got a constant: OK */
-	    add0 = 0;
+	if (list[i] == 0) {
+	    /* got the constant: OK */
+	    cpos = i;
 	    break;
 	}
     }
 
+    /* OPT_N means don't auto-add a constant */
+    if (cpos == 0 && !(opt & OPT_N)) {
+	add0 = 1;
+    } 
+
+    *ifc = (cpos > 0 || add0);
+
     glist = gretl_list_new(list[0] + add0);
 
     if (glist == NULL) {
 	*err = E_ALLOC;
     } else {
+	int j = 1;
+
 	/* transcribe first portion of original list */
 	for (i=1; i<XPOS; i++) {
-	    glist[i] = list[i];
+	    glist[j++] = list[i];
 	}
 
-	/* insert constant if not already present */
-	if (add0) {
-	    glist[XPOS] = 0;
-	}
+	if (add0 || (cpos > 0 && cpos != XPOS)) {
+	    /* insert constant here if not already present,
+	       or if originally placed later */ 
+	    glist[j++] = 0;
+	} 
 
-	/* transcribe original regressors, if any */
+	/* transcribe the original regressors, if any */
 	for (i=XPOS; i<=list[0]; i++) {
-	    glist[i+add0] = list[i];
+	    if (i == XPOS || list[i] != 0) {
+		glist[j++] = list[i];
+	    }
 	}
     }
 
     return glist;
 }
 
-/* make regression list for initial OLS: we skip three terms 
-   from the GARCH list, namely p, q and the separator 
-   that follows.
-*/
-
-static int *make_ols_list (const int *list, int *err)
-{
-    int *olist;
-    int i;
-
-    olist = gretl_list_new(list[0] - 3);
-
-    if (olist == NULL) {
-	*err = E_ALLOC;
-    } else {
-	for (i=4; i<=list[0]; i++) {
-	    olist[i-3] = list[i];
-	}
-    }
-
-    return olist;
-}
-
 #define GARCH_AUTOCORR_TEST 1
 
 #if GARCH_AUTOCORR_TEST
@@ -774,20 +767,80 @@ static void garch_undo_scaling (int yno, double scale, double **Z,
 
 /* default variance parameter initialization */
 
-static void garch_vparm_init (const int *list, const MODEL *pmod,
+static void garch_vparm_init (const int *list, double sigma,
 			      double *vparm)
 {
     int i, q = list[1], p = list[2];
+    double den = 1.0;
+    double tmp = (q>0) ? 0.2 : 0.8;
+
+    if (p > 0) {
+	for (i=1; i<=p; i++) {
+	    vparm[i] = tmp / p;
+	    den -= vparm[i];
+	}
+    }
 
-    vparm[0] = pmod->sigma * pmod->sigma * 0.1;
     if (q > 0) {
-	vparm[p+1] = 0.7;
+	for (i=p+1; i<=p+q; i++) {
+	    vparm[i] = 0.7 / q;
+	    den -= vparm[i];
+	}
     }
 
-    vparm[1] = ((q == 0)? 0.9 : 0.2) / p;
-    for (i=2; i<=p; i++) {
-	vparm[i] = vparm[1];
+    vparm[0] = sigma * sigma * den;
+}
+
+/* make regression list for initial OLS: we skip three terms 
+   from the GARCH list, namely p, q and the separator 
+   that follows.
+*/
+
+static int *make_ols_list (const int *list, int *err)
+{
+    int *olist;
+    int i;
+
+    olist = gretl_list_new(list[0] - 3);
+
+    if (olist == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (i=4; i<=list[0]; i++) {
+	    olist[i-3] = list[i];
+	}
     }
+
+    return olist;
+}
+
+static MODEL garch_run_ols (const int *list, double ***pZ,
+			    DATAINFO *pdinfo, PRN *prn)
+{
+    int *ols_list;
+    MODEL model;
+    int err = 0;
+
+    ols_list = make_ols_list(list, &err);
+    if (err) {
+	gretl_model_init(&model);
+	model.errcode = err;
+	return model;
+    }
+
+    model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M | OPT_U);
+
+#if 0
+    fprintf(stderr, "errcode=%d, ess=%g, sigma=%g\n",
+	    model.errcode, model.ess, model.sigma);
+    if (!model.errcode) {
+	printmodel(&model, pdinfo, OPT_NONE, prn);
+    }
+#endif
+
+    free(ols_list);
+
+    return model;
 }
 
 /* the driver function for the plugin */
@@ -797,38 +850,28 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 {
     MODEL model;
     int *list = NULL;
-    int *ols_list = NULL;
     double vparm[VPARM_MAX] = {0};
     double LMF = NADBL;
     double pvF = NADBL;
     double scale = 1.0;
-    int yno = 0;
-
-    gretl_model_init(&model);
+    int ifc, yno = 0;
+    int err = 0;
 
     list = get_garch_list(cmdlist, (const double **) *pZ,
-			  pdinfo, &model.errcode);
-    if (model.errcode != 0) {
-	return model;
-    }
-
-    ols_list = make_ols_list(list, &model.errcode);
-    if (model.errcode != 0) {
+			  pdinfo, opt, &ifc, &err);
+    if (err) {
+	gretl_model_init(&model);
+	model.errcode = err;
 	return model;
     }
 
     /* run initial OLS */
-    model = lsq(ols_list, pZ, pdinfo, OLS, OPT_A | OPT_M);
+    model = garch_run_ols(list, pZ, pdinfo, prn);
     if (model.errcode) {
-	free(ols_list);
 	free(list);
 	return model;
     }
 
-#if 0
-    printmodel(&model, pdinfo, OPT_NONE, prn);
-#endif
-
 #if GARCH_AUTOCORR_TEST
     /* pretest the residuals for autocorrelation */
     if (prn != NULL) {
@@ -837,12 +880,12 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 #endif
 
 #if GARCH_SCALE_SIGMA
-    yno = ols_list[1];
+    yno = list[4];
     scale = garch_scale_sigma(yno, &model, *pZ, pdinfo);
 #endif 
 
     /* variance parameter initialization */
-    garch_vparm_init(list, &model, vparm);
+    garch_vparm_init(list, model.sigma, vparm);
 
     if (opt & OPT_A) {
 	/* "--arma-init": try initializing params via ARMA */
@@ -850,7 +893,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     }
 
     garch_driver(list, *pZ, scale, pdinfo, &model, vparm,
-		 opt, prn); 
+		 ifc, opt, prn); 
 
 #if GARCH_SCALE_SIGMA
     garch_undo_scaling(yno, scale, *pZ, pdinfo);
@@ -866,9 +909,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
     }
 #endif
 
-    free(ols_list);
     free(list);
 
     return model;
 }
-
