@@ -270,11 +270,32 @@ static void cancel_mail_settings (GtkWidget *w, struct mail_dialog *md)
     gtk_widget_destroy(md->dlg);
 }
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
+
+static gchar *my_combo_box_get_active_text (GtkComboBox *box)
+{
+    GtkWidget *w = gtk_bin_get_child(GTK_BIN(box));
+    gchar *ret = NULL;
+
+    if (GTK_IS_ENTRY(w)) {
+	const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
+
+	if (s != NULL) {
+	    ret = g_strdup(s);
+	}
+    } 
+
+    return ret;
+}
+
+#endif
+
 static void finalize_mail_settings (GtkWidget *w, struct mail_dialog *md)
 {
     GList *list = NULL;
     struct mail_info *minfo = md->minfo;
     struct msg_info *msg = md->msg;
+    gchar *recip;
     const gchar *txt;
     int err = MAIL_OK;
     int save = 0;
@@ -282,15 +303,20 @@ static void finalize_mail_settings (GtkWidget *w, struct mail_dialog *md)
     list = minfo->addrs;
 
     /* recipient */
-    txt = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(md->recip_combo)->entry));
-    if (txt != NULL && *txt != '\0') {
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
+    recip = my_combo_box_get_active_text(GTK_COMBO_BOX(md->recip_combo));
+#else
+    recip = gtk_combo_box_get_active_text(GTK_COMBO_BOX(md->recip_combo));
+#endif
+
+    if (recip != NULL && *recip != '\0') {
 	int i = 0;
 
-	msg->recip = g_strdup(txt);
+	msg->recip = g_strdup(recip);
 	fprintf(stderr, "targ = '%s'\n", msg->recip);
 	save = 1;
 	while (list) {
-	    if (!strcmp(txt, (char *) list->data)) {
+	    if (!strcmp(recip, (char *) list->data)) {
 		if (i == 0) {
 		    /* current recipient is top of the list already */
 		    save = 0;
@@ -304,12 +330,14 @@ static void finalize_mail_settings (GtkWidget *w, struct mail_dialog *md)
 	    i++;
 	}
 	if (save) {
-	    minfo->addrs = g_list_prepend(minfo->addrs, g_strdup(txt));
+	    minfo->addrs = g_list_prepend(minfo->addrs, g_strdup(recip));
 	} 
     } else {
 	err = MAIL_NO_RECIPIENT;
     }
 
+    g_free(recip);
+
     if (!err) {
 	/* reply-to address */
 	txt = gtk_entry_get_text(GTK_ENTRY(md->reply_entry));
@@ -524,6 +552,16 @@ static void mail_dialog_quit (GtkWidget *w, gpointer p)
     gtk_main_quit();
 }
 
+static void set_combo_strings_from_list (GtkComboBox *box, GList *list)
+{
+    GList *mylist = list;
+
+    while (mylist != NULL) {
+	gtk_combo_box_append_text(box, mylist->data);
+	mylist = mylist->next;
+    }
+}
+
 static void 
 mail_to_dialog (const char *fname, struct mail_info *minfo, 
 		struct msg_info *msg)
@@ -592,9 +630,9 @@ mail_to_dialog (const char *fname, struct mail_info *minfo,
 	gtk_table_attach(GTK_TABLE(tbl), lbl, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
 
 	if (i == 0) {
-	    w = gtk_combo_new();
+	    w = gtk_combo_box_entry_new_text();
 	    if (minfo->addrs != NULL) {
-		gtk_combo_set_popdown_strings(GTK_COMBO(w), minfo->addrs);
+		set_combo_strings_from_list(GTK_COMBO_BOX(w), minfo->addrs);
 	    } 
 	} else {
 	    w = gtk_entry_new();
@@ -621,7 +659,9 @@ mail_to_dialog (const char *fname, struct mail_info *minfo,
 	}
 
 	if (i == 0) {
-	    gtk_entry_set_activates_default(GTK_ENTRY(GTK_COMBO(w)->entry), TRUE);
+	    GtkWidget *entry = gtk_bin_get_child(GTK_BIN(w));
+
+	    gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 	} else {
 	    gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE);
 	}
