@@ -24,10 +24,11 @@
 */
 
 #include "libgretl.h"
-#include "libset.h"    /* unused */
+#include "libset.h"
 #include "garch.h"
 
 #define GDEBUG 0
+#define MOD_DEBUG 0
 
 enum {
     INIT_VAR_THEO,
@@ -48,7 +49,7 @@ struct garch_container_ {
     int t1;              /* beginning of sample */
     int t2;              /* end of sample */
     int nobs;            /* number of observations */
-    int ncm;             /* number of regressors (constant excluded) */
+    int ncm;             /* number of regressors, including the constant */
     int p;               /* GARCH p */
     int q;               /* GARCH q */
     int k;               /* total number of parameters */
@@ -57,7 +58,6 @@ struct garch_container_ {
     double *e;           /* residuals */
     double *e2;          /* squared residuals */
     double *h;           /* conditional variance */
-    int ascore;          /* 1 for analytical score provided */
     double **score_e;    /* derivatives of the residuals wrt the parameters */
     double **score_h;    /* derivatives of the variances wrt the parameters */
     double **blockglue;  /* derivatives of the loglik wrt residuals and variances */
@@ -79,8 +79,6 @@ static void mark (int *n)
 }
 #endif
 
-#define MOD_DEBUG 0
-
 static void free_eh_derivs (garch_container *DH)
 {
     doubles_array_free(DH->score_e, DH->k);
@@ -113,10 +111,10 @@ static int allocate_eh_derivs (garch_container *DH)
 
 static garch_container *
 garch_container_new (const double *y, const double **X, 
-		     int t1, int t2, int nobs, int nc,
+		     int t1, int t2, int nobs, int nc, 
 		     int p, int q, int init_method, 
 		     double *e, double *e2, double *h, 
-		     double scale, int analytical)
+		     double scale)
 {
     garch_container *DH = malloc(sizeof *DH);
 
@@ -129,14 +127,13 @@ garch_container_new (const double *y, const double **X,
     DH->t1 = t1;
     DH->t2 = t2;
     DH->nobs = nobs;
-    DH->ncm = nc - 1;
+    DH->ncm = nc;
     DH->p = p;
     DH->q = q;
     DH->init = init_method;
     DH->e = e;
     DH->e2 = e2;
     DH->h = h;
-    DH->ascore = analytical;
     DH->k = nc + 1 + p + q;
     DH->boundcheck = 1;
     DH->scale = scale;
@@ -146,11 +143,9 @@ garch_container_new (const double *y, const double **X,
     DH->G = NULL;
     DH->blockglue = NULL;
 
-    if (DH->ascore) {
-	if (allocate_eh_derivs(DH)) {
-	    free(DH);
-	    DH = NULL;
-	}
+    if (allocate_eh_derivs(DH)) {
+	free(DH);
+	DH = NULL;
     }
 
     return DH;
@@ -158,9 +153,7 @@ garch_container_new (const double *y, const double **X,
 
 static void garch_container_destroy (garch_container *DH)
 {
-    if (DH->ascore) {
-	free_eh_derivs(DH);
-    }
+    free_eh_derivs(DH);
     free(DH);
 }
 
@@ -197,64 +190,56 @@ static void normal_score (const garch_container *DH)
     }
 } 
 
-/* Compute the GARCH quantities */
-
-static int check_nonnegative (const double *par, int ncm, int k)
+static int params_in_bounds (const double *par, int ncm, int k)
 {
-    int nonzero = (par[ncm+1] >= 0.0);
+    int ok = (par[ncm] >= 0.0); /* omega */
     double sum = 0.0;
     int i;
 
-    for (i=ncm+2; i<k; i++) {
-	nonzero &= (par[i] >= 1.0e-12);
+    for (i=ncm+1; i<k && ok; i++) {
+	ok &= (par[i] >= 1.0e-12);
 	sum += par[i];
-	if (!nonzero) {
-	    break;
-	}
     }
 
-    nonzero &= (sum <= 1.0);
+    ok &= (sum <= 1.0);
 
-    return nonzero;
+    return ok;
 }
 
-static int garch_etht (const double *par, void *ptr)
+/* Compute the GARCH quantities */
+
+static int garch_etht (const double *par, garch_container *DH)
 {
-    garch_container *DH = (garch_container *) ptr;
     double **dedq = DH->score_e;
     double **dhdq = DH->score_h;
+    int ncm = DH->ncm;
     int t1 = DH->t1;
     int t2 = DH->t2;
     int p = DH->p;
     int q = DH->q;
     int maxlag = (p > q)? p : q;
-    int i, j, k, ret = 0;
-    int ncm = DH->ncm;
+    int t0 = t1 - maxlag;
     int t, T = t2 - t1 + 1;
     double et, ht, tmp, h0 = 0.0;
-    double u_var = 0.0;
+    double dh0, u_var = 0.0;
+    int i, j, k;
 
     /* check for nonnegative params */
 
-    if (DH->boundcheck) {
-	ret = !check_nonnegative(par, ncm, DH->k);
-	if (ret) {
-	    return E_DATA;
-	}
+    if (DH->boundcheck && !params_in_bounds(par, ncm, DH->k)) {
+	return E_DATA;
     }
 
     /* compute residuals */
 
     tmp = 0.0;
-    for (t = t1-maxlag; t <= t2; t++) {
+    for (t=t0; t<=t2; t++) {
 	if (t < t1) {
 	    et = 0.0;
 	} else {
-	    et = DH->y[t] - par[0];
-	    if (DH->X != NULL) {
-		for (i=1; i<=ncm; i++) {
-		    et -= DH->X[i][t]*par[i];
-		}
+	    et = DH->y[t];
+	    for (i=0; i<ncm; i++) {
+		et -= DH->X[i][t] * par[i];
 	    }
 	    DH->e[t] = et;
 	    DH->e2[t] = et * et;
@@ -262,195 +247,155 @@ static int garch_etht (const double *par, void *ptr)
 	}
     }
 
-    if (DH->ascore) {
-	for (t=t-maxlag; t<t1; t++) {
-	    for (i=0; i<DH->k; i++) {
-		dedq[i][t] = 0.0;
-	    }
+    for (t=t0; t<t1; t++) {
+	for (i=0; i<DH->k; i++) {
+	    dedq[i][t] = 0.0;
 	}
     }
 	
     /* h0 and derivatives */
 
-    switch (DH->init) {
-    case INIT_VAR_OLS:
+    if (DH->init == INIT_VAR_OLS) {
 	h0 = 1.0;
-	break;
-    case INIT_VAR_RESID:
+    } else if (DH->init == INIT_VAR_RESID) {
 	h0 = tmp / T;
-	break;
-    case INIT_VAR_THEO:
+    } else if (DH->init == INIT_VAR_THEO) {
 	tmp = 1.0;
-	for (i=ncm+2; i<DH->k; i++) {
+	for (i=ncm+1; i<DH->k; i++) {
 	    tmp -= par[i];
 	}
-	u_var = par[ncm+1] / tmp;
-	h0 = u_var;
-	break;
+	h0 = u_var = par[ncm] / tmp;
     }
 
-    for (t=t1-maxlag; t<t1; t++) {
-	DH->h[t] = h0;
-	DH->e2[t] = h0;
+#if GDEBUG
+    fprintf(stderr, "garch_etht: h0 = %g\n", h0);
+#endif
+
+    for (t=t0; t<t1; t++) {
+	DH->h[t] = DH->e2[t] = h0;
     }
 
-    if (DH->ascore) {
-	double dh0;
-
-	switch (DH->init) {
-	case INIT_VAR_OLS:
-	    for (t=t1-maxlag; t<t1; t++) {
-		for (i=0; i<DH->k; i++) {
-		    dhdq[i][t] = 0.0;
-		}
+    if (DH->init == INIT_VAR_OLS) {
+	for (t=t0; t<t1; t++) {
+	    for (i=0; i<DH->k; i++) {
+		dhdq[i][t] = 0.0;
 	    }
-	    break;
-
-	case INIT_VAR_RESID:
+	}
+    } else if (DH->init == INIT_VAR_RESID) {
+	for (i=0; i<ncm; i++) {
 	    dh0 = 0.0;
 	    for (t=t1; t<=t2; t++) {
-		dh0 -= DH->e[t];
+		dh0 -= DH->e[t] * DH->X[i][t];
 	    }
-	    for (t=t1-maxlag; t<t1; t++) {
-		dhdq[0][t] = dh0  * 2.0 / T;
+	    for (t=t0; t<t1; t++) {
+		dhdq[i][t] = dh0 * 2.0 / T;
+	    }
+	}
+	for (t=t0; t<t1; t++) {
+	    for (i=ncm; i<DH->k; i++) {
+		dhdq[i][t] = 0.0;
+	    }
+	}
+    } else if (DH->init == INIT_VAR_THEO) {
+	for (t=t0; t<t1; t++) {
+	    for (i=0; i<ncm; i++) {
+		dhdq[i][t] = 0.0;
+	    }
+	}
+	dh0 = u_var / par[ncm];
+	for (t=t0; t<t1; t++) {
+	    dhdq[ncm][t] = dh0;
+	}
+	dh0 *= u_var;
+	for (t=t0; t<t1; t++) {
+	    for (i=ncm+1; i<DH->k; i++) {
+		dhdq[i][t] = dh0;
 	    }
-	    
-	    for (i=0; i<ncm; i++) {
-		dh0 = 0.0;
-		for (t=t1; t<=t2; t++) {
-		    dh0 -= DH->e[t] * DH->X[i+1][t];
-		}
-		for (t=t1-maxlag; t<t1; t++) {
-		    dhdq[i+1][t] = dh0  * 2.0 / T;
-		}
-	    }
-	    
-	    for (t=t1-maxlag; t<t1; t++) {
-		for (i=ncm+1; i<DH->k; i++) {
-		    dhdq[i][t] = 0.0;
-		}
-	    }
-	    
-	    break;
-	    
-	case INIT_VAR_THEO:
-	    for (t=t1-maxlag; t<t1; t++) {
-		for (i=0; i<=ncm; i++) {
-		    dhdq[i][t] = 0.0;
-		}
-	    }
-	    dh0 = u_var / par[ncm+1];
-	    for (t=t1-maxlag; t<t1; t++) {
-		dhdq[ncm+1][t] = dh0;
-	    }
-	    dh0 *= u_var;
-	    for (t=t1-maxlag; t<t1; t++) {
-		for (i=ncm+2; i<DH->k; i++) {
-		    dhdq[i][t] = dh0;
-		}
-	    }
-	    break;
 	}
-	
     }
 
     /* in-sample loop */
 
     for (t=t1; t<=t2; t++) {
-	ht = par[ncm+1];
+	ht = par[ncm]; /* omega */
 
 	for (i=1; i<=q; i++) {
-	    ht += DH->e2[t-i] * par[ncm+i+1];
+	    ht += DH->e2[t-i] * par[ncm+i];
 	}
 
 	for (i=1; i<=p; i++) {
-	    ht += DH->h[t-i] * par[ncm+i+q+1];
+	    ht += DH->h[t-i] * par[ncm+i+q];
 	}
 	
 	DH->h[t] = ht;
 	    
-	if (DH->ascore) {
-	    
-	    /* constant */
-	    dedq[0][t] = -1.0;
-	    k = ncm+1;
-	    dhdq[0][t] = 0.0;
-	    for (i=1; i<=q; i++) {
-		if (t - q < t1 && DH->init == INIT_VAR_RESID) {
-		    dhdq[0][t] += par[k+i] * dhdq[0][t1-1];
+	/* regressors in mean equation */
+	for (i=0; i<ncm; i++) {
+	    dedq[i][t] = -(DH->X[i][t]);
+	    k = ncm;
+	    dhdq[i][t] = 0.0;
+	    for (j=1; j<=q; j++) {
+		if (t - q < t1 && DH->init == INIT_VAR_RESID) { 
+		    /* add INIT_THEO here */
+		    dhdq[i][t] += par[k+j] * dhdq[i][t1-1];
 		} else {	
-		    dhdq[0][t] += 2.0 * par[k+i] * DH->e[t-i] * dedq[0][t-i];
+		    dhdq[i][t] += 2.0 * par[k+j] * DH->e[t-j] * dedq[i][t-j];
 		}
 	    }
+	}
 	    
-	    /* regressors */
-	    for (i=1; i<=ncm; i++) {
-		dedq[i][t] = -(DH->X[i][t]);
-		k = ncm+1;
-		dhdq[i][t] = 0.0;
-		for (j=1; j<=q; j++) {
-		    if (t - q < t1 && DH->init == INIT_VAR_RESID) { 
-			// add INIT_THEO here
-			dhdq[i][t] += par[k+j] * dhdq[i][t1-1];
-		    } else {	
-			dhdq[i][t] += 2.0 * par[k+j] * DH->e[t-j] * dedq[i][t-j];
-		    }
-		}
-	    }
-	    
-	    /* garch params: omega */
-	    dedq[ncm+1][t] = 0.0;
-	    dhdq[ncm+1][t] = 1.0;
-	    if (t - p < t1 && DH->init == INIT_VAR_THEO) {
-		for (i=1; i<=p; i++) {
-		    dhdq[ncm+1][t] += par[ncm+1+i] * dhdq[ncm+1][t1-1];
-		}
-	    }
-	    
-	    /* garch params: alphas */
-	    k = ncm + 2;
-	    for (i=1; i<=q; i++) {
-		dedq[k][t] = 0.0;
-		dhdq[k][t] = DH->e2[t-i];
-		if (t - p < t1 && DH->init == INIT_VAR_THEO) {
-		    for (j=0; j<p; j++) {
-			dhdq[k][t] += par[k+j] * dhdq[k][t1-1];
-		    }
-		}
-		k++;
-	    }
-	    
-	    /* garch params: betas */
-	    k = ncm + p + 2;
+	/* garch params: omega */
+	dedq[ncm][t] = 0.0;
+	dhdq[ncm][t] = 1.0;
+	if (t - p < t1 && DH->init == INIT_VAR_THEO) {
 	    for (i=1; i<=p; i++) {
-		dedq[k][t] = 0.0;
-		dhdq[k][t] = DH->h[t-i];
-		if (t - p < t1 && DH->init == INIT_VAR_THEO) {
-		    for (j=0; j<p; j++) {
-			dhdq[k][t] += par[k+j-p] * dhdq[k][t1-1];
-		    }
-		}
-		k++;
+		dhdq[ncm][t] += par[ncm+i] * dhdq[ncm][t1-1];
 	    }
+	}
 	    
-	    /* "real" recursive part */
-	    for (i=0; i<DH->k; i++) {
-		k = ncm + p + 2;
-		for (j=1; j<=p; j++) {
-		    dhdq[i][t] += par[k++] * dhdq[i][t-j];
+	/* garch params: alphas */
+	k = ncm + 1;
+	for (i=1; i<=q; i++) {
+	    dedq[k][t] = 0.0;
+	    dhdq[k][t] = DH->e2[t-i];
+	    if (t - p < t1 && DH->init == INIT_VAR_THEO) {
+		for (j=0; j<p; j++) {
+		    dhdq[k][t] += par[k+j] * dhdq[k][t1-1];
 		}
 	    }
+	    k++;
+	}
 	    
-	}	
-    }
+	/* garch params: betas */
+	k = ncm + q + 1;
+	for (i=1; i<=p; i++) {
+	    dedq[k][t] = 0.0;
+	    dhdq[k][t] = DH->h[t-i];
+	    if (t - p < t1 && DH->init == INIT_VAR_THEO) {
+		for (j=0; j<p; j++) {
+		    dhdq[k][t] += par[k+j-p] * dhdq[k][t1-1];
+		}
+	    }
+	    k++;
+	}
+	    
+	/* "real" recursive part */
+	for (i=0; i<DH->k; i++) {
+	    k = ncm + q + 1;
+	    for (j=1; j<=p; j++) {
+		dhdq[i][t] += par[k++] * dhdq[i][t-j];
+	    }
+	}
+    }	
     
 #if MOD_DEBUG
     fputs("\n\n", stderr);
+    fputs("garch_etht:\n", stderr);
     for (i=0; i<DH->k; i++) {
-	fprintf(stderr, "garch_etht: par[%d] = %9.6f ", i, par[i]);
+	fprintf(stderr, " par[%d] = %9.6f\n", i, par[i]);
     }
     fputc('\n', stderr);
-    for (t=t1-maxlag; t<=20; t++) {
+    for (t=t0; t<=20; t++) {
 	if (t < t1) {
 	    fputc('*', stderr); 
 	} else {
@@ -460,13 +405,13 @@ static int garch_etht (const double *par, void *ptr)
 	fprintf(stderr, " %8.4f", DH->e[t]);
 	fprintf(stderr, " %8.4f", DH->e2[t]);
 	fprintf(stderr, " %8.4f", DH->h[t]);
-	fprintf(stderr, " %12.8f", dedq[ncm+2][t]);
-	fprintf(stderr, " %12.8f", dhdq[ncm+2][t]);
+	fprintf(stderr, " %12.8f", dedq[ncm][t]);
+	fprintf(stderr, " %12.8f", dhdq[ncm][t]);
 	fputc('\n', stderr);
     }
 #endif
 
-    return ret;
+    return 0;
 } 
 
 static double loglik (const double *theta, void *ptr)
@@ -483,9 +428,8 @@ static double loglik (const double *theta, void *ptr)
     return ll;
 }
 
-static int score_fill_matrices (const double *theta, void *ptr)
+static int score_fill_matrices (const double *theta, garch_container *DH)
 {
-    garch_container *DH = (garch_container *) ptr;
     int i, t, err;
 
     err = garch_etht(theta, DH);
@@ -533,6 +477,7 @@ static gretl_matrix *garch_iinfo (garch_container *DH, int *err)
     gretl_matrix *info;
     double **tmp_info;
     double tmpi, tmpj, tmpx1, tmpx2, x;
+    int ncm = DH->ncm;
     int i, j, t;
 
     info = gretl_matrix_alloc(DH->k, DH->k);
@@ -555,21 +500,21 @@ static gretl_matrix *garch_iinfo (garch_container *DH, int *err)
     }
 
     for (t=DH->t1; t<=DH->t2; t++) {
-	for (i=0; i<=DH->ncm; i++) {
+	for (i=0; i<ncm; i++) {
 	    tmpi = DH->score_h[i][t] / DH->h[t];
-	    tmpx1 = (i==0)? 2.0 : 2.0*(DH->X[i][t]);
+	    tmpx1 = 2.0 * DH->X[i][t];
 	    tmpx1 /= DH->h[t];
 	    for (j=0; j<=i; j++) {
 		tmpj = DH->score_h[j][t] / DH->h[t];
-		tmpx2 = (j==0)? 1.0 : 1.0*(DH->X[j][t]);
+		tmpx2 = DH->X[j][t];
 		x = tmpx1 * tmpx2 + tmpi * tmpj;
 		tmp_info[i][j] += x;
 	    }
 	}
 
-	for (i=DH->ncm+1; i<DH->k; i++) {
+	for (i=ncm; i<DH->k; i++) {
 	    tmpi = DH->score_h[i][t] / DH->h[t];
-	    for (j=DH->ncm+1; j<=i; j++) {
+	    for (j=ncm; j<=i; j++) {
 		tmpj = DH->score_h[j][t] / DH->h[t];
 		x = tmpi * tmpj;
 		tmp_info[i][j] += x;
@@ -579,9 +524,9 @@ static gretl_matrix *garch_iinfo (garch_container *DH, int *err)
 
     for (i=0; i<DH->k; i++) {
 	for (j=0; j<=i; j++) {
-	    gretl_matrix_set(info, i, j, 0.5*tmp_info[i][j]);
+	    gretl_matrix_set(info, i, j, 0.5 * tmp_info[i][j]);
 	    if (j < i) {
-		gretl_matrix_set(info, j, i, 0.5*tmp_info[i][j]);
+		gretl_matrix_set(info, j, i, 0.5 * tmp_info[i][j]);
 	    }
 	}
     }
@@ -655,68 +600,14 @@ static gretl_matrix *garch_opg (garch_container *DH, int *err)
     return GG;
 }
 
-#define ANALYTICAL_HESSIAN 1
-
-#if ANALYTICAL_HESSIAN
-
 static gretl_matrix * 
 garch_ihess (garch_container *DH, double *theta, int *err)
 {
-    gretl_matrix *H;
-
-    H = garch_analytical_hessian(DH->y, DH->X, DH->t1, DH->t2, DH->nobs, 
-				 DH->ncm + 1, DH->p, DH->q, theta, DH->e, 
-				 DH->e2, DH->h, DH->scale, err);
-
-    return H;
+    return garch_analytical_hessian(DH->y, DH->X, DH->t1, DH->t2, DH->nobs, 
+				    DH->ncm, DH->p, DH->q, theta, DH->e, 
+				    DH->e2, DH->h, DH->scale, err);
 }
 
-#else
-
-static gretl_matrix *
-garch_ihess (garch_container *DH, double *theta, int *err)
-{
-    gretl_matrix *iH;
-    double vij, *V;
-    int i, j, k, npar = DH->k;
-    int tmp = DH->boundcheck;
-
-    iH = gretl_matrix_alloc(npar, npar);
-    if (iH == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    DH->boundcheck = 0;
-    V = numerical_hessian(theta, npar, loglik, DH, err);
-    DH->boundcheck = tmp;
-
-    if (*err) {
-	return NULL;
-    }
-
-    k = 0;
-    for (i=0; i<npar; i++) {
-	for (j=i; j<npar; j++) {
-	    vij = V[k++];
-	    gretl_matrix_set(iH, i, j, vij);
-	    if (i != j) {
-		gretl_matrix_set(iH, j, i, vij);
-	    }
-	}
-    }
-
-# if GDEBUG
-    gretl_matrix_print(iH, "Hessian (inverse)");
-# endif
-
-    free(V);
-
-    return iH;
-}
-
-#endif
-
 static int
 garch_covariance_matrix (int vopt, double *theta, garch_container *DH,
 			 gretl_matrix *V) 
@@ -836,7 +727,7 @@ static void test_score (garch_container *DH, double *theta)
 */
 
 int garch_estimate_mod (const double *y, const double **X,
-			int t1, int t2, int nobs, int nc,
+			int t1, int t2, int nobs, int nc, 
 			int p, int q, double *theta,  gretl_matrix *V,
 			double *e, double *e2, double *h,
 			double scale, double *pll, 
@@ -844,45 +735,46 @@ int garch_estimate_mod (const double *y, const double **X,
 			int vopt, PRN *prn)
 {
     garch_container *DH;
-    int npar = nc + 1 + p + q; 
-    int analytical = 1;
-    int err = 0;
+    int npar, err;
 
     /* BFGS apparatus */
-    int maxit = 10000;
+    int maxit = 1000;
     double reltol = 1.0e-13;
 
     DH = garch_container_new(y, X, t1, t2, nobs, nc, p, q, 
 			     INIT_VAR_RESID, e, e2, h, 
-			     scale, analytical);
+			     scale);
 
     if (DH == NULL) {
 	return E_ALLOC;
     }
 
     theta[0] = gretl_mean(t1, t2, y);
-
-#if 0
-    if (DH->ascore) {
-	fputs("\nUsing analytical score\n", stderr);
-    } else {
-	fputs("\nUsing numerical score\n", stderr);
-    } 
-#endif
+    npar = nc + 1 + p + q; 
 
     err = BFGS_max(theta, npar, maxit, reltol, 
 		   fncount, grcount, loglik, C_LOGLIK,
-		   (DH->ascore)? anal_score : NULL, 
-		   DH, (prn != NULL)? OPT_V : OPT_NONE, 
+		   anal_score, DH, (prn != NULL)? OPT_V : OPT_NONE, 
 		   prn);
-    
-    *pll = loglik(theta, DH) - (t2 - t1 + 1) * log(scale);
+
+#if GDEBUG
+    fprintf(stderr, "maxit = %d, fncount = %d, grcount = %d\n",
+	    maxit, *fncount, *grcount);
+#endif
+
+    if (err) {
+	*pll = NADBL;
+    } else {
+	*pll = loglik(theta, DH) - (t2 - t1 + 1) * log(scale);
+    }
     
 #if GDEBUG
     test_score(DH, theta);
 #endif
 
-    err = garch_covariance_matrix(vopt, theta, DH, V);
+    if (!err) {
+	err = garch_covariance_matrix(vopt, theta, DH, V);
+    }
 
     garch_container_destroy(DH);
 
