@@ -22,23 +22,25 @@
 
 #define ODEBUG 0
 
+#define USE_LOGDIFF 1
+
 #define OPROBIT_TOL 1.0e-12
 
 typedef struct op_container_ op_container;
 
 struct op_container_ {
-    int type;         /* model type */
+    int type;         /* model type (probit or logit) */
     gretlopt opt;     /* option flags */
     int *y;           /* dependent variable */
     double **Z;       /* data */
-    int *list;
-    int m;            /* min of y */
-    int M;            /* max of y */
+    int *list;        /* dependent var plus regular regressors */
+    int M;            /* max of (possibly normalized) y */
     int t1;           /* beginning of sample */
     int t2;           /* end of sample */
     int nobs;         /* number of observations */
     int nx;           /* number of explanatory variables */
     int k;            /* total number of parameters */
+    double *theta;    /* real parameter estimates */
     double *ndx;      /* index variable */
     double *dP;       /* probabilities */
     MODEL *pmod;      /* model struct, initially containing OLS */
@@ -81,15 +83,17 @@ static void op_container_destroy (op_container *OC)
     free(OC->list);
     doubles_array_free(OC->G, OC->k);
     free(OC->g);
+    free(OC->theta);
 
     free(OC);
 }
 
-static op_container *op_container_new (int type, double **Z, MODEL *pmod,  
+static op_container *op_container_new (int type, int ndum,
+				       double **Z, MODEL *pmod,  
 				       gretlopt opt)
 {
     op_container *OC;
-    int i, t, nx, vy;
+    int i, t, vy = pmod->list[1];
     int nobs = pmod->nobs;
 
     OC = malloc(sizeof *OC);
@@ -105,13 +109,9 @@ static op_container *op_container_new (int type, double **Z, MODEL *pmod,
     OC->t2 = pmod->t2;
     OC->nobs = nobs;
     OC->k = pmod->ncoeff;
+    OC->M = ndum;
+    OC->nx = OC->k - ndum;
 
-    vy = pmod->list[1];
-    OC->m = (int) gretl_min(OC->t1, OC->t2, Z[vy]);
-    OC->M = (int) gretl_max(OC->t1, OC->t2, Z[vy]);
-    OC->M -= OC->m;
-    nx = OC->k - (OC->M - 1);
-    OC->nx = nx;
     OC->opt = opt;
 
     OC->y = NULL;
@@ -124,13 +124,16 @@ static op_container *op_container_new (int type, double **Z, MODEL *pmod,
     OC->y = malloc(nobs * sizeof *OC->y);
     OC->ndx = malloc(nobs * sizeof *OC->ndx);
     OC->dP = malloc(nobs * sizeof *OC->dP);
-    OC->list = gretl_list_new(1 + nx);
+
+    OC->list = gretl_list_new(1 + OC->nx);
     OC->G = doubles_array_new(OC->k, nobs);
     OC->g = malloc(OC->k * sizeof *OC->g);
+    OC->theta = malloc(OC->k * sizeof *OC->theta);
 
     if (OC->y == NULL || OC->ndx == NULL || 
 	OC->dP == NULL || OC->list == NULL ||
-	OC->G == NULL || OC->g == NULL) {
+	OC->G == NULL || OC->g == NULL ||
+	OC->theta == NULL) {
 	op_container_destroy(OC);
 	return NULL;
     }
@@ -138,44 +141,81 @@ static op_container *op_container_new (int type, double **Z, MODEL *pmod,
     i = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	if (!na(pmod->uhat[t])) {
-	    OC->y[i++] = (int) Z[vy][t] - OC->m;
+	    OC->y[i++] = (int) Z[vy][t];
 	}
     }
 
+    OC->list[1] = vy;
+    for (i=0; i<OC->nx; i++) {
+	OC->list[i+2] = pmod->list[i+2];
+    }
+
 #if ODEBUG
     fprintf(stderr, "nobs = %d\n", OC->nobs);
     fprintf(stderr, "t1-t2 = %d-%d\n", OC->t1, OC->t2);
     fprintf(stderr, "k = %d\n", OC->k);
     fprintf(stderr, "nx = %d\n", OC->nx);
-    fprintf(stderr, "Max(y) = %d\n", OC->M);
-#endif
-
-    OC->list[1] = vy;
-    for (i=1; i<=nx; i++) {
-	OC->list[i+1] = pmod->list[i+1];
-    }
-
-#if ODEBUG > 1
+    fprintf(stderr, "Max(y) = M = %d\n", OC->M);
     printlist(OC->list, "list, in op_container_new");
 #endif
 
     return OC;
 }
 
+static int compute_score (op_container *OC, int yt, 
+			  double ystar0, double ystar1,
+			  double dP, int t, int s)
+{
+    double mills0;
+    double mills1;
+    double dm;
+    int M = OC->M;
+    int i, v;
+
+    if (ystar1 < 6.0 || OC->type == LOGIT) {
+	mills0 = (yt == 0)? 0.0 : densfunc(ystar0, OC->type) / dP;
+	mills1 = (yt == M)? 0.0 : densfunc(ystar1, OC->type) / dP;
+    } else { 
+	/* L'Hopital-based approximation */
+	mills0 = (yt == 0)? 0.0 : -ystar0;
+	mills1 = (yt == M)? 0.0 : -ystar1;
+    }
+
+    dm = mills1 - mills0;
+
+    for (i=0; i<OC->nx; i++) {
+	v = OC->list[i+2];
+	OC->G[i][s] = -dm * OC->Z[v][t];
+	OC->g[i] += OC->G[i][s];
+    }
+
+    for (i=OC->nx; i<OC->k; i++) {
+	OC->G[i][s] = 0.0;
+	if (i == OC->nx + yt - 1) {
+	    OC->G[i][s] = -mills0;
+	    OC->g[i] += OC->G[i][s];
+	}
+	if (i == OC->nx + yt) {
+	    OC->G[i][s] = mills1;
+	    OC->g[i] += OC->G[i][s];
+	}
+    }
+
+    return 0;
+}
+
 static int compute_probs (const double *theta, op_container *OC)
 {
     double m0, m1, ystar0 = 0.0, ystar1 = 0.0;
     int M = OC->M;
     int nx = OC->nx;
     double P0, P1, h, adj, dP;
-    int i, t, s, yt, v;
+    int i, t, s, yt;
     int type = OC->type;
 
-    if (OC->opt & OPT_A) {
-	/* analytical score */
-	for (i=0; i<OC->k; i++) {
-	    OC->g[i] = 0.0;
-	}
+    /* initialize analytical score */
+    for (i=0; i<OC->k; i++) {
+	OC->g[i] = 0.0;
     }
 
     s = 0;
@@ -191,16 +231,13 @@ static int compute_probs (const double *theta, op_container *OC)
 	yt = OC->y[s];
 
 	if (yt == 0) {
-	    ystar1 = OC->ndx[s];
-	} else if (yt == 1) {
-	    m1 = theta[nx];
-	    ystar0 = OC->ndx[s];
-	    ystar1 = OC->ndx[s] + m1;
+	    m0 = theta[nx];
+	    ystar1 = OC->ndx[s] + m0;
 	} else {
-	    m0 = theta[nx + yt - 2];
+	    m0 = theta[nx + yt - 1];
 	    ystar0 = OC->ndx[s] + m0;
 	    if (yt < M) {
-		m1 = theta[nx + yt - 1];
+		m1 = theta[nx + yt];
 		ystar1 = OC->ndx[s] + m1;
 	    }
 	} 
@@ -231,58 +268,73 @@ static int compute_probs (const double *theta, op_container *OC)
 	    return 1;
 	} 
 
-	if (OC->opt & OPT_A) {
-	    double mills0;
-	    double mills1;
-	    double dm;
-
-	    if (ystar1 < 6.0 || OC->type == LOGIT) {
-		mills0 = (yt == 0)? 0.0 : densfunc(ystar0, type) / dP;
-		mills1 = (yt == M)? 0.0 : densfunc(ystar1, type) / dP;
-	    } else { 
-		/* L'Hopital-based approximation */
-		mills0 = (yt == 0)? 0.0 : -ystar0;
-		mills1 = (yt == M)? 0.0 : -ystar1;
-	    }
-
-	    dm = mills1 - mills0;
-
-	    for (i=0; i<nx; i++) {
-		v = OC->list[i+2];
-		OC->G[i][s] = -dm * OC->Z[v][t];
-		OC->g[i] += OC->G[i][s];
-	    }
-
-	    for (i=nx; i<OC->k; i++) {
-		OC->G[i][s] = 0.0;
-		if (i == nx + yt - 2) {
-		    OC->G[i][s] = -mills0;
-		    OC->g[i] += OC->G[i][s];
-		}
-		if (i == nx + yt - 1) {
-		    OC->G[i][s] = mills1;
-		    OC->g[i] += OC->G[i][s];
-		}
-	    }
-	}
+	compute_score(OC, yt, ystar0, ystar1, dP, t, s);
 
 	s++;
-
     }
 
     return 0;
 }
 
+#if USE_LOGDIFF
+
+/* Below: a method for getting around the "non-increasing cut point"
+   issue by construction: the 2nd and higher cut points are
+   represented to BFGS in the form of the log-difference from the
+   previous cut point.  This produces faster convergence in some
+   cases.  Could do with some more testing.
+
+   Note that if we're _not_ doing this monkey business, there's no
+   need for conversion of theta, and the "theta" pointer is just set
+   equal to OC->theta (in do_ordered(), further below).
+*/
+
+static void get_real_theta (op_container *OC, const double *theta)
+{
+    int i;
+
+    if (theta == OC->theta) {
+	return;
+    }
+
+    for (i=0; i<=OC->nx; i++) {
+	OC->theta[i] = theta[i];
+    }
+
+    for (i=OC->nx+1; i<OC->k; i++) {
+	/* retrieve cut point 2 and higher from log-difference form */
+	OC->theta[i] = exp(theta[i]) + OC->theta[i-1];
+#if ODEBUG > 1
+	fprintf(stderr, "BFGS theta[%d] = %g -> theta = %g\n", i, theta[i], OC->theta[i]);
+#endif
+    }
+}
+
+static void make_BFGS_theta (op_container *OC, double *theta)
+{
+    int i;
+
+    for (i=0; i<=OC->nx; i++) {
+	theta[i] = OC->theta[i];
+    }
+
+    for (i=OC->nx+1; i<OC->k; i++) {
+	/* convert cut point 2 and higher to log-difference form */
+	theta[i] = log(OC->theta[i] - OC->theta[i-1]);
+    }
+}
+
+#else
+
 static int bad_cutpoints (const double *theta, const op_container *OC)
 {
     int i;
-    
-    if (theta[OC->nx] <= 0.0) {
-	return 1;
-    }
-    
+
     for (i=OC->nx+1; i<OC->k; i++) {
 	if (theta[i] <= theta[i-1]) {
+#if ODEBUG
+	    fprintf(stderr, "theta[%d]: non-increasing cutpoint!\n", i);
+#endif
 	    return 1;
 	}
     }
@@ -290,19 +342,23 @@ static int bad_cutpoints (const double *theta, const op_container *OC)
     return 0;
 }
 
+#endif /* USE_LOGDIFF, or not */
+
 static double op_loglik (const double *theta, void *ptr)
 {
     op_container *OC = (op_container *) ptr;
     double x, ll = 0.0;
     int err;
     int i, s, t, v;
-    
+
+#if USE_LOGDIFF
+    /* bad cut points not possible */
+    get_real_theta(OC, theta);
+#else
     if (bad_cutpoints(theta, OC)) {
-#if ODEBUG > 1
-	fputs("Non-increasing cutpoint!\n", stderr);
-#endif
 	return NADBL;
     }
+#endif
 
     s = 0;
     for (t=OC->t1; t<=OC->t2; t++) {
@@ -311,16 +367,17 @@ static double op_loglik (const double *theta, void *ptr)
 	}
 	x = 0.0;
 	for (i=0; i<OC->nx; i++) {
+	    /* the independent variables */
 	    v = OC->list[i+2];
-	    x -= theta[i] * OC->Z[v][t];
+	    x -= OC->theta[i] * OC->Z[v][t];
 	}
 	OC->ndx[s++] = x;
-#if ODEBUG > 1
+#if ODEBUG > 2
 	fprintf(stderr, "t = %d, s = %d, x = %g\n", t, s, x);
 #endif
     }
     
-    err = compute_probs(theta, OC);
+    err = compute_probs(OC->theta, OC);
     if (err) {
 	ll = NADBL;
     } else {
@@ -349,6 +406,20 @@ static int op_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 	s[i] = OC->g[i];
     }
 
+#if USE_LOGDIFF
+    for (i=OC->nx; i<npar; i++) {
+	int j;
+
+	for (j=i+1; j<npar; j++) {
+	    /* add effects of changes in subsequent cut points */
+	    s[i] += OC->g[j];
+	}
+	if (i > OC->nx) {
+	    s[i] *= exp(theta[i]); /* apply jacobian */
+	}
+    }
+#endif
+
     return 1;
 }
 
@@ -422,45 +493,18 @@ static int ihess_from_ascore (op_container *OC, double *theta, gretl_matrix *inH
 
     free(g0);
 
-    gretl_matrix_print(inH, "H");
+#if ODEBUG
+    gretl_matrix_print(inH, "inverse of Hessian");
+#endif
 
     err = gretl_invert_symmetric_matrix(inH);
+    if (err) {
+	fprintf(stderr, "ihess_from_ascore: failed to invert numerical Hessian\n");
+    }
 
     return err;
 }
 
-static int 
-numerical_ihess (op_container *OC, double *theta, gretl_matrix *invH)
-{
-    double vij, *V;
-    int i, j, k, npar = OC->k;
-    int err = 0;
-
-    if (invH == NULL) {
-	return E_ALLOC;
-    }
-
-    V = numerical_hessian(theta, npar, op_loglik, OC, &err);
-    if (V == NULL) {
-	return err;
-    }
-
-    k = 0;
-    for (i=0; i<npar; i++) {
-	for (j=i; j<npar; j++) {
-	    vij = V[k++];
-	    gretl_matrix_set(invH, i, j, vij);
-	    if (i != j) {
-		gretl_matrix_set(invH, j, i, vij);
-	    }
-	}
-    }
-
-    free(V);
-
-    return 0;
-}
-
 static int get_pred (op_container *OC, const MODEL *pmod,
 		     double Xb)
 {
@@ -480,7 +524,7 @@ static int get_pred (op_container *OC, const MODEL *pmod,
     return pred;
 }
 
-static double gen_resid (const double *theta, op_container *OC, int t) 
+static double gen_resid (op_container *OC, const double *theta, int t) 
 {
     double ndxt, m0, m1, ystar0, f0, f1;
     double ret, dP, ystar1 = 0.0;
@@ -493,16 +537,13 @@ static double gen_resid (const double *theta, op_container *OC, int t)
     ndxt = OC->ndx[t];
 
     if (yt == 0) {
-	ystar1 = ndxt;
-    } else if (yt == 1) {
-	m1 = theta[nx];
-	ystar0 = ndxt;
-	ystar1 = ndxt + m1;
+	m0 = theta[nx];
+	ystar1 = ndxt + m0;
     } else {
-	m0 = theta[nx + yt - 2];
+	m0 = theta[nx + yt - 1];
 	ystar0 = ndxt + m0;
 	if (yt < M) {
-	    m1 = theta[nx + yt - 1];
+	    m1 = theta[nx + yt];
 	    ystar1 = ndxt + m1;
 	}
     } 
@@ -521,20 +562,21 @@ static double gen_resid (const double *theta, op_container *OC, int t)
     return ret;
 } 
 
-static void fill_model (MODEL *pmod, const DATAINFO *pdinfo, 
+static void fill_model (MODEL *pmod, const int *list,
+			const DATAINFO *pdinfo, 
 			op_container *OC, double *theta, 
-			gretl_matrix *V)
+			gretl_matrix *V, int fnc, int grc)
 {
     int npar = OC->k;
     int nx = OC->nx;
     double x;
-    int i, k, s, t, v;
+    int i, s, t, v;
 
-    pmod->t1 = OC->t1;
-    pmod->t2 = OC->t2;
-    pmod->nobs = OC->nobs;
     pmod->ci = OC->type;
+
     gretl_model_set_int(pmod, "ordered", 1);
+    gretl_model_set_int(pmod, "fncount", fnc);
+    gretl_model_set_int(pmod, "grcount", grc);
 
     pmod->ncoeff = npar;
 
@@ -547,9 +589,6 @@ static void fill_model (MODEL *pmod, const DATAINFO *pdinfo,
 
     for (i=0; i<npar; i++) {
 	pmod->coeff[i] = theta[i];
-#if ODEBUG > 1
-	fprintf(stderr,"theta[%d] = %12.8f\n", i, theta[i]);
-#endif
     }
 
     if (OC->opt & OPT_R) {
@@ -570,26 +609,40 @@ static void fill_model (MODEL *pmod, const DATAINFO *pdinfo,
 	    x += theta[i] * OC->Z[v][t];
 	}
 
-	/* X\hat{beta} */
+	/* yhat = X\hat{beta} */
 	pmod->yhat[t] = x;
 	pred = get_pred(OC, pmod, x); /* should we do anything with this? */
 	/* compute generalized residual */
-	pmod->uhat[t] = gen_resid(theta, OC, s++);
+	pmod->uhat[t] = gen_resid(OC, OC->theta, s++);
     }
 
     pmod->lnL = op_loglik(theta, OC);
+    mle_criteria(pmod, 0);
 
     gretl_model_allocate_params(pmod, npar);
 
     if (pmod->errcode == 0) {
 	for (i=0; i<nx; i++) {
-	    strcpy(pmod->params[i], pdinfo->varname[OC->list[i+2]]);
+	    v = OC->list[i+2];
+	    strcpy(pmod->params[i], pdinfo->varname[v]);
 	}
-	k = 1;
+	s = 1;
 	for (i=nx; i<npar; i++) {
-	    sprintf(pmod->params[i], "cut%d", k++);
+	    sprintf(pmod->params[i], "cut%d", s++);
 	}
     }
+
+    /* trim the model list: remove references to the 'cut'
+       dummy variables */
+    for (i=pmod->list[0]; i>1; i--) {
+	if (!in_gretl_list(list, pmod->list[i])) {
+	    gretl_list_delete_at_pos(pmod->list, i);
+	}
+    }
+
+    if (nx > 0) {
+	gretl_model_set_coeff_separator(pmod, NULL, nx);
+    }
 }
 
 static gretl_matrix *oprobit_vcv (op_container *OC, double *theta, int *err)
@@ -603,13 +656,8 @@ static gretl_matrix *oprobit_vcv (op_container *OC, double *theta, int *err)
 	return NULL;
     }
 
-    if (OC->opt & OPT_A) {
-	/* hessian from analytical score */
-	*err = ihess_from_ascore(OC, theta, V);
-    } else {
-	/* numerical hessian */
-	*err = numerical_ihess(OC, theta, V);
-    }
+    /* hessian from analytical score */
+    *err = ihess_from_ascore(OC, theta, V);
 
     if (!*err && (OC->opt & OPT_R)) {
 	gretl_matrix *GG = NULL;
@@ -643,18 +691,48 @@ static gretl_matrix *oprobit_vcv (op_container *OC, double *theta, int *err)
 	V = NULL;
     }
 
+#if ODEBUG > 1
+    gretl_matrix_print(V, "Covariance matrix");
+#endif
+
     return V;
 }
 
+/* Initialize the cut-points by counting the occurrences of each value
+   of the (normalized) dependent variable, finding the sample
+   proportion (cumulating as we go), and taking the inverse of the
+   normal CDF.  It may well be possible to do better than this.
+*/
+
+static double naive_prob (MODEL *pmod, double **Z, int j,
+			  double *p)
+{
+    const double *y = Z[pmod->list[1]];
+    int t, nj = 0;
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t]) && y[t] == j) {
+	    nj++;
+	}
+    }
+
+    *p += (double) nj / pmod->nobs;
+
+    return normal_cdf_inverse(*p);
+}
+
 /* Main ordered estimation function */
 
-static int do_ordered (int ci, double **Z, DATAINFO *pdinfo, MODEL *pmod,
+static int do_ordered (int ci, int ndum, 
+		       double **Z, DATAINFO *pdinfo, 
+		       MODEL *pmod, const int *list, 
 		       gretlopt opt, PRN *prn)
 {
     op_container *OC;
-    int i, npar;
+    int i, j, npar;
     gretl_matrix *V = NULL;
     double *theta = NULL;
+    double p;
     int err;
 
     /* BFGS apparatus */
@@ -662,58 +740,49 @@ static int do_ordered (int ci, double **Z, DATAINFO *pdinfo, MODEL *pmod,
     int fncount = 0;
     int grcount = 0;
 
-    /* set analytical score option... */
-    opt |= OPT_A;
-
-    /* ...but if not analytical, remove robust option */
-    if (!(opt & OPT_A)) {
-	opt &= ~OPT_R;
-    }
-
-    OC = op_container_new(ci, Z, pmod, opt);
+    OC = op_container_new(ci, ndum, Z, pmod, opt);
     if (OC == NULL) {
 	return E_ALLOC;
     }
 
     npar = OC->k;
 
+#if USE_LOGDIFF
+    /* we need a distinct theta to pass to BFGS */
     theta = malloc(npar * sizeof *theta);
     if (theta == NULL) {
 	op_container_destroy(OC);
 	return E_ALLOC;
     }
+#else
+    /* we only need one 'theta' */
+    theta = OC->theta;
+#endif
 
     for (i=0; i<OC->nx; i++) {
-	theta[i] = pmod->coeff[i];
+	OC->theta[i] = 0.0001;
     }
 
-    if (OC->m != 0) {
-	theta[0] -= OC->m;
+    p = 0.0;
+    for (i=OC->nx, j=0; i<npar; i++, j++) {
+	OC->theta[i] = naive_prob(pmod, Z, j, &p);
     }
 
-    /* 
-       the following is very heuristic, has no sound theoretical 
-       justification but seems to work in practice. So much
-       for the Scientific Method (TM) !!!
-    */
- 
-    for (i=OC->nx; i<npar; i++) {
-	theta[i] = 0.5 * pmod->coeff[i];
-    }
+#if USE_LOGDIFF
+    make_BFGS_theta(OC, theta);
+#endif
 
 #if ODEBUG
-    fputs("Starting values:\n", stderr);
     for (i=0; i<npar; i++) {
-	fprintf(stderr, "theta[%d] = % .12E\n", i, theta[i]);
+	fprintf(stderr, "theta[%d]: 'real' = %g, BFGS = %g\n", i, OC->theta[i], theta[i]);
     }
-    fprintf(stderr, "\ninitial loglikelihood = %12.8f\n", 
+    fprintf(stderr, "\ninitial loglikelihood = %.12g\n", 
 	    op_loglik(theta, OC));
 #endif
 
     err = BFGS_max(theta, npar, maxit, OPROBIT_TOL, 
 		   &fncount, &grcount, op_loglik, C_LOGLIK,
-		   (OC->opt & OPT_A)? op_score : NULL, 
-		   OC, (prn != NULL)? OPT_V : OPT_NONE,
+		   op_score, OC, (prn != NULL)? OPT_V : OPT_NONE,
 		   prn);
 
     if (err) {
@@ -722,106 +791,300 @@ static int do_ordered (int ci, double **Z, DATAINFO *pdinfo, MODEL *pmod,
 	fprintf(stderr, "Number of iterations = %d (%d)\n", fncount, grcount);
     }
 
-    for (i=0; i<npar; i++) {
-	pmod->coeff[i] = theta[i];
-    }
-
-    V = oprobit_vcv(OC, theta, &err);
-
-#if ODEBUG > 1
-    debug_print_matrix(V, "Covariance matrix");
+#if USE_LOGDIFF
+    get_real_theta(OC, theta);
 #endif
 
+    V = oprobit_vcv(OC, OC->theta, &err);
+
     if (!err) {
-	fill_model(pmod, pdinfo, OC, theta, V);
+	fill_model(pmod, list, pdinfo, OC, OC->theta, V, fncount, grcount);
     }
 
  bailout:
 
+#if USE_LOGDIFF
     free(theta);
+#endif
     gretl_matrix_free(V);
     op_container_destroy(OC);
 
     return err;
 }
 
-/* the driver function for the plugin: note, if prn is non-NULL,
-   the verbose option has been selected
+struct sorter {
+    double x;
+    int t;
+};
+
+/* We want to ensure that the values of the dependent variable
+   actually used in the analysis (after dropping any bad
+   observations) form a zero-based series of consecutive 
+   integers.
 */
 
-MODEL ordered_estimate (const int *list, int ci, double ***pZ, DATAINFO *pdinfo,
-			gretlopt opt, PRN *prn) 
+static int maybe_fix_ordered_depvar (MODEL *pmod, double **Z,
+				     const DATAINFO *pdinfo,
+				     double **orig_y, int *ndum)
 {
-    MODEL model;
-    int *newlist = NULL;
-    int *dumlist = NULL;
+    struct sorter *s = NULL;
+    double *sy = NULL;
+    double nexty, bady;
+    int dv = pmod->list[1];
+    int i, t, n = 0;
+    int fixit = 0;
+    int err = 0;
+
+    /* make a copy of the observations on the dep. var. that
+       were actually used in the initial OLS, recording
+       the observation numbers, then sort
+    */
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    n++;
+	}
+    }
+
+    s = malloc(n * sizeof *s);
+    if (s == NULL) {
+	return E_ALLOC;
+    }
+
+    i = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    s[i].x = Z[dv][t];
+	    s[i].t = t;
+	    i++;
+	}
+    }
+
+    qsort(s, n, sizeof *s, gretl_compare_doubles);
+
+    /* normalize to minimum zero */
+    if (s[0].x != 0) {
+	double ymin = s[0].x;
+
+	for (i=0; i<n && s[i].x == ymin; i++) {
+	    s[i].x = 0.0;
+	}
+	fixit = 1;
+    }
+
+    /* ensure that the sorted values increase by steps of one */
+    for (i=1; i<n; i++) {
+	if (s[i].x != s[i-1].x) {
+	    nexty = s[i-1].x + 1;
+	    if (s[i].x != nexty) {
+		bady = s[i].x;
+		while (i < n && s[i].x == bady) {
+		    s[i++].x = nexty;
+		}
+		i--; /* compensate for outer i++ */
+		fixit = 1;
+	    } 
+	} 
+    }
+
+    /* the number of dummies actually used will equal the
+       max of normalized y
+    */
+    *ndum = (int) s[n-1].x;
+
+    if (fixit) {
+	/* the dependent var needs transforming */
+	sy = copyvec(Z[dv], pdinfo->n);
+	if (sy == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    for (i=0; i<n; i++) {
+		sy[s[i].t] = s[i].x;
+	    }
+	    /* back up the original dep. var and replace it for
+	       the duration of the ordered analysis */
+	    *orig_y = Z[dv];
+	    Z[dv] = sy;
+	}
+    }
+
+    if (fixit) {
+	fputs("oprobit: using normalized y\n", stderr);
+    } else {
+	fputs("oprobit: using original y\n", stderr);
+    }
+
+    free(s);
+
+    return err;
+}
+
+static void restore_depvar (double **Z, double *y, int v)
+{
+    free(Z[v]);
+    Z[v] = y;
+}
+
+static int *make_dummies_list (const int *list, 
+			       double ***pZ, DATAINFO *pdinfo,
+			       int *err)
+{
+    int *dumlist = gretl_list_new(1);
+
+    if (dumlist == NULL) {
+	*err = E_ALLOC;
+    } else {
+	dumlist[1] = list[1];
+
+	/* OPT_F -> drop first value */
+	*err = list_dumgenr(&dumlist, pZ, pdinfo, OPT_F);
+	if (*err) {
+	    free(dumlist);
+	    dumlist = NULL;
+	}
+    }
+
+    return dumlist;
+}
+
+static int *make_big_list (const int *list, double ***pZ,
+			   DATAINFO *pdinfo, int **pdumlist,
+			   int *err)
+{
+    int *dumlist;
+    int *biglist;
     int i, k, nv;
 
-    gretl_model_init(&model);
-
-    dumlist = gretl_list_new(1);
+    dumlist = make_dummies_list(list, pZ, pdinfo, err);
     if (dumlist == NULL) {
-	model.errcode = E_ALLOC;
-	return model;
+	return NULL;
     }
 
-    dumlist[1] = list[1];
+    nv = list[0] + dumlist[0];
 
-    model.errcode = list_dumgenr(&dumlist, pZ, pdinfo, OPT_F);
-    if (model.errcode) {
+    biglist = gretl_list_new(nv);
+    if (biglist == NULL) {
 	free(dumlist);
-	return model;
-    }
-
-    /* we drop 2 dummies, not just the first one */
-    nv = list[0] + dumlist[0] - 1;
-
-    newlist = gretl_list_new(nv);
-    if (newlist == NULL) {
-	model.errcode = E_ALLOC;
-	goto bailout;
+	*err = E_ALLOC;
+	return NULL;
     }
 
     k = 1;
     for (i=1; i<=list[0]; i++) {
-	newlist[k++] = list[i];
+	biglist[k++] = list[i];
     }
-    for (i=2; i<=dumlist[0]; i++) {
-	newlist[k++] = dumlist[i];
+    for (i=1; i<=dumlist[0]; i++) {
+	biglist[k++] = dumlist[i];
     }
 
-    /* run initial OLS */
-    model = lsq(newlist, pZ, pdinfo, OLS, OPT_A);
+    *pdumlist = dumlist;
+
+    return biglist;
+}
+
+static void list_purge_const (int *list)
+{
+    int i;
+
+    for (i=2; i<=list[0]; i++) {
+	if (list[i] == 0) {
+	    gretl_list_delete_at_pos(list, i);
+	    return;
+	}
+    }
+}
+
+static int check_for_missing_dummies (MODEL *pmod, int *dlist)
+{
+    int i, dv;
+
+    for (i=1; i<=dlist[0]; i++) {
+	dv = dlist[i];
+	if (!in_gretl_list(pmod->list, dv)) {
+	    fprintf(stderr, "check for missing dummies: var %d is gone!\n", dv);
+	    return E_SINGULAR;
+	}
+    }
+
+    return 0;
+}
+
+/* the driver function for the plugin: note, prn is non-NULL
+   only if the verbose option has been selected
+*/
+
+MODEL ordered_estimate (int *list, int ci, double ***pZ, DATAINFO *pdinfo,
+			gretlopt opt, PRN *prn) 
+{
+    MODEL model;
+    int orig_v = pdinfo->v;
+    double *orig_y = NULL;
+    int *biglist = NULL;
+    int *dumlist = NULL;
+    int ndum = 0;
+
+    gretl_model_init(&model);
+
+    /* remove the constant from the incoming list, if present */
+    list_purge_const(list);
+
+    /* construct augmented regression list, including dummies 
+       for the level of the dependent variable
+    */
+    biglist = make_big_list(list, pZ, pdinfo, &dumlist, &model.errcode);
+    if (model.errcode) {
+	return model;
+    }
+
+    /* run initial OLS, with dummies added */
+    model = lsq(biglist, pZ, pdinfo, OLS, OPT_A);
     if (model.errcode) {
 	fprintf(stderr, "ordered_estimate: initial OLS failed\n");
-	goto bailout;
+	free(dumlist);
+	free(biglist);
+	return model;
     }
 
-    if (model.ifc == 0) {
-	/* we assume the base regression has an intercept included */
-	model.errcode = E_NOCONST;
-	goto bailout;
-    }    
-
-#if ODEBUG > 1
-    pprintf(prn, "oprobit_estimate: initial OLS\n");
-    if (1) {
-	PRN *errprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
-    
-	printmodel(&model, pdinfo, OPT_NONE, errprn);
-	gretl_print_destroy(errprn);
-    }
+#if ODEBUG
+    pprintf(prn, "ordered_estimate: initial OLS\n");
+    printmodel(&model, pdinfo, OPT_S, prn);
 #endif
 
-    /* do the actual ordered probit analysis */
-    if (model.errcode == 0) {
-	model.errcode = do_ordered(ci, *pZ, pdinfo, &model, opt, prn);
+    if (model.list[0] < biglist[0]) {
+	/* One or more regressors were dropped in OLS, most likely due
+	   to collinearity.  We can accept the dropping of
+	   user-selected regressors, but all the added dummies must be
+	   retained, so we'd better investigate.
+	*/
+	model.errcode = check_for_missing_dummies(&model, dumlist);
+    }
+	
+    if (!model.errcode) {
+	/* after accounting for any missing observations, check that
+	   the dependent variable is acceptable 
+	*/
+	model.errcode = maybe_fix_ordered_depvar(&model, *pZ, pdinfo,
+						 &orig_y, &ndum);
     }
 
- bailout:
+    /* do the actual ordered probit analysis */
+    if (!model.errcode) {
+	model.errcode = do_ordered(ci, ndum, *pZ, pdinfo, &model, list, 
+				   opt, prn);
+    }
 
     free(dumlist);
-    free(newlist);
+    free(biglist);
+
+    if (orig_y != NULL) {
+	/* if we messed with the dependent var, put the original back */
+	restore_depvar(*pZ, orig_y, list[1]);
+    }
+
+    if (pdinfo->v > orig_v) {
+	/* clean up any automatically-added dummies */
+	dataset_drop_last_variables(pdinfo->v - orig_v, pZ, pdinfo);
+    }
 
     return model;
 }
