@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.26 2008/04/22 20:25:36 allin Exp $
+     * $Id: stata_import.c,v 1.28 2008/09/19 14:24:01 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -34,54 +34,63 @@
 #include "gretl_string_table.h"
 #include "swap_bytes.h"
 
-enum {
-    CN_TYPE_BIG = 1,
-    CN_TYPE_LITTLE,
-};
-
 #ifdef WORDS_BIGENDIAN
-# define CN_TYPE_NATIVE CN_TYPE_BIG
+# define HOST_ENDIAN G_BIG_ENDIAN
 #else
-# define CN_TYPE_NATIVE CN_TYPE_LITTLE
+# define HOST_ENDIAN G_LITTLE_ENDIAN
 #endif
 
 /* Stata versions */
-#define VERSION_5 0x69
-#define VERSION_6 'l'
-#define VERSION_7 0x6e
-#define VERSION_7SE 111
-#define VERSION_8 113
+#define VERSION_5   0x69
+#define VERSION_6     'l'
+#define VERSION_7   0x6e
+#define VERSION_7SE  111
+#define VERSION_8    113
 
 /* Stata format constants */
 #define STATA_STRINGOFFSET 0x7f
 #define STATA_FLOAT    'f'
 #define STATA_DOUBLE   'd'
-#define STATA_INT      'l'
-#define STATA_SHORTINT 'i'
+#define STATA_LONG     'l'
+#define STATA_INT      'i'
 #define STATA_BYTE     'b'
 
 /* Stata SE format constants */
 #define STATA_SE_STRINGOFFSET 0
 #define STATA_SE_FLOAT    254
 #define STATA_SE_DOUBLE   255
-#define STATA_SE_INT      253
-#define STATA_SE_SHORTINT 252
+#define STATA_SE_LONG     253
+#define STATA_SE_INT      252
 #define STATA_SE_BYTE     251
 
-/* Stata missing value codes */
-#define STATA_FLOAT_NA pow(2.0, 127)
-#define STATA_DOUBLE_NA pow(2.0, 1023)
-#define STATA_INT_NA 2147483647
-#define STATA_SHORTINT_NA 32767
+/* see http://www.stata.com/help.cgi?dta */
+#define STATA_FLOAT_MAX  1.701e+38
+#define STATA_DOUBLE_MAX 8.988e+307
+#define STATA_LONG_MAX   2147483620
+#define STATA_INT_MAX    32740
+
+/* values from R's stataread.c -- these were labeled "*NA" */
+#if 0
+# define STATA_FLOAT_CUT  pow(2.0, 127)
+# define STATA_DOUBLE_CUT pow(2.0, 1023)
+# define STATA_LONG_CUT   2147483647
+#endif
+#define STATA_INT_CUT    32767
+
+/* Stata missing value codes: see http://www.stata.com/help.cgi?dta */
+#define STATA_FLOAT_NA(x)  (x > STATA_FLOAT_MAX)
+#define STATA_DOUBLE_NA(x) (x > STATA_DOUBLE_MAX)
+#define STATA_LONG_NA(i)   (i > STATA_LONG_MAX)
+#define STATA_INT_NA(i)    (i > STATA_INT_MAX)
 
 #define STATA_BYTE_NA(b,v) ((v<8 && b==127) || b>=101)
 
 #define NA_INT -999
 
 /* it's convenient to have these as file-scope globals */
-int stata_version;
-int stata_endian;
-int swapends;
+static int stata_version;
+static int stata_endian;
+static int swapends;
 
 static void bin_error (int *err)
 {
@@ -89,21 +98,25 @@ static void bin_error (int *err)
     *err = 1;
 }
 
-static int read_int (FILE *fp, int naok, int *err)
+/* actually an int (4-byte signed int) */
+
+static int stata_read_long (FILE *fp, int naok, int *err)
 {
     int i;
 
     if (fread(&i, sizeof i, 1, fp) != 1) {
 	bin_error(err);
+	return NA_INT;
     }
+
     if (swapends) {
 	reverse_int(i);
     }
 
-    return ((i == STATA_INT_NA) & !naok ? NA_INT : i);
+    return (STATA_LONG_NA(i) & !naok)? NA_INT : i;
 }
 
-static int read_signed_byte (FILE *fp, int naok, int *err)
+static int stata_read_signed_byte (FILE *fp, int naok, int *err)
 { 
     signed char b;
     int ret;
@@ -126,7 +139,7 @@ static int read_signed_byte (FILE *fp, int naok, int *err)
     return ret;
 }
 
-static int read_byte (FILE *fp, int *err)
+static int stata_read_byte (FILE *fp, int *err)
 { 
     unsigned char u;
 
@@ -138,56 +151,61 @@ static int read_byte (FILE *fp, int *err)
     return (int) u;
 }
 
-static int read_short (FILE *fp, int naok, int *err)
+/* actually a short (2-byte signed int) */
+
+static int stata_read_int (FILE *fp, int naok, int *err)
 {
     unsigned first, second;
     int s;
 	
-    first = read_byte(fp, err);
-    second = read_byte(fp, err);
+    first = stata_read_byte(fp, err);
+    second = stata_read_byte(fp, err);
 
-    if (stata_endian == CN_TYPE_BIG) {
+    if (stata_endian == G_BIG_ENDIAN) {
 	s = (first << 8) | second;
     } else {
 	s = (second << 8) | first;
     }
 
-    if (s > STATA_SHORTINT_NA) {
+    if (s > STATA_INT_CUT) { 
+	/* ?? */
 	s -= 65536;
     }
 
-    return ((s == STATA_SHORTINT_NA) & !naok)? NA_INT : s;
+    return (STATA_INT_NA(s) && !naok)? NA_INT : s;
 }
 
-static double read_double (FILE *fp, int *err)
+static double stata_read_double (FILE *fp, int *err)
 {
     double d;
 
     if (fread(&d, sizeof d, 1, fp) != 1) {
 	bin_error(err);
     }
+
     if (swapends) {
 	reverse_double(d);
     }
 
-    return (d == STATA_DOUBLE_NA)? NADBL : d;
+    return (STATA_DOUBLE_NA(d))? NADBL : d;
 }
 
-static double read_float (FILE *fp, int *err)
+static double stata_read_float (FILE *fp, int *err)
 {
     float f;
 
     if (fread(&f, sizeof f, 1, fp) != 1) {
 	bin_error(err);
     }
+
     if (swapends) {
 	reverse_float(f);
     }
 
-    return (f == STATA_FLOAT_NA)? NADBL : (double) f;
+    return (STATA_FLOAT_NA(f))? NADBL : (double) f;
 }
 
-static void read_string (FILE *fp, int nc, char *buf, int *err)
+static void stata_read_string (FILE *fp, int nc, char *buf, int *err)
 {
     if (fread(buf, 1, nc, fp) != nc) {
 	bin_error(err);
@@ -195,10 +213,12 @@ static void read_string (FILE *fp, int nc, char *buf, int *err)
 }
 
 static int 
-get_version_and_namelen (unsigned char u, int *vnamelen)
+stata_get_version_and_namelen (unsigned char u, int *vnamelen)
 {
     int err = 0;
 
+    /* a negative value of stata_version indicates "SE" */
+
     switch (u) {
     case VERSION_5:
         stata_version = 5;
@@ -227,10 +247,17 @@ get_version_and_namelen (unsigned char u, int *vnamelen)
     return err;
 }
 
+static int stata_get_endianness (FILE *fp, int *err)
+{
+    int i = (int) stata_read_byte(fp, err);
+
+    return (i == 0x01)? G_BIG_ENDIAN : G_LITTLE_ENDIAN;
+}
+
 #define stata_type_float(t)  ((stata_version > 0 && t == STATA_FLOAT) || t == STATA_SE_FLOAT)
 #define stata_type_double(t) ((stata_version > 0 && t == STATA_DOUBLE) || t == STATA_SE_DOUBLE)
+#define stata_type_long(t)   ((stata_version > 0 && t == STATA_LONG) || t == STATA_SE_LONG)
 #define stata_type_int(t)    ((stata_version > 0 && t == STATA_INT) || t == STATA_SE_INT)
-#define stata_type_short(t)  ((stata_version > 0 && t == STATA_SHORTINT) || t == STATA_SE_SHORTINT)
 #define stata_type_byte(t)   ((stata_version > 0 && t == STATA_BYTE) || t == STATA_SE_BYTE)
 #define stata_type_string(t) ((stata_version > 0 && t >= STATA_STRINGOFFSET) || t <= 244)
 
@@ -241,17 +268,19 @@ static int check_variable_types (FILE *fp, int *types, int nvar, int *nsv)
     *nsv = 0;
 
     for (i=0; i<nvar && !err; i++) {
-   	unsigned char u = read_byte(fp, &err);
+   	unsigned char u = stata_read_byte(fp, &err);
 
 	types[i] = u;
 	if (stata_type_float(u) || stata_type_double(u)) {
-	    printf("variable %d: float type\n", i);
-	} else if (stata_type_int(u) ||
-		   stata_type_short(u) ||
-		   stata_type_byte(u)) {
-	    printf("variable %d: int type\n", i);
+	    printf("variable %d: float type\n", i+1);
+	} else if (stata_type_long(u)) {
+	    printf("variable %d: long type\n", i+1);
+	} else if (stata_type_int(u)) {
+	    printf("variable %d: int type\n", i+1);
+	} else if (stata_type_byte(u)) {
+	    printf("variable %d: byte type\n", i+1);
 	} else if (stata_type_string(u)) {
-	    printf("variable %d: string type\n", i);
+	    printf("variable %d: string type\n", i+1);
 	    *nsv += 1;
 	} else {
 	    fputs(_("unknown data type"), stderr);
@@ -281,8 +310,8 @@ dta_make_string_table (int *types, int nvar, int ncols)
     for (i=0; i<nvar && j<=list[0]; i++) {
 	if (!stata_type_float(types[i]) &&
 	    !stata_type_double(types[i]) &&
+	    !stata_type_long(types[i]) &&
 	    !stata_type_int(types[i]) &&
-	    !stata_type_short(types[i]) &&
 	    !stata_type_byte(types[i])) {
 	    list[j++] = i + 1;
 	}
@@ -387,7 +416,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     char *txt = NULL; 
     int *off = NULL;
     int err = 0;
-    
+
     labellen = (stata_version == 5)? 32 : 81;
     soffset = (stata_version > 0)? STATA_STRINGOFFSET : STATA_SE_STRINGOFFSET;
     *nvread = nvar;
@@ -395,11 +424,11 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     printf("Max length of labels = %d\n", labellen);
 
     /* data label - zero terminated string */
-    read_string(fp, labellen, datalabel, &err);
+    stata_read_string(fp, labellen, datalabel, &err);
     printf("datalabel: '%s'\n", datalabel);
 
     /* file creation time - zero terminated string */
-    read_string(fp, 18, c18, &err);  
+    stata_read_string(fp, 18, c18, &err);  
     printf("timestamp: '%s'\n", c18);
 
     if (*datalabel != '\0' || *c18 != '\0') {
@@ -428,7 +457,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* names */
     for (i=0; i<nvar && !err; i++) {
-        read_string(fp, namelen + 1, aname, &err);
+        stata_read_string(fp, namelen + 1, aname, &err);
 	printf("variable %d: name = '%s'\n", i+1, aname);
 	if (check_varname(aname) && try_fix_varname(aname)) {
 	    err = 1;
@@ -439,12 +468,12 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* sortlist -- not relevant */
     for (i=0; i<2*(nvar+1) && !err; i++) {
-        read_byte(fp, &err);
+        stata_read_byte(fp, &err);
     }
     
     /* format list (use it to identify date variables?) */
     for (i=0; i<nvar && !err; i++){
-        read_string(fp, 12, c18, &err);
+        stata_read_string(fp, 12, c18, &err);
 	if (*c18 != '\0' && c18[strlen(c18)-1] != 'g') {
 	    printf("variable %d: format = '%s'\n", i+1, c18);
 	    if (!strcmp(c18, "%tm")) {
@@ -463,7 +492,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     /* "value labels": these are stored as the names of label formats, 
        which are themselves stored later in the file. */
     for (i=0; i<nvar && !err; i++) {
-        read_string(fp, namelen + 1, aname, &err);
+        stata_read_string(fp, namelen + 1, aname, &err);
 	if (*aname != '\0') {
 	    printf("variable %d: \"value label\" = '%s'\n", i+1, aname);
 	}
@@ -471,7 +500,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* variable descriptive labels */
     for (i=0; i<nvar && !err; i++) {
-	read_string(fp, labellen, datalabel, &err);
+	stata_read_string(fp, labellen, datalabel, &err);
 	if (*datalabel != '\0') {
 	    printf("variable %d: label = '%s'\n", i+1, datalabel);
 	    if (!g_utf8_validate(datalabel, -1, NULL)) {
@@ -491,20 +520,20 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* variable 'characteristics' -- not handled */
     if (!err) {
-	while (read_byte(fp, &err)) {
+	while (stata_read_byte(fp, &err)) {
 	    if (abs(stata_version) >= 7) { /* manual is wrong here */
-		clen = read_int(fp, 1, &err);
+		clen = stata_read_long(fp, 1, &err);
 	    } else {
-		clen = read_short(fp, 1, &err);
+		clen = stata_read_int(fp, 1, &err);
 	    }
 	    for (i=0; i<clen; i++) {
-		read_signed_byte(fp, 1, &err);
+		stata_read_signed_byte(fp, 1, &err);
 	    }
 	}
 	if (abs(stata_version) >= 7) {
-	    clen = read_int(fp, 1, &err);
+	    clen = stata_read_long(fp, 1, &err);
 	} else {
-	    clen = read_short(fp, 1, &err);
+	    clen = stata_read_int(fp, 1, &err);
 	}
 	if (clen != 0) {
 	    fputs(_("something strange in the file\n"
@@ -521,21 +550,21 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 	    Z[v][t] = NADBL; 
 
 	    if (stata_type_float(types[i])) {
-		Z[v][t] = read_float(fp, &err);
+		Z[v][t] = stata_read_float(fp, &err);
 	    } else if (stata_type_double(types[i])) {
-		Z[v][t] = read_double(fp, &err);
-	    } else if (stata_type_int(types[i])) {
-		ix = read_int(fp, 0, &err);
+		Z[v][t] = stata_read_double(fp, &err);
+	    } else if (stata_type_long(types[i])) {
+		ix = stata_read_long(fp, 0, &err);
 		Z[v][t] = (ix == NA_INT)? NADBL : ix;
-	    } else if (stata_type_short(types[i])) {
-		ix = read_short(fp, 0, &err);
+	    } else if (stata_type_int(types[i])) {
+		ix = stata_read_int(fp, 0, &err);
 		Z[v][t] = (ix == NA_INT)? NADBL : ix;
 	    } else if (stata_type_byte(types[i])) {
-		ix = read_signed_byte(fp, 0, &err);
+		ix = stata_read_signed_byte(fp, 0, &err);
 		Z[v][t] = (ix == NA_INT)? NADBL : ix;
 	    } else {
 		clen = types[i] - soffset;
-		read_string(fp, clen, strbuf, &err);
+		stata_read_string(fp, clen, strbuf, &err);
 		strbuf[clen] = 0;
 #if 0
 		printf("Z[%d][%d] = '%s'\n", v, t, strbuf);
@@ -568,32 +597,32 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 		break;
 	    }
 
-	    read_string(fp, namelen + 1, aname, &err);
+	    stata_read_string(fp, namelen + 1, aname, &err);
 	    printf("variable %d: \"aname\" = '%s'\n", i, aname);
 
 	    /* padding */
-	    read_byte(fp, &err);
-	    read_byte(fp, &err);
-	    read_byte(fp, &err);
+	    stata_read_byte(fp, &err);
+	    stata_read_byte(fp, &err);
+	    stata_read_byte(fp, &err);
 
-	    nlabels = read_int(fp, 1, &err);
-	    totlen = read_int(fp, 1, &err);
+	    nlabels = stata_read_long(fp, 1, &err);
+	    totlen = stata_read_long(fp, 1, &err);
 
 	    off = malloc(nlabels * sizeof *off);
 
 	    for (i=0; i<nlabels && !err; i++) {
-		off[i] = read_int(fp, 1, &err);
+		off[i] = stata_read_long(fp, 1, &err);
 		printf("label offset %d = %d\n", i, off[i]);
 	    }
 
 	    for (i=0; i<nlabels && !err; i++) {
-		double lev = (double) read_int(fp, 0, &err);
+		double lev = (double) stata_read_long(fp, 0, &err);
 
 		printf("level %d = %g\n", i, lev);
 	    }
 
 	    txt = calloc(totlen, 1);
-	    read_string(fp, totlen, txt, &err);
+	    stata_read_string(fp, totlen, txt, &err);
 	    for (i=0; i<nlabels; i++) {
 		printf("label %d = '%s'\n", i, txt + off[i]);
 	    }
@@ -613,10 +642,10 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     unsigned char u;
     int err = 0;
     
-    u = read_byte(fp, &err);   /* release version */
+    u = stata_read_byte(fp, &err);   /* release version */
 
     if (!err) {
-	err = get_version_and_namelen(u, namelen);
+	err = stata_get_version_and_namelen(u, namelen);
     }
 
     if (err) {
@@ -627,19 +656,20 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     printf("Stata file version %d\n", abs(stata_version));
 
     /* these are file-scope globals */
-    stata_endian = (int) read_byte(fp, &err);
-    swapends = stata_endian != CN_TYPE_NATIVE;
+    stata_endian = stata_get_endianness(fp, &err);
+    swapends = stata_endian != HOST_ENDIAN;
 
-    read_byte(fp, &err);              /* filetype -- junk */
-    read_byte(fp, &err);              /* padding */
-    *nvar = read_short(fp, 1, &err);  /* number of variables */
-    *nobs = read_int(fp, 1, &err);    /* number of observations */
+    stata_read_byte(fp, &err);              /* filetype -- junk */
+    stata_read_byte(fp, &err);              /* padding */
+    *nvar = stata_read_int(fp, 1, &err);    /* number of variables */
+    *nobs = stata_read_long(fp, 1, &err);   /* number of observations */
 
     if (!err && (*nvar <= 0 || *nobs <= 0)) {
 	err = 1;
     }
 
     if (!err) {
+	printf("endianness: %s\n", (stata_endian == G_BIG_ENDIAN)? "big" : "little");
 	printf("number of variables = %d\n", *nvar);
 	printf("number of observations = %d\n", *nobs);
 	printf("length of varnames = %d\n", *namelen);
