@@ -123,13 +123,13 @@ static void free_zfiles (zfile *zf)
 	z = zfiles->nxt;
 
 	if (zfiles->zname && zfiles->zname != zfiles->name) {
-	    free(zfiles->zname);
+	    g_free(zfiles->zname);
 	}
 	if (zfiles->name) {
-	    free(zfiles->name);
+	    g_free(zfiles->name);
 	}
 	if (zfiles->iname) {
-	    free(zfiles->iname);
+	    g_free(zfiles->iname);
 	}
 	if (zfiles->cextlen && zfiles->cextra && zfiles->cextra != zfiles->extra) {
 	    free(zfiles->cextra);
@@ -307,12 +307,34 @@ static zlist *zlist_entry_new (flist *f)
 
 static void free_zlist_entry (zlist *z)
 {
-    free(z->name);
-    free(z->iname);
-    free(z->zname);
+    g_free(z->name);
+    g_free(z->iname);
+    g_free(z->zname);
     free(z);
 }
 
+static int gretl_file_test (const gchar *fname, GFileTest test)
+{
+#ifdef G_OS_WIN32
+    if (!g_utf8_validate(fname, -1, NULL)) {
+	gchar *altname;
+	gsize bytes;
+	int ret = 0;
+
+	altname = g_locale_to_utf8(fname, -1, NULL, &bytes, NULL);
+	if (altname != NULL) {
+	    ret = g_file_test(altname, test);
+	    g_free(altname);
+	}
+	return ret;
+    } else {
+	return g_file_test(fname, test);
+    }
+#else
+    return g_file_test(fname, test);
+#endif
+}
+
 static int zipfile_write_check (zfile *zf, int task, int *attr)
 {
     FILE *fp = NULL;
@@ -329,7 +351,7 @@ static int zipfile_write_check (zfile *zf, int task, int *attr)
     if (task == ZIP_DO_NEW) {
 	/* overwriting: don't destroy the original archive until we're
 	   fairly confident we have a good replacement */
-	if (g_file_test(zf->fname, G_FILE_TEST_EXISTS)) {
+	if (gretl_file_test(zf->fname, G_FILE_TEST_EXISTS)) {
 	    fmode = "r+";
 	}
     } else if (zfiles != NULL || zf->zstart != 0) {
@@ -508,7 +530,7 @@ static int real_archive_files (const char *targ, const char **filenames,
     /* assemble list of filenames to be added/updated */
     k = 0;
     for (i=0; filenames[i] != NULL; i++) {
-	if (!g_file_test(filenames[i], G_FILE_TEST_EXISTS)) {
+	if (!gretl_file_test(filenames[i], G_FILE_TEST_EXISTS)) {
 	    err = ziperr(ZE_OPEN, "file '%s' was not found", filenames[i]);
 	    goto bailout;
 	}
@@ -751,7 +773,7 @@ int zipfile_archive_files (const char *targ, const char **filenames,
     g_return_val_if_fail(filenames != NULL, 1);
 
     return real_archive_files(targ, filenames, level, opt,
-			      ZIP_DO_ZIP, gerr);
+			      ZIP_DO_NEW, gerr);
 }
 
 static int process_zipfile (zfile *zf, const char *targ, int task)
